@@ -235,57 +235,73 @@ pcb_bool saved_mode = pcb_false;
 
 static void AdjustAttachedBox(void);
 
+
 void pcb_clear_warnings()
 {
+	pcb_rtree_it_t it;
+	pcb_box_t *n;
+	int li;
+	pcb_layer_t *l;
+
 	conf_core.temp.rat_warn = pcb_false;
-	PCB_PIN_ALL_LOOP(PCB->Data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, pin)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, pin);
-			DrawPin(pin);
+
+	for(n = pcb_r_first(PCB->Data->pin_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+			pcb_pin_invalidate_draw((pcb_pin_t *)n);
 		}
 	}
-	PCB_ENDALL_LOOP;
-	PCB_VIA_LOOP(PCB->Data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, via)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, via);
-			DrawVia(via);
+	pcb_r_end(&it);
+
+	for(n = pcb_r_first(PCB->Data->via_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+			pcb_via_invalidate_draw((pcb_pin_t *)n);
 		}
 	}
-	PCB_END_LOOP;
-	PCB_PAD_ALL_LOOP(PCB->Data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, pad)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, pad);
-			DrawPad(pad);
+	pcb_r_end(&it);
+
+	for(n = pcb_r_first(PCB->Data->pad_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+		if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+			PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+			pcb_pad_invalidate_draw((pcb_pad_t *)n);
 		}
 	}
-	PCB_ENDALL_LOOP;
-	PCB_LINE_ALL_LOOP(PCB->Data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, line)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, line);
-			DrawLine(layer, line);
+	pcb_r_end(&it);
+
+	for(li = 0, l = PCB->Data->Layer; li < PCB->Data->LayerN; li++,l++) {
+		for(n = pcb_r_first(l->line_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+			if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+				pcb_line_invalidate_draw(l, (pcb_line_t *)n);
+			}
 		}
-	}
-	PCB_ENDALL_LOOP;
-	PCB_ARC_ALL_LOOP(PCB->Data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, arc)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, arc);
-			DrawArc(layer, arc);
+		pcb_r_end(&it);
+
+		for(n = pcb_r_first(l->arc_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+			if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+				pcb_arc_invalidate_draw(l, (pcb_arc_t *)n);
+			}
 		}
-	}
-	PCB_ENDALL_LOOP;
-	PCB_POLY_ALL_LOOP(PCB->Data);
-	{
-		if (PCB_FLAG_TEST(PCB_FLAG_WARN, polygon)) {
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN, polygon);
-			DrawPolygon(layer, polygon);
+		pcb_r_end(&it);
+
+		for(n = pcb_r_first(l->polygon_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+			if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+				pcb_poly_invalidate_draw(l, (pcb_polygon_t *)n);
+			}
 		}
+		pcb_r_end(&it);
+
+		for(n = pcb_r_first(l->text_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+			if (PCB_FLAG_TEST(PCB_FLAG_WARN, (pcb_any_obj_t *)n)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_WARN, (pcb_any_obj_t *)n);
+				pcb_text_invalidate_draw(l, (pcb_text_t *)n);
+			}
+		}
+		pcb_r_end(&it);
 	}
-	PCB_ENDALL_LOOP;
 
 	pcb_draw();
 }
@@ -602,7 +618,7 @@ void pcb_notify_mode(void)
 				if (pcb_gui->shift_is_pressed())
 					pcb_chg_obj_thermal(PCB_TYPE_VIA, via, via, via, PCB->ThermStyle);
 				pcb_undo_inc_serial();
-				DrawVia(via);
+				pcb_via_invalidate_draw(via);
 				pcb_draw();
 			}
 			break;
@@ -654,7 +670,7 @@ void pcb_notify_mode(void)
 						pcb_undo_add_obj_to_create(PCB_TYPE_ARC, CURRENT, arc, arc);
 						pcb_undo_inc_serial();
 						pcb_added_lines++;
-						DrawArc(CURRENT, arc);
+						pcb_arc_invalidate_draw(CURRENT, arc);
 						pcb_draw();
 						pcb_crosshair.AttachedBox.State = PCB_CH_STATE_THIRD;
 					}
@@ -686,10 +702,18 @@ void pcb_notify_mode(void)
 				/* always re-draw it since I'm too lazy
 				 * to tell if a selected flag changed
 				 */
-				DrawElement(element);
+				pcb_elem_invalidate_draw(element);
 				pcb_draw();
 				pcb_hid_actionl("Report", "Object", NULL);
 			}
+			else if (type == PCB_TYPE_SUBC) {
+				pcb_subc_t *subc = (pcb_subc_t *)ptr2;
+				pcb_flag_change(PCB, PCB_CHGFLG_TOGGLE, PCB_FLAG_LOCK, PCB_TYPE_SUBC, ptr1, ptr2, ptr3);
+
+				DrawSubc(subc);
+				pcb_draw();
+				pcb_hid_actionl("Report", "Subc", NULL);
+			}
 			else if (type != PCB_TYPE_NONE) {
 				pcb_text_t *thing = (pcb_text_t *) ptr3;
 				PCB_FLAG_TOGGLE(PCB_FLAG_LOCK, thing);
@@ -697,7 +721,7 @@ void pcb_notify_mode(void)
 						&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, thing)) {
 					/* this is not un-doable since LOCK isn't */
 					PCB_FLAG_CLEAR(PCB_FLAG_SELECTED, thing);
-					pcb_draw_obj(type, ptr1, ptr2);
+					pcb_draw_obj((pcb_any_obj_t *)ptr2);
 					pcb_draw();
 				}
 				pcb_hid_actionl("Report", "Object", NULL);
@@ -745,7 +769,7 @@ void pcb_notify_mode(void)
 				pcb_added_lines++;
 				pcb_undo_add_obj_to_create(PCB_TYPE_RATLINE, line, line, line);
 				pcb_undo_inc_serial();
-				DrawRat(line);
+				pcb_rat_invalidate_draw(line);
 				pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X;
 				pcb_crosshair.AttachedLine.Point1.Y = pcb_crosshair.AttachedLine.Point2.Y;
 				pcb_draw();
@@ -798,7 +822,7 @@ void pcb_notify_mode(void)
 			}
 
 			if(via)
-				DrawVia(via);
+				pcb_via_invalidate_draw(via);
 
 			pcb_draw();
 			pcb_undo_inc_serial();
@@ -846,7 +870,7 @@ void pcb_notify_mode(void)
 				pcb_added_lines++;
 				pcb_obj_add_attribs(line, PCB->pen_attr);
 				pcb_undo_add_obj_to_create(PCB_TYPE_LINE, CURRENT, line, line);
-				DrawLine(CURRENT, line);
+				pcb_line_invalidate_draw(CURRENT, line);
 				/* place a via if vias are visible, the layer is
 				   in a new group since the last line and there
 				   isn't a pin already here */
@@ -866,7 +890,7 @@ void pcb_notify_mode(void)
 														 2 * conf_core.design.clearance, 0, conf_core.design.via_drilling_hole, NULL, pcb_no_flags())) != NULL) {
 					pcb_obj_add_attribs(via, PCB->pen_attr);
 					pcb_undo_add_obj_to_create(PCB_TYPE_VIA, via, via, via);
-					DrawVia(via);
+					pcb_via_invalidate_draw(via);
 				}
 				/* copy the coordinates */
 				pcb_crosshair.AttachedLine.Point1.X = pcb_crosshair.AttachedLine.Point2.X;
@@ -888,7 +912,7 @@ void pcb_notify_mode(void)
 				pcb_obj_add_attribs(line, PCB->pen_attr);
 				pcb_undo_add_obj_to_create(PCB_TYPE_LINE, CURRENT, line, line);
 				pcb_undo_inc_serial();
-				DrawLine(CURRENT, line);
+				pcb_line_invalidate_draw(CURRENT, line);
 				/* move to new start point */
 				pcb_crosshair.AttachedLine.Point1.X = Note.X;
 				pcb_crosshair.AttachedLine.Point1.Y = Note.Y;
@@ -924,15 +948,19 @@ void pcb_notify_mode(void)
 			int flags = PCB_FLAG_CLEARPOLY;
 			if (conf_core.editor.full_poly)
 				flags |= PCB_FLAG_FULLPOLY;
+			if (conf_core.editor.clear_polypoly)
+				flags |= PCB_FLAG_CLEARPOLYPOLY;
 			if ((polygon = pcb_poly_new_from_rectangle(CURRENT,
 																									 pcb_crosshair.AttachedBox.Point1.X,
 																									 pcb_crosshair.AttachedBox.Point1.Y,
 																									 pcb_crosshair.AttachedBox.Point2.X,
-																									 pcb_crosshair.AttachedBox.Point2.Y, pcb_flag_make(flags))) != NULL) {
+																									 pcb_crosshair.AttachedBox.Point2.Y,
+																									 2 * conf_core.design.clearance,
+																									 pcb_flag_make(flags))) != NULL) {
 				pcb_obj_add_attribs(polygon, PCB->pen_attr);
 				pcb_undo_add_obj_to_create(PCB_TYPE_POLYGON, CURRENT, polygon, polygon);
 				pcb_undo_inc_serial();
-				DrawPolygon(CURRENT, polygon);
+				pcb_poly_invalidate_draw(CURRENT, polygon);
 				pcb_draw();
 			}
 
@@ -956,7 +984,7 @@ void pcb_notify_mode(void)
 																		Note.Y, 0, conf_core.design.text_scale, string, pcb_flag_make(flag))) != NULL) {
 						pcb_undo_add_obj_to_create(PCB_TYPE_TEXT, CURRENT, text, text);
 						pcb_undo_inc_serial();
-						DrawText(CURRENT, text);
+						pcb_text_invalidate_draw(CURRENT, text);
 						pcb_draw();
 					}
 				}
@@ -1034,8 +1062,6 @@ void pcb_notify_mode(void)
 				{
 					pcb_point_t *points = pcb_crosshair.AttachedPolygon.Points;
 					pcb_cardinal_t n = pcb_crosshair.AttachedPolygon.PointN;
-					pcb_polyarea_t *original, *new_hole, *result;
-					pcb_flag_t Flags;
 
 					/* do update of position; use the 'PCB_MODE_LINE' mechanism */
 					pcb_notify_line();
@@ -1108,14 +1134,14 @@ void pcb_notify_mode(void)
 
 					for (i = 0; i < PCB_MAX_ELEMENTNAMES; i++) {
 						if (i == save_n)
-							EraseElementName(e);
+							pcb_elem_name_invalidate_erase(e);
 						pcb_r_delete_entry(PCB->Data->name_tree[i], (pcb_box_t *) & (e->Name[i]));
 						memcpy(&(e->Name[i]), &(estr[i]), sizeof(pcb_text_t));
 						e->Name[i].Element = e;
 						pcb_text_bbox(pcb_font(PCB, 0, 1), &(e->Name[i]));
 						pcb_r_insert_entry(PCB->Data->name_tree[i], (pcb_box_t *) & (e->Name[i]), 0);
 						if (i == save_n)
-							DrawElementName(e);
+							pcb_elem_name_invalidate_draw(e);
 					}
 				}
 			}
