@@ -48,6 +48,7 @@
 #include "obj_subc_op.h"
 #include "layer_grp.h"
 #include "event.h"
+#include "safe_fs.h"
 
 /* ---------------------------------------------------------------------------
  * some local identifiers
@@ -613,9 +614,12 @@ pcb_bool pcb_buffer_copy_to_layout(pcb_board_t *pcb, pcb_coord_t X, pcb_coord_t
 		pcb_layer_t *destlayer = pcb_layer_resolve_binding(pcb, sourcelayer);
 
 		if (destlayer == NULL) {
-#warning TODO: refine this message with some info about the layer details
-			if (!pcb_layer_is_pure_empty(sourcelayer))
-				pcb_message(PCB_MSG_WARNING, "Couldn't resolve a buffer layer on the current board\n");
+			if ((!(sourcelayer->meta.bound.type & PCB_LYT_VIRTUAL)) && (!pcb_layer_is_pure_empty(sourcelayer))) {
+				const char *src_name = sourcelayer->meta.bound.name;
+				if ((src_name == NULL) || (*src_name == '\0'))
+					src_name = "<anonymous>";
+				pcb_message(PCB_MSG_WARNING, "Couldn't resolve buffer layer #%d (%s) on the current board\n", i, src_name);
+			}
 			continue;
 		}
 
@@ -848,7 +852,7 @@ static int pcb_act_PasteBuffer(int argc, const char **argv, pcb_coord_t x, pcb_c
 			{
 				FILE *exist;
 
-				if ((!force) && ((exist = fopen(name, "r")))) {
+				if ((!force) && ((exist = pcb_fopen(name, "r")))) {
 					fclose(exist);
 					if (pcb_gui->confirm_dialog(_("File exists!  Ok to overwrite?"), 0))
 						pcb_save_buffer_elements(name, fmt);
