@@ -40,6 +40,8 @@
 #include "hid_actions.h"
 #include "compat_nls.h"
 #include "obj_all_op.h"
+#include "obj_subc_parent.h"
+#include "obj_term.h"
 
 /* ---------------------------------------------------------------------------
  * some local identifiers
@@ -200,7 +202,7 @@ static pcb_opfunc_t ChangeNonetlistFunctions = {
 static pcb_opfunc_t ChangeJoinFunctions = {
 	pcb_lineop_change_join,
 	pcb_textop_change_join,
-	NULL,
+	pcb_polyop_change_join,
 	NULL,
 	NULL,
 	NULL,
@@ -392,7 +394,7 @@ pcb_opfunc_t ChgFlagFunctions = {
 	pcb_lineop_change_flag,
 	pcb_textop_change_flag,
 	pcb_polyop_change_flag,
-	pcb_pinop_change_flag,
+	pcb_viaop_change_flag,
 	NULL,
 	NULL,
 	pcb_pinop_change_flag,
@@ -405,6 +407,22 @@ pcb_opfunc_t ChgFlagFunctions = {
 	pcb_subcop_change_flag
 };
 
+static pcb_opfunc_t InvalLabelFunctions = {
+	pcb_lineop_invalidate_label,
+	pcb_textop_invalidate_label,
+	pcb_polyop_invalidate_label,
+	pcb_pinop_invalidate_label,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	pcb_arcop_invalidate_label,
+	NULL,
+	NULL,
+	/*pcb_subcop_invalidate_flag*/ NULL
+};
 
 
 /* ----------------------------------------------------------------------
@@ -438,8 +456,8 @@ pcb_bool pcb_chg_selected_size(int types, pcb_coord_t Difference, pcb_bool fixIt
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = pcb_selected_operation(PCB, &ChangeSizeFunctions, &ctx, pcb_false, types);
 	if (change) {
@@ -460,8 +478,8 @@ pcb_bool pcb_chg_selected_clear_size(int types, pcb_coord_t Difference, pcb_bool
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	if (pcb_mask_on(PCB))
 		change = pcb_selected_operation(PCB, &ChangeMaskSizeFunctions, &ctx, pcb_false, types);
@@ -485,8 +503,8 @@ pcb_bool pcb_chg_selected_2nd_size(int types, pcb_coord_t Difference, pcb_bool f
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = pcb_selected_operation(PCB, &Change2ndSizeFunctions, &ctx, pcb_false, types);
 	if (change) {
@@ -642,8 +660,8 @@ pcb_bool pcb_chg_selected_angle(int types, int is_start, pcb_angle_t Difference,
 
 	ctx.chgangle.pcb = PCB;
 	ctx.chgangle.is_primary = is_start;
-	ctx.chgangle.absolute = (fixIt) ? Difference : 0;
-	ctx.chgangle.delta = Difference;
+	ctx.chgangle.is_absolute = fixIt;
+	ctx.chgangle.value = Difference;
 
 	change = pcb_selected_operation(PCB, &ChangeAngleFunctions, &ctx, pcb_false, types);
 	if (change) {
@@ -664,8 +682,8 @@ pcb_bool pcb_chg_selected_radius(int types, int is_start, pcb_angle_t Difference
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = is_start;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = pcb_selected_operation(PCB, &ChangeRadiusFunctions, &ctx, pcb_false, types);
 	if (change) {
@@ -826,8 +844,8 @@ pcb_bool pcb_chg_obj_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_coor
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = (pcb_object_operation(&ChangeSizeFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
 	if (change) {
@@ -848,8 +866,8 @@ pcb_bool pcb_chg_obj_1st_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = (pcb_object_operation(&Change1stSizeFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
 	if (change) {
@@ -870,8 +888,8 @@ pcb_bool pcb_chg_obj_radius(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int is
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = is_x;
-	ctx.chgsize.absolute = (fixIt) ? r : 0;
-	ctx.chgsize.delta = r;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = r;
 
 	change = (pcb_object_operation(&ChangeRadiusFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
 	if (change) {
@@ -882,7 +900,7 @@ pcb_bool pcb_chg_obj_radius(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int is
 }
 
 /* ---------------------------------------------------------------------------
- * changes the angles of the passed object (e.g. arc start/ctx->chgsize.delta)
+ * changes the angles of the passed object (e.g. arc start/ctx->chgsize.value)
  * Returns pcb_true if anything is changed
  */
 pcb_bool pcb_chg_obj_angle(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int is_start, pcb_angle_t a, pcb_bool fixIt)
@@ -892,8 +910,8 @@ pcb_bool pcb_chg_obj_angle(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int is_
 
 	ctx.chgangle.pcb = PCB;
 	ctx.chgangle.is_primary = is_start;
-	ctx.chgangle.absolute = (fixIt) ? a : 0;
-	ctx.chgangle.delta = a;
+	ctx.chgangle.is_absolute = fixIt;
+	ctx.chgangle.value = a;
 
 	change = (pcb_object_operation(&ChangeAngleFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
 	if (change) {
@@ -915,8 +933,8 @@ pcb_bool pcb_chg_obj_clear_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pc
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	if (pcb_mask_on(PCB))
 		change = (pcb_object_operation(&ChangeMaskSizeFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
@@ -961,8 +979,8 @@ pcb_bool pcb_chg_obj_2nd_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb_
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = (pcb_object_operation(&Change2ndSizeFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
 	if (change) {
@@ -984,8 +1002,8 @@ pcb_bool pcb_chg_obj_mask_size(int Type, void *Ptr1, void *Ptr2, void *Ptr3, pcb
 
 	ctx.chgsize.pcb = PCB;
 	ctx.chgsize.is_primary = 1;
-	ctx.chgsize.absolute = (fixIt) ? Difference : 0;
-	ctx.chgsize.delta = Difference;
+	ctx.chgsize.is_absolute = fixIt;
+	ctx.chgsize.value = Difference;
 
 	change = (pcb_object_operation(&ChangeMaskSizeFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL);
 	if (change) {
@@ -1116,7 +1134,7 @@ pcb_bool pcb_chg_obj_square(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int st
 	pcb_opctx_t ctx;
 
 	ctx.chgsize.pcb = PCB;
-	ctx.chgsize.absolute = style;
+	ctx.chgsize.value = style;
 
 	if (pcb_object_operation(&ChangeSquareFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3) != NULL) {
 		pcb_draw();
@@ -1226,6 +1244,8 @@ void *pcb_chg_obj_name_query(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int p
 {
 	char *name = NULL;
 	char msg[513];
+	pcb_any_obj_t *obj = Ptr2;
+	pcb_subc_t *parent_subc;
 
 	/* if passed an element name, make it an element reference instead */
 	if (Type == PCB_TYPE_ELEMENT_NAME) {
@@ -1233,6 +1253,17 @@ void *pcb_chg_obj_name_query(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int p
 		Ptr2 = Ptr1;
 		Ptr3 = Ptr1;
 	}
+
+	parent_subc = pcb_obj_parent_subc(obj);
+	if (parent_subc != NULL) {
+		name = pcb_gui->prompt_for(_("Enter terminal ID:"), PCB_EMPTY(obj->term));
+		if (name != NULL) {
+			pcb_term_undoable_rename(PCB, obj, name);
+			pcb_draw();
+		}
+		return Ptr3;
+	}
+
 	switch (Type) {
 	case PCB_TYPE_LINE:
 		name = pcb_gui->prompt_for(_("Linename:"), PCB_EMPTY(((pcb_line_t *) Ptr2)->Number));
@@ -1265,6 +1296,10 @@ void *pcb_chg_obj_name_query(int Type, void *Ptr1, void *Ptr2, void *Ptr3, int p
 	case PCB_TYPE_ELEMENT:
 		name = pcb_gui->prompt_for(_("Elementname:"), PCB_EMPTY(PCB_ELEM_NAME_VISIBLE(PCB, (pcb_element_t *) Ptr2)));
 		break;
+
+	case PCB_TYPE_SUBC:
+		name = pcb_gui->prompt_for(_("Subcircuit refdes:"), PCB_EMPTY(((pcb_subc_t *)Ptr2)->refdes));
+		break;
 	}
 	if (name) {
 		/* NB: ChangeObjectName takes ownership of the passed memory */
@@ -1297,3 +1332,10 @@ void pcb_flag_change(pcb_board_t *pcb, pcb_change_flag_t how, pcb_flag_values_t
 
 	pcb_object_operation(&ChgFlagFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3);
 }
+
+void *pcb_obj_invalidate_label(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
+{
+	pcb_opctx_t ctx;
+	ctx.noarg.pcb = PCB;
+	return pcb_object_operation(&InvalLabelFunctions, &ctx, Type, Ptr1, Ptr2, Ptr3);
+}
