@@ -15,3 +15,6 @@ int pcb_tempfile_unlink(char *name);
 
 /* Return non-zero if path is a directory */
 int pcb_is_dir(const char *path);
+
+/* Return non-zero if fn is an absolute path */
+int pcb_is_path_abs(const char *fn);
