@@ -36,6 +36,7 @@
 #include "paths.h"
 #include "compat_fs.h"
 #include "compat_misc.h"
+#include "safe_fs.h"
 
 /* conf list node's name */
 const char *conf_list_name = "pcb-rnd-conf-v1";
@@ -124,10 +125,11 @@ int conf_load_as(conf_role_t role, const char *fn, int fn_is_text)
 	if (d == NULL) {
 		FILE *f;
 		char *efn;
-		pcb_path_resolve(fn, &efn, 0);
-		f = fopen(efn, "r");
+
+		f = pcb_fopen_fn(fn, "r", &efn);
 		if (f != NULL) { /* warn only if the file is there - missing file is normal */
-			pcb_message(PCB_MSG_ERROR, "error: failed to load lht config: %s\n", efn);
+			pcb_message(PCB_MSG_ERROR, "error: failed to load lht config: %s (%s)\n", fn, efn);
+			free(efn);
 			fclose(f);
 		}
 		free(efn);
@@ -327,9 +329,10 @@ const char *conf_get_user_conf_name()
 
 const char *conf_get_project_conf_name(const char *project_fn, const char *pcb_fn, const char **try)
 {
-	static char res[PCB_PATH_MAX];
+	static char res[PCB_PATH_MAX+1];
 	static const char *project_name = "project.lht";
 	FILE *f;
+	char *efn;
 
 	if (project_fn != NULL) {
 		strncpy(res, project_fn, sizeof(res)-1);
@@ -360,9 +363,12 @@ const char *conf_get_project_conf_name(const char *project_fn, const char *pcb_f
 
 	check:;
 	*try = res;
-	f = fopen(res, "r");
+	f = pcb_fopen_fn(res, "r", &efn);
 	if (f != NULL) {
 		fclose(f);
+		strncpy(res, efn, sizeof(res)-1);
+		res[sizeof(res)-1] = '\0';
+		free(efn);
 		return res;
 	}
 
@@ -791,7 +797,7 @@ static void conf_field_clear(conf_native_t *f)
 }
 
 int conf_rev = 0;
-void conf_update(const char *path)
+void conf_update(const char *path, int arr_idx)
 {
 	conf_native_t *n;
 
@@ -799,8 +805,8 @@ void conf_update(const char *path)
 	if (path == NULL) {
 		htsp_entry_t *e;
 		conf_fields_foreach(e) {
-			conf_hid_global_cb((conf_native_t *)e->value, val_change_pre);
-			conf_hid_local_cb((conf_native_t *)e->value, val_change_pre);
+			conf_hid_global_cb((conf_native_t *)e->value, -1, val_change_pre);
+			conf_hid_local_cb((conf_native_t *)e->value, -1, val_change_pre);
 			conf_field_clear(e->value);
 		}
 	}
@@ -813,6 +819,7 @@ void conf_update(const char *path)
 			/* It might be an array element - truncate */
 			field = strrchr(path_, '[');
 			if (field != NULL) {
+				arr_idx = atoi(field+1);
 				*field = '\0';
 				n = conf_get_field(path_);
 			}
@@ -832,14 +839,14 @@ void conf_update(const char *path)
 
 			/* if a valid node is found, update it */
 			if (n != NULL)
-				conf_update(path_);
+				conf_update(path_, arr_idx);
 
 			free(path_);
 			return;
 		}
 		conf_field_clear(n);
-		conf_hid_global_cb(n, val_change_pre);
-		conf_hid_local_cb(n, val_change_pre);
+		conf_hid_global_cb(n, arr_idx, val_change_pre);
+		conf_hid_local_cb(n, arr_idx, val_change_pre);
 	}
 
 	/* merge all memory-lht data to memory-bin */
@@ -849,13 +856,13 @@ void conf_update(const char *path)
 	if (path == NULL) {
 		htsp_entry_t *e;
 		conf_fields_foreach(e) {
-			conf_hid_local_cb((conf_native_t *)e->value, val_change_post);
-			conf_hid_global_cb((conf_native_t *)e->value, val_change_post);
+			conf_hid_local_cb((conf_native_t *)e->value, -1, val_change_post);
+			conf_hid_global_cb((conf_native_t *)e->value, -1, val_change_post);
 		}
 	}
 	else {
-		conf_hid_local_cb(n, val_change_post);
-		conf_hid_global_cb(n, val_change_post);
+		conf_hid_local_cb(n, arr_idx, val_change_post);
+		conf_hid_global_cb(n, arr_idx, val_change_post);
 	}
 	conf_rev++;
 }
@@ -986,7 +993,7 @@ void conf_load_project(const char *project_fn, const char *pcb_fn)
 			pc = NULL;
 	if (pc == NULL)
 		conf_reset(CFR_PROJECT, "<conf_load_project>");
-	conf_update(NULL);
+	conf_update(NULL, -1);
 }
 
 void conf_reg_field_(void *value, int array_size, conf_native_type_t type, const char *path, const char *desc, conf_flag_t flags)
@@ -1279,7 +1286,7 @@ int conf_set(conf_role_t target, const char *path, int arr_idx, const char *new_
 	res = conf_set_dry(target, path, arr_idx, new_val, pol);
 	if (res < 0)
 		return res;
-	conf_update(path);
+	conf_update(path, arr_idx);
 	return 0;
 }
 
@@ -1289,7 +1296,7 @@ int conf_del(conf_role_t target, const char *path, int arr_idx)
 	res = conf_set_dry(target, path, arr_idx, NULL, POL_OVERWRITE);
 	if (res < 0)
 		return res;
-	conf_update(path);
+	conf_update(path, arr_idx);
 	return 0;
 }
 
@@ -1556,12 +1563,10 @@ int conf_save_file(const char *project_fn, const char *pcb_fn, conf_role_t role,
 		}
 	}
 
-	pcb_path_resolve(fn, &efn, 0);
-
 	if (r != NULL) {
 		FILE *f;
-		f = fopen(efn, "w");
 
+		f = pcb_fopen_fn(fn, "w", &efn);
 		if ((f == NULL) && (role == CFR_USER)) {
 			/* create the directory and try again */
 			char *path = pcb_strdup(efn), *end;
@@ -1570,7 +1575,7 @@ int conf_save_file(const char *project_fn, const char *pcb_fn, conf_role_t role,
 				*end = '\0';
 				if (pcb_mkdir(path, 0755) == 0) {
 					pcb_message(PCB_MSG_INFO, "Created directory %s for saving %s\n", path, fn);
-					f = fopen(efn, "w");
+					f = pcb_fopen(efn, "w");
 				}
 				else
 					pcb_message(PCB_MSG_ERROR, "Error: failed to create directory %s for saving %s\n", path, efn);
@@ -1602,7 +1607,7 @@ int conf_export_to_file(const char *fn, conf_role_t role, const char *conf_path)
 	if (at == NULL)
 		return -1;
 
-	f = fopen(fn, "w");
+	f = pcb_fopen(fn, "w");
 	if (f == NULL)
 		return -1;
 
@@ -1869,3 +1874,65 @@ void conf_setf(conf_role_t role, const char *path, int idx, const char *fmt, ...
 
 	free(tmp);
 }
+
+int pcb_conf_cmd_is_safe_(const char *path_, const char *value, const char **val_out, int msg)
+{
+	const char *reason;
+	conf_native_t *nd;
+	char *path, *s;
+
+	if (val_out != NULL)
+		*val_out = NULL;
+
+	if (value == NULL)
+		goto accept;
+
+	path = pcb_strdup(path_);
+	for(s = path; *s != '\0'; s++)
+		if (*s == '.')
+			*s = '/';
+	nd = conf_get_field(path);
+	free(path);
+
+	if (nd == NULL) {
+		if (msg)
+			pcb_message(PCB_MSG_ERROR, "pcb_conf_cmd_is_safe(): invalid node path '%s' (looks like an internal error)\n", path_);
+		return 0;
+	}
+
+	if (nd->array_size > 1) {
+		if (msg)
+			pcb_message(PCB_MSG_ERROR, "pcb_conf_cmd_is_safe(): invalid node path '%s' (it is an array)\n", path_);
+		return 0;
+	}
+
+	switch(conf_lookup_role(nd->prop[0].src)) {
+		/* these are considered safe, because: */
+		case CFR_INTERNAL: /* the attacker would have access to the source code and compilation, could place system() anyway */
+		case CFR_SYSTEM:   /* system admin - lets trust them */
+		case CFR_USER:     /* user config is written by the user, attackers have no better chance to overwrite that than overwriting the the shell's rc */
+		case CFR_CLI:      /* command line arguments: the user specified those; who has access to that potentially has access to the shell anyway */
+			goto accept;
+
+		/* these are considered unsafe, because: */
+		case CFR_DEFAULTPCB: /* the default pcb path may be manipulated; the user has the chance to specify the command path setting from safe config files */
+			reason = "default pcb"; break;
+		case CFR_ENV: /* env vars may be inherited from who-knows-where */
+			reason = "env var"; break;
+		case CFR_PROJECT: /* malicous file prepared by the attacker */
+			reason = "project file"; break;
+		case CFR_DESIGN: /* malicous file prepared by the attacker */
+			reason = "board file"; break;
+		default:
+			reason = "unknown source";
+	}
+
+	if (msg)
+		pcb_message(PCB_MSG_ERROR, "pcb_conf_cmd_is_safe(): refusing to use the value of '%s' because it is from unsafe source %s\n", path_, reason);
+	return 0;
+
+	accept:;
+	if (val_out != NULL)
+		*val_out = value;
+	return 1;
+}
