@@ -34,6 +34,7 @@
 #include "crosshair.h"
 #include "data.h"
 #include "draw.h"
+#include "draw_wireframe.h"
 #include "search.h"
 #include "polygon.h"
 #include "hid_actions.h"
@@ -49,6 +50,7 @@
 #include "obj_arc_draw.h"
 #include "obj_text_draw.h"
 #include "obj_arc_ui.h"
+#include "obj_all_op.h"
 
 typedef struct {
 	int x, y;
@@ -114,25 +116,6 @@ void XORPolygon(pcb_polygon_t *polygon, pcb_coord_t dx, pcb_coord_t dy, int dash
 	}
 }
 
-/*-----------------------------------------------------------
- * Draws the outline of an arc
- */
-static void XORDrawArc(pcb_arc_t * arc)
-{
-	pcb_coord_t wid = arc->Thickness / 2;
-	pcb_coord_t x1,y1,x2,y2;
-
-	pcb_arc_get_end(arc, 0, &x1, &y1);
-	pcb_arc_get_end(arc, 1, &x2, &y2);
-
-	pcb_gui->draw_arc(pcb_crosshair.GC, arc->X, arc->Y, arc->Width + wid, arc->Height + wid, arc->StartAngle, arc->Delta);
-	if (wid > pcb_pixel_slop) {
-		pcb_gui->draw_arc(pcb_crosshair.GC, arc->X, arc->Y, arc->Width - wid, arc->Height - wid, arc->StartAngle, arc->Delta);
-		pcb_gui->draw_arc(pcb_crosshair.GC, x1, y1, wid, wid, arc->StartAngle, -180 * SGN(arc->Delta));
-		pcb_gui->draw_arc(pcb_crosshair.GC, x2, y2, wid, wid, arc->StartAngle + arc->Delta, 180 * SGN(arc->Delta));
-	}
-}
-
 /*-----------------------------------------------------------
  * Draws the outline of an attached arc
  */
@@ -165,32 +148,7 @@ static void XORDrawAttachedArc(pcb_coord_t thick)
 	arc.Width = arc.Height = wy;
 	arc.Thickness = thick;
 
-	XORDrawArc(&arc);
-}
-
-/*-----------------------------------------------------------
- * Draws the outline of a line
- */
-void XORDrawAttachedLine(pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, pcb_coord_t thick)
-{
-	pcb_coord_t dx, dy, ox, oy;
-	double h;
-
-	dx = x2 - x1;
-	dy = y2 - y1;
-	if (dx != 0 || dy != 0)
-		h = 0.5 * thick / sqrt(PCB_SQUARE(dx) + PCB_SQUARE(dy));
-	else
-		h = 0.0;
-	ox = dy * h + 0.5 * SGN(dy);
-	oy = -(dx * h + 0.5 * SGN(dx));
-	pcb_gui->draw_line(pcb_crosshair.GC, x1 + ox, y1 + oy, x2 + ox, y2 + oy);
-	if (coord_abs(ox) >= pcb_pixel_slop || coord_abs(oy) >= pcb_pixel_slop) {
-		pcb_angle_t angle = atan2(dx, dy) * 57.295779;
-		pcb_gui->draw_line(pcb_crosshair.GC, x1 - ox, y1 - oy, x2 - ox, y2 - oy);
-		pcb_gui->draw_arc(pcb_crosshair.GC, x1, y1, thick / 2, thick / 2, angle - 180, 180);
-		pcb_gui->draw_arc(pcb_crosshair.GC, x2, y2, thick / 2, thick / 2, angle, 180);
-	}
+	pcb_draw_wireframe_arc(pcb_crosshair.GC,&arc);
 }
 
 /* ---------------------------------------------------------------------------
@@ -301,7 +259,7 @@ static void XORDrawBuffer(pcb_buffer_t *Buffer)
 			PCB_END_LOOP;
 			PCB_TEXT_LOOP(layer);
 			{
-				XORDrawText(text, x, y);
+				pcb_text_draw_xor(text, x, y);
 			}
 			PCB_END_LOOP;
 			/* the tmp polygon has n+1 points because the first
@@ -359,8 +317,10 @@ static void XORDrawInsertPointObject(void)
  */
 static void XORDrawMoveOrCopy(void)
 {
-	pcb_coord_t dx = pcb_crosshair.X - pcb_crosshair.AttachedObject.X, dy = pcb_crosshair.Y - pcb_crosshair.AttachedObject.Y;
-
+	pcb_coord_t dx = pcb_crosshair.X - pcb_crosshair.AttachedObject.X;
+	pcb_coord_t dy = pcb_crosshair.Y - pcb_crosshair.AttachedObject.Y;
+	int event_sent = 0;
+	
 	switch (pcb_crosshair.AttachedObject.Type) {
 	case PCB_TYPE_VIA:
 		{
@@ -374,29 +334,36 @@ static void XORDrawMoveOrCopy(void)
 			/* We move a local copy of the line -the real line hasn't moved, 
 			 * only the preview.
 			 */			
-			int moved = 0;
+			int constrained = 0;
 			pcb_line_t line;
+			
+			int dx1 = dx, dx2 = dx;
+			int dy1 = dy, dy2 = dy;
+			
 			memcpy(&line, (pcb_line_t *) pcb_crosshair.AttachedObject.Ptr2, sizeof(line));
 
-			pcb_event(PCB_EVENT_RUBBER_CONSTRAIN_MAIN_LINE, "pp", &line, &moved);
-			
-			if (!moved)
-			{
-				line.Point1.X += dx;
-				line.Point1.Y += dy;
-				line.Point2.X += dx;
-				line.Point2.Y += dy;
-			}
+			if(conf_core.editor.rubber_band_keep_midlinedir)
+				pcb_event(PCB_EVENT_RUBBER_CONSTRAIN_MAIN_LINE, "pppppp", &line, &constrained, &dx1, &dy1, &dx2, &dy2);
+			pcb_event(PCB_EVENT_RUBBER_MOVE_DRAW, "icccc", constrained, dx1, dy1, dx2, dy2);
 
-			XORDrawAttachedLine(line.Point1.X, line.Point1.Y,
-								line.Point2.X, line.Point2.Y, line.Thickness);
+			event_sent = 1;
+
+			line.Point1.X += dx1;
+			line.Point1.Y += dy1;
+			line.Point2.X += dx2;
+			line.Point2.Y += dy2;
+
+			pcb_draw_wireframe_line(pcb_crosshair.GC,
+															line.Point1.X, line.Point1.Y,
+															line.Point2.X, line.Point2.Y, 
+															line.Thickness);
 			
 			/* Draw the DRC outline if it is enabled */
 			if (conf_core.editor.show_drc) {
 				pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.cross);
-				XORDrawAttachedLine(line.Point1.X, line.Point1.Y,
-									line.Point2.X, line.Point2.Y,
-									line.Thickness + 2 * (PCB->Bloat + 1) );
+				pcb_draw_wireframe_line(pcb_crosshair.GC,line.Point1.X, line.Point1.Y,
+																line.Point2.X, line.Point2.Y,
+																line.Thickness + 2 * (PCB->Bloat + 1) );
 				pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 			}
 			break;
@@ -406,16 +373,19 @@ static void XORDrawMoveOrCopy(void)
 		{
 			/* Make a temporary arc and move it by dx,dy */
 			pcb_arc_t arc = *((pcb_arc_t *) pcb_crosshair.AttachedObject.Ptr2);
+			pcb_event(PCB_EVENT_RUBBER_MOVE_DRAW, "icccc", 0, dx, dy, dx, dy);
+			event_sent = 1;
+			
 			arc.X += dx;
 			arc.Y += dy;
 
-			XORDrawArc(&arc);
+			pcb_draw_wireframe_arc(pcb_crosshair.GC,&arc);
 
 			/* Draw the DRC outline if it is enabled */
 			if (conf_core.editor.show_drc) {
 				pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.cross);
 				arc.Thickness += 2 * (PCB->Bloat + 1);
-				XORDrawArc(&arc);
+				pcb_draw_wireframe_arc(pcb_crosshair.GC,&arc);
 				pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 			}
 			break;
@@ -445,12 +415,13 @@ static void XORDrawMoveOrCopy(void)
 			point2.Y += dy;				
 
 			if(conf_core.editor.move_linepoint_uses_route == 0) {/* config setting for selecting new 45/90 method */ 
-				XORDrawAttachedLine(point1->X, point1->Y, point2.X, point2.Y, line->Thickness);
+				pcb_draw_wireframe_line(pcb_crosshair.GC,point1->X, point1->Y, point2.X, point2.Y, line->Thickness);
 
 				/* Draw the DRC outline if it is enabled */
 				if (conf_core.editor.show_drc) {
 					pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.cross);
-					XORDrawAttachedLine(point1->X, point1->Y, point2.X, point2.Y,line->Thickness + 2 * (PCB->Bloat + 1) );
+					pcb_draw_wireframe_line(pcb_crosshair.GC,point1->X, point1->Y, point2.X, 
+																	point2.Y,line->Thickness + 2 * (PCB->Bloat + 1) );
 					pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 				}
 			}
@@ -478,6 +449,7 @@ static void XORDrawMoveOrCopy(void)
 
 	case PCB_TYPE_ARC_POINT:
 		pcb_arc_ui_move_or_copy(&pcb_crosshair);
+		event_sent = 1;
 		break;
 
 	case PCB_TYPE_POLYGON_POINT:
@@ -511,7 +483,7 @@ static void XORDrawMoveOrCopy(void)
 	case PCB_TYPE_TEXT:
 		{
 			pcb_text_t *text = (pcb_text_t *) pcb_crosshair.AttachedObject.Ptr2;
-			XORDrawText(text, dx, dy);
+			pcb_text_draw_xor(text, dx, dy);
 			break;
 		}
 
@@ -527,7 +499,8 @@ static void XORDrawMoveOrCopy(void)
 		break;
 	}
 
-	pcb_event(PCB_EVENT_RUBBER_MOVE_DRAW, "cc", dx, dy);
+	if(!event_sent)
+		pcb_event(PCB_EVENT_RUBBER_MOVE_DRAW, "icc", 0, dx, dy );
 }
 
 /* ---------------------------------------------------------------------------
@@ -648,11 +621,8 @@ XORDrawPinViaDRCOutline(pcb_pin_t * pv,pcb_coord_t clearance)
 	if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pv) || (clearance == 0))
 		return;
 	
-	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pv))
-	{
-		
-		if((style==0) || (style==1))
-		{
+	if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pv))	{
+		if((style==0) || (style==1)) {
 			pcb_coord_t w = pv->Thickness;
 			pcb_coord_t l = pv->X - (w/2);
 			pcb_coord_t b = pv->Y - (w/2);
@@ -679,8 +649,7 @@ XORDrawPinViaDRCOutline(pcb_pin_t * pv,pcb_coord_t clearance)
 	}
 	else if (PCB_FLAG_TEST(PCB_FLAG_OCTAGON, pv))
 			draw_pinvia_shape_drc_outline(pcb_crosshair.GC,pv->X,pv->Y,pv->Thickness,clearance,17);
-	else
-	{
+	else {
 			pcb_coord_t r = (pv->Thickness/2)+clearance;
 
 			pcb_gui->set_line_cap(pcb_crosshair.GC, Round_Cap);
@@ -773,6 +742,9 @@ XORDrawPadDRCOutline(pcb_pad_t * pad,pcb_coord_t clearance)
  */
 void pcb_draw_attached(void)
 {
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, 1, NULL);
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, 1, NULL);
+
 	switch (conf_core.editor.mode) {
 	case PCB_MODE_VIA:
 		{
@@ -794,8 +766,8 @@ void pcb_draw_attached(void)
 				XORDrawPinViaDRCOutline(&via,PCB->Bloat);
 				pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 			}
-			break;
 		}
+		break;
 
 		/* the attached line is used by both LINEMODE, PCB_MODE_POLYGON and PCB_MODE_POLYGON_HOLE */
 	case PCB_MODE_POLYGON:
@@ -820,22 +792,20 @@ void pcb_draw_attached(void)
 				XORDrawAttachedArc(conf_core.design.line_thickness + 2 * (PCB->Bloat + 1));
 				pcb_gui->set_color(pcb_crosshair.GC, conf_core.appearance.color.crosshair);
 			}
-
 		}
 		break;
 
 	case PCB_MODE_LINE:
-		if(PCB->RatDraw)
-		{
+		if(PCB->RatDraw) {
 			/* draw only if starting point exists and the line has length */
 			if (pcb_crosshair.AttachedLine.State != PCB_CH_STATE_FIRST && pcb_crosshair.AttachedLine.draw) 
-				XORDrawAttachedLine(pcb_crosshair.AttachedLine.Point1.X,
-														pcb_crosshair.AttachedLine.Point1.Y,
-														pcb_crosshair.AttachedLine.Point2.X,
-														pcb_crosshair.AttachedLine.Point2.Y, 10 );
+				pcb_draw_wireframe_line(pcb_crosshair.GC,
+																pcb_crosshair.AttachedLine.Point1.X,
+																pcb_crosshair.AttachedLine.Point1.Y,
+																pcb_crosshair.AttachedLine.Point2.X,
+																pcb_crosshair.AttachedLine.Point2.Y, 10 );
 		}
-		else if(pcb_crosshair.Route.size > 0)
-		{	
+		else if(pcb_crosshair.Route.size > 0)	{	
 			pcb_route_draw(&pcb_crosshair.Route,pcb_crosshair.GC);
 			if(conf_core.editor.show_drc)
 				pcb_route_draw_drc(&pcb_crosshair.Route,pcb_crosshair.GC);
@@ -867,6 +837,8 @@ void pcb_draw_attached(void)
 		y2 = pcb_crosshair.AttachedBox.Point2.Y;
 		pcb_gui->draw_rect(pcb_crosshair.GC, x1, y1, x2, y2);
 	}
+
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, 1, NULL);
 }
 
 
@@ -881,8 +853,13 @@ void pcb_draw_mark(void)
 	if (!pcb_marked.status)
 		return;
 
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, 1, NULL);
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, 1, NULL);
+
 	pcb_gui->draw_line(pcb_crosshair.GC, pcb_marked.X - ms, pcb_marked.Y - ms, pcb_marked.X + ms, pcb_marked.Y + ms);
 	pcb_gui->draw_line(pcb_crosshair.GC, pcb_marked.X + ms, pcb_marked.Y - ms, pcb_marked.X - ms, pcb_marked.Y + ms);
+
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, 1, NULL);
 }
 
 /* ---------------------------------------------------------------------------
@@ -967,7 +944,7 @@ static pcb_r_dir_t onpoint_line_callback(const pcb_box_t * box, void *cl)
 		op.obj.line = line;
 		vtop_append(&crosshair->onpoint_objs, op);
 		PCB_FLAG_SET(PCB_FLAG_ONPOINT, (pcb_any_obj_t *) line);
-		DrawLine(NULL, line);
+		pcb_line_invalidate_draw(NULL, line);
 		return PCB_R_DIR_FOUND_CONTINUE;
 	}
 	else {
@@ -997,7 +974,7 @@ static pcb_r_dir_t onpoint_arc_callback(const pcb_box_t * box, void *cl)
 		op.obj.arc = arc;
 		vtop_append(&crosshair->onpoint_objs, op);
 		PCB_FLAG_SET(PCB_FLAG_ONPOINT, (pcb_any_obj_t *) arc);
-		DrawArc(NULL, arc);
+		pcb_arc_invalidate_draw(NULL, arc);
 		return PCB_R_DIR_FOUND_CONTINUE;
 	}
 	else {
@@ -1010,15 +987,15 @@ void DrawLineOrArc(int type, void *obj)
 	switch (type) {
 	case PCB_TYPE_LINE_POINT:
 		/* Attention: We can use a NULL pointer here for the layer,
-		 * because it is not used in the DrawLine() function anyways.
-		 * ATM DrawLine() only calls AddPart() internally, which invalidates
+		 * because it is not used in the pcb_line_invalidate_draw() function anyways.
+		 * ATM pcb_line_invalidate_draw() only calls AddPart() internally, which invalidates
 		 * the area specified by the line's bounding box.
 		 */
-		DrawLine(NULL, (pcb_line_t *) obj);
+		pcb_line_invalidate_draw(NULL, (pcb_line_t *) obj);
 		break;
 	case PCB_TYPE_ARC_POINT:
 		/* See comment above */
-		DrawArc(NULL, (pcb_arc_t *) obj);
+		pcb_arc_invalidate_draw(NULL, (pcb_arc_t *) obj);
 		break;
 	}
 }
@@ -1256,9 +1233,6 @@ void pcb_crosshair_grid_fit(pcb_coord_t X, pcb_coord_t Y)
 
 	}
 
-	pcb_event(PCB_EVENT_RUBBER_FIT_CROSSHAIR, "pppp", &pcb_crosshair, &pcb_marked,
-											&nearest_grid_x, &nearest_grid_y);
-
 	snap_data.crosshair = &pcb_crosshair;
 	snap_data.nearest_sq_dist = crosshair_sq_dist(&pcb_crosshair, nearest_grid_x, nearest_grid_y);
 	snap_data.nearest_is_grid = pcb_true;
@@ -1329,7 +1303,12 @@ void pcb_crosshair_grid_fit(pcb_coord_t X, pcb_coord_t Y)
 	if (PCB->RatDraw || conf_core.editor.snap_pin)
 		ans = pcb_search_grid_slop(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_PIN, &ptr1, &ptr2, &ptr3);
 
+	if (ans == PCB_TYPE_NONE)
+		if (PCB->RatDraw || conf_core.editor.snap_pin)
+			ans = pcb_search_grid_slop(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_VIA | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3);
+
 	/* Avoid self-snapping when moving */
+	/* SUBC/TERM note: this behavior is wrong, let the user use shift to verride */
 	if (ans != PCB_TYPE_NONE &&
 			conf_core.editor.mode == PCB_MODE_MOVE && pcb_crosshair.AttachedObject.Type == PCB_TYPE_ELEMENT && ptr1 == pcb_crosshair.AttachedObject.Ptr1)
 		ans = PCB_TYPE_NONE;
@@ -1358,7 +1337,7 @@ void pcb_crosshair_grid_fit(pcb_coord_t X, pcb_coord_t Y)
 
 	ans = PCB_TYPE_NONE;
 	if (conf_core.editor.snap_pin)
-		ans = pcb_search_grid_slop(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_LINE_POINT | PCB_TYPE_ARC_POINT, &ptr1, &ptr2, &ptr3);
+		ans = pcb_search_grid_slop(pcb_crosshair.X, pcb_crosshair.Y, PCB_TYPE_LINE_POINT | PCB_TYPE_ARC_POINT | PCB_TYPE_SUBC_PART, &ptr1, &ptr2, &ptr3);
 
 	if (ans == PCB_TYPE_ARC_POINT) {
 		/* Arc point needs special handling as it's not a real point but has to be calculated */
