@@ -69,12 +69,15 @@ pcb_box_t pcb_draw_invalidated = { COORD_MAX, COORD_MAX, -COORD_MAX, -COORD_MAX
 
 int pcb_draw_doing_pinout = 0;
 pcb_bool pcb_draw_doing_assy = pcb_false;
+static vtp0_t delayed_labels;
+pcb_bool delayed_labels_enabled = pcb_false;
 
 /* ---------------------------------------------------------------------------
  * some local prototypes
  */
 static void DrawEverything(const pcb_box_t *);
 static void DrawLayerGroup(int, const pcb_box_t *);
+static void pcb_draw_obj_label(pcb_any_obj_t *obj);
 
 /* In draw_ly_spec.c: */
 static void pcb_draw_paste(int side, const pcb_box_t *drawn_area);
@@ -84,8 +87,14 @@ static void pcb_draw_rats(const pcb_box_t *);
 static void pcb_draw_assembly(unsigned int lyt_side, const pcb_box_t *drawn_area);
 
 
+void pcb_draw_delay_label_add(pcb_any_obj_t *obj)
+{
+	if (delayed_labels_enabled)
+		vtp0_append(&delayed_labels, obj);
+}
 
-#warning TODO: this should be cached
+
+#warning cleanup TODO: this should be cached
 void pcb_lighten_color(const char *orig, char buf[8], double factor)
 {
 	unsigned int r, g, b;
@@ -166,8 +175,8 @@ static void DrawHoles(pcb_bool draw_plated, pcb_bool draw_unplated, const pcb_bo
 	if (!draw_plated && draw_unplated)
 		plated = 0;
 
-	pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, draw_hole_callback, &plated, NULL);
-	pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, draw_hole_callback, &plated, NULL);
+	pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_hole_draw_callback, &plated, NULL);
+	pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_hole_draw_callback, &plated, NULL);
 }
 
 static void DrawEverything_holes(const pcb_box_t * drawn_area)
@@ -207,6 +216,8 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	PCB->Data->SILKLAYER.meta.real.color = conf_core.appearance.color.element;
 	PCB->Data->BACKSILKLAYER.meta.real.color = conf_core.appearance.color.invisible_objects;
 
+	pcb_gui->render_burst(PCB_HID_BURST_START, drawn_area);
+
 	memset(do_group, 0, sizeof(do_group));
 	for (ngroups = 0, i = 0; i < pcb_max_layer; i++) {
 		pcb_layer_t *l = LAYER_ON_STACK(i);
@@ -232,7 +243,7 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	if (!conf_core.editor.check_planes && pcb_layer_gui_set_vlayer(PCB_VLY_INVISIBLE, 0)) {
 		side = PCB_SWAP_IDENT ? PCB_COMPONENT_SIDE : PCB_SOLDER_SIDE;
 		pcb_draw_silk(PCB_LYT_INVISIBLE_SIDE(), drawn_area);
-		pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, draw_pad_callback, &side, NULL);
+		pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_draw_callback, &side, NULL);
 		pcb_gui->end_layer();
 	}
 
@@ -247,13 +258,16 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	}
 
 	if (conf_core.editor.check_planes && pcb_gui->gui)
-		return;
+		goto finish;
 
 	/* Draw pins, pads, vias below silk */
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area);
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
 	if (pcb_gui->gui)
 		pcb_draw_ppv(PCB_SWAP_IDENT ? solder : component, drawn_area);
 	else if (!pcb_gui->holes_after)
 		DrawEverything_holes(drawn_area);
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
 
 	/* Draw the solder mask if turned on */
 	gid = pcb_layergrp_get_top_mask();
@@ -279,22 +293,38 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 	}
 
 	if (pcb_gui->holes_after)
+	{
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area); 
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
 		DrawEverything_holes(drawn_area);
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
+	}
 
 	if (pcb_gui->gui) {
 		/* Draw pins' and pads' names */
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area);
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
 		pcb_draw_ppv_names(PCB_SWAP_IDENT ? solder : component, drawn_area);
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
 		
 		/* Draw element Marks */
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area);
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
+	
 		if (PCB->PinOn)
-			pcb_r_search(PCB->Data->element_tree, drawn_area, NULL, draw_element_mark_callback, NULL, NULL);
+			pcb_r_search(PCB->Data->element_tree, drawn_area, NULL, pcb_elem_mark_draw_callback, NULL, NULL);
 
 		if (PCB->SubcOn)
 			pcb_r_search(PCB->Data->subc_tree, drawn_area, NULL, draw_subc_mark_callback, NULL, NULL);
+		
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
 
 		/* Draw rat lines on top */
 		if (pcb_layer_gui_set_vlayer(PCB_VLY_RATS, 0)) {
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area);
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
 			pcb_draw_rats(drawn_area);
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
 			pcb_gui->end_layer();
 		}
 	}
@@ -344,12 +374,24 @@ static void DrawEverything(const pcb_box_t * drawn_area)
 
 	/* if there's any UI layer, try to draw them */
 	if ((first != NULL) && pcb_layer_gui_set_g_ui(first, 0)) {
+		int have_canvas = 0;
 		for(i = 0; i < vtlayer_len(&pcb_uilayer); i++)
-			if ((pcb_uilayer.array[i].meta.real.cookie != NULL) && (pcb_uilayer.array[i].meta.real.vis))
+			if ((pcb_uilayer.array[i].meta.real.cookie != NULL) && (pcb_uilayer.array[i].meta.real.vis)) {
+				if (!have_canvas) {
+					pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area);
+					pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
+					have_canvas = 1;
+				}
 				pcb_draw_layer(pcb_uilayer.array+i, drawn_area);
+			}
 		pcb_gui->end_layer();
+		if (have_canvas)
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
 	}
 
+
+	finish:;
+	pcb_gui->render_burst(PCB_HID_BURST_END, drawn_area);
 }
 
 /* ---------------------------------------------------------------------------
@@ -363,27 +405,27 @@ void pcb_draw_ppv(pcb_layergrp_id_t group, const pcb_box_t * drawn_area)
 
 	if (PCB->PinOn || !pcb_gui->gui) {
 		/* draw element pins */
-		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, draw_pin_callback, NULL, NULL);
+		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_pin_draw_callback, NULL, NULL);
 
 		/* draw element pads */
 		if (gflg & PCB_LYT_TOP) {
 			side = PCB_COMPONENT_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, draw_pad_callback, &side, NULL);
+			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_draw_callback, &side, NULL);
 		}
 
 		if (gflg & PCB_LYT_BOTTOM) {
 			side = PCB_SOLDER_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, draw_pad_callback, &side, NULL);
+			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_draw_callback, &side, NULL);
 		}
 	}
 
 	/* draw vias */
 	if (PCB->ViaOn || !pcb_gui->gui) {
-		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, draw_via_callback, NULL, NULL);
-		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, draw_hole_callback, NULL, NULL);
+		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_via_draw_callback, NULL, NULL);
+		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_hole_draw_callback, NULL, NULL);
 	}
 	if (PCB->PinOn || pcb_draw_doing_assy)
-		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, draw_hole_callback, NULL, NULL);
+		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_hole_draw_callback, NULL, NULL);
 }
 
 /* ---------------------------------------------------------------------------
@@ -397,19 +439,30 @@ void pcb_draw_ppv_names(pcb_layergrp_id_t group, const pcb_box_t * drawn_area)
 
 	if (PCB->PinOn || !pcb_gui->gui) {
 		/* draw element pins' names */
-		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, draw_pin_name_callback, NULL, NULL);
+		pcb_r_search(PCB->Data->pin_tree, drawn_area, NULL, pcb_pin_name_draw_callback, NULL, NULL);
 
 		/* draw element pads' names */
 		if (gflg & PCB_LYT_TOP) {
 			side = PCB_COMPONENT_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, draw_pad_name_callback, &side, NULL);
+			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_name_draw_callback, &side, NULL);
 		}
 
 		if (gflg & PCB_LYT_BOTTOM) {
 			side = PCB_SOLDER_SIDE;
-			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, draw_pad_name_callback, &side, NULL);
+			pcb_r_search(PCB->Data->pad_tree, drawn_area, NULL, pcb_pad_name_draw_callback, &side, NULL);
 		}
 	}
+
+	if (PCB->ViaOn || !pcb_gui->gui) {
+		/* draw element pins' names */
+		pcb_r_search(PCB->Data->via_tree, drawn_area, NULL, pcb_pin_name_draw_callback, NULL, NULL);
+	}
+
+	if (PCB->PinOn || !pcb_gui->gui) {
+		size_t n;
+		for(n = 0; n < delayed_labels.used; n++)
+			pcb_draw_obj_label(delayed_labels.array[n]);
+	}
 }
 
 #include "draw_composite.c"
@@ -433,23 +486,44 @@ void pcb_draw_layer(pcb_layer_t *Layer, const pcb_box_t * screen)
 	info.drawn_area = screen;
 	info.layer = Layer;
 
+	lflg = pcb_layer_flags_(PCB, Layer);
+	if (PCB_LAYERFLG_ON_VISIBLE_SIDE(lflg))
+		Output.active_padGC = Output.padGC;
+	else
+		Output.active_padGC = Output.backpadGC;
 
-	/* print the non-clearing polys */
-	pcb_r_search(Layer->polygon_tree, screen, NULL, draw_poly_callback, &info, NULL);
+	if (lflg & PCB_LYT_COPPER) {
+		/* print the non-clearing polys */
+		pcb_r_search(Layer->polygon_tree, screen, NULL, pcb_poly_draw_term_callback, &info, NULL);
+	}
+	else {
+		/* print the non-clearing polys */
+		pcb_r_search(Layer->polygon_tree, screen, NULL, pcb_poly_draw_callback, &info, NULL);
+	}
 
 	if (conf_core.editor.check_planes)
-		return;
+		goto out;
 
-	/* draw all visible lines this layer */
-	pcb_r_search(Layer->line_tree, screen, NULL, draw_line_callback, Layer, NULL);
+	if (lflg & PCB_LYT_COPPER) {
+		/* draw all visible lines this layer - with terminal gfx */
+		pcb_r_search(Layer->line_tree, screen, NULL, pcb_line_draw_term_callback, Layer, NULL);
 
-	/* draw the layer arcs on screen */
-	pcb_r_search(Layer->arc_tree, screen, NULL, draw_arc_callback, Layer, NULL);
+		/* draw the layer arcs on screen */
+		pcb_r_search(Layer->arc_tree, screen, NULL, pcb_arc_draw_term_callback, Layer, NULL);
 
-	/* draw the layer text on screen */
-	pcb_r_search(Layer->text_tree, screen, NULL, draw_text_callback, Layer, NULL);
+		/* draw the layer text on screen */
+		pcb_r_search(Layer->text_tree, screen, NULL, pcb_text_draw_term_callback, Layer, NULL);
+	}
+	else {
+		/* draw all visible lines this layer */
+		pcb_r_search(Layer->line_tree, screen, NULL, pcb_line_draw_callback, Layer, NULL);
 
-	lflg = pcb_layer_flags_(PCB, Layer);
+		/* draw the layer arcs on screen */
+		pcb_r_search(Layer->arc_tree, screen, NULL, pcb_arc_draw_callback, Layer, NULL);
+
+		/* draw the layer text on screen */
+		pcb_r_search(Layer->text_tree, screen, NULL, pcb_text_draw_callback, Layer, NULL);
+	}
 
 	/* The implicit outline rectangle (or automatic outline rectanlge).
 	   We should check for pcb_gui->gui here, but it's kinda cool seeing the
@@ -460,6 +534,9 @@ void pcb_draw_layer(pcb_layer_t *Layer, const pcb_box_t * screen)
 		pcb_gui->set_line_width(Output.fgGC, PCB->minWid);
 		pcb_gui->draw_rect(Output.fgGC, 0, 0, PCB->MaxWidth, PCB->MaxHeight);
 	}
+
+	out:;
+		Output.active_padGC = NULL;
 }
 
 /* ---------------------------------------------------------------------------
@@ -475,6 +552,9 @@ static void DrawLayerGroup(int group, const pcb_box_t * drawn_area)
 	pcb_layer_id_t *layers = PCB->LayerGroups.grp[group].lid;
 	unsigned int gflg = pcb_layergrp_flags(PCB, group);
 
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, Output.direct, drawn_area);
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, Output.direct, drawn_area);
+
 	if (gflg & PCB_LYT_OUTLINE)
 		rv = 0;
 
@@ -489,6 +569,8 @@ static void DrawLayerGroup(int group, const pcb_box_t * drawn_area)
 
 	if (rv && !pcb_gui->gui)
 		pcb_draw_ppv(group, drawn_area);
+
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, Output.direct, drawn_area);
 }
 
 void pcb_erase_obj(int type, void *lptr, void *ptr)
@@ -496,17 +578,17 @@ void pcb_erase_obj(int type, void *lptr, void *ptr)
 	switch (type) {
 	case PCB_TYPE_VIA:
 	case PCB_TYPE_PIN:
-		ErasePin((pcb_pin_t *) ptr);
+		pcb_pin_invalidate_erase((pcb_pin_t *) ptr);
 		break;
 	case PCB_TYPE_TEXT:
 	case PCB_TYPE_ELEMENT_NAME:
-		EraseText((pcb_layer_t *) lptr, (pcb_text_t *) ptr);
+		pcb_text_invalidate_erase((pcb_layer_t *) lptr, (pcb_text_t *) ptr);
 		break;
 	case PCB_TYPE_POLYGON:
-		ErasePolygon((pcb_polygon_t *) ptr);
+		pcb_poly_invalidate_erase((pcb_polygon_t *) ptr);
 		break;
 	case PCB_TYPE_ELEMENT:
-		EraseElement((pcb_element_t *) ptr);
+		pcb_elem_invalidate_erase((pcb_element_t *) ptr);
 		break;
 	case PCB_TYPE_SUBC:
 		EraseSubc((pcb_subc_t *)ptr);
@@ -514,14 +596,14 @@ void pcb_erase_obj(int type, void *lptr, void *ptr)
 	case PCB_TYPE_LINE:
 	case PCB_TYPE_ELEMENT_LINE:
 	case PCB_TYPE_RATLINE:
-		EraseLine((pcb_line_t *) ptr);
+		pcb_line_invalidate_erase((pcb_line_t *) ptr);
 		break;
 	case PCB_TYPE_PAD:
-		ErasePad((pcb_pad_t *) ptr);
+		pcb_pad_invalidate_erase((pcb_pad_t *) ptr);
 		break;
 	case PCB_TYPE_ARC:
 	case PCB_TYPE_ELEMENT_ARC:
-		EraseArc((pcb_arc_t *) ptr);
+		pcb_arc_invalidate_erase((pcb_arc_t *) ptr);
 		break;
 	default:
 		pcb_message(PCB_MSG_ERROR, "hace: Internal ERROR, trying to erase an unknown type\n");
@@ -529,49 +611,70 @@ void pcb_erase_obj(int type, void *lptr, void *ptr)
 }
 
 
-void pcb_draw_obj(int type, void *ptr1, void *ptr2)
+void pcb_draw_obj(pcb_any_obj_t *obj)
 {
-	switch (type) {
-	case PCB_TYPE_VIA:
+	switch (obj->type) {
+	case PCB_OBJ_VIA:
 		if (PCB->ViaOn)
-			DrawVia((pcb_pin_t *) ptr2);
+			pcb_via_invalidate_draw((pcb_pin_t *)obj);
 		break;
-	case PCB_TYPE_LINE:
-		if (((pcb_layer_t *) ptr1)->meta.real.vis)
-			DrawLine((pcb_layer_t *) ptr1, (pcb_line_t *) ptr2);
+	case PCB_OBJ_LINE:
+		if (obj->parent.layer->meta.real.vis)
+			pcb_line_invalidate_draw(obj->parent.layer, (pcb_line_t *)obj);
 		break;
-	case PCB_TYPE_ARC:
-		if (((pcb_layer_t *) ptr1)->meta.real.vis)
-			DrawArc((pcb_layer_t *) ptr1, (pcb_arc_t *) ptr2);
+	case PCB_OBJ_ARC:
+		if (obj->parent.layer->meta.real.vis)
+			pcb_arc_invalidate_draw(obj->parent.layer, (pcb_arc_t *)obj);
 		break;
-	case PCB_TYPE_TEXT:
-		if (((pcb_layer_t *) ptr1)->meta.real.vis)
-			DrawText((pcb_layer_t *) ptr1, (pcb_text_t *) ptr2);
+	case PCB_OBJ_TEXT:
+		if (obj->parent.layer->meta.real.vis)
+			pcb_text_invalidate_draw(obj->parent.layer, (pcb_text_t *)obj);
 		break;
-	case PCB_TYPE_POLYGON:
-		if (((pcb_layer_t *) ptr1)->meta.real.vis)
-			DrawPolygon((pcb_layer_t *) ptr1, (pcb_polygon_t *) ptr2);
+	case PCB_OBJ_POLYGON:
+		if (obj->parent.layer->meta.real.vis)
+			pcb_poly_invalidate_draw(obj->parent.layer, (pcb_polygon_t *)obj);
 		break;
-	case PCB_TYPE_ELEMENT:
-		if (pcb_silk_on(PCB) && (PCB_FRONT((pcb_element_t *) ptr2) || PCB->InvisibleObjectsOn))
-			DrawElement((pcb_element_t *) ptr2);
+	case PCB_OBJ_ELEMENT:
+		if (pcb_silk_on(PCB) && (PCB_FRONT((pcb_element_t *)obj) || PCB->InvisibleObjectsOn))
+			pcb_elem_invalidate_draw((pcb_element_t *)obj);
 		break;
-	case PCB_TYPE_RATLINE:
+	case PCB_OBJ_RAT:
 		if (PCB->RatOn)
-			DrawRat((pcb_rat_t *) ptr2);
+			pcb_rat_invalidate_draw((pcb_rat_t *)obj);
 		break;
-	case PCB_TYPE_PIN:
+	case PCB_OBJ_PIN:
 		if (PCB->PinOn)
-			DrawPin((pcb_pin_t *) ptr2);
+			pcb_pin_invalidate_draw((pcb_pin_t *)obj);
 		break;
-	case PCB_TYPE_PAD:
+	case PCB_OBJ_PAD:
 		if (PCB->PinOn)
-			DrawPad((pcb_pad_t *) ptr2);
+			pcb_pad_invalidate_draw((pcb_pad_t *)obj);
 		break;
-	case PCB_TYPE_ELEMENT_NAME:
-		if (pcb_silk_on(PCB) && (PCB_FRONT((pcb_element_t *) ptr2) || PCB->InvisibleObjectsOn))
-			DrawElementName((pcb_element_t *) ptr1);
+	case PCB_OBJ_ETEXT:
+		if (pcb_silk_on(PCB) && (PCB_FRONT(obj->parent.element) || PCB->InvisibleObjectsOn))
+			pcb_elem_name_invalidate_draw(obj->parent.element);
 		break;
+	case PCB_OBJ_POINT:
+	case PCB_OBJ_SUBC:
+	case PCB_OBJ_NET:
+	case PCB_OBJ_LAYER:
+	case PCB_OBJ_ELINE:
+	case PCB_OBJ_EARC:
+	case PCB_OBJ_CLASS_MASK:
+	case PCB_OBJ_CLASS_OBJ:
+	case PCB_OBJ_ANY:
+		break;
+	}
+}
+
+static void pcb_draw_obj_label(pcb_any_obj_t *obj)
+{
+	switch(obj->type) {
+		case PCB_OBJ_LINE:    pcb_line_draw_label((pcb_line_t *)obj); return;
+		case PCB_OBJ_ARC:     pcb_arc_draw_label((pcb_arc_t *)obj); return;
+		case PCB_OBJ_POLYGON: pcb_poly_draw_label((pcb_polygon_t *)obj); return;
+		case PCB_OBJ_TEXT:    pcb_text_draw_label((pcb_text_t *)obj); return;
+		default: break;
 	}
 }
 
@@ -583,22 +686,39 @@ static pcb_hid_t *expose_begin(pcb_hid_t *hid)
 {
 	pcb_hid_t *old_gui = pcb_gui;
 
+	delayed_labels_enabled = pcb_true;
+	vtp0_truncate(&delayed_labels, 0);
+
 	pcb_gui = hid;
 	Output.fgGC = pcb_gui->make_gc();
-	Output.bgGC = pcb_gui->make_gc();
+	Output.padGC = pcb_gui->make_gc();
+	Output.backpadGC = pcb_gui->make_gc();
+	Output.padselGC = pcb_gui->make_gc();
+	Output.drillGC = pcb_gui->make_gc();
 	Output.pmGC = pcb_gui->make_gc();
+	Output.direct = 1;
 
 	hid->set_color(Output.pmGC, "erase");
-	hid->set_color(Output.bgGC, "drill");
+	hid->set_color(Output.drillGC, "drill");
+	hid->set_color(Output.padGC, conf_core.appearance.color.pin);
+	hid->set_color(Output.backpadGC, conf_core.appearance.color.invisible_objects);
+	hid->set_color(Output.padselGC, conf_core.appearance.color.pin_selected);
+
 	return old_gui;
 }
 
 static void expose_end(pcb_hid_t *old_gui)
 {
 	pcb_gui->destroy_gc(Output.fgGC);
-	pcb_gui->destroy_gc(Output.bgGC);
+	pcb_gui->destroy_gc(Output.padGC);
+	pcb_gui->destroy_gc(Output.backpadGC);
+	pcb_gui->destroy_gc(Output.padselGC);
+	pcb_gui->destroy_gc(Output.drillGC);
 	pcb_gui->destroy_gc(Output.pmGC);
 	pcb_gui = old_gui;
+
+	delayed_labels_enabled = pcb_false;
+	vtp0_truncate(&delayed_labels, 0);
 }
 
 void pcb_hid_expose_all(pcb_hid_t * hid, const pcb_hid_expose_ctx_t *ctx)
@@ -614,9 +734,14 @@ void pcb_hid_expose_pinout(pcb_hid_t * hid, const pcb_hid_expose_ctx_t *ctx)
 {
 	pcb_hid_t *old_gui = expose_begin(hid);
 
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, 1, &ctx->view);
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, 1, &ctx->view);
+
 	pcb_draw_doing_pinout = pcb_true;
-	draw_element(ctx->content.elem);
+	pcb_elem_draw(ctx->content.elem);
 	pcb_draw_doing_pinout = pcb_false;
+			
+	pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, 1, &ctx->view);
 
 	expose_end(old_gui);
 }
@@ -636,18 +761,28 @@ void pcb_hid_expose_layer(pcb_hid_t *hid, const pcb_hid_expose_ctx_t *e)
 
 	if (lflg & PCB_LYT_CSECT) {
 		if ((pcb_layer_gui_set_vlayer(PCB_VLY_CSECT, 0)) || (e->force)) {
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, 1, &e->view);
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, 1, &e->view);
 			pcb_stub_draw_csect(Output.fgGC, e);
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, 1, &e->view);
 			pcb_gui->end_layer();
 		}
 	}
 	else if (lflg & PCB_LYT_DIALOG) {
 		if ((pcb_layer_gui_set_vlayer(PCB_VLY_DIALOG, 0)) || (e->force)) {
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, 1, &e->view);
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, 1, &e->view);
 			e->dialog_draw(Output.fgGC, e);
+			pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, 1, &e->view);
 			pcb_gui->end_layer();
 		}
 	}
-	else if ((e->content.layer_id >= 0) && (e->content.layer_id < pcb_max_layer))
+	else if ((e->content.layer_id >= 0) && (e->content.layer_id < pcb_max_layer)) {
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_RESET, 1, &e->view);
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_POSITIVE, 1, &e->view);
 		pcb_draw_layer(&(PCB->Data->Layer[e->content.layer_id]), &e->view);
+		pcb_gui->set_drawing_mode(PCB_HID_COMP_FLUSH, 1, &e->view);
+	}
 	else
 		pcb_message(PCB_MSG_ERROR, "Internal error: don't know how to draw layer %ld for preview; please report this bug.\n", e->content.layer_id);
 
@@ -658,3 +793,77 @@ void pcb_hid_expose_layer(pcb_hid_t *hid, const pcb_hid_expose_ctx_t *e)
 		conf_force_set_bool(conf_core.editor.view.flip_y, fy);
 	}
 }
+
+static const char *lab_with_intconn(int intconn, const char *lab, char *buff, int bufflen)
+{
+	if (intconn <= 0)
+		return lab;
+	pcb_snprintf(buff, bufflen, "%s[%d]", lab, intconn);
+	return buff;
+}
+
+static void pcb_term_label_setup_(pcb_text_t *text, pcb_coord_t x, pcb_coord_t y, double scale, pcb_bool vert, pcb_bool centered, const char *label)
+{
+	pcb_bool flip_x = conf_core.editor.view.flip_x;
+	pcb_bool flip_y = conf_core.editor.view.flip_y;
+
+	text->TextString = (char *)label;
+	text->Flags = (flip_x ^ flip_y) ? pcb_flag_make(PCB_FLAG_ONSOLDER) : pcb_no_flags();
+	text->X = x;
+	text->Y = y;
+	text->fid = 0;
+	text->Scale = scale;
+	text->Direction = (vert ? 1 : 0) + (flip_x ? 2 : 0);
+
+	pcb_text_bbox(NULL, text);
+
+	if (centered) {
+		pcb_coord_t dx, dy;
+		dx = (text->BoundingBox.X2 - text->BoundingBox.X1) / 2;
+		dy = (text->BoundingBox.Y2 - text->BoundingBox.Y1) / 2;
+
+		/* make sure the offset is in-line with the flip and our sick y coords for vertical */
+		if ((vert) && (!(flip_x ^ flip_y)))
+			dy = -dy;
+		if (flip_x)
+			dx = -dx;
+		if (flip_y)
+			dy = -dy;
+
+		text->X -= dx;
+		text->Y -= dy;
+		text->BoundingBox.X1 -= dx;
+		text->BoundingBox.X2 -= dx;
+		text->BoundingBox.Y1 -= dy;
+		text->BoundingBox.Y2 -= dy;
+	}
+}
+
+void pcb_term_label_setup(pcb_text_t *text, pcb_coord_t x, pcb_coord_t y, double scale, pcb_bool vert, pcb_bool centered, const char *lab, int intconn)
+{
+	const char *label;
+	char buff[128];
+
+	label = lab_with_intconn(intconn, lab, buff, sizeof(buff));
+	pcb_term_label_setup_(text, x, y, scale, vert, centered, label);
+}
+
+
+void pcb_term_label_draw(pcb_coord_t x, pcb_coord_t y, double scale, pcb_bool vert, pcb_bool centered, const char *lab, int intconn)
+{
+	pcb_text_t text;
+	const char *label;
+	char buff[128];
+	label = lab_with_intconn(intconn, lab, buff, sizeof(buff));
+
+	pcb_gui->set_color(Output.fgGC, conf_core.appearance.color.pin_name);
+
+	pcb_term_label_setup_(&text, x, y, scale, vert, centered, label);
+
+	if (pcb_gui->gui)
+		pcb_draw_doing_pinout++;
+	pcb_text_draw_(&text, 0, 0);
+	if (pcb_gui->gui)
+		pcb_draw_doing_pinout--;
+}
+
