@@ -244,7 +244,7 @@ static pcb_r_dir_t drc_callback(pcb_data_t *data, pcb_layer_t *layer, pcb_polygo
 	switch (type) {
 	case PCB_TYPE_LINE:
 		if (line->Clearance < 2 * PCB->Bloat) {
-			pcb_undo_add_obj_to_flag(type, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_SET(TheFlag, line);
 			message = _("Line with insufficient clearance inside polygon\n");
 			goto doIsBad;
@@ -252,7 +252,7 @@ static pcb_r_dir_t drc_callback(pcb_data_t *data, pcb_layer_t *layer, pcb_polygo
 		break;
 	case PCB_TYPE_ARC:
 		if (arc->Clearance < 2 * PCB->Bloat) {
-			pcb_undo_add_obj_to_flag(type, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_SET(TheFlag, arc);
 			message = _("Arc with insufficient clearance inside polygon\n");
 			goto doIsBad;
@@ -261,7 +261,7 @@ static pcb_r_dir_t drc_callback(pcb_data_t *data, pcb_layer_t *layer, pcb_polygo
 	case PCB_TYPE_PAD:
 		if (pad->Clearance && pad->Clearance < 2 * PCB->Bloat)
 			if (pcb_is_pad_in_poly(pad, polygon)) {
-				pcb_undo_add_obj_to_flag(type, ptr1, ptr2, ptr2);
+				pcb_undo_add_obj_to_flag(ptr2);
 				PCB_FLAG_SET(TheFlag, pad);
 				message = _("Pad with insufficient clearance inside polygon\n");
 				goto doIsBad;
@@ -269,7 +269,7 @@ static pcb_r_dir_t drc_callback(pcb_data_t *data, pcb_layer_t *layer, pcb_polygo
 		break;
 	case PCB_TYPE_PIN:
 		if (pin->Clearance && pin->Clearance < 2 * PCB->Bloat) {
-			pcb_undo_add_obj_to_flag(type, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_SET(TheFlag, pin);
 			message = _("Pin with insufficient clearance inside polygon\n");
 			goto doIsBad;
@@ -277,7 +277,7 @@ static pcb_r_dir_t drc_callback(pcb_data_t *data, pcb_layer_t *layer, pcb_polygo
 		break;
 	case PCB_TYPE_VIA:
 		if (pin->Clearance && pin->Clearance < 2 * PCB->Bloat) {
-			pcb_undo_add_obj_to_flag(type, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_SET(TheFlag, pin);
 			message = _("Via with insufficient clearance inside polygon\n");
 			goto doIsBad;
@@ -289,10 +289,10 @@ static pcb_r_dir_t drc_callback(pcb_data_t *data, pcb_layer_t *layer, pcb_polygo
 	return PCB_R_DIR_NOT_FOUND;
 
 doIsBad:
-	pcb_undo_add_obj_to_flag(PCB_TYPE_POLYGON, layer, polygon, polygon);
+	pcb_undo_add_obj_to_flag(polygon);
 	PCB_FLAG_SET(PCB_FLAG_FOUND, polygon);
-	DrawPolygon(layer, polygon);
-	pcb_draw_obj(type, ptr1, ptr2);
+	pcb_poly_invalidate_draw(layer, polygon);
+	pcb_draw_obj((pcb_any_obj_t *)ptr2);
 	drcerr_count++;
 	LocateError(&x, &y);
 	BuildObjectList(&object_count, &object_id_list, &object_type_list);
@@ -401,9 +401,9 @@ int pcb_drc_all(void)
 			if (IsBad)
 				break;
 			if (line->Thickness < PCB->minWid) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_LINE, layer, line, line);
+				pcb_undo_add_obj_to_flag(line);
 				PCB_FLAG_SET(TheFlag, line);
-				DrawLine(layer, line);
+				pcb_line_invalidate_draw(layer, line);
 				drcerr_count++;
 				SetThing(PCB_TYPE_LINE, layer, line, line);
 				LocateError(&x, &y);
@@ -432,9 +432,9 @@ int pcb_drc_all(void)
 			if (IsBad)
 				break;
 			if (arc->Thickness < PCB->minWid) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_ARC, layer, arc, arc);
+				pcb_undo_add_obj_to_flag(arc);
 				PCB_FLAG_SET(TheFlag, arc);
-				DrawArc(layer, arc);
+				pcb_arc_invalidate_draw(layer, arc);
 				drcerr_count++;
 				SetThing(PCB_TYPE_ARC, layer, arc, arc);
 				LocateError(&x, &y);
@@ -463,9 +463,9 @@ int pcb_drc_all(void)
 			if (IsBad)
 				break;
 			if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) && pin->Thickness - pin->DrillingHole < 2 * PCB->minRing) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, pin, pin);
+				pcb_undo_add_obj_to_flag(pin);
 				PCB_FLAG_SET(TheFlag, pin);
-				DrawPin(pin);
+				pcb_pin_invalidate_draw(pin);
 				drcerr_count++;
 				SetThing(PCB_TYPE_PIN, element, pin, pin);
 				LocateError(&x, &y);
@@ -486,9 +486,9 @@ int pcb_drc_all(void)
 				pcb_undo(pcb_false);
 			}
 			if (pin->DrillingHole < PCB->minDrill) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, pin, pin);
+				pcb_undo_add_obj_to_flag(pin);
 				PCB_FLAG_SET(TheFlag, pin);
-				DrawPin(pin);
+				pcb_pin_invalidate_draw(pin);
 				drcerr_count++;
 				SetThing(PCB_TYPE_PIN, element, pin, pin);
 				LocateError(&x, &y);
@@ -517,9 +517,9 @@ int pcb_drc_all(void)
 			if (IsBad)
 				break;
 			if (pad->Thickness < PCB->minWid) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, element, pad, pad);
+				pcb_undo_add_obj_to_flag(pad);
 				PCB_FLAG_SET(TheFlag, pad);
-				DrawPad(pad);
+				pcb_pad_invalidate_draw(pad);
 				drcerr_count++;
 				SetThing(PCB_TYPE_PAD, element, pad, pad);
 				LocateError(&x, &y);
@@ -548,9 +548,9 @@ int pcb_drc_all(void)
 			if (IsBad)
 				break;
 			if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, via) && via->Thickness - via->DrillingHole < 2 * PCB->minRing) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, via, via, via);
+				pcb_undo_add_obj_to_flag(via);
 				PCB_FLAG_SET(TheFlag, via);
-				DrawVia(via);
+				pcb_via_invalidate_draw(via);
 				drcerr_count++;
 				SetThing(PCB_TYPE_VIA, via, via, via);
 				LocateError(&x, &y);
@@ -571,9 +571,9 @@ int pcb_drc_all(void)
 				pcb_undo(pcb_false);
 			}
 			if (via->DrillingHole < PCB->minDrill) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, via, via, via);
+				pcb_undo_add_obj_to_flag(via);
 				PCB_FLAG_SET(TheFlag, via);
-				DrawVia(via);
+				pcb_via_invalidate_draw(via);
 				drcerr_count++;
 				SetThing(PCB_TYPE_VIA, via, via, via);
 				LocateError(&x, &y);
@@ -608,7 +608,7 @@ int pcb_drc_all(void)
 		{
 			if (line->Thickness < PCB->minSlk) {
 				PCB_FLAG_SET(TheFlag, line);
-				DrawLine(layer, line);
+				pcb_line_invalidate_draw(layer, line);
 				drcerr_count++;
 				SetThing(PCB_TYPE_LINE, layer, line, line);
 				LocateError(&x, &y);
@@ -649,7 +649,7 @@ int pcb_drc_all(void)
 				int buflen;
 
 				PCB_FLAG_SET(TheFlag, element);
-				DrawElement(element);
+				pcb_elem_invalidate_draw(element);
 				drcerr_count++;
 				SetThing(PCB_TYPE_ELEMENT, element, element, element);
 				LocateError(&x, &y);
@@ -681,7 +681,7 @@ int pcb_drc_all(void)
 			}
 		}
 		PCB_END_LOOP;
-#warning subc term TODO
+#warning subc TODO term TODO: implement this for subcircuits
 	}
 
 
@@ -717,12 +717,12 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 	if (PCB->Shrink != 0) {
 		Bloat = -PCB->Shrink;
 		TheFlag = PCB_FLAG_DRC | PCB_FLAG_SELECTED;
-		ListStart(What, ptr1, ptr2, ptr3);
+		ListStart(ptr2);
 		DoIt(pcb_true, pcb_false);
 		/* ok now the shrunk net has the PCB_FLAG_SELECTED set */
 		DumpList();
 		TheFlag = PCB_FLAG_FOUND;
-		ListStart(What, ptr1, ptr2, ptr3);
+		ListStart(ptr2);
 		Bloat = 0;
 		drc = pcb_true;									/* abort the search if we find anything not already found */
 		if (DoIt(pcb_true, pcb_false)) {
@@ -734,10 +734,10 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 			drc = pcb_false;
 			Bloat = -PCB->Shrink;
 			TheFlag = PCB_FLAG_SELECTED;
-			ListStart(What, ptr1, ptr2, ptr3);
+			ListStart(ptr2);
 			DoIt(pcb_true, pcb_true);
 			DumpList();
-			ListStart(What, ptr1, ptr2, ptr3);
+			ListStart(ptr2);
 			TheFlag = PCB_FLAG_FOUND;
 			Bloat = 0;
 			drc = pcb_true;
@@ -768,7 +768,7 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 	drc = pcb_false;
 	pcb_reset_conns(pcb_false);
 	TheFlag = PCB_FLAG_FOUND;
-	ListStart(What, ptr1, ptr2, ptr3);
+	ListStart(ptr2);
 	Bloat = PCB->Bloat;
 	drc = pcb_true;
 	while (DoIt(pcb_true, pcb_false)) {
@@ -780,11 +780,11 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 		drc = pcb_false;
 		Bloat = 0;
 		TheFlag = PCB_FLAG_SELECTED;
-		ListStart(What, ptr1, ptr2, ptr3);
+		ListStart(ptr2);
 		DoIt(pcb_true, pcb_true);
 		DumpList();
 		TheFlag = PCB_FLAG_FOUND;
-		ListStart(What, ptr1, ptr2, ptr3);
+		ListStart(ptr2);
 		Bloat = PCB->Bloat;
 		drc = pcb_true;
 		DoIt(pcb_true, pcb_true);
@@ -809,12 +809,12 @@ static pcb_bool DRCFind(int What, void *ptr1, void *ptr2, void *ptr3)
 		/* highlight the rest of the encroaching net so it's not reported again */
 		TheFlag |= PCB_FLAG_SELECTED;
 		Bloat = 0;
-		ListStart(thing_type, thing_ptr1, thing_ptr2, thing_ptr3);
+		ListStart(thing_ptr2);
 		DoIt(pcb_true, pcb_true);
 		DumpList();
 		drc = pcb_true;
 		Bloat = PCB->Bloat;
-		ListStart(What, ptr1, ptr2, ptr3);
+		ListStart(ptr2);
 	}
 	drc = pcb_false;
 	DumpList();
