@@ -41,6 +41,7 @@
  */
 
 #include "macro.h"
+#include "obj_arc_ui.h"
 
 #define EXPAND_BOUNDS(p) if (Bloat > 0) {\
        (p)->BoundingBox.X1 -= Bloat; \
@@ -229,16 +230,70 @@ static pcb_bool ArcArcIntersect(pcb_arc_t *Arc1, pcb_arc_t *Arc2)
 }
 
 /* ---------------------------------------------------------------------------
- * Tests if point is same as line end point
+ * Tests if point is same as line end point or center point
  */
-static pcb_bool IsRatPointOnLineEnd(pcb_point_t *Point, pcb_line_t *Line)
+static pcb_bool IsRatPointOnLineSpec(pcb_point_t *Point, pcb_line_t *Line)
 {
+	pcb_coord_t cx, cy;
+
+	/* either end */
 	if ((Point->X == Line->Point1.X && Point->Y == Line->Point1.Y)
 			|| (Point->X == Line->Point2.X && Point->Y == Line->Point2.Y))
 		return (pcb_true);
+
+	/* middle point */
+	pcb_obj_center((pcb_any_obj_t *)Line, &cx, &cy);
+	if ((Point->X == cx) && (Point->Y == cy))
+		return pcb_true;
+
 	return (pcb_false);
 }
 
+/* ---------------------------------------------------------------------------
+ * Tests if point is same as arc end point or center point
+ */
+static pcb_bool IsRatPointOnArcSpec(pcb_point_t *Point, pcb_arc_t *arc)
+{
+	pcb_coord_t cx, cy;
+
+	/* either end */
+	pcb_arc_get_end(arc, 0, &cx, &cy);
+	if ((Point->X == cx) && (Point->Y == cy))
+		return pcb_true;
+
+	pcb_arc_get_end(arc, 1, &cx, &cy);
+	if ((Point->X == cx) && (Point->Y == cy))
+		return pcb_true;
+
+	/* middle point */
+	pcb_arc_middle(arc, &cx, &cy);
+	if ((Point->X == cx) && (Point->Y == cy))
+		return pcb_true;
+
+	return (pcb_false);
+}
+
+/* ---------------------------------------------------------------------------
+ * Tests if rat line point is connected to a polygon
+ */
+static pcb_bool IsRatPointOnPoly(pcb_point_t *Point, pcb_polygon_t *polygon)
+{
+	pcb_coord_t cx, cy;
+
+	/* canonical point */
+	cx = polygon->Clipped->contours->head.point[0];
+	cy = polygon->Clipped->contours->head.point[1];
+	if ((Point->X == cx) && (Point->Y == cy))
+		return pcb_true;
+
+	/* middle point */
+	pcb_obj_center((pcb_any_obj_t *)polygon, &cx, &cy);
+	if ((Point->X == cx) && (Point->Y == cy))
+		return pcb_true;
+
+	return pcb_false;
+}
+
 static void form_slanted_rectangle(pcb_point_t p[4], pcb_line_t *l)
 /* writes vertices of a squared line */
 {
@@ -479,7 +534,7 @@ pcb_bool pcb_is_arc_in_poly(pcb_arc_t *Arc, pcb_polygon_t *Polygon)
 	pcb_box_t *Box = (pcb_box_t *) Arc;
 
 	/* arcs with clearance never touch polys */
-	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY, Polygon) && PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc))
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY, Polygon) && PCB_OBJ_HAS_CLEARANCE(Arc))
 		return pcb_false;
 	if (!Polygon->Clipped)
 		return pcb_false;
@@ -509,7 +564,7 @@ pcb_bool pcb_is_line_in_poly(pcb_line_t *Line, pcb_polygon_t *Polygon)
 	pcb_polyarea_t *lp;
 
 	/* lines with clearance never touch polygons */
-	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY, Polygon) && PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line))
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLY, Polygon) && PCB_OBJ_HAS_CLEARANCE(Line))
 		return pcb_false;
 	if (!Polygon->Clipped)
 		return pcb_false;
