@@ -27,6 +27,7 @@
 
 #include "compat_nls.h"
 #include "board.h"
+#include "obj_subc_parent.h"
 
 static inline pcb_r_dir_t r_search_pt(pcb_rtree_t * rtree, const pcb_point_t * pt,
 															int radius,
@@ -50,7 +51,7 @@ static inline pcb_r_dir_t r_search_pt(pcb_rtree_t * rtree, const pcb_point_t * p
 static pcb_bool ADD_PV_TO_LIST(pcb_pin_t *Pin, int from_type, void *from_ptr, pcb_found_conn_type_t type)
 {
 	if (User)
-		pcb_undo_add_obj_to_flag(Pin->Element ? PCB_TYPE_PIN : PCB_TYPE_VIA, Pin->Element ? Pin->Element : Pin, Pin, Pin);
+		pcb_undo_add_obj_to_flag(Pin);
 	PCB_FLAG_SET(TheFlag, Pin);
 	make_callback(PCB_TYPE_PIN, Pin, from_type, from_ptr, type);
 	PVLIST_ENTRY(PVList.Number) = Pin;
@@ -71,7 +72,7 @@ static pcb_bool ADD_PAD_TO_LIST(pcb_cardinal_t L, pcb_pad_t *Pad, int from_type,
 
 /*fprintf(stderr, "ADD_PAD_TO_LIST cardinal %d %p %d\n", L, Pad, from_type);*/
 	if (User)
-		pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, Pad->Element, Pad, Pad);
+		pcb_undo_add_obj_to_flag(Pad);
 	PCB_FLAG_SET(TheFlag, Pad);
 	make_callback(PCB_TYPE_PAD, Pad, from_type, from_ptr, type);
 	PADLIST_ENTRY((L), PadList[(L)].Number) = Pad;
@@ -90,7 +91,7 @@ static pcb_bool ADD_LINE_TO_LIST(pcb_layer_id_t L, pcb_line_t *Ptr, int from_typ
 	if (LineList[L].Data == NULL)
 		return pcb_false;
 	if (User)
-		pcb_undo_add_obj_to_flag(PCB_TYPE_LINE, LAYER_PTR(L), (Ptr), (Ptr));
+		pcb_undo_add_obj_to_flag(Ptr);
 	PCB_FLAG_SET(TheFlag, (Ptr));
 	make_callback(PCB_TYPE_LINE, Ptr, from_type, from_ptr, type);
 	LINELIST_ENTRY((L), LineList[(L)].Number) = (Ptr);
@@ -110,7 +111,7 @@ static pcb_bool ADD_ARC_TO_LIST(pcb_cardinal_t L, pcb_arc_t *Ptr, int from_type,
 		return pcb_false;
 
 	if (User)
-		pcb_undo_add_obj_to_flag(PCB_TYPE_ARC, LAYER_PTR(L), (Ptr), (Ptr));
+		pcb_undo_add_obj_to_flag(Ptr);
 	PCB_FLAG_SET(TheFlag, (Ptr));
 	make_callback(PCB_TYPE_ARC, Ptr, from_type, from_ptr, type);
 	ARCLIST_ENTRY((L), ArcList[(L)].Number) = (Ptr);
@@ -127,7 +128,7 @@ static pcb_bool ADD_ARC_TO_LIST(pcb_cardinal_t L, pcb_arc_t *Ptr, int from_type,
 static pcb_bool ADD_RAT_TO_LIST(pcb_rat_t *Ptr, int from_type, void *from_ptr, pcb_found_conn_type_t type)
 {
 	if (User)
-		pcb_undo_add_obj_to_flag(PCB_TYPE_RATLINE, (Ptr), (Ptr), (Ptr));
+		pcb_undo_add_obj_to_flag(Ptr);
 	PCB_FLAG_SET(TheFlag, (Ptr));
 	make_callback(PCB_TYPE_RATLINE, Ptr, from_type, from_ptr, type);
 	RATLIST_ENTRY(RatList.Number) = (Ptr);
@@ -147,7 +148,7 @@ static pcb_bool ADD_POLYGON_TO_LIST(pcb_cardinal_t L, pcb_polygon_t *Ptr, int fr
 		return pcb_false;
 
 	if (User)
-		pcb_undo_add_obj_to_flag(PCB_TYPE_POLYGON, LAYER_PTR(L), (Ptr), (Ptr));
+		pcb_undo_add_obj_to_flag(Ptr);
 	PCB_FLAG_SET(TheFlag, (Ptr));
 	make_callback(PCB_TYPE_POLYGON, Ptr, from_type, from_ptr, type);
 	POLYGONLIST_ENTRY((L), PolygonList[(L)].Number) = (Ptr);
@@ -174,6 +175,8 @@ pcb_bool SetThing(int type, void *ptr1, void *ptr2, void *ptr3)
 	return pcb_true;
 }
 
+#include "find_intconn.c"
+
 /* ---------------------------------------------------------------------------
  * releases all allocated memory
  */
@@ -398,13 +401,19 @@ static pcb_bool LookupLOConnectionsToPVList(pcb_bool AndRats)
 {
 	pcb_cardinal_t layer;
 	struct pv_info info;
+	pcb_pin_t *orig_pin;
 
 	/* loop over all PVs currently on list */
 	while (PVList.Location < PVList.Number) {
 		/* get pointer to data */
-		info.pv = *(PVLIST_ENTRY(PVList.Location));
+		orig_pin = (PVLIST_ENTRY(PVList.Location));
+		info.pv = *orig_pin;
 		EXPAND_BOUNDS(&info.pv);
 
+		/* subc intconn jumps */
+		if ((orig_pin->term != NULL) && (orig_pin->intconn > 0))
+			LOC_int_conn_subc(pcb_gobj_parent_subc(orig_pin->parent_type, &orig_pin->parent), orig_pin->intconn, PCB_TYPE_PIN, orig_pin);
+
 		/* check pads */
 		if (setjmp(info.env) == 0)
 			pcb_r_search(PCB->Data->pad_tree, (pcb_box_t *) & info.pv, NULL, LOCtoPVpad_callback, &info, NULL);
@@ -585,21 +594,13 @@ static pcb_bool LookupPVConnectionsToPVList(void)
 		orig_pin = (PVLIST_ENTRY(PVList.Location));
 		info.pv = *orig_pin;
 
-		/* Internal connection: if pins in the same element have the same
-		   internal connection group number, they are connected */
-		ic = PCB_FLAG_INTCONN_GET(orig_pin);
-		if ((info.pv.Element != NULL) && (ic > 0)) {
-			pcb_element_t *e = info.pv.Element;
-			PCB_PIN_LOOP(e);
-			{
-				if ((orig_pin != pin) && (ic == PCB_FLAG_INTCONN_GET(pin))) {
-					if (!PCB_FLAG_TEST(TheFlag, pin))
-						ADD_PV_TO_LIST(pin, PCB_TYPE_PIN, orig_pin, PCB_FCT_INTERNAL);
-				}
-			}
-			PCB_END_LOOP;
-		}
+		ic = orig_pin->intconn;
+		if ((info.pv.Element != NULL) && (ic > 0))
+			LOC_int_conn_element(info.pv.Element, ic, PCB_TYPE_PIN, orig_pin);
 
+		/* subc intconn jumps */
+		if ((orig_pin->term != NULL) && (orig_pin->intconn > 0))
+			LOC_int_conn_subc(pcb_gobj_parent_subc(orig_pin->parent_type, &orig_pin->parent), orig_pin->intconn, PCB_TYPE_PIN, orig_pin);
 
 		EXPAND_BOUNDS(&info.pv);
 		if (setjmp(info.env) == 0)
@@ -745,8 +746,14 @@ static pcb_bool LookupPVConnectionsToLOList(pcb_bool AndRats)
 
 		/* check all lines */
 		while (LineList[layer].Location < LineList[layer].Number) {
-			info.line = *(LINELIST_ENTRY(layer, LineList[layer].Location));
+			pcb_line_t *orig_line = (LINELIST_ENTRY(layer, LineList[layer].Location));
+			info.line = *orig_line;
 			EXPAND_BOUNDS(&info.line);
+
+			/* subc intconn jumps */
+			if ((orig_line->term != NULL) && (orig_line->intconn > 0))
+				LOC_int_conn_subc(pcb_lobj_parent_subc(orig_line->parent_type, &orig_line->parent), orig_line->intconn, PCB_TYPE_LINE, orig_line);
+
 			if (setjmp(info.env) == 0)
 				pcb_r_search(PCB->Data->via_tree, (pcb_box_t *) & info.line, NULL, pv_line_callback, &info, NULL);
 			else
@@ -760,8 +767,14 @@ static pcb_bool LookupPVConnectionsToLOList(pcb_bool AndRats)
 
 		/* check all arcs */
 		while (ArcList[layer].Location < ArcList[layer].Number) {
-			info.arc = *(ARCLIST_ENTRY(layer, ArcList[layer].Location));
+			pcb_arc_t *orig_arc = (ARCLIST_ENTRY(layer, ArcList[layer].Location));
+			info.arc = *orig_arc;
 			EXPAND_BOUNDS(&info.arc);
+
+			/* subc intconn jumps */
+			if ((orig_arc->term != NULL) && (orig_arc->intconn > 0))
+				LOC_int_conn_subc(pcb_lobj_parent_subc(orig_arc->parent_type, &orig_arc->parent), orig_arc->intconn, PCB_TYPE_LINE, orig_arc);
+
 			if (setjmp(info.env) == 0)
 				pcb_r_search(PCB->Data->via_tree, (pcb_box_t *) & info.arc, NULL, pv_arc_callback, &info, NULL);
 			else
@@ -776,8 +789,14 @@ static pcb_bool LookupPVConnectionsToLOList(pcb_bool AndRats)
 		/* now all polygons */
 		info.layer = layer;
 		while (PolygonList[layer].Location < PolygonList[layer].Number) {
-			info.polygon = *(POLYGONLIST_ENTRY(layer, PolygonList[layer].Location));
+			pcb_polygon_t *orig_poly = (POLYGONLIST_ENTRY(layer, PolygonList[layer].Location));
+			info.polygon = *orig_poly;
 			EXPAND_BOUNDS(&info.polygon);
+
+			/* subc intconn jumps */
+			if ((orig_poly->term != NULL) && (orig_poly->intconn > 0))
+				LOC_int_conn_subc(pcb_lobj_parent_subc(orig_poly->parent_type, &orig_poly->parent), orig_poly->intconn, PCB_TYPE_LINE, orig_poly);
+
 			if (setjmp(info.env) == 0)
 				pcb_r_search(PCB->Data->via_tree, (pcb_box_t *) & info.polygon, NULL, pv_poly_callback, &info, NULL);
 			else
@@ -882,6 +901,26 @@ static pcb_r_dir_t LOCtoArcPad_callback(const pcb_box_t * b, void *cl)
 	return PCB_R_DIR_NOT_FOUND;
 }
 
+static pcb_r_dir_t LOCtoArcRat_callback(const pcb_box_t *b, void *cl)
+{
+	pcb_rat_t *rat = (pcb_rat_t *)b;
+	struct lo_info *i = (struct lo_info *)cl;
+
+	if (!PCB_FLAG_TEST(TheFlag, rat)) {
+		if ((rat->group1 == i->layer)
+				&& IsRatPointOnArcSpec(&rat->Point1, &i->arc)) {
+			if (ADD_RAT_TO_LIST(rat, PCB_TYPE_ARC, &i->arc, PCB_FCT_RAT))
+				longjmp(i->env, 1);
+		}
+		else if ((rat->group2 == i->layer)
+						 && IsRatPointOnArcSpec(&rat->Point2, &i->arc)) {
+			if (ADD_RAT_TO_LIST(rat, PCB_TYPE_ARC, &i->arc, PCB_FCT_RAT))
+				longjmp(i->env, 1);
+		}
+	}
+	return PCB_R_DIR_NOT_FOUND;
+}
+
 /* ---------------------------------------------------------------------------
  * searches all LOs that are connected to the given arc on the given
  * layergroup. All found connections are added to the list
@@ -897,15 +936,24 @@ static pcb_bool LookupLOConnectionsToArc(pcb_arc_t *Arc, pcb_cardinal_t LayerGro
 
 	info.arc = *Arc;
 	EXPAND_BOUNDS(&info.arc);
+
+
 	/* loop over all layers of the group */
 	for (entry = 0; entry < PCB->LayerGroups.grp[LayerGroup].len; entry++) {
 		pcb_layer_id_t layer;
-		pcb_polygon_t *polygon;
-		gdl_iterator_t it;
 
 		layer = PCB->LayerGroups.grp[LayerGroup].lid[entry];
 
+		info.layer = LayerGroup;
+
+		/* add the new rat lines */
+		if (setjmp(info.env) == 0)
+			pcb_r_search(PCB->Data->rat_tree, &info.arc.BoundingBox, NULL, LOCtoArcRat_callback, &info, NULL);
+		else
+			return pcb_true;
+
 		info.layer = layer;
+
 		/* add arcs */
 		if (setjmp(info.env) == 0)
 			pcb_r_search(LAYER_PTR(layer)->line_tree, &info.arc.BoundingBox, NULL, LOCtoArcLine_callback, &info, NULL);
@@ -978,12 +1026,12 @@ static pcb_r_dir_t LOCtoLineRat_callback(const pcb_box_t * b, void *cl)
 
 	if (!PCB_FLAG_TEST(TheFlag, rat)) {
 		if ((rat->group1 == i->layer)
-				&& IsRatPointOnLineEnd(&rat->Point1, &i->line)) {
+				&& IsRatPointOnLineSpec(&rat->Point1, &i->line)) {
 			if (ADD_RAT_TO_LIST(rat, PCB_TYPE_LINE, &i->line, PCB_FCT_RAT))
 				longjmp(i->env, 1);
 		}
 		else if ((rat->group2 == i->layer)
-						 && IsRatPointOnLineEnd(&rat->Point2, &i->line)) {
+						 && IsRatPointOnLineSpec(&rat->Point2, &i->line)) {
 			if (ADD_RAT_TO_LIST(rat, PCB_TYPE_LINE, &i->line, PCB_FCT_RAT))
 				longjmp(i->env, 1);
 		}
@@ -1019,7 +1067,7 @@ static pcb_bool LookupLOConnectionsToLine(pcb_line_t *Line, pcb_cardinal_t Layer
 	info.layer = LayerGroup;
 	EXPAND_BOUNDS(&info.line)
 		/* add the new rat lines */
-		if (setjmp(info.env) == 0)
+	if (setjmp(info.env) == 0)
 		pcb_r_search(PCB->Data->rat_tree, &info.line.BoundingBox, NULL, LOCtoLineRat_callback, &info, NULL);
 	else
 		return pcb_true;
@@ -1076,28 +1124,36 @@ struct rat_info {
 	jmp_buf env;
 };
 
-static pcb_r_dir_t LOCtoRat_callback(const pcb_box_t * b, void *cl)
+static pcb_r_dir_t LOCtoRatLine_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_line_t *line = (pcb_line_t *) b;
 	struct rat_info *i = (struct rat_info *) cl;
 
-	if (!PCB_FLAG_TEST(TheFlag, line) &&
-			((line->Point1.X == i->Point->X &&
-				line->Point1.Y == i->Point->Y) || (line->Point2.X == i->Point->X && line->Point2.Y == i->Point->Y))) {
+	if (!PCB_FLAG_TEST(TheFlag, line) && IsRatPointOnLineSpec(i->Point, line)) {
 		if (ADD_LINE_TO_LIST(i->layer, line, PCB_TYPE_RATLINE, &i->Point, PCB_FCT_RAT))
 			longjmp(i->env, 1);
 	}
 	return PCB_R_DIR_NOT_FOUND;
 }
 
-static pcb_r_dir_t PolygonToRat_callback(const pcb_box_t * b, void *cl)
+static pcb_r_dir_t LOCtoRatArc_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_arc_t *arc = (pcb_arc_t *) b;
+	struct rat_info *i = (struct rat_info *) cl;
+
+	if (!PCB_FLAG_TEST(TheFlag, arc) && IsRatPointOnArcSpec(i->Point, arc)) {
+		if (ADD_ARC_TO_LIST(i->layer, arc, PCB_TYPE_RATLINE, &i->Point, PCB_FCT_RAT))
+			longjmp(i->env, 1);
+	}
+	return PCB_R_DIR_NOT_FOUND;
+}
+
+static pcb_r_dir_t LOCtoRatPoly_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_polygon_t *polygon = (pcb_polygon_t *) b;
 	struct rat_info *i = (struct rat_info *) cl;
 
-	if (!PCB_FLAG_TEST(TheFlag, polygon) && polygon->Clipped &&
-			(i->Point->X == polygon->Clipped->contours->head.point[0]) &&
-			(i->Point->Y == polygon->Clipped->contours->head.point[1])) {
+	if (!PCB_FLAG_TEST(TheFlag, polygon) && polygon->Clipped && IsRatPointOnPoly(i->Point, polygon)) {
 		if (ADD_POLYGON_TO_LIST(i->layer, polygon, PCB_TYPE_RATLINE, &i->Point, PCB_FCT_RAT))
 			longjmp(i->env, 1);
 	}
@@ -1139,14 +1195,17 @@ static pcb_bool LookupLOConnectionsToRatEnd(pcb_point_t *Point, pcb_cardinal_t L
 		pcb_layer_id_t layer;
 
 		layer = PCB->LayerGroups.grp[LayerGroup].lid[entry];
-		/* handle normal layers rats don't ever touch arcs by definition */
 		info.layer = layer;
 		if (setjmp(info.env) == 0)
-			r_search_pt(LAYER_PTR(layer)->line_tree, Point, 1, NULL, LOCtoRat_callback, &info, NULL);
+			r_search_pt(LAYER_PTR(layer)->arc_tree, Point, 1, NULL, LOCtoRatArc_callback, &info, NULL);
 		else
 			return pcb_true;
 		if (setjmp(info.env) == 0)
-			r_search_pt(LAYER_PTR(layer)->polygon_tree, Point, 1, NULL, PolygonToRat_callback, &info, NULL);
+			r_search_pt(LAYER_PTR(layer)->line_tree, Point, 1, NULL, LOCtoRatLine_callback, &info, NULL);
+		else
+			return pcb_true;
+		if (setjmp(info.env) == 0)
+			r_search_pt(LAYER_PTR(layer)->polygon_tree, Point, 1, NULL, LOCtoRatPoly_callback, &info, NULL);
 	}
 
 	/* handle the special pad layers */
@@ -1254,43 +1313,9 @@ static pcb_bool LookupLOConnectionsToPad(pcb_pad_t *Pad, pcb_cardinal_t LayerGro
 
 	/* Internal connection: if pads in the same element have the same
 	   internal connection group number, they are connected */
-	ic = PCB_FLAG_INTCONN_GET(Pad);
-	if ((Pad->Element != NULL) && (ic > 0)) {
-		pcb_element_t *e = Pad->Element;
-		pcb_pad_t *orig_pad = Pad;
-		int tlayer = -1;
-
-/*fprintf(stderr, "lg===\n");*/
-		for (entry = 0; entry < PCB->LayerGroups.grp[LayerGroup].len; entry++) {
-			pcb_layer_id_t layer;
-			layer = PCB->LayerGroups.grp[LayerGroup].lid[entry];
-/*fprintf(stderr, "lg: %d\n", layer);*/
-			if (layer == PCB_COMPONENT_SIDE)
-				tlayer = PCB_COMPONENT_SIDE;
-			else if (layer == PCB_SOLDER_SIDE)
-				tlayer = PCB_SOLDER_SIDE;
-		}
-
-/*fprintf(stderr, "tlayer=%d\n", tlayer);*/
-
-		if (tlayer >= 0) {
-			PCB_PAD_LOOP(e);
-			{
-				if ((orig_pad != pad) && (ic == PCB_FLAG_INTCONN_GET(pad))) {
-					int padlayer = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) ? PCB_SOLDER_SIDE : PCB_COMPONENT_SIDE;
-/*fprintf(stderr, "layergroup1: %d {%d %d %d} %d \n", tlayer, PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad), PCB_SOLDER_SIDE, PCB_COMPONENT_SIDE, padlayer);*/
-					if ((!PCB_FLAG_TEST(TheFlag, pad)) && (tlayer != padlayer)) {
-/*fprintf(stderr, "layergroup2\n");*/
-						ADD_PAD_TO_LIST(padlayer, pad, PCB_TYPE_PAD, orig_pad, PCB_FCT_INTERNAL);
-						if (LookupLOConnectionsToPad(pad, LayerGroup))
-							retv = pcb_true;
-					}
-				}
-			}
-			PCB_END_LOOP;
-		}
-	}
-
+	ic = Pad->intconn;
+	if ((Pad->Element != NULL) && (ic > 0))
+		LOC_int_conn_element(Pad->Element, ic, PCB_TYPE_PAD, Pad);
 
 	if (!PCB_FLAG_TEST(PCB_FLAG_SQUARE, Pad))
 		return (LookupLOConnectionsToLine((pcb_line_t *) Pad, LayerGroup, pcb_false));
@@ -1388,13 +1413,11 @@ static pcb_r_dir_t LOCtoPolyRat_callback(const pcb_box_t * b, void *cl)
 	struct lo_info *i = (struct lo_info *) cl;
 
 	if (!PCB_FLAG_TEST(TheFlag, rat)) {
-		if ((rat->Point1.X == (i->polygon.Clipped->contours->head.point[0]) &&
-				 rat->Point1.Y == (i->polygon.Clipped->contours->head.point[1]) &&
-				 rat->group1 == i->layer) ||
-				(rat->Point2.X == (i->polygon.Clipped->contours->head.point[0]) &&
-				 rat->Point2.Y == (i->polygon.Clipped->contours->head.point[1]) && rat->group2 == i->layer))
+		if (((rat->group1 == i->layer) && IsRatPointOnPoly(&rat->Point1, &i->polygon))
+			|| ((rat->group2 == i->layer) && IsRatPointOnPoly(&rat->Point2, &i->polygon))) {
 			if (ADD_RAT_TO_LIST(rat, PCB_TYPE_POLYGON, &i->polygon, PCB_FCT_RAT))
 				longjmp(i->env, 1);
+		}
 	}
 	return PCB_R_DIR_NOT_FOUND;
 }
@@ -1423,8 +1446,6 @@ static pcb_bool LookupLOConnectionsToPolygon(pcb_polygon_t *Polygon, pcb_cardina
 /* loop over all layers of the group */
 	for (entry = 0; entry < PCB->LayerGroups.grp[LayerGroup].len; entry++) {
 		pcb_layer_id_t layer;
-		gdl_iterator_t it;
-		pcb_polygon_t *polygon;
 
 		layer = PCB->LayerGroups.grp[LayerGroup].lid[entry];
 
