@@ -33,6 +33,7 @@
 #include "error.h"
 #include "undo.h"
 #include "funchash_core.h"
+#include "change.h"
 
 #include "draw.h"
 #include "search.h"
@@ -48,6 +49,8 @@
 #include "layer_vis.h"
 #include "attrib.h"
 #include "hid_attrib.h"
+#include "operation.h"
+#include "obj_subc_op.h"
 
 #include "obj_elem_draw.h"
 #include "obj_pinvia_draw.h"
@@ -213,6 +216,7 @@ static enum pcb_crosshair_shape_e CrosshairShapeIncrement(enum pcb_crosshair_sha
 	return shape;
 }
 
+extern pcb_opfunc_t ChgFlagFunctions;
 static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_coord_t childY)
 {
 	const char *function, *str_dir;
@@ -237,7 +241,7 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 		case F_Description:
 			PCB_ELEMENT_LOOP(PCB->Data);
 			{
-				EraseElementName(element);
+				pcb_elem_name_invalidate_erase(element);
 			}
 			PCB_END_LOOP;
 			switch (id) {
@@ -268,7 +272,7 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 			}
 			PCB_ELEMENT_LOOP(PCB->Data);
 			{
-				DrawElementName(element);
+				pcb_elem_name_invalidate_draw(element);
 			}
 			PCB_END_LOOP;
 			pcb_draw();
@@ -450,32 +454,34 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 			/* toggle displaying of pin/pad/via names */
 		case F_PinOrPadName:
 			{
+				int type;
 				void *ptr1, *ptr2, *ptr3;
 				pcb_coord_t x, y;
 				pcb_gui->get_coords(_("Click on an element"), &x, &y);
 
+				/* old code for elements */
 				switch (pcb_search_screen(x, y,
 														 PCB_TYPE_ELEMENT | PCB_TYPE_PIN | PCB_TYPE_PAD |
 														 PCB_TYPE_VIA, (void **) &ptr1, (void **) &ptr2, (void **) &ptr3)) {
 				case PCB_TYPE_ELEMENT:
 					PCB_PIN_LOOP((pcb_element_t *) ptr1);
 					{
-						if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, pin))
-							ErasePinName(pin);
+						if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pin))
+							pcb_pin_name_invalidate_erase(pin);
 						else
-							DrawPinName(pin);
-						pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, ptr1, pin, pin);
-						PCB_FLAG_TOGGLE(PCB_FLAG_DISPLAYNAME, pin);
+							pcb_pin_name_invalidate_draw(pin);
+						pcb_undo_add_obj_to_flag(pin);
+						PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, pin);
 					}
 					PCB_END_LOOP;
 					PCB_PAD_LOOP((pcb_element_t *) ptr1);
 					{
-						if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, pad))
-							ErasePadName(pad);
+						if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pad))
+							pcb_pad_name_invalidate_erase(pad);
 						else
-							DrawPadName(pad);
-						pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, ptr1, pad, pad);
-						PCB_FLAG_TOGGLE(PCB_FLAG_DISPLAYNAME, pad);
+							pcb_pad_name_invalidate_draw(pad);
+						pcb_undo_add_obj_to_flag(pad);
+						PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, pad);
 					}
 					PCB_END_LOOP;
 					pcb_board_set_changed_flag(pcb_true);
@@ -484,40 +490,73 @@ static int pcb_act_Display(int argc, const char **argv, pcb_coord_t childX, pcb_
 					break;
 
 				case PCB_TYPE_PIN:
-					if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, (pcb_pin_t *) ptr2))
-						ErasePinName((pcb_pin_t *) ptr2);
+					if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2))
+						pcb_pin_name_invalidate_erase((pcb_pin_t *) ptr2);
 					else
-						DrawPinName((pcb_pin_t *) ptr2);
-					pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, ptr1, ptr2, ptr3);
-					PCB_FLAG_TOGGLE(PCB_FLAG_DISPLAYNAME, (pcb_pin_t *) ptr2);
+						pcb_pin_name_invalidate_draw((pcb_pin_t *) ptr2);
+					pcb_undo_add_obj_to_flag(ptr2);
+					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2);
 					pcb_board_set_changed_flag(pcb_true);
 					pcb_undo_inc_serial();
 					pcb_draw();
 					break;
 
 				case PCB_TYPE_PAD:
-					if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, (pcb_pad_t *) ptr2))
-						ErasePadName((pcb_pad_t *) ptr2);
+					if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, (pcb_pad_t *) ptr2))
+						pcb_pad_name_invalidate_erase((pcb_pad_t *) ptr2);
 					else
-						DrawPadName((pcb_pad_t *) ptr2);
-					pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, ptr1, ptr2, ptr3);
-					PCB_FLAG_TOGGLE(PCB_FLAG_DISPLAYNAME, (pcb_pad_t *) ptr2);
+						pcb_pad_name_invalidate_draw((pcb_pad_t *) ptr2);
+					pcb_undo_add_obj_to_flag(ptr2);
+					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, (pcb_pad_t *) ptr2);
 					pcb_board_set_changed_flag(pcb_true);
 					pcb_undo_inc_serial();
 					pcb_draw();
 					break;
 				case PCB_TYPE_VIA:
-					if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, (pcb_pin_t *) ptr2))
-						EraseViaName((pcb_pin_t *) ptr2);
+					if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2))
+						pcb_via_name_invalidate_erase((pcb_pin_t *) ptr2);
 					else
-						DrawViaName((pcb_pin_t *) ptr2);
-					pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, ptr1, ptr2, ptr3);
-					PCB_FLAG_TOGGLE(PCB_FLAG_DISPLAYNAME, (pcb_pin_t *) ptr2);
+						pcb_via_name_invalidate_draw((pcb_pin_t *) ptr2);
+					pcb_undo_add_obj_to_flag(ptr2);
+					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, (pcb_pin_t *) ptr2);
 					pcb_board_set_changed_flag(pcb_true);
 					pcb_undo_inc_serial();
 					pcb_draw();
 					break;
 				}
+
+				/* toggle terminal ID print for subcircuit parts */
+				type = pcb_search_screen(x, y, PCB_TYPE_SUBC | PCB_TYPE_SUBC_PART | PCB_TYPE_VIA | PCB_TYPE_LINE | PCB_TYPE_ARC | PCB_TYPE_POLYGON | PCB_TYPE_TEXT, (void **)&ptr1, (void **)&ptr2, (void **)&ptr3);
+				if (type) {
+					pcb_any_obj_t *obj = ptr2;
+					pcb_opctx_t opctx;
+					
+					switch(type) {
+						case PCB_TYPE_SUBC:
+							opctx.chgflag.pcb = PCB;
+							opctx.chgflag.how = PCB_CHGFLG_TOGGLE;
+							opctx.chgflag.flag = PCB_FLAG_TERMNAME;
+							pcb_subc_op(PCB->Data, (pcb_subc_t *)obj, &ChgFlagFunctions, &opctx);
+							pcb_undo_inc_serial();
+							return 0;
+							break;
+						case PCB_TYPE_VIA:
+						case PCB_TYPE_LINE:
+						case PCB_TYPE_ARC:
+						case PCB_TYPE_POLYGON:
+						case PCB_TYPE_TEXT:
+							pcb_obj_invalidate_label(type, ptr1, ptr2, ptr3);
+							break;
+						default:
+							return 0; /* nothing else can have a displayed name */
+					}
+
+					pcb_undo_add_obj_to_flag(ptr2);
+					PCB_FLAG_TOGGLE(PCB_FLAG_TERMNAME, obj);
+					pcb_board_set_changed_flag(pcb_true);
+					pcb_undo_inc_serial();
+					pcb_draw();
+				}
 				break;
 			}
 		default:
@@ -663,6 +702,7 @@ static int pcb_act_Mode(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 				case PCB_MODE_LOCK:
 					pcb_crosshair_set_mode(PCB_MODE_NO);
 					pcb_crosshair_set_mode(PCB_MODE_ARROW);
+					Note.Hit = Note.Click = 0; /* if the mouse button is still pressed, don't start selecting a box */
 					break;
 
 				case PCB_MODE_LINE:
@@ -940,10 +980,10 @@ static int pcb_act_ToggleHideName(int argc, const char **argv, pcb_coord_t x, pc
 
 				pcb_gui->get_coords(_("Select an Object"), &x, &y);
 				if ((type = pcb_search_screen(x, y, PCB_TYPE_ELEMENT, &ptr1, &ptr2, &ptr3)) != PCB_TYPE_NONE) {
-					pcb_undo_add_obj_to_flag(type, ptr1, ptr2, ptr3);
-					EraseElementName((pcb_element_t *) ptr2);
+					pcb_undo_add_obj_to_flag(ptr2);
+					pcb_elem_name_invalidate_erase((pcb_element_t *) ptr2);
 					PCB_FLAG_TOGGLE(PCB_FLAG_HIDENAME, (pcb_element_t *) ptr2);
-					DrawElementName((pcb_element_t *) ptr2);
+					pcb_elem_name_invalidate_draw((pcb_element_t *) ptr2);
 					pcb_draw();
 					pcb_undo_inc_serial();
 				}
@@ -957,10 +997,10 @@ static int pcb_act_ToggleHideName(int argc, const char **argv, pcb_coord_t x, pc
 				{
 					if ((PCB_FLAG_TEST(PCB_FLAG_SELECTED, element) || PCB_FLAG_TEST(PCB_FLAG_SELECTED, &PCB_ELEM_TEXT_REFDES(element)))
 							&& (PCB_FRONT(element) || PCB->InvisibleObjectsOn)) {
-						pcb_undo_add_obj_to_flag(PCB_TYPE_ELEMENT, element, element, element);
-						EraseElementName(element);
+						pcb_undo_add_obj_to_flag(element);
+						pcb_elem_name_invalidate_erase(element);
 						PCB_FLAG_TOGGLE(PCB_FLAG_HIDENAME, element);
-						DrawElementName(element);
+						pcb_elem_name_invalidate_draw(element);
 						changed = pcb_true;
 					}
 				}
@@ -1007,14 +1047,24 @@ static int pcb_act_MarkCrosshair(int argc, const char **argv, pcb_coord_t x, pcb
 		else {
 			pcb_notify_mark_change(pcb_false);
 			pcb_marked.status = pcb_true;
-			pcb_gui->get_coords(_("Click on new mark"), &pcb_marked.X, &pcb_marked.Y);
+			if (conf_core.editor.marker_snaps) {
+				pcb_marked.X = pcb_crosshair.X;
+				pcb_marked.Y = pcb_crosshair.Y;
+			}
+			else
+				pcb_gui->get_coords(_("Click on new mark"), &pcb_marked.X, &pcb_marked.Y);
 			pcb_notify_mark_change(pcb_true);
 		}
 	}
 	else if (pcb_funchash_get(function, NULL) == F_Center) {
 		pcb_notify_mark_change(pcb_false);
 		pcb_marked.status = pcb_true;
-		pcb_gui->get_coords(_("Click on new mark"), &pcb_marked.X, &pcb_marked.Y);
+		if (conf_core.editor.marker_snaps) {
+			pcb_marked.X = pcb_crosshair.X;
+			pcb_marked.Y = pcb_crosshair.Y;
+		}
+		else
+			pcb_gui->get_coords(_("Click on new mark"), &pcb_marked.X, &pcb_marked.Y);
 		pcb_notify_mark_change(pcb_true);
 	}
 	return 0;
@@ -1139,10 +1189,10 @@ static void set_same_(pcb_coord_t Thick, pcb_coord_t Diameter, pcb_coord_t Hole,
 	known = pcb_route_style_lookup(&PCB->RouteStyle, Thick, Diameter, Hole, Clearance, Name);
 	if (known < 0) {
 		/* unknown style, set properties */
-		if (Thick != 0)     { pcb_custom_route_style.Thick     = Thick;     conf_set_design("design/line_thickness", "%$mS", Thick); }
-		if (Clearance != 0) { pcb_custom_route_style.Clearance = Clearance; conf_set_design("design/clearance", "%$mS", Clearance); }
-		if (Diameter != 0)  { pcb_custom_route_style.Diameter  = Diameter;  conf_set_design("design/via_thickness", "%$mS", Diameter); }
-		if (Hole != 0)      { pcb_custom_route_style.Hole      = Hole;      conf_set_design("design/via_drilling_hole", "%$mS", Hole); }
+		if (Thick != -1)     { pcb_custom_route_style.Thick     = Thick;     conf_set_design("design/line_thickness", "%$mS", Thick); }
+		if (Clearance != -1) { pcb_custom_route_style.Clearance = Clearance; conf_set_design("design/clearance", "%$mS", Clearance); }
+		if (Diameter != -1)  { pcb_custom_route_style.Diameter  = Diameter;  conf_set_design("design/via_thickness", "%$mS", Diameter); }
+		if (Hole != -1)      { pcb_custom_route_style.Hole      = Hole;      conf_set_design("design/via_drilling_hole", "%$mS", Hole); }
 		PCB->pen_attr = NULL;
 	}
 	else
@@ -1160,7 +1210,7 @@ static int pcb_act_SetSame(int argc, const char **argv, pcb_coord_t x, pcb_coord
 	switch (type) {
 	case PCB_TYPE_LINE:
 		pcb_notify_crosshair_change(pcb_false);
-		set_same_(((pcb_line_t *) ptr2)->Thickness, 0, 0, ((pcb_line_t *) ptr2)->Clearance / 2, NULL);
+		set_same_(((pcb_line_t *) ptr2)->Thickness, -1, -1, ((pcb_line_t *) ptr2)->Clearance / 2, NULL);
 		layer = (pcb_layer_t *) ptr1;
 		if (conf_core.editor.mode != PCB_MODE_LINE)
 			pcb_crosshair_set_mode(PCB_MODE_LINE);
@@ -1170,7 +1220,7 @@ static int pcb_act_SetSame(int argc, const char **argv, pcb_coord_t x, pcb_coord
 
 	case PCB_TYPE_ARC:
 		pcb_notify_crosshair_change(pcb_false);
-		set_same_(((pcb_arc_t *) ptr2)->Thickness, 0, 0, ((pcb_arc_t *) ptr2)->Clearance / 2, NULL);
+		set_same_(((pcb_arc_t *) ptr2)->Thickness, -1, -1, ((pcb_arc_t *) ptr2)->Clearance / 2, NULL);
 		layer = (pcb_layer_t *) ptr1;
 		if (conf_core.editor.mode != PCB_MODE_ARC)
 			pcb_crosshair_set_mode(PCB_MODE_ARC);
@@ -1475,7 +1525,7 @@ static int pcb_act_EditLayer(int argc, const char **argv, pcb_coord_t x, pcb_coo
 			if (val == NULL)
 				ret |= pcb_attribute_remove(&ly->meta.real.Attributes, key);
 			else
-				ret |= pcb_attribute_put(&ly->meta.real.Attributes, key, val, 1);
+				ret |= pcb_attribute_put(&ly->meta.real.Attributes, key, val);
 			free(key);
 			pcb_board_set_changed_flag(pcb_true);
 		}
@@ -1498,7 +1548,7 @@ static int pcb_act_EditLayer(int argc, const char **argv, pcb_coord_t x, pcb_coo
 		attr[1].default_val.int_value = ly->comb & PCB_LYC_SUB;
 		attr[2].default_val.int_value = ly->comb & PCB_LYC_AUTO;
 
-		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "edit layer properies", "Edit the properties of a logical layer");
+		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "edit layer properies", "Edit the properties of a logical layer", NULL);
 
 		if (ar == 0) {
 			pcb_layer_combining_t comb = 0;
@@ -1590,7 +1640,7 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 			if (val == NULL)
 				ret |= pcb_attribute_remove(&g->Attributes, key);
 			else
-				ret |= pcb_attribute_put(&g->Attributes, key, val, 1);
+				ret |= pcb_attribute_put(&g->Attributes, key, val);
 			free(key);
 		}
 #endif
@@ -1609,7 +1659,7 @@ static int pcb_act_EditGroup(int argc, const char **argv, pcb_coord_t x, pcb_coo
 
 		attr[0].default_val.str_value = pcb_strdup(g->name);
 
-		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "edit layer properies", "Edit the properties of a logical layer");
+		ar = pcb_gui->attribute_dialog(attr,sizeof(attr)/sizeof(attr[0]), rv, "edit layer properies", "Edit the properties of a logical layer", NULL);
 
 		if (ar == 0) {
 			if (strcmp(g->name, attr[0].default_val.str_value) != 0) {
@@ -1737,9 +1787,6 @@ static int pcb_act_ToggleView(int argc, const char **argv, pcb_coord_t x, pcb_co
 	return 1;
 }
 
-
-
-
 pcb_hid_action_t gui_action_list[] = {
 	{"Display", 0, pcb_act_Display,
 	 pcb_acth_Display, pcb_acts_Display}
