@@ -11,6 +11,13 @@
 #include "layer.h"
 #include "layer_grp.h"
 
+/* attribute dialog properties */
+typedef enum pcb_hat_property_e {
+	PCB_HATP_GLOBAL_CALLBACK,
+	PCB_HATP_max
+} pcb_hat_property_t;
+
+
 typedef enum {
 	PCB_HID_MOUSE_PRESS,
 	PCB_HID_MOUSE_RELEASE,
@@ -128,17 +135,17 @@ typedef struct {
 	int (*throw_drc_dialog) (void);
 } pcb_hid_drc_gui_t;
 
-typedef enum pcb_mask_op_s {
-	HID_MASK_OFF,    /* Flushes the buffer and return to non-mask operation.  */
+typedef enum pcb_composite_op_s {
+	PCB_HID_COMP_RESET,         /* reset (allocate and clear) the sketch canvas */
+	PCB_HID_COMP_POSITIVE,      /* draw subsequent objects in positive, with color */
+	PCB_HID_COMP_NEGATIVE,      /* draw subsequent objects in negative, ignore color */
+	PCB_HID_COMP_FLUSH          /* combine the sketch canvas onto the output buffer and discard the sketch canvas */
+} pcb_composite_op_t;
 
-	HID_MASK_BEFORE, /* Polygons being drawn before clears.  */
-
-	HID_MASK_INIT,   /* Initialize a new rendering layer for compositing layers  */
-	HID_MASK_CLEAR,  /* negative-draw on the rendering layer  */
-	HID_MASK_SET,    /* positive-draw on the rendering layer */
-
-	HID_MASK_AFTER   /* Polygons being drawn after clears.  */
-} pcb_mask_op_t;
+typedef enum pcb_burst_op_s {
+	PCB_HID_BURST_START,
+	PCB_HID_BURST_END
+} pcb_burst_op_t;
 
 typedef struct hid_s pcb_hid_t;
 
@@ -176,15 +183,6 @@ struct hid_s {
 	   and EPS exporters.  */
 	unsigned exporter:1;
 
-	/* If set, the redraw code will draw polygons before erasing the
-	   clearances.  */
-	unsigned poly_before:1;
-
-	/* If set, the redraw code will draw polygons after erasing the
-	   clearances.  Note that HIDs may set both of these, in which case
-	   polygons will be drawn twice.  */
-	unsigned poly_after:1;
-
 	/* If set, draw the mask layer inverted. Normally the mask is a filled
 	   rectangle over the board with cutouts at pins/pads. The HIDs
 	   use render in normal mode, gerber renders in inverse mode. */
@@ -195,14 +193,6 @@ struct hid_s {
 	   punches through everything. */
 	unsigned holes_after:1;
 
-	/* If set, enable the fake composite layer draw optimization, that is:
-	   if all layers in a compositing layer group are positive, no composition
-	   is done but layers are just drawn. Useful for software render HIDs with
-	   no transluency; causes problems with transluent rendering: different
-	   layers within the same group will be darker on overlap */
-	unsigned enable_fake_composite:1;
-
-
 	/* lesstif allows positive AND negative drawing in HID_MASK_CLEAR.
 	   gtk only allows negative drawing.
 	   using the mask is to get rat transparency */
@@ -262,15 +252,12 @@ struct hid_s {
 	pcb_hid_gc_t (*make_gc) (void);
 	void (*destroy_gc) (pcb_hid_gc_t gc_);
 
-	/* Special note about the "erase" color: To use this color, you must
-	   use this function to tell the HID when you're using it.  At the
-	   beginning of a layer redraw cycle (i.e. after set_layer), call
-	   use_mask() to redirect output to a buffer.  Draw to the buffer
-	   (using regular HID calls) using regular and "erase" colors.  Then
-	   call use_mask(HID_MASK_OFF) to flush the buffer to the HID.  If
-	   you use the "erase" color when use_mask is disabled, it simply
-	   draws in the background color.  */
-	void (*use_mask) (pcb_mask_op_t use_it_);
+	/* Composite layer drawing: manipulate the sketch canvas and set
+	   positive or negative drawing mode. The canvas covers the screen box. */
+	void (*set_drawing_mode)(pcb_composite_op_t op, pcb_bool direct, const pcb_box_t *screen);
+
+	/* Announce start/end of a render burst for a specific screen screen box. */
+	void (*render_burst)(pcb_burst_op_t op, const pcb_box_t *screen);
 
 	/* Sets a color.  Names can be like "red" or "#rrggbb" or special
 	   names like "erase".  *Always* use the "erase" color for removing
@@ -447,8 +434,25 @@ struct hid_s {
 	   may use it for a tooltip or text in a dialog box, or a help
 	   string.
 	 */
-	int (*attribute_dialog) (pcb_hid_attribute_t * attrs_,
-													 int n_attrs_, pcb_hid_attr_val_t * results_, const char *title_, const char *descr_);
+	int (*attribute_dialog)(pcb_hid_attribute_t * attrs_, int n_attrs_, pcb_hid_attr_val_t * results_, const char *title_, const char *descr_, void *caller_data);
+
+	/* The same API in 3 stages: */
+	void *(*attr_dlg_new)(pcb_hid_attribute_t *attrs_, int n_attrs_, pcb_hid_attr_val_t * results_, const char *title_, const char *descr_, void *caller_data); /* returns hid_ctx */
+	int (*attr_dlg_run)(void *hid_ctx);
+	void (*attr_dlg_free)(void *hid_ctx); /* results_ is avalibale after this call */
+
+	/* Set a property of an attribute dialog (typical call is between
+	   attr_dlg_new() and attr_dlg_run()) */
+	void (*attr_dlg_property)(void *hid_ctx, pcb_hat_property_t prop, const pcb_hid_attr_val_t *val);
+
+
+	/* Disable or enable a widget of an active attribute dialog */
+	int (*attr_dlg_widget_state)(void *hid_ctx, int idx, pcb_bool enabled);
+
+	/* Change the current value of a widget; same as if the user chaged it,
+	   except the value-changed callback is inhibited */
+	int (*attr_dlg_set_value)(void *hid_ctx, int idx, const pcb_hid_attr_val_t *val);
+
 
 	/* This causes a second window to display, which only shows the
 	   selected item_. The expose callback is called twice; once to size
