@@ -57,6 +57,7 @@ typedef struct {
 	const char *name;
 } pcb_layer_type_name_t;
 
+/* update docs: lihata format tree */
 static const pcb_layer_type_name_t pcb_layer_type_names[] = {
 	{ PCB_LYT_TOP,     1, "top" },
 	{ PCB_LYT_BOTTOM,  1, "bottom" },
@@ -115,7 +116,13 @@ pcb_bool pcb_layer_is_pure_empty(pcb_layer_t *layer)
 
 pcb_bool pcb_layer_is_empty_(pcb_board_t *pcb, pcb_layer_t *layer)
 {
-	unsigned int flags = pcb_layer_flags_(pcb, layer);
+	pcb_layer_type_t flags;
+
+#warning layer TODO#5: temporary is-bound hack until we get an explicit flag */
+	if (layer->parent == pcb->Data)
+		flags = pcb_layer_flags_(pcb, layer);
+	else
+		flags = layer->meta.bound.type;
 
 	if (flags == 0)
 		return 1;
@@ -405,7 +412,7 @@ static int is_last_bottom_copper_layer(int layer)
 
 int pcb_layer_rename_(pcb_layer_t *Layer, char *Name)
 {
-#warning TODO: duplicate of pcb_layer_rename()?
+#warning cleanup TODO: duplicate of pcb_layer_rename()?
 	if (PCB_LAYER_IS_REAL(Layer)) {
 		free((char*)Layer->meta.real.name);
 		Layer->meta.real.name = Name;
@@ -633,10 +640,11 @@ const char *pcb_layer_name(pcb_layer_id_t id)
 	return NULL;
 }
 
-pcb_layer_t *pcb_get_layer(pcb_layer_id_t id)
+#warning layer TODO: this should be the new pcb_get_layer()
+pcb_layer_t *pcb_get_layer2(pcb_board_t *pcb, pcb_layer_id_t id)
 {
-	if ((id >= 0) && (id < PCB->Data->LayerN))
-		return &PCB->Data->Layer[id];
+	if ((id >= 0) && (id < pcb->Data->LayerN))
+		return &pcb->Data->Layer[id];
 	if (id & PCB_LYT_UI) {
 		id &= ~(PCB_LYT_VIRTUAL | PCB_LYT_UI);
 		if ((id >= 0) && (id < vtlayer_len(&pcb_uilayer)))
@@ -645,6 +653,11 @@ pcb_layer_t *pcb_get_layer(pcb_layer_id_t id)
 	return NULL;
 }
 
+pcb_layer_t *pcb_get_layer(pcb_layer_id_t id)
+{
+	return pcb_get_layer2(PCB, id);
+}
+
 void pcb_layer_link_trees(pcb_layer_t *dst, pcb_layer_t *src)
 {
 	/* we can't link non-existing trees - make sure src does have the trees initialized */
@@ -663,7 +676,6 @@ void pcb_layer_real2bound(pcb_layer_t *dst, pcb_layer_t *src, int share_rtrees)
 {
 	dst->grp = src->grp;
 	dst->comb = src->comb;
-	dst->meta.bound.comb = src->comb;
 
 	if (PCB_LAYER_IS_REAL(src)) {
 		dst->meta.bound.real = src;
@@ -719,10 +731,10 @@ pcb_layer_t *pcb_layer_resolve_binding(pcb_board_t *pcb, pcb_layer_t *src)
 	{
 		pcb_layergrp_t *grp = pcb->LayerGroups.grp+gid;
 		for(l = 0; l < grp->len; l++) {
-			pcb_layer_t *ly = pcb_get_layer(grp->lid[l]);
-			if ((ly->comb & PCB_LYC_SUB) == (src->meta.bound.comb & PCB_LYC_SUB)) {
+			pcb_layer_t *ly = pcb_get_layer2(pcb, grp->lid[l]);
+			if ((ly->comb & PCB_LYC_SUB) == (src->comb & PCB_LYC_SUB)) {
 				score = 1;
-				if (ly->comb == src->meta.bound.comb)
+				if (ly->comb == src->comb)
 					score++;
 
 				if (ly->meta.real.name == src->meta.bound.name) /* mainly for NULL = NULL */
@@ -746,6 +758,18 @@ pcb_layer_t *pcb_layer_resolve_binding(pcb_board_t *pcb, pcb_layer_t *src)
 	return best;
 }
 
+pcb_layer_t *pcb_layer_new_bound(pcb_data_t *data, pcb_layer_type_t type, const char *name)
+{
+	pcb_layer_t *lay = &data->Layer[data->LayerN++];
+
+	memset(lay, 0, sizeof(pcb_layer_t));
+	lay->meta.bound.name = pcb_strdup(name);
+	lay->meta.bound.type = type;
+	lay->grp = -1;
+	lay->parent = data;
+
+	return lay;
+}
 
 int pcb_layer_type_map(pcb_layer_type_t type, void *ctx, void (*cb)(void *ctx, pcb_layer_type_t bit, const char *name, int class, const char *class_name))
 {
@@ -836,6 +860,13 @@ int pcb_layer_gui_set_g_ui(pcb_layer_t *first, int is_empty)
 	return 1;
 }
 
+void pcb_layer_edit_attrib(pcb_layer_t *layer)
+{
+	char *buf = pcb_strdup_printf(buf, "Layer %s Attributes", layer->meta.real.name);
+	pcb_gui->edit_attributes(buf, &(layer->meta.real.Attributes));
+	free(buf);
+}
+
 static pcb_layer_id_t pcb_layer_get_cached(pcb_layer_id_t *cache, unsigned int loc, unsigned int typ)
 {
 	pcb_layergrp_t *g;
