@@ -63,7 +63,13 @@ pcb_layergrp_id_t pcb_layergrp_id(pcb_board_t *pcb, pcb_layergrp_t *grp)
 
 pcb_layergrp_id_t pcb_layer_get_group_(pcb_layer_t *Layer)
 {
-	return Layer->grp;
+	if (PCB_LAYER_IS_REAL(Layer))
+		return Layer->grp;
+
+	if (Layer->meta.bound.real == NULL) /* bound layer without a binding */
+		return -1;
+
+	return Layer->meta.bound.real->grp;
 }
 
 pcb_layergrp_t *pcb_get_layergrp(pcb_board_t *pcb, pcb_layergrp_id_t gid)
@@ -162,6 +168,15 @@ pcb_bool pcb_layergrp_is_empty(pcb_board_t *pcb, pcb_layergrp_id_t num)
 			return pcb_layer_is_paste_empty(pcb, PCB_SOLDER_SIDE);
 	}
 
+#warning padstack TODO: not true with pad stacks
+	/* copper layers are always non-empty if there are vias or pins because of the rings */
+	if (g->type & PCB_LYT_COPPER) {
+		if ((pcb->Data->via_tree != NULL) && (pcb->Data->via_tree->size > 0))
+			return pcb_false;
+		if ((pcb->Data->pin_tree != NULL) && (pcb->Data->pin_tree->size > 0))
+			return pcb_false;
+	}
+
 	for (i = 0; i < g->len; i++)
 		if (!pcb_layer_is_empty(pcb, g->lid[i]))
 			return pcb_false;
@@ -258,8 +273,9 @@ static pcb_layergrp_t *pcb_get_grp_new_intern_(pcb_board_t *pcb, int omit_substr
 {
 	pcb_layer_stack_t *stack = &pcb->LayerGroups;
 	int bl, n;
+	int room = omit_substrate ? 1 : 2;
 
-	if (stack->len+2 >= PCB_MAX_LAYERGRP)
+	if ((stack->len + room) >= PCB_MAX_LAYERGRP)
 		return NULL;
 
 	/* seek the bottom copper layer */
@@ -268,9 +284,9 @@ static pcb_layergrp_t *pcb_get_grp_new_intern_(pcb_board_t *pcb, int omit_substr
 
 			/* insert a new internal layer: move existing layers to make room */
 			for(n = stack->len-1; n >= bl; n--)
-				pcb_layergrp_move_onto(pcb, n+2, n);
+				pcb_layergrp_move_onto(pcb, n+room, n);
 
-			stack->len += 2;
+			stack->len += room;
 
 			stack->grp[bl].name = pcb_strdup("Intern");
 			stack->grp[bl].type = PCB_LYT_INTERN | PCB_LYT_COPPER;
@@ -581,7 +597,7 @@ int pcb_layergrp_list(pcb_board_t *pcb, pcb_layer_type_t mask, pcb_layergrp_id_t
 	return used;
 }
 
-int pcb_layer_group_list_any(pcb_board_t *pcb, pcb_layer_type_t mask, pcb_layergrp_id_t *res, int res_len)
+int pcb_layergrp_list_any(pcb_board_t *pcb, pcb_layer_type_t mask, pcb_layergrp_id_t *res, int res_len)
 {
 	int group, used = 0;
 	for (group = 0; group < pcb->LayerGroups.len; group++) {
@@ -629,16 +645,16 @@ void pcb_layer_group_setup_default(pcb_layer_stack_t *newg)
 
 	memset(newg, 0, sizeof(pcb_layer_stack_t));
 
-	NEWG(g, PCB_LYT_TOP | PCB_LYT_PASTE, "top paste");
-	NEWG(g, PCB_LYT_TOP | PCB_LYT_SILK, "top silk");
-	NEWG(g, PCB_LYT_TOP | PCB_LYT_MASK, "top mask");
-	NEWG(g, PCB_LYT_TOP | PCB_LYT_COPPER, "top copper");
+	NEWG(g, PCB_LYT_TOP | PCB_LYT_PASTE, "top_paste");
+	NEWG(g, PCB_LYT_TOP | PCB_LYT_SILK, "top_silk");
+	NEWG(g, PCB_LYT_TOP | PCB_LYT_MASK, "top_mask");
+	NEWG(g, PCB_LYT_TOP | PCB_LYT_COPPER, "top_copper");
 	NEWG(g, PCB_LYT_INTERN | PCB_LYT_SUBSTRATE, NULL);
 
-	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_COPPER, "bottom copper");
-	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_MASK, "bottom mask");
-	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_SILK, "bottom silk");
-	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_PASTE, "bottom paste");
+	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_COPPER, "bottom_copper");
+	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_MASK, "bottom_mask");
+	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_SILK, "bottom_silk");
+	NEWG(g, PCB_LYT_BOTTOM | PCB_LYT_PASTE, "bottom_paste");
 
 /*	NEWG(g, PCB_LYT_INTERN | PCB_LYT_OUTLINE, "outline");*/
 }
@@ -703,7 +719,7 @@ int pcb_layergrp_dist(pcb_board_t *pcb, pcb_layergrp_id_t gid1, pcb_layergrp_id_
 
 pcb_layergrp_id_t pcb_layergrp_step(pcb_board_t *pcb, pcb_layergrp_id_t gid, int steps, pcb_layer_type_t mask)
 {
-	int d, cnt;
+	int d;
 
 	if (gid < 0)
 		return -1;
@@ -730,6 +746,97 @@ pcb_layergrp_id_t pcb_layergrp_step(pcb_board_t *pcb, pcb_layergrp_id_t gid, int
 	return -1;
 }
 
+int pcb_layer_create_all_for_recipe(pcb_board_t *pcb, pcb_layer_t *layer, int num_layer)
+{
+	int n, existing_intern = 0, want_intern = 0;
+	static char *anon = "anon";
+
+	for(n = 0; n < pcb->LayerGroups.len; n++)
+		if ((pcb->LayerGroups.grp[n].type & PCB_LYT_COPPER) && (pcb->LayerGroups.grp[n].type & PCB_LYT_INTERN))
+			existing_intern++;
+
+	for(n = 0; n < num_layer; n++) {
+		pcb_layer_t *ly = layer + n;
+		pcb_layer_t *dst = pcb_layer_resolve_binding(pcb, ly);
+		pcb_layergrp_t *grp;
+
+		if ((ly->meta.bound.type & PCB_LYT_COPPER) && (ly->meta.bound.type & PCB_LYT_INTERN)) {
+			want_intern++;
+			continue;
+		}
+
+		if (dst != NULL)
+			continue;
+
+		if (ly->meta.bound.type & PCB_LYT_VIRTUAL) {
+			/* no chance to have this yet */
+			continue;
+		}
+
+		if (ly->meta.bound.type & PCB_LYT_OUTLINE) {
+			pcb_layergrp_t *grp = pcb_get_grp_new_misc(pcb);
+			grp->type = PCB_LYT_OUTLINE | PCB_LYT_INTERN;
+			grp->name = pcb_strdup("outline");
+			pcb_layer_create(pcb_layergrp_id(pcb, grp), ly->meta.bound.name);
+			continue;
+		}
+
+
+		if (ly->meta.bound.type & PCB_LYT_COPPER) { /* top or bottom copper */
+			grp = pcb_get_grp(&pcb->LayerGroups, ly->meta.bound.type & PCB_LYT_ANYWHERE, PCB_LYT_COPPER);
+			if (grp != NULL) {
+				pcb_layer_create(pcb_layergrp_id(pcb, grp), ly->meta.bound.name);
+				continue;
+			}
+		}
+
+		grp = pcb_get_grp(&pcb->LayerGroups, ly->meta.bound.type & PCB_LYT_ANYWHERE, ly->meta.bound.type & PCB_LYT_ANYTHING);
+		if (grp != NULL) {
+			pcb_layer_id_t lid = pcb_layer_create(pcb_layergrp_id(pcb, grp), ly->meta.bound.name);
+			pcb_layer_t *nly = pcb_get_layer(lid);
+			nly->comb = ly->comb;
+			continue;
+		}
+
+		pcb_message(PCB_MSG_ERROR, "Failed to create layer from recipe %s\n", ly->meta.bound.name);
+	}
+
+	if (want_intern > existing_intern) {
+		int int_ofs = 0;
+/*pcb_trace("want: %d have: %d\n", want_intern, existing_intern);*/
+		/* create enough dummy internal layers, mark them by name anon */
+		while(want_intern > existing_intern) {
+			pcb_layergrp_t *grp = pcb_get_grp_new_intern(pcb, -1);
+			grp->name = anon;
+			existing_intern++;
+		}
+		/* rename new interns from the recipe layer names */
+		for(n = 0; n < pcb->LayerGroups.len; n++) {
+			if (pcb->LayerGroups.grp[n].name == anon) {
+				int m;
+				int_ofs++;
+				for(m = 0; m < num_layer; m++) {
+					pcb_layer_t *ly = layer + m;
+					if ((ly->meta.bound.type & PCB_LYT_COPPER) && (ly->meta.bound.type & PCB_LYT_INTERN)) {
+						int offs = ly->meta.bound.stack_offs;
+/*pcb_trace("offs: %d (%d) == %d\n", offs, existing_intern + offs + 1, int_ofs);*/
+						if (offs < 0)
+							offs = existing_intern + offs + 1;
+						if ((offs == int_ofs) && (ly->meta.bound.name != NULL)) {
+							pcb->LayerGroups.grp[n].name = pcb_strdup("internal");
+							pcb_layer_create(n, ly->meta.bound.name);
+							goto found;
+						}
+					}
+				}
+				pcb->LayerGroups.grp[n].name = pcb_strdup("anon-recipe");
+				found:;
+			}
+		}
+	}
+	return 0;
+}
+
 
 static pcb_layergrp_id_t pcb_layergrp_get_cached(pcb_board_t *pcb, pcb_layer_id_t *cache, unsigned int loc, unsigned int typ)
 {
