@@ -37,6 +37,7 @@
 #include "move.h"
 #include "conf_core.h"
 #include "compat_misc.h"
+#include "draw_wireframe.h"
 
 #include "obj_arc.h"
 #include "obj_arc_op.h"
@@ -56,6 +57,7 @@ pcb_arc_t *pcb_arc_alloc(pcb_layer_t * layer)
 
 	new_obj = calloc(sizeof(pcb_arc_t), 1);
 	new_obj->type = PCB_OBJ_ARC;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, layer, layer);
 	arclist_append(&layer->Arc, new_obj);
 
@@ -229,7 +231,7 @@ static pcb_arc_t *pcb_arc_copy_meta(pcb_arc_t *dst, pcb_arc_t *src)
 {
 	if (dst == NULL)
 		return NULL;
-	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes, 0);
+	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes);
 	return dst;
 }
 
@@ -350,20 +352,20 @@ void *pcb_arcop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_arc_t
 /* changes the size of an arc */
 void *pcb_arcop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Arc->Thickness + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Arc->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc))
 		return (NULL);
 	if (value <= PCB_MAX_LINESIZE && value >= PCB_MIN_LINESIZE && value != Arc->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_ARC, Layer, Arc, Arc);
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		Arc->Thickness = value;
 		pcb_arc_bbox(Arc);
 		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
-		DrawArc(Layer, Arc);
+		pcb_arc_invalidate_draw(Layer, Arc);
 		return (Arc);
 	}
 	return (NULL);
@@ -372,27 +374,27 @@ void *pcb_arcop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc
 /* changes the clearance size of an arc */
 void *pcb_arcop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Arc->Clearance + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Arc->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc))
 		return (NULL);
 	if (value < 0)
 		value = 0;
-	value = MIN(PCB_MAX_LINESIZE, MAX(value, PCB->Bloat * 2 + 2));
+	value = MIN(PCB_MAX_LINESIZE, value);
+	if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < PCB->Bloat * 2)
+		value = 0;
+	if (ctx->chgsize.value > 0 && value < PCB->Bloat * 2)
+		value = PCB->Bloat * 2 + 2;
 	if (value != Arc->Clearance) {
 		pcb_undo_add_obj_to_clear_size(PCB_TYPE_ARC, Layer, Arc, Arc);
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		Arc->Clearance = value;
-		if (Arc->Clearance == 0) {
-			PCB_FLAG_CLEAR(PCB_FLAG_CLEARLINE, Arc);
-			Arc->Clearance = PCB_MIL_TO_COORD(10);
-		}
 		pcb_arc_bbox(Arc);
 		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
-		DrawArc(Layer, Arc);
+		pcb_arc_invalidate_draw(Layer, Arc);
 		return (Arc);
 	}
 	return (NULL);
@@ -418,18 +420,18 @@ void *pcb_arcop_change_radius(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *A
 			return NULL;
 	}
 
-	value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : (*dst) + ctx->chgsize.delta;
+	value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : (*dst) + ctx->chgsize.value;
 	value = MIN(PCB_MAX_ARCSIZE, MAX(value, PCB_MIN_ARCSIZE));
 	if (value != *dst) {
 		pcb_undo_add_obj_to_change_radii(PCB_TYPE_ARC, Layer, Arc, Arc);
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		*dst = value;
 		pcb_arc_bbox(Arc);
 		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
-		DrawArc(Layer, Arc);
+		pcb_arc_invalidate_draw(Layer, Arc);
 		return (Arc);
 	}
 	return (NULL);
@@ -455,21 +457,21 @@ void *pcb_arcop_change_angle(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Ar
 			return NULL;
 	}
 
-	value = (ctx->chgangle.absolute) ? ctx->chgangle.absolute : (*dst) + ctx->chgangle.delta;
+	value = (ctx->chgangle.is_absolute) ? ctx->chgangle.value : (*dst) + ctx->chgangle.value;
 	value = fmod(value, 360.0);
 	if (value < 0)
 		value += 360;
 
 	if (value != *dst) {
 		pcb_undo_add_obj_to_change_angles(PCB_TYPE_ARC, Layer, Arc, Arc);
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 		pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		*dst = value;
 		pcb_arc_bbox(Arc);
 		pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
-		DrawArc(Layer, Arc);
+		pcb_arc_invalidate_draw(Layer, Arc);
 		return (Arc);
 	}
 	return (NULL);
@@ -480,18 +482,18 @@ void *pcb_arcop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc))
 		return (NULL);
-	EraseArc(Arc);
+	pcb_arc_invalidate_erase(Arc);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc)) {
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_ARC, Layer, Arc, Arc, pcb_false);
 	}
-	pcb_undo_add_obj_to_flag(PCB_TYPE_ARC, Layer, Arc, Arc);
+	pcb_undo_add_obj_to_flag(Arc);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARLINE, Arc);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Arc)) {
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_ARC, Layer, Arc, Arc, pcb_true);
 	}
-	DrawArc(Layer, Arc);
+	pcb_arc_invalidate_draw(Layer, Arc);
 	return (Arc);
 }
 
@@ -522,7 +524,7 @@ void *pcb_arcop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 	if (!arc)
 		return (arc);
 	pcb_arc_copy_meta(arc, Arc);
-	DrawArc(Layer, arc);
+	pcb_arc_invalidate_draw(Layer, arc);
 	pcb_undo_add_obj_to_create(PCB_TYPE_ARC, Layer, arc, arc);
 	return (arc);
 }
@@ -533,9 +535,9 @@ void *pcb_arcop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	if (Layer->meta.real.vis) {
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 		pcb_arc_move(Arc, ctx->move.dx, ctx->move.dy);
-		DrawArc(Layer, Arc);
+		pcb_arc_invalidate_draw(Layer, Arc);
 		pcb_draw();
 	}
 	else {
@@ -574,19 +576,19 @@ void *pcb_arcop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_arc_t *
 		return NULL;
 	}
 	if (ctx->move.dst_layer == Layer && Layer->meta.real.vis) {
-		DrawArc(Layer, Arc);
+		pcb_arc_invalidate_draw(Layer, Arc);
 		pcb_draw();
 	}
-	if (((long int) ctx->move.dst_layer == -1) || ctx->move.dst_layer == Layer)
+	if (ctx->move.dst_layer == Layer)
 		return (Arc);
 	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_ARC, Layer, Arc, Arc);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	if (Layer->meta.real.vis)
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 	newone = (pcb_arc_t *) pcb_arcop_move_to_layer_low(ctx, Layer, Arc, ctx->move.dst_layer);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, ctx->move.dst_layer, Arc);
 	if (ctx->move.dst_layer->meta.real.vis)
-		DrawArc(ctx->move.dst_layer, newone);
+		pcb_arc_invalidate_draw(ctx->move.dst_layer, newone);
 	pcb_draw();
 	return (newone);
 }
@@ -607,7 +609,7 @@ void *pcb_arcop_remve(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	/* erase from screen */
 	if (Layer->meta.real.vis) {
-		EraseArc(Arc);
+		pcb_arc_invalidate_erase(Arc);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -685,13 +687,13 @@ void pcb_arc_flip_side(pcb_layer_t *layer, pcb_arc_t *arc)
 /* rotates an arc */
 void *pcb_arcop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
-	EraseArc(Arc);
+	pcb_arc_invalidate_erase(Arc);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
 	pcb_r_delete_entry(Layer->arc_tree, (pcb_box_t *) Arc);
 	pcb_arc_rotate90(Arc, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
 	pcb_r_insert_entry(Layer->arc_tree, (pcb_box_t *) Arc, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_ARC, Layer, Arc);
-	DrawArc(Layer, Arc);
+	pcb_arc_invalidate_draw(Layer, Arc);
 	pcb_draw();
 	return (Arc);
 }
@@ -725,35 +727,121 @@ void *pcb_arc_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *arc)
 	return new_arc;
 }
 
-#define PCB_ARC_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_CLEARLINE | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_RUBBEREND | PCB_FLAG_LOCK | PCB_FLAG_VISIT)
 void *pcb_arcop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
-	if ((ctx->chgflag.flag & PCB_ARC_FLAGS) != ctx->chgflag.flag)
+	static pcb_flag_values_t pcb_arc_flags = 0;
+	if (pcb_arc_flags == 0)
+		pcb_arc_flags = pcb_obj_valid_flags(PCB_TYPE_ARC);
+
+	if ((ctx->chgflag.flag & pcb_arc_flags) != ctx->chgflag.flag)
 		return NULL;
-	pcb_undo_add_obj_to_flag(PCB_TYPE_ARC, Arc, Arc, Arc);
+	if ((ctx->chgflag.flag & PCB_FLAG_TERMNAME) && (Arc->term == NULL))
+		return NULL;
+	pcb_undo_add_obj_to_flag(Arc);
+
+	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
+		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_TYPE_ARC, Arc->parent.layer, Arc);
+
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, Arc);
+
+	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
+		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_TYPE_ARC, Arc->parent.layer, Arc);
+
 	return Arc;
 }
 
+void *pcb_arcop_invalidate_label(pcb_opctx_t *ctx, pcb_layer_t *layer, pcb_arc_t *arc)
+{
+	pcb_arc_name_invalidate_draw(arc);
+	return arc;
+}
+
 
 /*** draw ***/
-void _draw_arc(pcb_arc_t * arc)
+
+static void arc_label_pos(const pcb_arc_t *arc, pcb_coord_t *x0, pcb_coord_t *y0, pcb_bool_t *vert)
+{
+	double da, ea, la;
+
+	da = PCB_CLAMP(arc->Delta, -360, 360);
+	ea = arc->StartAngle + da;
+	while(ea < -360) ea += 360;
+	while(ea > +360) ea -= 360;
+
+	la = (arc->StartAngle+ea)/2.0;
+
+	*x0 = pcb_round((double)arc->X - (double)arc->Width * cos(la * PCB_M180));
+	*y0 = pcb_round((double)arc->Y + (double)arc->Height * sin(la * PCB_M180));
+	*vert = (((la < 45) && (la > -45)) || ((la > 135) && (la < 225)));
+}
+
+void pcb_arc_middle(const pcb_arc_t *arc, pcb_coord_t *x, pcb_coord_t *y)
+{
+	pcb_bool_t waste;
+	arc_label_pos(arc, x, y, &waste);
+}
+
+void pcb_arc_name_invalidate_draw(pcb_arc_t *arc)
+{
+	if (arc->term != NULL) {
+		pcb_text_t text;
+		pcb_coord_t x0, y0;
+		pcb_bool_t vert;
+
+		arc_label_pos(arc, &x0, &y0, &vert);
+		pcb_term_label_setup(&text, x0, y0, 100.0, vert, pcb_true, arc->term, arc->intconn);
+		pcb_draw_invalidate(&text);
+	}
+}
+
+void pcb_arc_draw_label(pcb_arc_t *arc)
+{
+	if (arc->term != NULL) {
+		pcb_coord_t x0, y0;
+		pcb_bool_t vert;
+
+		arc_label_pos(arc, &x0, &y0, &vert);
+		pcb_term_label_draw(x0, y0, 100.0, vert, pcb_true, arc->term, arc->intconn);
+	}
+}
+
+void pcb_arc_draw_(pcb_arc_t * arc, int allow_term_gfx)
 {
 	if (!arc->Thickness)
 		return;
 
 	PCB_DRAW_BBOX(arc);
 
-	if (conf_core.editor.thin_draw)
-		pcb_gui->set_line_width(Output.fgGC, 0);
-	else
+	if (!conf_core.editor.thin_draw && !conf_core.editor.wireframe_draw)
+	{
+		if ((allow_term_gfx) && pcb_draw_term_need_gfx(arc)) {
+			pcb_gui->set_line_width(Output.active_padGC, arc->Thickness);
+			pcb_gui->draw_arc(Output.active_padGC, arc->X, arc->Y, arc->Width, arc->Height, arc->StartAngle, arc->Delta);
+			pcb_gui->set_line_width(Output.fgGC, pcb_draw_term_gfx_thick(arc->Thickness));
+		}
+		else
 		pcb_gui->set_line_width(Output.fgGC, arc->Thickness);
-	pcb_gui->set_line_cap(Output.fgGC, Trace_Cap);
+		pcb_gui->set_line_cap(Output.fgGC, Trace_Cap);
+		pcb_gui->draw_arc(Output.fgGC, arc->X, arc->Y, arc->Width, arc->Height, arc->StartAngle, arc->Delta);
+	}
+	else
+	{
+		pcb_gui->set_line_width(Output.fgGC, 0);
+		pcb_gui->set_line_cap(Output.fgGC, Trace_Cap);
 
-	pcb_gui->draw_arc(Output.fgGC, arc->X, arc->Y, arc->Width, arc->Height, arc->StartAngle, arc->Delta);
+		if(conf_core.editor.thin_draw)
+			pcb_gui->draw_arc(Output.fgGC, arc->X, arc->Y, arc->Width, arc->Height, arc->StartAngle, arc->Delta);
+
+		if(conf_core.editor.wireframe_draw)
+			pcb_draw_wireframe_arc(Output.fgGC,arc);
+	}
+	if (arc->term != NULL) {
+		if ((pcb_draw_doing_pinout) || PCB_FLAG_TEST(PCB_FLAG_TERMNAME, arc))
+			pcb_draw_delay_label_add((pcb_any_obj_t *)arc);
+	}
 }
 
-void draw_arc(pcb_layer_t * layer, pcb_arc_t * arc)
+static void pcb_arc_draw(pcb_layer_t * layer, pcb_arc_t * arc, int allow_term_gfx)
 {
 	const char *color;
 	char buf[sizeof("#XXXXXX")];
@@ -775,22 +863,33 @@ void draw_arc(pcb_layer_t * layer, pcb_arc_t * arc)
 		color = buf;
 	}
 	pcb_gui->set_color(Output.fgGC, color);
-	_draw_arc(arc);
+	pcb_arc_draw_(arc, allow_term_gfx);
 }
 
-pcb_r_dir_t draw_arc_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_arc_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_arc_t *arc = (pcb_arc_t *)b;
 
-	if (!PCB->SubcPartsOn && pcb_is_lobj_in_subc(arc->parent_type, &arc->parent))
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(arc->parent_type, &arc->parent))
 		return PCB_R_DIR_NOT_FOUND;
 
-	draw_arc((pcb_layer_t *)cl, arc);
+	pcb_arc_draw((pcb_layer_t *)cl, arc, 0);
+	return PCB_R_DIR_FOUND_CONTINUE;
+}
+
+pcb_r_dir_t pcb_arc_draw_term_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_arc_t *arc = (pcb_arc_t *)b;
+
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(arc->parent_type, &arc->parent))
+		return PCB_R_DIR_NOT_FOUND;
+
+	pcb_arc_draw((pcb_layer_t *)cl, arc, 1);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
 /* erases an arc on a layer */
-void EraseArc(pcb_arc_t *Arc)
+void pcb_arc_invalidate_erase(pcb_arc_t *Arc)
 {
 	if (!Arc->Thickness)
 		return;
@@ -798,7 +897,7 @@ void EraseArc(pcb_arc_t *Arc)
 	pcb_flag_erase(&Arc->Flags);
 }
 
-void DrawArc(pcb_layer_t *Layer, pcb_arc_t *Arc)
+void pcb_arc_invalidate_draw(pcb_layer_t *Layer, pcb_arc_t *Arc)
 {
 	pcb_draw_invalidate(Arc);
 }
