@@ -62,11 +62,17 @@
 pcb_element_t *pcb_element_alloc(pcb_data_t * data)
 {
 	pcb_element_t *new_obj;
+	int n;
 
 	new_obj = calloc(sizeof(pcb_element_t), 1);
 	new_obj->type = PCB_OBJ_ELEMENT;
 	PCB_SET_PARENT(new_obj, data, data);
 
+	for(n = 0; n < PCB_MAX_ELEMENTNAMES; n++) {
+		new_obj->Name[n].type = PCB_OBJ_ETEXT;
+		PCB_SET_PARENT(&new_obj->Name[n], element, new_obj);
+	}
+
 	elementlist_append(&data->Element, new_obj);
 
 	return new_obj;
@@ -131,21 +137,23 @@ pcb_line_t *pcb_element_line_alloc(pcb_element_t *Element)
  */
 pcb_bool pcb_element_load_to_buffer(pcb_buffer_t *Buffer, const char *Name)
 {
-	pcb_element_t *element;
-
 	pcb_buffer_clear(PCB, Buffer);
 	if (!pcb_parse_element(Buffer->Data, Name)) {
 		if (conf_core.editor.show_solder_side)
 			pcb_buffer_flip_side(PCB, Buffer);
 		pcb_set_buffer_bbox(Buffer);
+
+		Buffer->X = 0;
+		Buffer->Y = 0;
+
 		if (elementlist_length(&Buffer->Data->Element)) {
-			element = elementlist_first(&Buffer->Data->Element);
+			pcb_element_t *element = elementlist_first(&Buffer->Data->Element);
 			Buffer->X = element->MarkX;
 			Buffer->Y = element->MarkY;
 		}
-		else {
-			Buffer->X = 0;
-			Buffer->Y = 0;
+		else if (elementlist_length(&Buffer->Data->subc)) {
+			pcb_subc_t *subc = elementlist_first(&Buffer->Data->subc);
+			pcb_subc_get_origin(subc, &Buffer->X, &Buffer->Y);
 		}
 		return (pcb_true);
 	}
@@ -170,6 +178,7 @@ pcb_bool pcb_element_smash_buffer(pcb_buffer_t *Buffer)
 	pcb_element_t *element;
 	pcb_layergrp_id_t group, gbottom, gtop;
 	pcb_layer_t *clayer, *slayer;
+	char tmp[128];
 
 	if (elementlist_length(&Buffer->Data->Element) != 1)
 		return (pcb_false);
@@ -202,12 +211,28 @@ pcb_bool pcb_element_smash_buffer(pcb_buffer_t *Buffer)
 	PCB_END_LOOP;
 	PCB_PIN_LOOP(element);
 	{
+		pcb_pin_t *via;
 		pcb_flag_t f = pcb_no_flags();
 		pcb_flag_add(f, PCB_FLAG_VIA);
 		if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin))
-			pcb_flag_add(f, PCB_FLAG_HOLE);
+			f = pcb_flag_add(f, PCB_FLAG_HOLE);
 
-		pcb_via_new(Buffer->Data, pin->X, pin->Y, pin->Thickness, pin->Clearance, pin->Mask, pin->DrillingHole, pin->Number, f);
+		if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pin))
+			f = pcb_flag_add(f, PCB_FLAG_SQUARE);
+		if (PCB_FLAG_TEST(PCB_FLAG_OCTAGON, pin))
+			f = pcb_flag_add(f, PCB_FLAG_OCTAGON);
+
+		via = pcb_via_new(Buffer->Data, pin->X, pin->Y, pin->Thickness, pin->Clearance, pin->Mask, pin->DrillingHole, pin->Number, f);
+		if (pin->Number != NULL)
+			pcb_attribute_put(&via->Attributes, "term", pin->Number);
+		if (pin->Flags.q != 0) {
+			char tmp[16];
+			sprintf(tmp, "%d", pin->Flags.q);
+			pcb_attribute_put(&via->Attributes, "elem_smash_shape_id", tmp);
+		}
+		pcb_attribute_put(&via->Attributes, "elem_smash_pad", "1");
+		pcb_sprintf(tmp, "%$mm", pin->Mask);
+		pcb_attribute_put(&pin->Attributes, "elem_smash_pad_mask", tmp);
 	}
 	PCB_END_LOOP;
 
@@ -225,8 +250,23 @@ pcb_bool pcb_element_smash_buffer(pcb_buffer_t *Buffer)
 		line = pcb_line_new(PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) ? slayer : clayer,
 																pad->Point1.X, pad->Point1.Y,
 																pad->Point2.X, pad->Point2.Y, pad->Thickness, pad->Clearance, pcb_no_flags());
-		if (line)
+		if (line) {
 			line->Number = pcb_strdup_null(pad->Number);
+			if (pad->Number != NULL)
+				pcb_attribute_put(&line->Attributes, "term", pad->Number);
+			if (pad->Flags.q != 0) {
+				char tmp[16];
+				sprintf(tmp, "%d", pad->Flags.q);
+				pcb_attribute_put(&line->Attributes, "elem_smash_shape_id", tmp);
+			}
+			if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, pad))
+				pcb_attribute_put(&line->Attributes, "elem_smash_shape_square", "1");
+			if (PCB_FLAG_TEST(PCB_FLAG_NOPASTE, pad))
+				pcb_attribute_put(&line->Attributes, "elem_smash_nopaste", "1");
+			pcb_attribute_put(&line->Attributes, "elem_smash_pad", "1");
+			pcb_sprintf(tmp, "%$mm", pad->Mask);
+			pcb_attribute_put(&line->Attributes, "elem_smash_pad_mask", tmp);
+		}
 	}
 	PCB_END_LOOP;
 	pcb_element_destroy(element);
@@ -457,10 +497,10 @@ pcb_bool pcb_element_change_side(pcb_element_t *Element, pcb_coord_t yoff)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
 		return (pcb_false);
-	EraseElement(Element);
+	pcb_elem_invalidate_erase(Element);
 	pcb_undo_add_obj_to_mirror(PCB_TYPE_ELEMENT, Element, Element, Element, yoff);
 	pcb_element_mirror(PCB->Data, Element, yoff);
-	DrawElement(Element);
+	pcb_elem_invalidate_draw(Element);
 	return (pcb_true);
 }
 
@@ -486,12 +526,14 @@ char *pcb_element_text_change(pcb_board_t * pcb, pcb_data_t * data, pcb_element_
 {
 	char *old = Element->Name[which].TextString;
 
+	Element->Name[which].type = PCB_OBJ_ETEXT;
+	PCB_SET_PARENT(&Element->Name[which], element, Element);
 #ifdef DEBUG
 	printf("In ChangeElementText, updating old TextString %s to %s\n", old, new_name);
 #endif
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
-		EraseElementName(Element);
+		pcb_elem_name_invalidate_erase(Element);
 
 	pcb_r_delete_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 
@@ -501,7 +543,7 @@ char *pcb_element_text_change(pcb_board_t * pcb, pcb_data_t * data, pcb_element_
 	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox, 0);
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
-		DrawElementName(Element);
+		pcb_elem_name_invalidate_draw(Element);
 
 	return old;
 }
@@ -509,20 +551,20 @@ char *pcb_element_text_change(pcb_board_t * pcb, pcb_data_t * data, pcb_element_
 void pcb_element_text_update(pcb_board_t *pcb, pcb_data_t *data, pcb_element_t *Element, int which)
 {
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
-		EraseElementName(Element);
+		pcb_elem_name_invalidate_erase(Element);
 
 	pcb_r_delete_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 	pcb_text_bbox(pcb_font(PCB, 0, 1), &Element->Name[which]);
 	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox, 0);
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
-		DrawElementName(Element);
+		pcb_elem_name_invalidate_draw(Element);
 }
 
 void pcb_element_text_set_font(pcb_board_t *pcb, pcb_data_t *data, pcb_element_t *Element, int which, pcb_font_id_t fid)
 {
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
-		EraseElementName(Element);
+		pcb_elem_name_invalidate_erase(Element);
 
 	pcb_r_delete_entry(data->name_tree[which], &Element->Name[which].BoundingBox);
 	Element->Name[which].fid = fid;
@@ -530,7 +572,7 @@ void pcb_element_text_set_font(pcb_board_t *pcb, pcb_data_t *data, pcb_element_t
 	pcb_r_insert_entry(data->name_tree[which], &Element->Name[which].BoundingBox, 0);
 
 	if (pcb && which == PCB_ELEMNAME_IDX_VISIBLE())
-		DrawElementName(Element);
+		pcb_elem_name_invalidate_draw(Element);
 }
 
 /* copies data from one element to another and creates the destination; if necessary */
@@ -580,7 +622,7 @@ pcb_element_t *pcb_element_copy(pcb_data_t *Data, pcb_element_t *Dest, pcb_eleme
 	PCB_END_LOOP;
 
 	for (i = 0; i < Src->Attributes.Number; i++)
-		pcb_attribute_put(&Dest->Attributes, Src->Attributes.List[i].name, Src->Attributes.List[i].value, 0);
+		pcb_attribute_put(&Dest->Attributes, Src->Attributes.List[i].name, Src->Attributes.List[i].value);
 
 	Dest->MarkX = Src->MarkX + dx;
 	Dest->MarkY = Src->MarkY + dy;
@@ -1337,13 +1379,13 @@ void *pcb_elemop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 		return (NULL);
 	PCB_PIN_LOOP(Element);
 	{
-		value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : pin->DrillingHole + ctx->chgsize.delta;
+		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pin->DrillingHole + ctx->chgsize.value;
 		if (value <= PCB_MAX_PINORVIASIZE &&
 				value >= PCB_MIN_PINORVIAHOLE && (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) || value <= pin->Thickness - PCB_MIN_PINORVIACOPPER)
 				&& value != pin->DrillingHole) {
 			changed = pcb_true;
 			pcb_undo_add_obj_to_2nd_size(PCB_TYPE_PIN, Element, pin, pin);
-			ErasePin(pin);
+			pcb_pin_invalidate_erase(pin);
 			pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, pin);
 			pin->DrillingHole = value;
 			if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin)) {
@@ -1351,7 +1393,7 @@ void *pcb_elemop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 				pin->Thickness = value;
 			}
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, pin);
-			DrawPin(pin);
+			pcb_pin_invalidate_draw(pin);
 		}
 	}
 	PCB_END_LOOP;
@@ -1371,11 +1413,11 @@ void *pcb_elemop_change_1st_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 		return (NULL);
 	PCB_PIN_LOOP(Element);
 	{
-		value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : pin->DrillingHole + ctx->chgsize.delta;
+		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pin->DrillingHole + ctx->chgsize.value;
 		if (value <= PCB_MAX_PINORVIASIZE && value >= pin->DrillingHole + PCB_MIN_PINORVIACOPPER && value != pin->Thickness) {
 			changed = pcb_true;
 			pcb_undo_add_obj_to_size(PCB_TYPE_PIN, Element, pin, pin);
-			ErasePin(pin);
+			pcb_pin_invalidate_erase(pin);
 			pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, pin);
 			pin->Thickness = value;
 			if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin)) {
@@ -1383,7 +1425,7 @@ void *pcb_elemop_change_1st_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 				pin->Thickness = value;
 			}
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, pin);
-			DrawPin(pin);
+			pcb_pin_invalidate_draw(pin);
 		}
 	}
 	PCB_END_LOOP;
@@ -1403,13 +1445,13 @@ void *pcb_elemop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 		return (NULL);
 	PCB_PIN_LOOP(Element);
 	{
-		value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : pin->Clearance + ctx->chgsize.delta;
+		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pin->Clearance + ctx->chgsize.value;
 		if (value <= PCB_MAX_PINORVIASIZE &&
 				value >= PCB_MIN_PINORVIAHOLE && (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) || value <= pin->Thickness - PCB_MIN_PINORVIACOPPER)
 				&& value != pin->Clearance) {
 			changed = pcb_true;
 			pcb_undo_add_obj_to_clear_size(PCB_TYPE_PIN, Element, pin, pin);
-			ErasePin(pin);
+			pcb_pin_invalidate_erase(pin);
 			pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, pin);
 			pin->Clearance = value;
 			if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pin)) {
@@ -1417,18 +1459,18 @@ void *pcb_elemop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 				pin->Thickness = value;
 			}
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, pin);
-			DrawPin(pin);
+			pcb_pin_invalidate_draw(pin);
 		}
 	}
 	PCB_END_LOOP;
 
 	PCB_PAD_LOOP(Element);
 	{
-		value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : pad->Clearance + ctx->chgsize.delta;
+		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : pad->Clearance + ctx->chgsize.value;
 		if (value <= PCB_MAX_PINORVIASIZE && value >= PCB_MIN_PINORVIAHOLE && value != pad->Clearance) {
 			changed = pcb_true;
 			pcb_undo_add_obj_to_clear_size(PCB_TYPE_PAD, Element, pad, pad);
-			ErasePad(pad);
+			pcb_pad_invalidate_erase(pad);
 			pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PAD, Element, pad);
 			pcb_r_delete_entry(PCB->Data->pad_tree, &pad->BoundingBox);
 			pad->Clearance = value;
@@ -1440,7 +1482,7 @@ void *pcb_elemop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 			pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PAD, Element, pad);
-			DrawPad(pad);
+			pcb_pad_invalidate_draw(pad);
 		}
 	}
 	PCB_END_LOOP;
@@ -1460,10 +1502,10 @@ void *pcb_elemop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Element))
 		return (NULL);
 	if (pcb_silk_on(PCB))
-		EraseElement(Element);
+		pcb_elem_invalidate_erase(Element);
 	PCB_ELEMENT_PCB_LINE_LOOP(Element);
 	{
-		value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : line->Thickness + ctx->chgsize.delta;
+		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : line->Thickness + ctx->chgsize.value;
 		if (value <= PCB_MAX_LINESIZE && value >= PCB_MIN_LINESIZE && value != line->Thickness) {
 			pcb_undo_add_obj_to_size(PCB_TYPE_ELEMENT_LINE, Element, line, line);
 			line->Thickness = value;
@@ -1473,7 +1515,7 @@ void *pcb_elemop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	PCB_END_LOOP;
 	PCB_ARC_LOOP(Element);
 	{
-		value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : arc->Thickness + ctx->chgsize.delta;
+		value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : arc->Thickness + ctx->chgsize.value;
 		if (value <= PCB_MAX_LINESIZE && value >= PCB_MIN_LINESIZE && value != arc->Thickness) {
 			pcb_undo_add_obj_to_size(PCB_TYPE_ELEMENT_ARC, Element, arc, arc);
 			arc->Thickness = value;
@@ -1482,7 +1524,7 @@ void *pcb_elemop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 	}
 	PCB_END_LOOP;
 	if (pcb_silk_on(PCB)) {
-		DrawElement(Element);
+		pcb_elem_invalidate_draw(Element);
 	}
 	if (changed)
 		return (Element);
@@ -1492,13 +1534,13 @@ void *pcb_elemop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 /* changes the scaling factor of a elementname object; returns pcb_true if changed */
 void *pcb_elemop_change_name_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
-	int value = ctx->chgsize.absolute ? PCB_COORD_TO_MIL(ctx->chgsize.absolute)
-		: PCB_ELEM_TEXT_DESCRIPTION(Element).Scale + PCB_COORD_TO_MIL(ctx->chgsize.delta);
+	int value = ctx->chgsize.is_absolute ? PCB_COORD_TO_MIL(ctx->chgsize.value)
+		: PCB_ELEM_TEXT_DESCRIPTION(Element).Scale + PCB_COORD_TO_MIL(ctx->chgsize.value);
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, &Element->Name[0]))
 		return (NULL);
 	if (value <= PCB_MAX_TEXTSCALE && value >= PCB_MIN_TEXTSCALE) {
-		EraseElementName(Element);
+		pcb_elem_name_invalidate_erase(Element);
 		PCB_ELEMENT_PCB_TEXT_LOOP(Element);
 		{
 			pcb_undo_add_obj_to_size(PCB_TYPE_ELEMENT_NAME, Element, text, text);
@@ -1508,7 +1550,7 @@ void *pcb_elemop_change_name_size(pcb_opctx_t *ctx, pcb_element_t *Element)
 			pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
 		}
 		PCB_END_LOOP;
-		DrawElementName(Element);
+		pcb_elem_name_invalidate_draw(Element);
 		return (Element);
 	}
 	return (NULL);
@@ -1662,11 +1704,11 @@ void *pcb_elemop_copy(pcb_opctx_t *ctx, pcb_element_t *Element)
 	/* this call clears the polygons */
 	pcb_undo_add_obj_to_create(PCB_TYPE_ELEMENT, element, element, element);
 	if (pcb_silk_on(PCB) && (PCB_FRONT(element) || PCB->InvisibleObjectsOn)) {
-		DrawElementName(element);
-		DrawElementPackage(element);
+		pcb_elem_name_invalidate_draw(element);
+		pcb_elem_package_invalidate_draw(element);
 	}
 	if (PCB->PinOn) {
-		DrawElementPinsAndPads(element);
+		pcb_elem_pp_invalidate_draw(element);
 	}
 #ifdef DEBUG
 	printf(" ... Leaving pcb_elemop_copy.\n");
@@ -1678,7 +1720,7 @@ void *pcb_elemop_copy(pcb_opctx_t *ctx, pcb_element_t *Element)
 void *pcb_elemop_move_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
 	if (pcb_silk_on(PCB) && (PCB_FRONT(Element) || PCB->InvisibleObjectsOn)) {
-		EraseElementName(Element);
+		pcb_elem_name_invalidate_erase(Element);
 		PCB_ELEMENT_PCB_TEXT_LOOP(Element);
 		{
 			if (PCB->Data->name_tree[n])
@@ -1688,7 +1730,7 @@ void *pcb_elemop_move_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 				pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
 		}
 		PCB_END_LOOP;
-		DrawElementName(Element);
+		pcb_elem_name_invalidate_draw(Element);
 		pcb_draw();
 	}
 	else {
@@ -1711,19 +1753,19 @@ void *pcb_elemop_move(pcb_opctx_t *ctx, pcb_element_t *Element)
 	pcb_bool didDraw = pcb_false;
 
 	if (pcb_silk_on(PCB) && (PCB_FRONT(Element) || PCB->InvisibleObjectsOn)) {
-		EraseElement(Element);
+		pcb_elem_invalidate_erase(Element);
 		pcb_element_move(PCB->Data, Element, ctx->move.dx, ctx->move.dy);
-		DrawElementName(Element);
-		DrawElementPackage(Element);
+		pcb_elem_name_invalidate_draw(Element);
+		pcb_elem_package_invalidate_draw(Element);
 		didDraw = pcb_true;
 	}
 	else {
 		if (PCB->PinOn)
-			EraseElementPinsAndPads(Element);
+			pcb_elem_pp_invalidate_erase(Element);
 		pcb_element_move(PCB->Data, Element, ctx->move.dx, ctx->move.dy);
 	}
 	if (PCB->PinOn) {
-		DrawElementPinsAndPads(Element);
+		pcb_elem_pp_invalidate_draw(Element);
 		didDraw = pcb_true;
 	}
 	if (didDraw)
@@ -1768,7 +1810,7 @@ void *pcb_elemop_remove(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
 	/* erase from screen */
 	if ((pcb_silk_on(PCB) || PCB->PinOn) && (PCB_FRONT(Element) || PCB->InvisibleObjectsOn)) {
-		EraseElement(Element);
+		pcb_elem_invalidate_erase(Element);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -1780,9 +1822,9 @@ void *pcb_elemop_remove(pcb_opctx_t *ctx, pcb_element_t *Element)
 /* rotates an element */
 void *pcb_elemop_rotate90(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
-	EraseElement(Element);
+	pcb_elem_invalidate_erase(Element);
 	pcb_element_rotate90(PCB->Data, Element, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
-	DrawElement(Element);
+	pcb_elem_invalidate_draw(Element);
 	pcb_draw();
 	return (Element);
 }
@@ -1792,7 +1834,7 @@ void *pcb_elemop_rotate90(pcb_opctx_t *ctx, pcb_element_t *Element)
  */
 void *pcb_elemop_rotate90_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 {
-	EraseElementName(Element);
+	pcb_elem_name_invalidate_erase(Element);
 	PCB_ELEMENT_PCB_TEXT_LOOP(Element);
 	{
 		pcb_r_delete_entry(PCB->Data->name_tree[n], (pcb_box_t *) text);
@@ -1800,13 +1842,13 @@ void *pcb_elemop_rotate90_name(pcb_opctx_t *ctx, pcb_element_t *Element)
 		pcb_r_insert_entry(PCB->Data->name_tree[n], (pcb_box_t *) text, 0);
 	}
 	PCB_END_LOOP;
-	DrawElementName(Element);
+	pcb_elem_name_invalidate_draw(Element);
 	pcb_draw();
 	return (Element);
 }
 
 /*** draw ***/
-void draw_element_name(pcb_element_t * element)
+void pcb_elem_name_draw(pcb_element_t * element)
 {
 	if ((conf_core.editor.hide_names && pcb_gui->gui) || PCB_FLAG_TEST(PCB_FLAG_HIDENAME, element))
 		return;
@@ -1824,11 +1866,11 @@ void draw_element_name(pcb_element_t * element)
 	else
 		pcb_gui->set_color(Output.fgGC, conf_core.appearance.color.invisible_objects);
 
-	DrawTextLowLevel(&PCB_ELEM_TEXT_VISIBLE(PCB, element), PCB->minSlk);
+	pcb_text_draw_(&PCB_ELEM_TEXT_VISIBLE(PCB, element), PCB->minSlk, 0);
 
 }
 
-pcb_r_dir_t draw_element_name_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_elem_name_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_text_t *text = (pcb_text_t *) b;
 	pcb_element_t *element = (pcb_element_t *) text->Element;
@@ -1838,27 +1880,27 @@ pcb_r_dir_t draw_element_name_callback(const pcb_box_t * b, void *cl)
 		return PCB_R_DIR_NOT_FOUND;
 
 	if (PCB_ON_SIDE(element, *side))
-		draw_element_name(element);
+		pcb_elem_name_draw(element);
 	return PCB_R_DIR_NOT_FOUND;
 }
 
-void draw_element_pins_and_pads(pcb_element_t * element)
+void pcb_elem_pp_draw(pcb_element_t * element)
 {
 	PCB_PAD_LOOP(element);
 	{
 		if (pcb_draw_doing_pinout || pcb_draw_doing_assy || PCB_FRONT(pad) || PCB->InvisibleObjectsOn)
-			draw_pad(pad);
+			pcb_pad_draw(pad);
 	}
 	PCB_END_LOOP;
 	PCB_PIN_LOOP(element);
 	{
-		draw_pin(pin, pcb_true);
+		pcb_pin_draw(pin, pcb_true);
 	}
 	PCB_END_LOOP;
 }
 
 
-void draw_element_package(pcb_element_t * element)
+void pcb_elem_package_draw(pcb_element_t * element)
 {
 	/* set color and draw lines, arcs, text and pins */
 	if (pcb_draw_doing_pinout || pcb_draw_doing_assy)
@@ -1873,30 +1915,30 @@ void draw_element_package(pcb_element_t * element)
 	/* draw lines, arcs, text and pins */
 	PCB_ELEMENT_PCB_LINE_LOOP(element);
 	{
-		_draw_line(line);
+		pcb_line_draw_(line, 0);
 	}
 	PCB_END_LOOP;
 	PCB_ARC_LOOP(element);
 	{
-		_draw_arc(arc);
+		pcb_arc_draw_(arc, 0);
 	}
 	PCB_END_LOOP;
 }
 
-void draw_element(pcb_element_t *element)
+void pcb_elem_draw(pcb_element_t *element)
 {
-	draw_element_package(element);
-	draw_element_name(element);
-	draw_element_pins_and_pads(element);
+	pcb_elem_package_draw(element);
+	pcb_elem_name_draw(element);
+	pcb_elem_pp_draw(element);
 }
 
-pcb_r_dir_t draw_element_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_elem_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_element_t *element = (pcb_element_t *) b;
 	int *side = cl;
 
 	if (PCB_ON_SIDE(element, *side))
-		draw_element_package(element);
+		pcb_elem_package_draw(element);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
@@ -1938,7 +1980,7 @@ static void DrawEMark(pcb_element_t *e, pcb_coord_t X, pcb_coord_t Y, pcb_bool i
 	}
 }
 
-pcb_r_dir_t draw_element_mark_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_elem_mark_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_element_t *element = (pcb_element_t *) b;
 
@@ -1946,84 +1988,84 @@ pcb_r_dir_t draw_element_mark_callback(const pcb_box_t * b, void *cl)
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-void EraseElement(pcb_element_t *Element)
+void pcb_elem_invalidate_erase(pcb_element_t *Element)
 {
 	PCB_ELEMENT_PCB_LINE_LOOP(Element);
 	{
-		EraseLine(line);
+		pcb_line_invalidate_erase(line);
 	}
 	PCB_END_LOOP;
 	PCB_ARC_LOOP(Element);
 	{
-		EraseArc(arc);
+		pcb_arc_invalidate_erase(arc);
 	}
 	PCB_END_LOOP;
-	EraseElementName(Element);
-	EraseElementPinsAndPads(Element);
+	pcb_elem_name_invalidate_erase(Element);
+	pcb_elem_pp_invalidate_erase(Element);
 	pcb_flag_erase(&Element->Flags);
 }
 
-void EraseElementPinsAndPads(pcb_element_t *Element)
+void pcb_elem_pp_invalidate_erase(pcb_element_t *Element)
 {
 	PCB_PIN_LOOP(Element);
 	{
-		ErasePin(pin);
+		pcb_pin_invalidate_erase(pin);
 	}
 	PCB_END_LOOP;
 	PCB_PAD_LOOP(Element);
 	{
-		ErasePad(pad);
+		pcb_pad_invalidate_erase(pad);
 	}
 	PCB_END_LOOP;
 }
 
-void EraseElementName(pcb_element_t *Element)
+void pcb_elem_name_invalidate_erase(pcb_element_t *Element)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_HIDENAME, Element)) {
 		return;
 	}
-	DrawText(NULL, &PCB_ELEM_TEXT_VISIBLE(PCB, Element));
+	pcb_text_invalidate_draw(NULL, &PCB_ELEM_TEXT_VISIBLE(PCB, Element));
 }
 
-void DrawElement(pcb_element_t *Element)
+void pcb_elem_invalidate_draw(pcb_element_t *Element)
 {
-	DrawElementPackage(Element);
-	DrawElementName(Element);
-	DrawElementPinsAndPads(Element);
+	pcb_elem_package_invalidate_draw(Element);
+	pcb_elem_name_invalidate_draw(Element);
+	pcb_elem_pp_invalidate_draw(Element);
 }
 
-void DrawElementName(pcb_element_t *Element)
+void pcb_elem_name_invalidate_draw(pcb_element_t *Element)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_HIDENAME, Element))
 		return;
-	DrawText(NULL, &PCB_ELEM_TEXT_VISIBLE(PCB, Element));
+	pcb_text_invalidate_draw(NULL, &PCB_ELEM_TEXT_VISIBLE(PCB, Element));
 }
 
-void DrawElementPackage(pcb_element_t *Element)
+void pcb_elem_package_invalidate_draw(pcb_element_t *Element)
 {
 	PCB_ELEMENT_PCB_LINE_LOOP(Element);
 	{
-		DrawLine(NULL, line);
+		pcb_line_invalidate_draw(NULL, line);
 	}
 	PCB_END_LOOP;
 	PCB_ARC_LOOP(Element);
 	{
-		DrawArc(NULL, arc);
+		pcb_arc_invalidate_draw(NULL, arc);
 	}
 	PCB_END_LOOP;
 }
 
-void DrawElementPinsAndPads(pcb_element_t *Element)
+void pcb_elem_pp_invalidate_draw(pcb_element_t *Element)
 {
 	PCB_PAD_LOOP(Element);
 	{
 		if (pcb_draw_doing_pinout || pcb_draw_doing_assy || PCB_FRONT(pad) || PCB->InvisibleObjectsOn)
-			DrawPad(pad);
+			pcb_pad_invalidate_draw(pad);
 	}
 	PCB_END_LOOP;
 	PCB_PIN_LOOP(Element);
 	{
-		DrawPin(pin);
+		pcb_pin_invalidate_draw(pin);
 	}
 	PCB_END_LOOP;
 }
