@@ -46,6 +46,7 @@
 
 /* TODO: maybe remove this and move lines from draw here? */
 #include "draw.h"
+#include "draw_wireframe.h"
 #include "obj_line_draw.h"
 #include "obj_rat_draw.h"
 #include "obj_pinvia_draw.h"
@@ -60,6 +61,7 @@ pcb_line_t *pcb_line_alloc(pcb_layer_t * layer)
 
 	new_obj = calloc(sizeof(pcb_line_t), 1);
 	new_obj->type = PCB_OBJ_LINE;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, layer, layer);
 
 	linelist_append(&layer->Line, new_obj);
@@ -77,7 +79,7 @@ void pcb_line_free(pcb_line_t * data)
 /**** utility ****/
 struct line_info {
 	pcb_coord_t X1, X2, Y1, Y2;
-	pcb_coord_t Thickness;
+	pcb_coord_t Thickness, Clearance;
 	pcb_flag_t Flags;
 	pcb_line_t test, *ans;
 	jmp_buf env;
@@ -103,8 +105,9 @@ static pcb_r_dir_t line_callback(const pcb_box_t * b, void *cl)
 	}
 	/* remove unnecessary line points */
 	if (line->Thickness == i->Thickness
-			/* don't merge lines if the clear flags differ  */
-			&& PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, line) == PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, i)) {
+			/* don't merge lines if the clear flags or clearance differ  */
+			&& PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, line) == PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, i)
+			&& line->Clearance == i->Clearance) {
 		if (line->Point1.X == i->X1 && line->Point1.Y == i->Y1) {
 			i->test.Point1.X = line->Point2.X;
 			i->test.Point1.Y = line->Point2.Y;
@@ -169,6 +172,7 @@ pcb_line_t *pcb_line_new_merge(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y
 	info.Y1 = Y1;
 	info.Y2 = Y2;
 	info.Thickness = Thickness;
+	info.Clearance = Clearance;
 	info.Flags = Flags;
 	info.test.Thickness = 0;
 	info.test.Flags = pcb_no_flags();
@@ -222,7 +226,7 @@ static pcb_line_t *pcb_line_copy_meta(pcb_line_t *dst, pcb_line_t *src)
 {
 	if (dst == NULL)
 		return NULL;
-	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes, 0);
+	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes);
 	if (src->Number != NULL)
 		dst->Number = pcb_strdup(src->Number);
 	return dst;
@@ -299,6 +303,40 @@ double pcb_line_area(const pcb_line_t *line)
 		+ (double)line->Thickness * (double)line->Thickness * M_PI; /* cap circles */
 }
 
+void pcb_sqline_to_rect(const pcb_line_t *line, pcb_coord_t *x, pcb_coord_t *y)
+{
+	double l, vx, vy, nx, ny, width, x1, y1, x2, y2, dx, dy;
+
+	x1 = line->Point1.X;
+	y1 = line->Point1.Y;
+	x2 = line->Point2.X;
+	y2 = line->Point2.Y;
+
+	width = (double)((line->Thickness + 1) / 2);
+	dx = x2-x1;
+	dy = y2-y1;
+
+	if ((dx == 0) && (dy == 0))
+		dx = 1;
+
+	l = sqrt((double)dx*(double)dx + (double)dy*(double)dy);
+
+	vx = dx / l;
+	vy = dy / l;
+	nx = -vy;
+	ny = vx;
+
+	x[0] = (pcb_coord_t)pcb_round(x1 - vx * width + nx * width);
+	y[0] = (pcb_coord_t)pcb_round(y1 - vy * width + ny * width);
+	x[1] = (pcb_coord_t)pcb_round(x1 - vx * width - nx * width);
+	y[1] = (pcb_coord_t)pcb_round(y1 - vy * width - ny * width);
+	x[2] = (pcb_coord_t)pcb_round(x2 + vx * width - nx * width);
+	y[2] = (pcb_coord_t)pcb_round(y2 + vy * width - ny * width);
+	x[3] = (pcb_coord_t)pcb_round(x2 + vx * width + nx * width);
+	y[3] = (pcb_coord_t)pcb_round(y2 + vy * width + ny * width);
+}
+
+
 
 /*** ops ***/
 /* copies a line to buffer */
@@ -338,20 +376,20 @@ void *pcb_lineop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_line_
 /* changes the size of a line */
 void *pcb_lineop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Line->Thickness + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Line->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line))
 		return (NULL);
 	if (value <= PCB_MAX_LINESIZE && value >= PCB_MIN_LINESIZE && value != Line->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_LINE, Layer, Line, Line);
-		EraseLine(Line);
+		pcb_line_invalidate_erase(Line);
 		pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 		Line->Thickness = value;
 		pcb_line_bbox(Line);
 		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
-		DrawLine(Layer, Line);
+		pcb_line_invalidate_draw(Layer, Line);
 		return (Line);
 	}
 	return (NULL);
@@ -360,27 +398,27 @@ void *pcb_lineop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 /*changes the clearance size of a line */
 void *pcb_lineop_change_clear_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Line->Clearance + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Line->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line) || !PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line))
 		return (NULL);
 	if (value < 0)
 		value = 0;
-	value = MIN(PCB_MAX_LINESIZE, MAX(value, PCB->Bloat * 2 + 2));
+	value = MIN(PCB_MAX_LINESIZE, value);
+	if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < PCB->Bloat * 2)
+		value = 0;
+	if (ctx->chgsize.value > 0 && value < PCB->Bloat * 2)
+		value = PCB->Bloat * 2 + 2;
 	if (value != Line->Clearance) {
 		pcb_undo_add_obj_to_clear_size(PCB_TYPE_LINE, Layer, Line, Line);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
-		EraseLine(Line);
+		pcb_line_invalidate_erase(Line);
 		pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
 		Line->Clearance = value;
-		if (Line->Clearance == 0) {
-			PCB_FLAG_CLEAR(PCB_FLAG_CLEARLINE, Line);
-			Line->Clearance = PCB_MIL_TO_COORD(10);
-		}
 		pcb_line_bbox(Line);
 		pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
-		DrawLine(Layer, Line);
+		pcb_line_invalidate_draw(Layer, Line);
 		return (Line);
 	}
 	return (NULL);
@@ -401,18 +439,18 @@ void *pcb_lineop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *L
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Line))
 		return (NULL);
-	EraseLine(Line);
+	pcb_line_invalidate_erase(Line);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_LINE, Layer, Line, Line, pcb_false);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	}
-	pcb_undo_add_obj_to_flag(PCB_TYPE_LINE, Layer, Line, Line);
+	pcb_undo_add_obj_to_flag(Line);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARLINE, Line);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Line)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_LINE, Layer, Line, Line, pcb_true);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	}
-	DrawLine(Layer, Line);
+	pcb_line_invalidate_draw(Layer, Line);
 	return (Line);
 }
 
@@ -444,7 +482,7 @@ void *pcb_lineop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 	if (!line)
 		return (line);
 	pcb_line_copy_meta(line, Line);
-	DrawLine(Layer, line);
+	pcb_line_invalidate_draw(Layer, line);
 	pcb_undo_add_obj_to_create(PCB_TYPE_LINE, Layer, line, line);
 	return (line);
 }
@@ -453,14 +491,14 @@ void *pcb_lineop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 void *pcb_lineop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	if (Layer->meta.real.vis)
-		EraseLine(Line);
+		pcb_line_invalidate_erase(Line);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
 	pcb_line_move(Line, ctx->move.dx, ctx->move.dy);
 	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	if (Layer->meta.real.vis) {
-		DrawLine(Layer, Line);
+		pcb_line_invalidate_draw(Layer, Line);
 		pcb_draw();
 	}
 	return (Line);
@@ -471,7 +509,7 @@ void *pcb_lineop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Li
 {
 	if (Layer) {
 		if (Layer->meta.real.vis)
-			EraseLine(Line);
+			pcb_line_invalidate_erase(Line);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 		pcb_r_delete_entry(Layer->line_tree, &Line->BoundingBox);
 		PCB_MOVE(Point->X, Point->Y, ctx->move.dx, ctx->move.dy);
@@ -479,7 +517,7 @@ void *pcb_lineop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Li
 		pcb_r_insert_entry(Layer->line_tree, &Line->BoundingBox, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 		if (Layer->meta.real.vis) {
-			DrawLine(Layer, Line);
+			pcb_line_invalidate_draw(Layer, Line);
 			pcb_draw();
 		}
 		return (Line);
@@ -487,13 +525,13 @@ void *pcb_lineop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Li
 	else {												/* must be a rat */
 
 		if (PCB->RatOn)
-			EraseRat((pcb_rat_t *) Line);
+			pcb_rat_invalidate_erase((pcb_rat_t *) Line);
 		pcb_r_delete_entry(PCB->Data->rat_tree, &Line->BoundingBox);
 		PCB_MOVE(Point->X, Point->Y, ctx->move.dx, ctx->move.dy);
 		pcb_line_bbox(Line);
 		pcb_r_insert_entry(PCB->Data->rat_tree, &Line->BoundingBox, 0);
 		if (PCB->RatOn) {
-			DrawRat((pcb_rat_t *) Line);
+			pcb_rat_invalidate_draw((pcb_rat_t *) Line);
 			pcb_draw();
 		}
 		return (Line);
@@ -570,7 +608,7 @@ static pcb_r_dir_t moveline_callback(const pcb_box_t * b, void *cl)
 			 pcb_via_new(PCB->Data, i->X, i->Y,
 										conf_core.design.via_thickness, 2 * conf_core.design.clearance, PCB_FLAG_NO, conf_core.design.via_drilling_hole, NULL, pcb_no_flags())) != NULL) {
 		pcb_undo_add_obj_to_create(PCB_TYPE_VIA, via, via, via);
-		DrawVia(via);
+		pcb_via_invalidate_draw(via);
 	}
 	longjmp(i->env, 1);
 }
@@ -587,21 +625,21 @@ void *pcb_lineop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_line_t
 		return NULL;
 	}
 	if (ctx->move.dst_layer == Layer && Layer->meta.real.vis) {
-		DrawLine(Layer, Line);
+		pcb_line_invalidate_draw(Layer, Line);
 		pcb_draw();
 	}
-	if (((long int) ctx->move.dst_layer == -1) || ctx->move.dst_layer == Layer)
+	if (ctx->move.dst_layer == Layer)
 		return (Line);
 
 	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_LINE, Layer, Line, Line);
 	if (Layer->meta.real.vis)
-		EraseLine(Line);
+		pcb_line_invalidate_erase(Line);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	newone = (pcb_line_t *) pcb_lineop_move_to_layer_low(ctx, Layer, Line, ctx->move.dst_layer);
 	Line = NULL;
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, ctx->move.dst_layer, newone);
 	if (ctx->move.dst_layer->meta.real.vis)
-		DrawLine(ctx->move.dst_layer, newone);
+		pcb_line_invalidate_draw(ctx->move.dst_layer, newone);
 	pcb_draw();
 	if (!PCB->ViaOn || ctx->move.more_to_come ||
 			pcb_layer_get_group_(Layer) ==
@@ -696,7 +734,7 @@ void *pcb_lineop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	/* erase from screen */
 	if (Layer->meta.real.vis) {
-		EraseLine(Line);
+		pcb_line_invalidate_erase(Line);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -777,7 +815,7 @@ void pcb_line_flip_side(pcb_layer_t *layer, pcb_line_t *line)
 
 static void rotate_line1(pcb_layer_t *Layer, pcb_line_t *Line)
 {
-	EraseLine(Line);
+	pcb_line_invalidate_erase(Line);
 	if (Layer) {
 		if (PCB_LAYER_IS_REAL(Layer))
 			pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
@@ -796,11 +834,11 @@ static void rotate_line2(pcb_layer_t *Layer, pcb_line_t *Line)
 			pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
 		if (PCB_LAYER_IS_REAL(Layer))
 			pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
-		DrawLine(Layer, Line);
+		pcb_line_invalidate_draw(Layer, Line);
 	}
 	else {
 		pcb_r_insert_entry(PCB->Data->rat_tree, (pcb_box_t *) Line, 0);
-		DrawRat((pcb_rat_t *) Line);
+		pcb_rat_invalidate_draw((pcb_rat_t *) Line);
 	}
 	pcb_draw();
 }
@@ -808,16 +846,9 @@ static void rotate_line2(pcb_layer_t *Layer, pcb_line_t *Line)
 /* rotates a line's point */
 void *pcb_lineop_rotate90_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line, pcb_point_t *Point)
 {
-	pcb_point_t *center;
-
 	rotate_line1(Layer, Line);
 
-	if ((Point->X == Line->Point1.X) && (Point->Y == Line->Point1.Y))
-		center = &Line->Point2;
-	else
-		center = &Line->Point1;
-
-	pcb_point_rotate90(Point, center->X, center->Y, ctx->rotate.number);
+	pcb_point_rotate90(Point, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
 
 	rotate_line2(Layer, Line);
 	return Line;
@@ -845,7 +876,7 @@ void *pcb_lineop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 	X = Line->Point2.X;
 	Y = Line->Point2.Y;
 	pcb_undo_add_obj_to_move(PCB_TYPE_LINE_POINT, Layer, Line, &Line->Point2, ctx->insert.x - X, ctx->insert.y - Y);
-	EraseLine(Line);
+	pcb_line_invalidate_erase(Line);
 	pcb_r_delete_entry(Layer->line_tree, (pcb_box_t *) Line);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
 	Line->Point2.X = ctx->insert.x;
@@ -853,14 +884,14 @@ void *pcb_lineop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 	pcb_line_bbox(Line);
 	pcb_r_insert_entry(Layer->line_tree, (pcb_box_t *) Line, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, Line);
-	DrawLine(Layer, Line);
+	pcb_line_invalidate_draw(Layer, Line);
 	/* we must create after playing with Line since creation may
 	 * invalidate the line pointer
 	 */
 	if ((line = pcb_line_new_merge(Layer, ctx->insert.x, ctx->insert.y, X, Y, Line->Thickness, Line->Clearance, Line->Flags))) {
 		pcb_line_copy_meta(line, Line);
 		pcb_undo_add_obj_to_create(PCB_TYPE_LINE, Layer, line, line);
-		DrawLine(Layer, line);
+		pcb_line_invalidate_draw(Layer, line);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_LINE, Layer, line);
 		/* creation call adds it to the rtree */
 	}
@@ -868,31 +899,105 @@ void *pcb_lineop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *
 	return (line);
 }
 
-#define PCB_LINE_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_RAT | PCB_FLAG_CLEARLINE | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_RUBBEREND | PCB_FLAG_LOCK | PCB_FLAG_VISIT)
 void *pcb_lineop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_line_t *Line)
 {
-	if ((ctx->chgflag.flag & PCB_LINE_FLAGS) != ctx->chgflag.flag)
+	static pcb_flag_values_t pcb_line_flags = 0;
+	if (pcb_line_flags == 0)
+		pcb_line_flags = pcb_obj_valid_flags(PCB_TYPE_LINE);
+
+	if ((ctx->chgflag.flag & pcb_line_flags) != ctx->chgflag.flag)
 		return NULL;
-	pcb_undo_add_obj_to_flag(PCB_TYPE_LINE, Line, Line, Line);
+	if ((ctx->chgflag.flag & PCB_FLAG_TERMNAME) && (Line->term == NULL))
+		return NULL;
+	pcb_undo_add_obj_to_flag(Line);
+
+	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
+		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_TYPE_LINE, Line->parent.layer, Line);
+
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, Line);
+
+	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
+		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_TYPE_LINE, Line->parent.layer, Line);
+
 	return Line;
 }
 
+void *pcb_lineop_invalidate_label(pcb_opctx_t *ctx, pcb_layer_t *layer, pcb_line_t *line)
+{
+	pcb_line_name_invalidate_draw(line);
+	return line;
+}
+
 
 /*** draw ***/
-void _draw_line(pcb_line_t * line)
+
+static pcb_bool is_line_term_vert(const pcb_line_t *line)
+{
+	pcb_coord_t dx, dy;
+
+	dx = line->Point1.X - line->Point2.X;
+	if (dx < 0)
+		dx = -dx;
+
+	dy = line->Point1.Y - line->Point2.Y;
+	if (dy < 0)
+		dy = -dy;
+
+	return dx < dy;
+}
+
+void pcb_line_name_invalidate_draw(pcb_line_t *line)
+{
+	if (line->term != NULL) {
+		pcb_text_t text;
+		pcb_term_label_setup(&text, (line->Point1.X + line->Point2.X)/2, (line->Point1.Y + line->Point2.Y)/2,
+			100.0, is_line_term_vert(line), pcb_true, line->term, line->intconn);
+		pcb_draw_invalidate(&text);
+	}
+}
+
+void pcb_line_draw_label(pcb_line_t *line)
+{
+	if (line->term != NULL)
+		pcb_term_label_draw((line->Point1.X + line->Point2.X)/2, (line->Point1.Y + line->Point2.Y)/2,
+			100.0, is_line_term_vert(line), pcb_true, line->term, line->intconn);
+}
+
+
+void pcb_line_draw_(pcb_line_t *line, int allow_term_gfx)
 {
 	PCB_DRAW_BBOX(line);
 	pcb_gui->set_line_cap(Output.fgGC, Trace_Cap);
-	if (conf_core.editor.thin_draw)
-		pcb_gui->set_line_width(Output.fgGC, 0);
+	if (!conf_core.editor.thin_draw && !conf_core.editor.wireframe_draw) {
+		if ((allow_term_gfx) && pcb_draw_term_need_gfx(line)) {
+			pcb_gui->set_line_width(Output.active_padGC, line->Thickness);
+			pcb_gui->draw_line(Output.active_padGC, line->Point1.X, line->Point1.Y, line->Point2.X, line->Point2.Y);
+			pcb_gui->set_line_width(Output.fgGC, pcb_draw_term_gfx_thick(line->Thickness));
+		}
+		else
+			pcb_gui->set_line_width(Output.fgGC, line->Thickness);
+		pcb_gui->draw_line(Output.fgGC, line->Point1.X, line->Point1.Y, line->Point2.X, line->Point2.Y);
+	}
 	else
-		pcb_gui->set_line_width(Output.fgGC, line->Thickness);
+	{
+		if(conf_core.editor.thin_draw) {
+			pcb_gui->set_line_width(Output.fgGC, 0);
+			pcb_gui->draw_line(Output.fgGC, line->Point1.X, line->Point1.Y, line->Point2.X, line->Point2.Y);
+		}
 
-	pcb_gui->draw_line(Output.fgGC, line->Point1.X, line->Point1.Y, line->Point2.X, line->Point2.Y);
+		if(conf_core.editor.wireframe_draw) { 
+			pcb_gui->set_line_width(Output.fgGC, 0);
+			pcb_draw_wireframe_line(Output.fgGC, line->Point1.X, line->Point1.Y, line->Point2.X, line->Point2.Y, line->Thickness);
+		}
+	}
+
+	if (line->term != NULL) {
+		if ((pcb_draw_doing_pinout) || PCB_FLAG_TEST(PCB_FLAG_TERMNAME, line))
+			pcb_draw_delay_label_add((pcb_any_obj_t *)line);
+	}
 }
 
-void draw_line(pcb_layer_t * layer, pcb_line_t * line)
+static void pcb_line_draw(pcb_layer_t *layer, pcb_line_t *line, int allow_term_gfx)
 {
 	const char *color;
 	char buf[sizeof("#XXXXXX")];
@@ -915,28 +1020,39 @@ void draw_line(pcb_layer_t * layer, pcb_line_t * line)
 	}
 
 	pcb_gui->set_color(Output.fgGC, color);
-	_draw_line(line);
+	pcb_line_draw_(line, allow_term_gfx);
 }
 
-pcb_r_dir_t draw_line_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_line_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_line_t *line = (pcb_line_t *)b;
 
-	if (!PCB->SubcPartsOn && pcb_is_lobj_in_subc(line->parent_type, &line->parent))
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(line->parent_type, &line->parent))
 		return PCB_R_DIR_NOT_FOUND;
 
-	draw_line((pcb_layer_t *)cl, line);
+	pcb_line_draw((pcb_layer_t *)cl, line, 0);
+	return PCB_R_DIR_FOUND_CONTINUE;
+}
+
+pcb_r_dir_t pcb_line_draw_term_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_line_t *line = (pcb_line_t *)b;
+
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(line->parent_type, &line->parent))
+		return PCB_R_DIR_NOT_FOUND;
+
+	pcb_line_draw((pcb_layer_t *)cl, line, 1);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
 /* erases a line on a layer */
-void EraseLine(pcb_line_t *Line)
+void pcb_line_invalidate_erase(pcb_line_t *Line)
 {
 	pcb_draw_invalidate(Line);
 	pcb_flag_erase(&Line->Flags);
 }
 
-void DrawLine(pcb_layer_t *Layer, pcb_line_t *Line)
+void pcb_line_invalidate_draw(pcb_layer_t *Layer, pcb_line_t *Line)
 {
 	pcb_draw_invalidate(Line);
 }
