@@ -53,6 +53,7 @@ pcb_pad_t *pcb_pad_alloc(pcb_element_t * element)
 
 	new_obj = calloc(sizeof(pcb_pad_t), 1);
 	new_obj->type = PCB_OBJ_PAD;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, element, element);
 
 	padlist_append(&element->Pad, new_obj);
@@ -208,10 +209,10 @@ pcb_bool pcb_pad_change_paste(pcb_pad_t *Pad)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
 		return (pcb_false);
-	ErasePad(Pad);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, Pad, Pad, Pad);
+	pcb_pad_invalidate_erase(Pad);
+	pcb_undo_add_obj_to_flag(Pad);
 	PCB_FLAG_TOGGLE(PCB_FLAG_NOPASTE, Pad);
-	DrawPad(Pad);
+	pcb_pad_invalidate_draw(Pad);
 	pcb_draw();
 	return (pcb_true);
 }
@@ -261,7 +262,7 @@ unsigned int pcb_pad_hash_padstack(const pcb_pad_t *p)
 /* changes the size of a pad */
 void *pcb_padop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *Pad)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pad->Thickness + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pad->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
 		return (NULL);
@@ -269,14 +270,14 @@ void *pcb_padop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 		pcb_undo_add_obj_to_size(PCB_TYPE_PAD, Element, Pad, Pad);
 		pcb_undo_add_obj_to_mask_size(PCB_TYPE_PAD, Element, Pad, Pad);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
-		ErasePad(Pad);
+		pcb_pad_invalidate_erase(Pad);
 		pcb_r_delete_entry(PCB->Data->pad_tree, &Pad->BoundingBox);
 		Pad->Mask += value - Pad->Thickness;
 		Pad->Thickness = value;
 		/* SetElementBB updates all associated rtrees */
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
-		DrawPad(Pad);
+		pcb_pad_invalidate_draw(Pad);
 		return (Pad);
 	}
 	return (NULL);
@@ -285,7 +286,7 @@ void *pcb_padop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 /* changes the clearance size of a pad */
 void *pcb_padop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *Pad)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pad->Clearance + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pad->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
 		return (NULL);
@@ -293,21 +294,21 @@ void *pcb_padop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_
 		value = 0;
 	else
 		value = MIN(PCB_MAX_LINESIZE, value);
-	if (ctx->chgsize.delta < 0 && value < PCB->Bloat * 2)
+	if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < PCB->Bloat * 2)
 		value = 0;
-	if ((ctx->chgsize.delta > 0 || ctx->chgsize.absolute) && value < PCB->Bloat * 2)
+	if (ctx->chgsize.value > 0 && value < PCB->Bloat * 2)
 		value = PCB->Bloat * 2 + 2;
 	if (value == Pad->Clearance)
 		return NULL;
 	pcb_undo_add_obj_to_clear_size(PCB_TYPE_PAD, Element, Pad, Pad);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
-	ErasePad(Pad);
+	pcb_pad_invalidate_erase(Pad);
 	pcb_r_delete_entry(PCB->Data->pad_tree, &Pad->BoundingBox);
 	Pad->Clearance = value;
 	/* SetElementBB updates all associated rtrees */
 	pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
-	DrawPad(Pad);
+	pcb_pad_invalidate_draw(Pad);
 	return Pad;
 }
 
@@ -317,10 +318,10 @@ void *pcb_padop_change_name(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 	char *old = Pad->Name;
 
 	Element = Element;						/* get rid of 'unused...' warnings */
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pad)) {
-		ErasePadName(Pad);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pad)) {
+		pcb_pad_name_invalidate_erase(Pad);
 		Pad->Name = ctx->chgname.new_name;
-		DrawPadName(Pad);
+		pcb_pad_name_invalidate_draw(Pad);
 	}
 	else
 		Pad->Name = ctx->chgname.new_name;
@@ -333,10 +334,10 @@ void *pcb_padop_change_num(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *
 	char *old = Pad->Number;
 
 	Element = Element;						/* get rid of 'unused...' warnings */
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pad)) {
-		ErasePadName(Pad);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pad)) {
+		pcb_pad_name_invalidate_erase(Pad);
 		Pad->Number = ctx->chgname.new_name;
-		DrawPadName(Pad);
+		pcb_pad_name_invalidate_draw(Pad);
 	}
 	else
 		Pad->Number = ctx->chgname.new_name;
@@ -348,14 +349,14 @@ void *pcb_padop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad))
 		return (NULL);
-	ErasePad(Pad);
+	pcb_pad_invalidate_erase(Pad);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PAD, Element, Pad, Pad, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, Element, Pad, Pad);
+	pcb_undo_add_obj_to_flag(Pad);
 	PCB_FLAG_TOGGLE(PCB_FLAG_SQUARE, Pad);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PAD, Element, Pad, Pad, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PAD, Element, Pad);
-	DrawPad(Pad);
+	pcb_pad_invalidate_draw(Pad);
 	return (Pad);
 }
 
@@ -383,27 +384,30 @@ void *pcb_padop_clear_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t
 /* changes the mask size of a pad */
 void *pcb_padop_change_mask_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pad_t *Pad)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pad->Mask + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pad->Mask + ctx->chgsize.value;
 
 	value = MAX(value, 0);
-	if (value == Pad->Mask && ctx->chgsize.absolute == 0)
+	if (value == Pad->Mask && ctx->chgsize.value == 0)
 		value = Pad->Thickness;
 	if (value != Pad->Mask) {
 		pcb_undo_add_obj_to_mask_size(PCB_TYPE_PAD, Element, Pad, Pad);
-		ErasePad(Pad);
+		pcb_pad_invalidate_erase(Pad);
 		pcb_r_delete_entry(PCB->Data->pad_tree, &Pad->BoundingBox);
 		Pad->Mask = value;
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
-		DrawPad(Pad);
+		pcb_pad_invalidate_draw(Pad);
 		return (Pad);
 	}
 	return (NULL);
 }
 
-#define PCB_PAD_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_NOPASTE | PCB_FLAG_PININPOLY | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_LOCK | PCB_FLAG_VISIT)
-void *pcb_padop_change_flag(pcb_opctx_t *ctx, pcb_data_t *data, pcb_pad_t *pad)
+void *pcb_padop_change_flag(pcb_opctx_t *ctx, pcb_element_t *elem, pcb_pad_t *pad)
 {
-	if ((ctx->chgflag.flag & PCB_PAD_FLAGS) != ctx->chgflag.flag)
+	static pcb_flag_values_t pcb_pad_flags = 0;
+	if (pcb_pad_flags == 0)
+		pcb_pad_flags = pcb_obj_valid_flags(PCB_TYPE_PAD);
+
+	if ((ctx->chgflag.flag & pcb_pad_flags) != ctx->chgflag.flag)
 		return NULL;
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, pad);
 	return pad;
@@ -428,8 +432,8 @@ static void draw_pad_name(pcb_pad_t * pad)
 			pn = "n/a";
 	}
 
-	if (PCB_FLAG_INTCONN_GET(pad) > 0)
-		pcb_snprintf(buff, sizeof(buff), "%s[%d]", pn, PCB_FLAG_INTCONN_GET(pad));
+	if (pad->intconn > 0)
+		pcb_snprintf(buff, sizeof(buff), "%s[%d]", pn, pad->intconn);
 	else
 		strcpy(buff, pn);
 	text.TextString = buff;
@@ -462,7 +466,7 @@ static void draw_pad_name(pcb_pad_t * pad)
 	text.fid = 0;
 	text.Direction = (vert ? 1 : 0) + (flip_x ? 2 : 0);
 
-	DrawTextLowLevel(&text, 0);
+	pcb_text_draw_(&text, 0, 0);
 }
 
 static void _draw_pad(pcb_hid_gc_t gc, pcb_pad_t * pad, pcb_bool clear, pcb_bool mask)
@@ -472,13 +476,13 @@ static void _draw_pad(pcb_hid_gc_t gc, pcb_pad_t * pad, pcb_bool clear, pcb_bool
 
 	PCB_DRAW_BBOX(pad);
 
-	if (conf_core.editor.thin_draw || (clear && conf_core.editor.thin_draw_poly))
+	if (conf_core.editor.thin_draw || (clear && conf_core.editor.thin_draw_poly) || conf_core.editor.wireframe_draw)
 		pcb_gui->thindraw_pcb_pad(gc, pad, clear, mask);
 	else
 		pcb_gui->fill_pcb_pad(gc, pad, clear, mask);
 }
 
-void draw_pad(pcb_pad_t * pad)
+void pcb_pad_draw(pcb_pad_t * pad)
 {
 	const char *color = NULL;
 	char buf[sizeof("#XXXXXX")];
@@ -513,29 +517,29 @@ void draw_pad(pcb_pad_t * pad)
 		draw_pad_name(pad);
 }
 
-pcb_r_dir_t draw_pad_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_pad_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pad_t *pad = (pcb_pad_t *) b;
 	int *side = cl;
 
 	if (PCB_ON_SIDE(pad, *side))
-		draw_pad(pad);
+		pcb_pad_draw(pad);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-pcb_r_dir_t draw_pad_name_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_pad_name_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pad_t *pad = (pcb_pad_t *) b;
 	int *side = cl;
 
 	if (PCB_ON_SIDE(pad, *side)) {
-		if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, pad))
+		if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pad))
 			draw_pad_name(pad);
 	}
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-pcb_r_dir_t clear_pad_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_pad_clear_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pad_t *pad = (pcb_pad_t *) b;
 	int *side = cl;
@@ -545,7 +549,7 @@ pcb_r_dir_t clear_pad_callback(const pcb_box_t * b, void *cl)
 }
 
 /* draws solder paste layer for a given side of the board - only pads get paste */
-void pcb_draw_paste_auto(int side, const pcb_box_t * drawn_area)
+void pcb_pad_paste_draw(int side, const pcb_box_t * drawn_area)
 {
 	pcb_gui->set_color(Output.fgGC, conf_core.appearance.color.paste);
 	PCB_PAD_ALL_LOOP(PCB->Data);
@@ -593,27 +597,27 @@ static void GatherPadName(pcb_pad_t *Pad)
 	return;
 }
 
-void ErasePad(pcb_pad_t *Pad)
+void pcb_pad_invalidate_erase(pcb_pad_t *Pad)
 {
 	pcb_draw_invalidate(Pad);
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pad))
-		ErasePadName(Pad);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pad))
+		pcb_pad_name_invalidate_erase(Pad);
 	pcb_flag_erase(&Pad->Flags);
 }
 
-void ErasePadName(pcb_pad_t *Pad)
+void pcb_pad_name_invalidate_erase(pcb_pad_t *Pad)
 {
 	GatherPadName(Pad);
 }
 
-void DrawPad(pcb_pad_t *Pad)
+void pcb_pad_invalidate_draw(pcb_pad_t *Pad)
 {
 	pcb_draw_invalidate(Pad);
-	if (pcb_draw_doing_pinout || PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pad))
-		DrawPadName(Pad);
+	if (pcb_draw_doing_pinout || PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pad))
+		pcb_pad_name_invalidate_draw(Pad);
 }
 
-void DrawPadName(pcb_pad_t *Pad)
+void pcb_pad_name_invalidate_draw(pcb_pad_t *Pad)
 {
 	GatherPadName(Pad);
 }
