@@ -55,6 +55,7 @@ pcb_pin_t *pcb_via_alloc(pcb_data_t * data)
 
 	new_obj = calloc(sizeof(pcb_pin_t), 1);
 	new_obj->type = PCB_OBJ_VIA;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, data, data);
 
 	pinlist_append(&data->Via, new_obj);
@@ -75,6 +76,7 @@ pcb_pin_t *pcb_pin_alloc(pcb_element_t * element)
 
 	new_obj = calloc(sizeof(pcb_pin_t), 1);
 	new_obj->type = PCB_OBJ_PIN;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, element, element);
 
 	pinlist_append(&element->Pin, new_obj);
@@ -149,7 +151,7 @@ static pcb_pin_t *pcb_via_copy_meta(pcb_pin_t *dst, pcb_pin_t *src)
 {
 	if (dst == NULL)
 		return NULL;
-	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes, 0);
+	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes);
 	if (src->Number != NULL)
 		dst->Number = pcb_strdup(src->Number);
 	if (src->Name != NULL)
@@ -381,14 +383,14 @@ void *pcb_viaop_change_thermal(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, Via, Via, Via, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, CURRENT, Via);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, Via, Via, Via);
+	pcb_undo_add_obj_to_flag(Via);
 	if (!ctx->chgtherm.style)										/* remove the thermals */
 		PCB_FLAG_THERM_CLEAR(INDEXOFCURRENT, Via);
 	else
 		PCB_FLAG_THERM_ASSIGN(INDEXOFCURRENT, ctx->chgtherm.style, Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, Via, Via, Via, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, CURRENT, Via);
-	DrawVia(Via);
+	pcb_via_invalidate_draw(Via);
 	return Via;
 }
 
@@ -397,28 +399,28 @@ void *pcb_pinop_change_thermal(pcb_opctx_t *ctx, pcb_element_t *element, pcb_pin
 {
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, element, Pin, Pin, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, CURRENT, Pin);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, Pin, Pin);
+	pcb_undo_add_obj_to_flag(Pin);
 	if (!ctx->chgtherm.style)										/* remove the thermals */
 		PCB_FLAG_THERM_CLEAR(INDEXOFCURRENT, Pin);
 	else
 		PCB_FLAG_THERM_ASSIGN(INDEXOFCURRENT, ctx->chgtherm.style, Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, element, Pin, Pin, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, CURRENT, Pin);
-	DrawPin(Pin);
+	pcb_pin_invalidate_draw(Pin);
 	return Pin;
 }
 
 /* changes the size of a via */
 void *pcb_viaop_change_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
-	pcb_coord_t value = ctx->chgsize.absolute ? ctx->chgsize.absolute : Via->Thickness + ctx->chgsize.delta;
+	pcb_coord_t value = ctx->chgsize.is_absolute ? ctx->chgsize.value : Via->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
 		return (NULL);
 	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, Via) && value <= PCB_MAX_PINORVIASIZE &&
 			value >= PCB_MIN_PINORVIASIZE && value >= Via->DrillingHole + PCB_MIN_PINORVIACOPPER && value != Via->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_VIA, Via, Via, Via);
-		EraseVia(Via);
+		pcb_via_invalidate_erase(Via);
 		pcb_r_delete_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Via, Via);
 		if (Via->Mask) {
@@ -429,7 +431,7 @@ void *pcb_viaop_change_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 		pcb_pin_bbox(Via);
 		pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
-		DrawVia(Via);
+		pcb_via_invalidate_draw(Via);
 		return (Via);
 	}
 	return (NULL);
@@ -438,7 +440,7 @@ void *pcb_viaop_change_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 /* changes the drilling hole of a via */
 void *pcb_viaop_change_2nd_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Via->DrillingHole + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Via->DrillingHole + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
 		return (NULL);
@@ -446,7 +448,7 @@ void *pcb_viaop_change_2nd_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 			value >= PCB_MIN_PINORVIAHOLE && (PCB_FLAG_TEST(PCB_FLAG_HOLE, Via) || value <= Via->Thickness - PCB_MIN_PINORVIACOPPER)
 			&& value != Via->DrillingHole) {
 		pcb_undo_add_obj_to_2nd_size(PCB_TYPE_VIA, Via, Via, Via);
-		EraseVia(Via);
+		pcb_via_invalidate_erase(Via);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 		Via->DrillingHole = value;
 		if (PCB_FLAG_TEST(PCB_FLAG_HOLE, Via)) {
@@ -454,7 +456,7 @@ void *pcb_viaop_change_2nd_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 			Via->Thickness = value;
 		}
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
-		DrawVia(Via);
+		pcb_via_invalidate_draw(Via);
 		return (Via);
 	}
 	return (NULL);
@@ -463,7 +465,7 @@ void *pcb_viaop_change_2nd_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 /* changes the drilling hole of a pin */
 void *pcb_pinop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pin->DrillingHole + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->DrillingHole + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
 		return (NULL);
@@ -471,7 +473,7 @@ void *pcb_pinop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pi
 			value >= PCB_MIN_PINORVIAHOLE && (PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin) || value <= Pin->Thickness - PCB_MIN_PINORVIACOPPER)
 			&& value != Pin->DrillingHole) {
 		pcb_undo_add_obj_to_2nd_size(PCB_TYPE_PIN, Element, Pin, Pin);
-		ErasePin(Pin);
+		pcb_pin_invalidate_erase(Pin);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 		Pin->DrillingHole = value;
 		if (PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin)) {
@@ -479,7 +481,7 @@ void *pcb_pinop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pi
 			Pin->Thickness = value;
 		}
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-		DrawPin(Pin);
+		pcb_pin_invalidate_draw(Pin);
 		return (Pin);
 	}
 	return (NULL);
@@ -489,28 +491,28 @@ void *pcb_pinop_change_2nd_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pi
 /* changes the clearance size of a via */
 void *pcb_viaop_change_clear_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Via->Clearance + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Via->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
 		return (NULL);
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
-	if (ctx->chgsize.delta < 0 && value < PCB->Bloat * 2)
+	if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < PCB->Bloat * 2)
 		value = 0;
-	if ((ctx->chgsize.delta > 0 || ctx->chgsize.absolute) && value < PCB->Bloat * 2)
+	if (ctx->chgsize.value > 0 && value < PCB->Bloat * 2)
 		value = PCB->Bloat * 2 + 2;
 	if (Via->Clearance == value)
 		return NULL;
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	pcb_undo_add_obj_to_clear_size(PCB_TYPE_VIA, Via, Via, Via);
-	EraseVia(Via);
+	pcb_via_invalidate_erase(Via);
 	pcb_r_delete_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	Via->Clearance = value;
 	pcb_pin_bbox(Via);
 	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
-	DrawVia(Via);
+	pcb_via_invalidate_draw(Via);
 	Via->Element = NULL;
 	return (Via);
 }
@@ -519,7 +521,7 @@ void *pcb_viaop_change_clear_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 /* changes the size of a pin */
 void *pcb_pinop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pin->Thickness + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->Thickness + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
 		return (NULL);
@@ -527,7 +529,7 @@ void *pcb_pinop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 			value >= PCB_MIN_PINORVIASIZE && value >= Pin->DrillingHole + PCB_MIN_PINORVIACOPPER && value != Pin->Thickness) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_PIN, Element, Pin, Pin);
 		pcb_undo_add_obj_to_mask_size(PCB_TYPE_PIN, Element, Pin, Pin);
-		ErasePin(Pin);
+		pcb_pin_invalidate_erase(Pin);
 		pcb_r_delete_entry(PCB->Data->pin_tree, &Pin->BoundingBox);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 		Pin->Mask += value - Pin->Thickness;
@@ -535,7 +537,7 @@ void *pcb_pinop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 		/* SetElementBB updates all associated rtrees */
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-		DrawPin(Pin);
+		pcb_pin_invalidate_draw(Pin);
 		return (Pin);
 	}
 	return (NULL);
@@ -544,28 +546,28 @@ void *pcb_pinop_change_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 /* changes the clearance size of a pin */
 void *pcb_pinop_change_clear_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pin->Clearance + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->Clearance + ctx->chgsize.value;
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
 		return (NULL);
 	if (value < 0)
 		value = 0;
 	value = MIN(PCB_MAX_LINESIZE, value);
-	if (ctx->chgsize.delta < 0 && value < PCB->Bloat * 2)
+	if (!ctx->chgsize.is_absolute && ctx->chgsize.value < 0 && value < PCB->Bloat * 2)
 		value = 0;
-	if ((ctx->chgsize.delta > 0 || ctx->chgsize.absolute) && value < PCB->Bloat * 2)
+	if (ctx->chgsize.value > 0 && value < PCB->Bloat * 2)
 		value = PCB->Bloat * 2 + 2;
 	if (Pin->Clearance == value)
 		return NULL;
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
 	pcb_undo_add_obj_to_clear_size(PCB_TYPE_PIN, Element, Pin, Pin);
-	ErasePin(Pin);
+	pcb_pin_invalidate_erase(Pin);
 	pcb_r_delete_entry(PCB->Data->pin_tree, &Pin->BoundingBox);
 	Pin->Clearance = value;
 	/* SetElementBB updates all associated rtrees */
 	pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-	DrawPin(Pin);
+	pcb_pin_invalidate_draw(Pin);
 	return (Pin);
 }
 
@@ -574,10 +576,10 @@ void *pcb_viaop_change_name(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	char *old = Via->Name;
 
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Via)) {
-		ErasePinName(Via);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Via)) {
+		pcb_pin_name_invalidate_erase(Via);
 		Via->Name = ctx->chgname.new_name;
-		DrawPinName(Via);
+		pcb_pin_name_invalidate_draw(Via);
 	}
 	else
 		Via->Name = ctx->chgname.new_name;
@@ -590,10 +592,10 @@ void *pcb_pinop_change_name(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t
 	char *old = Pin->Name;
 
 	Element = Element;						/* get rid of 'unused...' warnings */
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pin)) {
-		ErasePinName(Pin);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pin)) {
+		pcb_pin_name_invalidate_erase(Pin);
 		Pin->Name = ctx->chgname.new_name;
-		DrawPinName(Pin);
+		pcb_pin_name_invalidate_draw(Pin);
 	}
 	else
 		Pin->Name = ctx->chgname.new_name;
@@ -606,10 +608,10 @@ void *pcb_pinop_change_num(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *
 	char *old = Pin->Number;
 
 	Element = Element;						/* get rid of 'unused...' warnings */
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pin)) {
-		ErasePinName(Pin);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pin)) {
+		pcb_pin_name_invalidate_erase(Pin);
 		Pin->Number = ctx->chgname.new_name;
-		DrawPinName(Pin);
+		pcb_pin_name_invalidate_draw(Pin);
 	}
 	else
 		Pin->Number = ctx->chgname.new_name;
@@ -622,19 +624,19 @@ void *pcb_viaop_change_square(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
 		return (NULL);
-	EraseVia(Via);
+	pcb_via_invalidate_erase(Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, NULL, Via, Via, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, NULL, Via);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, NULL, Via, Via);
-	PCB_FLAG_SQUARE_ASSIGN(ctx->chgsize.absolute, Via);
-	if (ctx->chgsize.absolute == 0)
+	pcb_undo_add_obj_to_flag(Via);
+	PCB_FLAG_SQUARE_ASSIGN(ctx->chgsize.value, Via);
+	if (ctx->chgsize.value == 0)
 		PCB_FLAG_CLEAR(PCB_FLAG_SQUARE, Via);
 	else
 		PCB_FLAG_SET(PCB_FLAG_SQUARE, Via);
 	pcb_pin_bbox(Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, NULL, Via, Via, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, NULL, Via);
-	DrawVia(Via);
+	pcb_via_invalidate_draw(Via);
 	return (Via);
 }
 
@@ -643,19 +645,19 @@ void *pcb_pinop_change_square(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
 		return (NULL);
-	ErasePin(Pin);
+	pcb_pin_invalidate_erase(Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, Element, Pin, Pin);
-	PCB_FLAG_SQUARE_ASSIGN(ctx->chgsize.absolute, Pin);
-	if (ctx->chgsize.absolute == 0)
+	pcb_undo_add_obj_to_flag(Pin);
+	PCB_FLAG_SQUARE_ASSIGN(ctx->chgsize.value, Pin);
+	if (ctx->chgsize.value == 0)
 		PCB_FLAG_CLEAR(PCB_FLAG_SQUARE, Pin);
 	else
 		PCB_FLAG_SET(PCB_FLAG_SQUARE, Pin);
 	pcb_pin_bbox(Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-	DrawPin(Pin);
+	pcb_pin_invalidate_draw(Pin);
 	return (Pin);
 }
 
@@ -682,14 +684,14 @@ void *pcb_viaop_change_octagon(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
 		return (NULL);
-	EraseVia(Via);
+	pcb_via_invalidate_erase(Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, Via, Via, Via, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, Via, Via, Via);
+	pcb_undo_add_obj_to_flag(Via);
 	PCB_FLAG_TOGGLE(PCB_FLAG_OCTAGON, Via);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_VIA, Via, Via, Via, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
-	DrawVia(Via);
+	pcb_via_invalidate_draw(Via);
 	return (Via);
 }
 
@@ -716,14 +718,14 @@ void *pcb_pinop_change_octagon(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin))
 		return (NULL);
-	ErasePin(Pin);
+	pcb_pin_invalidate_erase(Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_false);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, Element, Pin, Pin);
+	pcb_undo_add_obj_to_flag(Pin);
 	PCB_FLAG_TOGGLE(PCB_FLAG_OCTAGON, Pin);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_PIN, Element, Pin, Pin, pcb_true);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_PIN, Element, Pin);
-	DrawPin(Pin);
+	pcb_pin_invalidate_draw(Pin);
 	return (Pin);
 }
 
@@ -750,8 +752,8 @@ pcb_bool pcb_pin_change_hole(pcb_pin_t *Via)
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Via))
 		return (pcb_false);
-	EraseVia(Via);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, Via, Via, Via);
+	pcb_via_invalidate_erase(Via);
+	pcb_undo_add_obj_to_flag(Via);
 	pcb_undo_add_obj_to_mask_size(PCB_TYPE_VIA, Via, Via, Via);
 	pcb_r_delete_entry(PCB->Data->via_tree, (pcb_box_t *) Via);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
@@ -774,7 +776,7 @@ pcb_bool pcb_pin_change_hole(pcb_pin_t *Via)
 	pcb_pin_bbox(Via);
 	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
-	DrawVia(Via);
+	pcb_via_invalidate_draw(Via);
 	pcb_draw();
 	return (pcb_true);
 }
@@ -782,18 +784,18 @@ pcb_bool pcb_pin_change_hole(pcb_pin_t *Via)
 /* changes the mask size of a pin */
 void *pcb_pinop_change_mask_size(pcb_opctx_t *ctx, pcb_element_t *Element, pcb_pin_t *Pin)
 {
-	pcb_coord_t value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Pin->Mask + ctx->chgsize.delta;
+	pcb_coord_t value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Pin->Mask + ctx->chgsize.value;
 
 	value = MAX(value, 0);
-	if (value == Pin->Mask && ctx->chgsize.absolute == 0)
+	if (value == Pin->Mask && ctx->chgsize.value == 0)
 		value = Pin->Thickness;
 	if (value != Pin->Mask) {
 		pcb_undo_add_obj_to_mask_size(PCB_TYPE_PIN, Element, Pin, Pin);
-		ErasePin(Pin);
+		pcb_pin_invalidate_erase(Pin);
 		pcb_r_delete_entry(PCB->Data->pin_tree, &Pin->BoundingBox);
 		Pin->Mask = value;
 		pcb_element_bbox(PCB->Data, Element, pcb_font(PCB, 0, 1));
-		DrawPin(Pin);
+		pcb_pin_invalidate_draw(Pin);
 		return (Pin);
 	}
 	return (NULL);
@@ -804,16 +806,16 @@ void *pcb_viaop_change_mask_size(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	pcb_coord_t value;
 
-	value = (ctx->chgsize.absolute) ? ctx->chgsize.absolute : Via->Mask + ctx->chgsize.delta;
+	value = (ctx->chgsize.is_absolute) ? ctx->chgsize.value : Via->Mask + ctx->chgsize.value;
 	value = MAX(value, 0);
 	if (value != Via->Mask) {
 		pcb_undo_add_obj_to_mask_size(PCB_TYPE_VIA, Via, Via, Via);
-		EraseVia(Via);
+		pcb_via_invalidate_erase(Via);
 		pcb_r_delete_entry(PCB->Data->via_tree, &Via->BoundingBox);
 		Via->Mask = value;
 		pcb_pin_bbox(Via);
 		pcb_r_insert_entry(PCB->Data->via_tree, &Via->BoundingBox, 0);
-		DrawVia(Via);
+		pcb_via_invalidate_draw(Via);
 		return (Via);
 	}
 	return (NULL);
@@ -828,7 +830,7 @@ void *pcb_viaop_copy(pcb_opctx_t *ctx, pcb_pin_t *Via)
 										 Via->Thickness, Via->Clearance, Via->Mask, Via->DrillingHole, Via->Name, pcb_flag_mask(Via->Flags, PCB_FLAG_FOUND));
 	if (!via)
 		return (via);
-	DrawVia(via);
+	pcb_via_invalidate_draw(via);
 	pcb_undo_add_obj_to_create(PCB_TYPE_VIA, via, via, via);
 	return (via);
 }
@@ -840,11 +842,11 @@ void *pcb_viaop_move(pcb_opctx_t *ctx, pcb_pin_t *Via)
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	pcb_via_move(Via, ctx->move.dx, ctx->move.dy);
 	if (PCB->ViaOn)
-		EraseVia(Via);
+		pcb_via_invalidate_erase(Via);
 	pcb_r_insert_entry(PCB->Data->via_tree, (pcb_box_t *) Via, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_VIA, Via, Via);
 	if (PCB->ViaOn) {
-		DrawVia(Via);
+		pcb_via_invalidate_draw(Via);
 		pcb_draw();
 	}
 	return (Via);
@@ -865,7 +867,7 @@ void *pcb_viaop_remove(pcb_opctx_t *ctx, pcb_pin_t *Via)
 {
 	/* erase from screen and memory */
 	if (PCB->ViaOn) {
-		EraseVia(Via);
+		pcb_via_invalidate_erase(Via);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -879,7 +881,7 @@ void *pcb_viaop_rotate90(pcb_opctx_t *ctx, pcb_pin_t *via)
 	pcb_data_t *data = via->parent.data;
 	assert(via->parent_type == PCB_PARENT_DATA);
 
-	EraseVia(via);
+	pcb_via_invalidate_erase(via);
 	if (data->via_tree != NULL) {
 		pcb_poly_restore_to_poly(ctx->rotate.pcb->Data, PCB_TYPE_VIA, via, via);
 		pcb_r_delete_entry(data->via_tree, (pcb_box_t *)via);
@@ -890,21 +892,39 @@ void *pcb_viaop_rotate90(pcb_opctx_t *ctx, pcb_pin_t *via)
 		pcb_r_insert_entry(data->via_tree, (pcb_box_t *)via, 0);
 		pcb_poly_clear_from_poly(ctx->rotate.pcb->Data, PCB_TYPE_VIA, via, via);
 	}
-	DrawVia(via);
+	pcb_via_invalidate_draw(via);
 	return via;
 }
 
-
-#define PCB_PIN_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_HOLE | PCB_FLAG_PININPOLY | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_LOCK | PCB_FLAG_VISIT)
-void *pcb_pinop_change_flag(pcb_opctx_t *ctx, pcb_pin_t *pin)
+void *pcb_viaop_change_flag(pcb_opctx_t *ctx, pcb_pin_t *pin)
 {
-	if ((ctx->chgflag.flag & PCB_PIN_FLAGS) != ctx->chgflag.flag)
+	static pcb_flag_values_t pcb_pin_flags = 0;
+	if (pcb_pin_flags == 0)
+		pcb_pin_flags = pcb_obj_valid_flags(PCB_TYPE_PIN);
+
+	if ((ctx->chgflag.flag & pcb_pin_flags) != ctx->chgflag.flag)
 		return NULL;
-	pcb_undo_add_obj_to_flag(PCB_FLAG_TEST(PCB_FLAG_PIN, pin) ? PCB_TYPE_PIN : PCB_TYPE_VIA, pin, pin, pin);
+	if ((ctx->chgflag.flag & PCB_FLAG_TERMNAME) && (pin->term == NULL))
+		return NULL;
+	pcb_undo_add_obj_to_flag(pin);
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, pin);
 	return pin;
 }
 
+void *pcb_pinop_change_flag(pcb_opctx_t *ctx, pcb_element_t *elem, pcb_pin_t *pin)
+{
+	return pcb_viaop_change_flag(ctx, pin);
+}
+
+void *pcb_pinop_invalidate_label(pcb_opctx_t *ctx, pcb_pin_t *pin)
+{
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pin))
+		pcb_pin_name_invalidate_erase(pin);
+	else
+		pcb_pin_name_invalidate_draw(pin);
+	return pin;
+}
+
 
 /*** draw ***/
 
@@ -914,7 +934,8 @@ static void SetPVColor(pcb_pin_t *Pin, int Type)
 	char *color;
 	char buf[sizeof("#XXXXXX")];
 
-	if (Type == PCB_TYPE_VIA) {
+	if ((Type == PCB_TYPE_VIA) && (Pin->term == NULL)) {
+		/* normal via, not a terminal */
 		if (!pcb_draw_doing_pinout && PCB_FLAG_TEST(PCB_FLAG_WARN | PCB_FLAG_SELECTED | PCB_FLAG_FOUND, Pin)) {
 			if (PCB_FLAG_TEST(PCB_FLAG_WARN, Pin))
 				color = conf_core.appearance.color.warn;
@@ -933,6 +954,7 @@ static void SetPVColor(pcb_pin_t *Pin, int Type)
 			color = conf_core.appearance.color.via;
 	}
 	else {
+		/* terminal */
 		if (!pcb_draw_doing_pinout && PCB_FLAG_TEST(PCB_FLAG_WARN | PCB_FLAG_SELECTED | PCB_FLAG_FOUND, Pin)) {
 			if (PCB_FLAG_TEST(PCB_FLAG_WARN, Pin))
 				color = conf_core.appearance.color.warn;
@@ -957,33 +979,28 @@ static void SetPVColor(pcb_pin_t *Pin, int Type)
 static void _draw_pv_name(pcb_pin_t * pv)
 {
 	pcb_box_t box;
-	pcb_bool vert, flip_x, flip_y;
+	pcb_bool vert;
 	pcb_coord_t x_off, y_off;
-	pcb_text_t text;
-	char buff[128];
 	const char *pn;
+	double scale;
+	pcb_bool flip_x = conf_core.editor.view.flip_x;
+	pcb_bool flip_y = conf_core.editor.view.flip_y;
 
 	PCB_DRAW_BBOX(pv);
 
-	if (!pv->Name || !pv->Name[0]) {
+#warning subc TODO: remove the name/number logics when elements are removed
+	if (pv->term != NULL)
+		pn = pv->term;
+	else if (!pv->Name || !pv->Name[0]) { /* old: elements */
 		pn = PCB_EMPTY(pv->Number);
 	}
-	else {
+	else { /* old: elements */
 		pn = PCB_EMPTY(conf_core.editor.show_number ? pv->Number : pv->Name);
 		if (pn == NULL)
 			pn = "n/a";
 	}
 
-	if (PCB_FLAG_INTCONN_GET(pv) > 0)
-		pcb_snprintf(buff, sizeof(buff), "%s[%d]", pn, PCB_FLAG_INTCONN_GET(pv));
-	else
-		strcpy(buff, pn);
-	text.TextString = buff;
-
 	vert = PCB_FLAG_TEST(PCB_FLAG_EDGE2, pv);
-	flip_x = conf_core.editor.view.flip_x;
-	flip_y = conf_core.editor.view.flip_y;
-
 	if (vert) {
 		x_off = -pv->Thickness / 2 + conf_core.appearance.pinout.text_offset_y;
 		y_off = -pv->DrillingHole / 2 - conf_core.appearance.pinout.text_offset_x;
@@ -997,62 +1014,51 @@ static void _draw_pv_name(pcb_pin_t * pv)
 		box.Y1 = pv->Y + (flip_y ? -y_off : y_off);
 	}
 
-	pcb_gui->set_color(Output.fgGC, conf_core.appearance.color.pin_name);
-
-	text.Flags = (flip_x ^ flip_y) ? pcb_flag_make (PCB_FLAG_ONSOLDER) : pcb_no_flags();
 	/* Set font height to approx 56% of pin thickness */
-	text.Scale = 56 * pv->Thickness / PCB_FONT_CAPHEIGHT;
-	text.X = box.X1;
-	text.Y = box.Y1;
-	text.fid = 0;
-	text.Direction = (vert ? 1 : 0) + (flip_x ? 2 : 0);
+	scale = 56 * pv->Thickness / PCB_FONT_CAPHEIGHT;
 
-	if (pcb_gui->gui)
-		pcb_draw_doing_pinout++;
-	DrawTextLowLevel(&text, 0);
-	if (pcb_gui->gui)
-		pcb_draw_doing_pinout--;
+	pcb_term_label_draw(box.X1, box.Y1, scale, vert, pcb_false, pn, pv->intconn);
 }
 
 static void _draw_pv(pcb_pin_t *pv, pcb_bool draw_hole)
 {
-	if (conf_core.editor.thin_draw)
+	if (conf_core.editor.thin_draw || conf_core.editor.wireframe_draw)
 		pcb_gui->thindraw_pcb_pv(Output.fgGC, Output.fgGC, pv, draw_hole, pcb_false);
 	else
-		pcb_gui->fill_pcb_pv(Output.fgGC, Output.bgGC, pv, draw_hole, pcb_false);
+		pcb_gui->fill_pcb_pv(Output.fgGC, Output.drillGC, pv, draw_hole, pcb_false);
 	
 	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pv) && pcb_draw_doing_pinout)
 		_draw_pv_name(pv);
 }
 
-void draw_pin(pcb_pin_t *pin, pcb_bool draw_hole)
+void pcb_pin_draw(pcb_pin_t *pin, pcb_bool draw_hole)
 {
 	SetPVColor(pin, PCB_TYPE_PIN);
 	_draw_pv(pin, draw_hole);
 }
 
-pcb_r_dir_t draw_pin_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_pin_draw_callback(const pcb_box_t * b, void *cl)
 {
-	draw_pin((pcb_pin_t *) b, pcb_false);
+	pcb_pin_draw((pcb_pin_t *) b, pcb_false);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-void draw_pin_name(pcb_pin_t *pin)
+static void draw_pin_name(pcb_pin_t *pin)
 {
-	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) && PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, pin))
-		_draw_pv_name(pin); 
+	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) && PCB_FLAG_TEST(PCB_FLAG_TERMNAME, pin))
+		_draw_pv_name(pin);
 }
 
-pcb_r_dir_t draw_pin_name_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_pin_name_draw_callback(const pcb_box_t * b, void *cl)
 {
 	draw_pin_name((pcb_pin_t *) b);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-pcb_r_dir_t clear_pin_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_pin_clear_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pin_t *pin = (pcb_pin_t *) b;
-	if (conf_core.editor.thin_draw || conf_core.editor.thin_draw_poly)
+	if (conf_core.editor.thin_draw || conf_core.editor.thin_draw_poly || conf_core.editor.wireframe_draw)
 		pcb_gui->thindraw_pcb_pv(Output.pmGC, Output.pmGC, pin, pcb_false, pcb_true);
 	else
 		pcb_gui->fill_pcb_pv(Output.pmGC, Output.pmGC, pin, pcb_false, pcb_true);
@@ -1066,16 +1072,16 @@ static void draw_via(pcb_pin_t *via, pcb_bool draw_hole)
 	_draw_pv(via, draw_hole);
 }
 
-pcb_r_dir_t draw_via_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_via_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pin_t *via = (pcb_pin_t *)b;
 
-	if (PCB->SubcPartsOn || !pcb_is_gobj_in_subc(via->parent_type, &via->parent))
+	if (PCB->SubcPartsOn || !pcb_gobj_parent_subc(via->parent_type, &via->parent))
 		draw_via(via, pcb_false);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
-pcb_r_dir_t draw_hole_callback(const pcb_box_t * b, void *cl)
+pcb_r_dir_t pcb_hole_draw_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pin_t *pv = (pcb_pin_t *) b;
 	int plated = cl ? *(int *) cl : -1;
@@ -1085,10 +1091,10 @@ pcb_r_dir_t draw_hole_callback(const pcb_box_t * b, void *cl)
 	if ((plated == 0 && !PCB_FLAG_TEST(PCB_FLAG_HOLE, pv)) || (plated == 1 && PCB_FLAG_TEST(PCB_FLAG_HOLE, pv)))
 		return PCB_R_DIR_FOUND_CONTINUE;
 
-	if (!PCB->SubcPartsOn && pcb_is_gobj_in_subc(pv->parent_type, &pv->parent))
+	if (!PCB->SubcPartsOn && pcb_gobj_parent_subc(pv->parent_type, &pv->parent))
 		return PCB_R_DIR_FOUND_CONTINUE;
 
-	if (conf_core.editor.thin_draw) {
+	if (conf_core.editor.thin_draw || conf_core.editor.wireframe_draw) {
 		if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pv)) {
 			pcb_gui->set_line_cap(Output.fgGC, Round_Cap);
 			pcb_gui->set_line_width(Output.fgGC, 0);
@@ -1096,7 +1102,7 @@ pcb_r_dir_t draw_hole_callback(const pcb_box_t * b, void *cl)
 		}
 	}
 	else
-		pcb_gui->fill_circle(Output.bgGC, pv->X, pv->Y, pv->DrillingHole / 2);
+		pcb_gui->fill_circle(Output.drillGC, pv->X, pv->Y, pv->DrillingHole / 2);
 
 	if (PCB_FLAG_TEST(PCB_FLAG_HOLE, pv)) {
 		if (PCB_FLAG_TEST(PCB_FLAG_WARN, pv))
@@ -1145,53 +1151,53 @@ static void GatherPVName(pcb_pin_t *Ptr)
 	pcb_draw_invalidate(&box);
 }
 
-void EraseVia(pcb_pin_t *Via)
+void pcb_via_invalidate_erase(pcb_pin_t *Via)
 {
 	pcb_draw_invalidate(Via);
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Via))
-		EraseViaName(Via);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Via))
+		pcb_via_name_invalidate_erase(Via);
 	pcb_flag_erase(&Via->Flags);
 }
 
-void EraseViaName(pcb_pin_t *Via)
+void pcb_via_name_invalidate_erase(pcb_pin_t *Via)
 {
 	GatherPVName(Via);
 }
 
-void ErasePin(pcb_pin_t *Pin)
+void pcb_pin_invalidate_erase(pcb_pin_t *Pin)
 {
 	pcb_draw_invalidate(Pin);
-	if (PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pin))
-		ErasePinName(Pin);
+	if (PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pin))
+		pcb_pin_name_invalidate_erase(Pin);
 	pcb_flag_erase(&Pin->Flags);
 }
 
-void ErasePinName(pcb_pin_t *Pin)
+void pcb_pin_name_invalidate_erase(pcb_pin_t *Pin)
 {
 	GatherPVName(Pin);
 }
 
-void DrawVia(pcb_pin_t *Via)
+void pcb_via_invalidate_draw(pcb_pin_t *Via)
 {
 	pcb_draw_invalidate(Via);
-	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, Via) && PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Via))
-		DrawViaName(Via);
+	if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, Via) && PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Via))
+		pcb_via_name_invalidate_draw(Via);
 }
 
-void DrawViaName(pcb_pin_t *Via)
+void pcb_via_name_invalidate_draw(pcb_pin_t *Via)
 {
 	GatherPVName(Via);
 }
 
-void DrawPin(pcb_pin_t *Pin)
+void pcb_pin_invalidate_draw(pcb_pin_t *Pin)
 {
 	pcb_draw_invalidate(Pin);
-	if ((!PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin) && PCB_FLAG_TEST(PCB_FLAG_DISPLAYNAME, Pin))
+	if ((!PCB_FLAG_TEST(PCB_FLAG_HOLE, Pin) && PCB_FLAG_TEST(PCB_FLAG_TERMNAME, Pin))
 			|| pcb_draw_doing_pinout)
-		DrawPinName(Pin);
+		pcb_pin_name_invalidate_draw(Pin);
 }
 
-void DrawPinName(pcb_pin_t *Pin)
+void pcb_pin_name_invalidate_draw(pcb_pin_t *Pin)
 {
 	GatherPVName(Pin);
 }
