@@ -34,6 +34,7 @@
 #include "compat_nls.h"
 #include "undo.h"
 #include "polygon.h"
+#include "polygon_offs.h"
 #include "rotate.h"
 #include "search.h"
 
@@ -61,6 +62,7 @@ pcb_polygon_t *pcb_poly_alloc(pcb_layer_t * layer)
 
 	new_obj = calloc(sizeof(pcb_polygon_t), 1);
 	new_obj->type = PCB_OBJ_POLYGON;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, layer, layer);
 
 	polylist_append(&layer->Polygon, new_obj);
@@ -193,9 +195,9 @@ void pcb_poly_bbox(pcb_polygon_t *Polygon)
 }
 
 /* creates a new polygon from the old formats rectangle data */
-pcb_polygon_t *pcb_poly_new_from_rectangle(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2, pcb_coord_t Y2, pcb_flag_t Flags)
+pcb_polygon_t *pcb_poly_new_from_rectangle(pcb_layer_t *Layer, pcb_coord_t X1, pcb_coord_t Y1, pcb_coord_t X2, pcb_coord_t Y2, pcb_coord_t Clearance, pcb_flag_t Flags)
 {
-	pcb_polygon_t *polygon = pcb_poly_new(Layer, Flags);
+	pcb_polygon_t *polygon = pcb_poly_new(Layer, Clearance, Flags);
 	if (!polygon)
 		return (polygon);
 
@@ -208,6 +210,32 @@ pcb_polygon_t *pcb_poly_new_from_rectangle(pcb_layer_t *Layer, pcb_coord_t X1, p
 	return (polygon);
 }
 
+pcb_polygon_t *pcb_poly_new_from_poly(pcb_layer_t *Layer, pcb_polygon_t *src, pcb_coord_t offs, pcb_coord_t Clearance, pcb_flag_t Flags)
+{
+	pcb_coord_t x, y;
+	pcb_poly_it_t it;
+	pcb_pline_t *pl;
+	int go;
+	pcb_polygon_t *polygon = pcb_poly_new(Layer, Clearance, Flags);
+
+	if (!polygon)
+		return (polygon);
+
+	it.pa = src->Clipped;
+	pcb_poly_island_first(src, &it);
+	pl = pcb_poly_contour(&it);
+	it.cntr = pcb_pline_dup_offset(pl, offs);
+	for(go = pcb_poly_vect_first(&it, &x, &y); go; go = pcb_poly_vect_next(&it, &x, &y))
+		pcb_poly_point_new(polygon, x, y);
+
+	pcb_poly_contours_free(&it.cntr);
+
+	pcb_add_polygon_on_layer(Layer, polygon);
+	return (polygon);
+}
+
+
+
 void pcb_add_polygon_on_layer(pcb_layer_t *Layer, pcb_polygon_t *polygon)
 {
 	pcb_poly_bbox(polygon);
@@ -215,16 +243,18 @@ void pcb_add_polygon_on_layer(pcb_layer_t *Layer, pcb_polygon_t *polygon)
 		Layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon, 0);
 	PCB_SET_PARENT(polygon, layer, Layer);
+	pcb_poly_clear_from_poly(Layer->parent, PCB_TYPE_POLYGON, Layer, polygon);
 }
 
 /* creates a new polygon on a layer */
-pcb_polygon_t *pcb_poly_new(pcb_layer_t *Layer, pcb_flag_t Flags)
+pcb_polygon_t *pcb_poly_new(pcb_layer_t *Layer, pcb_coord_t Clearance, pcb_flag_t Flags)
 {
 	pcb_polygon_t *polygon = pcb_poly_alloc(Layer);
 
 	/* copy values */
 	polygon->Flags = Flags;
 	polygon->ID = pcb_create_ID_get();
+	polygon->Clearance = Clearance;
 	polygon->Clipped = NULL;
 	polygon->NoHoles = NULL;
 	polygon->NoHolesValid = 0;
@@ -235,14 +265,14 @@ static pcb_polygon_t *pcb_poly_copy_meta(pcb_polygon_t *dst, pcb_polygon_t *src)
 {
 	if (dst == NULL)
 		return NULL;
-	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes, 0);
+	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes);
 	return dst;
 }
 
 pcb_polygon_t *pcb_poly_dup(pcb_layer_t *dst, pcb_polygon_t *src)
 {
 	pcb_board_t *pcb;
-	pcb_polygon_t *p = pcb_poly_new(dst, src->Flags);
+	pcb_polygon_t *p = pcb_poly_new(dst, src->Clearance, src->Flags);
 	pcb_poly_copy(p, src, 0, 0);
 	pcb_add_polygon_on_layer(dst, p);
 	pcb_poly_copy_meta(p, src);
@@ -256,7 +286,7 @@ pcb_polygon_t *pcb_poly_dup(pcb_layer_t *dst, pcb_polygon_t *src)
 pcb_polygon_t *pcb_poly_dup_at(pcb_layer_t *dst, pcb_polygon_t *src, pcb_coord_t dx, pcb_coord_t dy)
 {
 	pcb_board_t *pcb;
-	pcb_polygon_t *p = pcb_poly_new(dst, src->Flags);
+	pcb_polygon_t *p = pcb_poly_new(dst, src->Clearance, src->Flags);
 	pcb_poly_copy(p, src, dx, dy);
 	pcb_add_polygon_on_layer(dst, p);
 	pcb_poly_copy_meta(p, src);
@@ -343,13 +373,26 @@ double pcb_poly_area(const pcb_polygon_t *poly)
 
 
 /*** ops ***/
+
+#define ppclear(poly) \
+do { \
+	if (PCB_POLY_HAS_CLEARANCE(poly) && (poly->parent.layer != NULL)) \
+		pcb_poly_clear_from_poly(poly->parent.layer->parent, PCB_TYPE_POLYGON, poly->parent.layer, poly); \
+} while(0)
+
+#define pprestore(poly) \
+do { \
+	if (PCB_POLY_HAS_CLEARANCE(poly) && (poly->parent.layer != NULL)) \
+		pcb_poly_restore_to_poly(poly->parent.layer->parent, PCB_TYPE_POLYGON, poly->parent.layer, poly); \
+} while(0)
+
 /* copies a polygon to buffer */
 void *pcb_polyop_add_to_buffer(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polygon)
 {
 	pcb_layer_t *layer = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, Layer)];
 	pcb_polygon_t *polygon;
 
-	polygon = pcb_poly_new(layer, Polygon->Flags);
+	polygon = pcb_poly_new(layer, Polygon->Clearance, Polygon->Flags);
 	pcb_poly_copy(polygon, Polygon, 0, 0);
 	pcb_poly_copy_meta(polygon, Polygon);
 
@@ -371,6 +414,8 @@ void *pcb_polyop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_polyg
 {
 	pcb_layer_t *lay = &ctx->buffer.dst->Layer[pcb_layer_id(ctx->buffer.src, layer)];
 
+	pprestore(polygon);
+
 	pcb_r_delete_entry(layer->polygon_tree, (pcb_box_t *) polygon);
 
 	polylist_remove(polygon);
@@ -384,6 +429,7 @@ void *pcb_polyop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_polyg
 
 	PCB_SET_PARENT(polygon, layer, lay);
 
+	ppclear(polygon);
 	return (polygon);
 }
 
@@ -408,10 +454,10 @@ void *pcb_polyop_change_clear(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Polygon))
 		return (NULL);
 	pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLYGON, Layer, Polygon, Polygon, pcb_true);
-	pcb_undo_add_obj_to_flag(PCB_TYPE_POLYGON, Layer, Polygon, Polygon);
+	pcb_undo_add_obj_to_flag(Polygon);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARPOLY, Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
-	DrawPolygon(Layer, Polygon);
+	pcb_poly_invalidate_draw(Layer, Polygon);
 	return (Polygon);
 }
 
@@ -422,6 +468,8 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 	pcb_cardinal_t n;
 	pcb_line_t line;
 
+	pprestore(Polygon);
+
 	if (!ctx->insert.forcible) {
 		/*
 		 * first make sure adding the point is sensible
@@ -435,7 +483,7 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 	/*
 	 * second, shift the points up to make room for the new point
 	 */
-	ErasePolygon(Polygon);
+	pcb_poly_invalidate_erase(Polygon);
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	save = *pcb_poly_point_new(Polygon, ctx->insert.x, ctx->insert.y);
 	for (n = Polygon->PointN - 1; n > ctx->insert.idx; n--)
@@ -454,12 +502,35 @@ void *pcb_polyop_insert_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	if (ctx->insert.forcible || !pcb_poly_remove_excess_points(Layer, Polygon)) {
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		pcb_draw();
 	}
+
+	ppclear(Polygon);
+
 	return (&Polygon->Points[ctx->insert.idx]);
 }
 
+/* changes the clearance flag of a polygon */
+void *pcb_polyop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *poly)
+{
+	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, poly))
+		return (NULL);
+	pcb_poly_invalidate_erase(poly);
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, poly)) {
+		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLYGON, Layer, poly, poly, pcb_false);
+		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_POLYGON, Layer, poly);
+	}
+	pcb_undo_add_obj_to_flag(poly);
+	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARPOLYPOLY, poly);
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, poly)) {
+		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_POLYGON, Layer, poly, poly, pcb_true);
+		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_POLYGON, Layer, poly);
+	}
+	pcb_poly_invalidate_draw(Layer, poly);
+	return poly;
+}
+
 /* low level routine to move a polygon */
 void pcb_poly_move(pcb_polygon_t *Polygon, pcb_coord_t DX, pcb_coord_t DY)
 {
@@ -474,25 +545,28 @@ void pcb_poly_move(pcb_polygon_t *Polygon, pcb_coord_t DX, pcb_coord_t DY)
 /* moves a polygon */
 void *pcb_polyop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polygon)
 {
+	pprestore(Polygon);
 	if (Layer->meta.real.vis) {
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 	}
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_move(Polygon, ctx->move.dx, ctx->move.dy);
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	if (Layer->meta.real.vis) {
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		pcb_draw();
 	}
+	ppclear(Polygon);
 	return (Polygon);
 }
 
 /* moves a polygon-point */
 void *pcb_polyop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polygon, pcb_point_t *Point)
 {
+	pprestore(Polygon);
 	if (Layer->meta.real.vis) {
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 	}
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	PCB_MOVE(Point->X, Point->Y, ctx->move.dx, ctx->move.dy);
@@ -501,9 +575,10 @@ void *pcb_polyop_move_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t
 	pcb_poly_remove_excess_points(Layer, Polygon);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	if (Layer->meta.real.vis) {
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		pcb_draw();
 	}
+	ppclear(Polygon);
 	return (Point);
 }
 
@@ -512,6 +587,7 @@ void *pcb_polyop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_p
 {
 	pcb_r_delete_entry(Source->polygon_tree, (pcb_box_t *) polygon);
 
+	pprestore(polygon);
 	polylist_remove(polygon);
 	polylist_append(&Destination->Polygon, polygon);
 
@@ -520,7 +596,7 @@ void *pcb_polyop_move_to_layer_low(pcb_opctx_t *ctx, pcb_layer_t * Source, pcb_p
 	pcb_r_insert_entry(Destination->polygon_tree, (pcb_box_t *) polygon, 0);
 
 	PCB_SET_PARENT(polygon, layer, Destination);
-
+	ppclear(polygon);
 	return polygon;
 }
 
@@ -537,9 +613,9 @@ pcb_r_dir_t mptl_pin_callback(const pcb_box_t * b, void *cl)
 	if (!PCB_FLAG_THERM_TEST(d->snum, pin) || !pcb_poly_is_point_in_p(pin->X, pin->Y, pin->Thickness + pin->Clearance + 2, d->polygon))
 		return PCB_R_DIR_NOT_FOUND;
 	if (d->type == PCB_TYPE_PIN)
-		pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, pin->Element, pin, pin);
+		pcb_undo_add_obj_to_flag(pin);
 	else
-		pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, pin, pin, pin);
+		pcb_undo_add_obj_to_flag(pin);
 	PCB_FLAG_THERM_ASSIGN(d->dnum, PCB_FLAG_THERM_GET(d->snum, pin), pin);
 	PCB_FLAG_THERM_CLEAR(d->snum, pin);
 	return PCB_R_DIR_FOUND_CONTINUE;
@@ -555,11 +631,11 @@ void *pcb_polyop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_polygo
 		pcb_message(PCB_MSG_WARNING, _("Sorry, the object is locked\n"));
 		return NULL;
 	}
-	if (((long int) ctx->move.dst_layer == -1) || (Layer == ctx->move.dst_layer))
+	if (Layer == ctx->move.dst_layer)
 		return (Polygon);
 	pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_POLYGON, Layer, Polygon, Polygon);
 	if (Layer->meta.real.vis)
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 	/* Move all of the thermals with the polygon */
 	d.snum = pcb_layer_id(PCB->Data, Layer);
 	d.dnum = pcb_layer_id(PCB->Data, ctx->move.dst_layer);
@@ -571,7 +647,7 @@ void *pcb_polyop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * Layer, pcb_polygo
 	newone = (struct pcb_polygon_s *) pcb_polyop_move_to_layer_low(ctx, Layer, Polygon, ctx->move.dst_layer);
 	pcb_poly_init_clip(PCB->Data, ctx->move.dst_layer, newone);
 	if (ctx->move.dst_layer->meta.real.vis) {
-		DrawPolygon(ctx->move.dst_layer, newone);
+		pcb_poly_invalidate_draw(ctx->move.dst_layer, newone);
 		pcb_draw();
 	}
 	return (newone);
@@ -597,6 +673,7 @@ void *pcb_polyop_destroy_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon
 	pcb_cardinal_t contour;
 	pcb_cardinal_t contour_start, contour_end, contour_points;
 
+	pprestore(Polygon);
 	point_idx = pcb_poly_point_idx(Polygon, Point);
 	contour = pcb_poly_contour_point(Polygon, point_idx);
 	contour_start = (contour == 0) ? 0 : Polygon->HoleIndex[contour - 1];
@@ -621,6 +698,7 @@ void *pcb_polyop_destroy_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon
 	pcb_poly_bbox(Polygon);
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
+	ppclear(Polygon);
 	return (Polygon);
 }
 
@@ -628,8 +706,9 @@ void *pcb_polyop_destroy_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon
 void *pcb_polyop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polygon)
 {
 	/* erase from screen */
+	pprestore(Polygon);
 	if (Layer->meta.real.vis) {
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -659,8 +738,10 @@ void *pcb_polyop_remove_counter(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygo
 	if (contour == 0)
 		return pcb_poly_remove(Layer, Polygon);
 
+	pprestore(Polygon);
+
 	if (Layer->meta.real.vis) {
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -683,9 +764,12 @@ void *pcb_polyop_remove_counter(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygo
 	Polygon->HoleIndexN--;
 
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
+
+	ppclear(Polygon);
+
 	/* redraw polygon if necessary */
 	if (Layer->meta.real.vis) {
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
@@ -700,6 +784,8 @@ void *pcb_polyop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 	pcb_cardinal_t contour;
 	pcb_cardinal_t contour_start, contour_end, contour_points;
 
+	pprestore(Polygon);
+
 	point_idx = pcb_poly_point_idx(Polygon, Point);
 	contour = pcb_poly_contour_point(Polygon, point_idx);
 	contour_start = (contour == 0) ? 0 : Polygon->HoleIndex[contour - 1];
@@ -710,7 +796,7 @@ void *pcb_polyop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 		return pcb_polyop_remove_counter(ctx, Layer, Polygon, contour);
 
 	if (Layer->meta.real.vis)
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 
 	/* insert the polygon-point into the undo list */
 	pcb_undo_add_obj_to_remove_point(PCB_TYPE_POLYGON_POINT, Layer, Polygon, point_idx);
@@ -733,10 +819,12 @@ void *pcb_polyop_remove_point(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_
 
 	/* redraw polygon if necessary */
 	if (Layer->meta.real.vis) {
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		if (!ctx->remove.bulk)
 			pcb_draw();
 	}
+
+	ppclear(Polygon);
 	return NULL;
 }
 
@@ -745,40 +833,60 @@ void *pcb_polyop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polyg
 {
 	pcb_polygon_t *polygon;
 
-	polygon = pcb_poly_new(Layer, pcb_no_flags());
+	polygon = pcb_poly_new(Layer, Polygon->Clearance, pcb_no_flags());
 	pcb_poly_copy(polygon, Polygon, ctx->copy.DeltaX, ctx->copy.DeltaY);
 	pcb_poly_copy_meta(polygon, Polygon);
 	if (!Layer->polygon_tree)
 		Layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) polygon, 0);
 	pcb_poly_init_clip(PCB->Data, Layer, polygon);
-	DrawPolygon(Layer, polygon);
+	pcb_poly_invalidate_draw(Layer, polygon);
 	pcb_undo_add_obj_to_create(PCB_TYPE_POLYGON, Layer, polygon, polygon);
+	ppclear(polygon);
 	return (polygon);
 }
 
 void *pcb_polyop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polygon)
 {
+	pprestore(Polygon);
 	if (Layer->meta.real.vis)
-		ErasePolygon(Polygon);
+		pcb_poly_invalidate_erase(Polygon);
 	pcb_r_delete_entry(Layer->polygon_tree, (pcb_box_t *) Polygon);
 	pcb_poly_rotate90(Polygon, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
 	pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
 	pcb_poly_init_clip(PCB->Data, Layer, Polygon);
 	if (Layer->meta.real.vis) {
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		pcb_draw();
 	}
+	ppclear(Polygon);
 	return Polygon;
 }
 
-#define PCB_POLY_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_CLEARPOLY | PCB_FLAG_FULLPOLY | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_LOCK | PCB_FLAG_VISIT)
 void *pcb_polyop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_polygon_t *Polygon)
 {
-	if ((ctx->chgflag.flag & PCB_POLY_FLAGS) != ctx->chgflag.flag)
+	static pcb_flag_values_t pcb_poly_flags = 0;
+	if (pcb_poly_flags == 0)
+		pcb_poly_flags = pcb_obj_valid_flags(PCB_TYPE_POLYGON);
+
+	if ((ctx->chgflag.flag & pcb_poly_flags) != ctx->chgflag.flag)
 		return NULL;
-	pcb_undo_add_obj_to_flag(PCB_TYPE_POLYGON, Polygon, Polygon, Polygon);
+	if ((ctx->chgflag.flag & PCB_FLAG_TERMNAME) && (Polygon->term == NULL))
+		return NULL;
+	pcb_undo_add_obj_to_flag(Polygon);
+
+
+	if (ctx->chgflag.flag & (PCB_FLAG_CLEARPOLY | PCB_FLAG_CLEARPOLYPOLY))
+		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_TYPE_POLYGON, Polygon->parent.layer, Polygon);
+
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, Polygon);
+
+	if (ctx->chgflag.flag & PCB_FLAG_FULLPOLY)
+		pcb_poly_init_clip(ctx->chgflag.pcb->Data, Polygon->parent.layer, Polygon);
+
+	if (ctx->chgflag.flag & (PCB_FLAG_CLEARPOLY | PCB_FLAG_CLEARPOLYPOLY))
+		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_TYPE_POLYGON, Polygon->parent.layer, Polygon);
+
 	return Polygon;
 }
 
@@ -806,27 +914,100 @@ void pcb_poly_map_contours(pcb_polygon_t *p, void *ctx, pcb_poly_map_cb_t *cb)
 	} while(pa != p->Clipped);
 }
 
-
-
+void *pcb_polyop_invalidate_label(pcb_opctx_t *ctx, pcb_layer_t *layer, pcb_polygon_t *poly)
+{
+	pcb_poly_name_invalidate_draw(poly);
+	return poly;
+}
 
 /*** draw ***/
-pcb_r_dir_t draw_poly_callback(const pcb_box_t * b, void *cl)
+
+static pcb_bool is_poly_term_vert(const pcb_polygon_t *poly)
+{
+	pcb_coord_t dx, dy;
+
+	dx = poly->BoundingBox.X2 - poly->BoundingBox.X1;
+	if (dx < 0)
+		dx = -dx;
+
+	dy = poly->BoundingBox.Y2 - poly->BoundingBox.Y1;
+	if (dy < 0)
+		dy = -dy;
+
+	return dx < dy;
+}
+
+
+void pcb_poly_name_invalidate_draw(pcb_polygon_t *poly)
+{
+	if (poly->term != NULL) {
+		pcb_text_t text;
+		pcb_term_label_setup(&text, (poly->BoundingBox.X1 + poly->BoundingBox.X2)/2, (poly->BoundingBox.Y1 + poly->BoundingBox.Y2)/2,
+			100.0, is_poly_term_vert(poly), pcb_true, poly->term, poly->intconn);
+		pcb_draw_invalidate(&text);
+	}
+}
+
+void pcb_poly_draw_label(pcb_polygon_t *poly)
+{
+	if (poly->term != NULL)
+		pcb_term_label_draw((poly->BoundingBox.X1 + poly->BoundingBox.X2)/2, (poly->BoundingBox.Y1 + poly->BoundingBox.Y2)/2,
+			100.0, is_poly_term_vert(poly), pcb_true, poly->term, poly->intconn);
+}
+
+void pcb_poly_draw_(pcb_polygon_t *polygon, const pcb_box_t *drawn_area, int allow_term_gfx)
+{
+	if ((pcb_gui->thindraw_pcb_polygon != NULL) && (conf_core.editor.thin_draw || conf_core.editor.thin_draw_poly || conf_core.editor.wireframe_draw))
+	{
+		pcb_gui->thindraw_pcb_polygon(Output.fgGC, polygon, drawn_area);
+	}
+	else {
+		if ((allow_term_gfx) && pcb_draw_term_need_gfx(polygon)) {
+			pcb_vnode_t *n, *head;
+			int i;
+
+			pcb_gui->fill_pcb_polygon(Output.active_padGC, polygon, drawn_area);
+			head = &polygon->Clipped->contours->head;
+
+			pcb_gui->set_line_cap(Output.fgGC, Square_Cap);
+			for(n = head, i = 0; (n != head) || (i == 0); n = n->next, i++) {
+				pcb_coord_t x, y, r;
+				x = (n->prev->point[0] + n->point[0] + n->next->point[0]) / 3;
+				y = (n->prev->point[1] + n->point[1] + n->next->point[1]) / 3;
+
+#warning subc TODO: check if x;y is within the poly, but use a cheaper method than the official
+				r = (PCB_DRAW_TERM_GFX_MIN_WIDTH + PCB_DRAW_TERM_GFX_MAX_WIDTH)/2;
+				pcb_gui->set_line_width(Output.fgGC, r);
+				pcb_gui->draw_line(Output.fgGC, x, y, x, y);
+			}
+		}
+		else
+			pcb_gui->fill_pcb_polygon(Output.fgGC, polygon, drawn_area);
+	}
+
+	/* If checking planes, thin-draw any pieces which have been clipped away */
+	if (pcb_gui->thindraw_pcb_polygon != NULL && conf_core.editor.check_planes && !PCB_FLAG_TEST(PCB_FLAG_FULLPOLY, polygon)) {
+		pcb_polygon_t poly = *polygon;
+
+		for (poly.Clipped = polygon->Clipped->f; poly.Clipped != polygon->Clipped; poly.Clipped = poly.Clipped->f)
+			pcb_gui->thindraw_pcb_polygon(Output.fgGC, &poly, drawn_area);
+	}
+
+	if (polygon->term != NULL) {
+		if ((pcb_draw_doing_pinout) || PCB_FLAG_TEST(PCB_FLAG_TERMNAME, polygon))
+			pcb_draw_delay_label_add((pcb_any_obj_t *)polygon);
+	}
+}
+
+static void pcb_poly_draw(pcb_layer_t *layer, pcb_polygon_t *polygon, const pcb_box_t *drawn_area, int allow_term_gfx)
 {
-	pcb_draw_info_t *i = cl;
-	pcb_polygon_t *polygon = (pcb_polygon_t *) b;
 	static const char *color;
 	char buf[sizeof("#XXXXXX")];
 
-	if (!polygon->Clipped)
-		return PCB_R_DIR_NOT_FOUND;
-
-	if (!PCB->SubcPartsOn && pcb_is_lobj_in_subc(polygon->parent_type, &polygon->parent))
-		return PCB_R_DIR_NOT_FOUND;
-
 	if (PCB_FLAG_TEST(PCB_FLAG_WARN, polygon))
 		color = conf_core.appearance.color.warn;
 	else if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, polygon))
-		color = i->layer->meta.real.selected_color;
+		color = layer->meta.real.selected_color;
 	else if (PCB_FLAG_TEST(PCB_FLAG_FOUND, polygon))
 		color = conf_core.appearance.color.connected;
 	else if (PCB_FLAG_TEST(PCB_FLAG_ONPOINT, polygon)) {
@@ -835,33 +1016,52 @@ pcb_r_dir_t draw_poly_callback(const pcb_box_t * b, void *cl)
 		color = buf;
 	}
 	else
-		color = i->layer->meta.real.color;
+		color = layer->meta.real.color;
 	pcb_gui->set_color(Output.fgGC, color);
 
-	if ((pcb_gui->thindraw_pcb_polygon != NULL) && (conf_core.editor.thin_draw || conf_core.editor.thin_draw_poly))
-		pcb_gui->thindraw_pcb_polygon(Output.fgGC, polygon, i->drawn_area);
-	else
-		pcb_gui->fill_pcb_polygon(Output.fgGC, polygon, i->drawn_area);
+	pcb_poly_draw_(polygon, drawn_area, allow_term_gfx);
+}
 
-	/* If checking planes, thin-draw any pieces which have been clipped away */
-	if (pcb_gui->thindraw_pcb_polygon != NULL && conf_core.editor.check_planes && !PCB_FLAG_TEST(PCB_FLAG_FULLPOLY, polygon)) {
-		pcb_polygon_t poly = *polygon;
+pcb_r_dir_t pcb_poly_draw_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_draw_info_t *i = cl;
+	pcb_polygon_t *polygon = (pcb_polygon_t *) b;
 
-		for (poly.Clipped = polygon->Clipped->f; poly.Clipped != polygon->Clipped; poly.Clipped = poly.Clipped->f)
-			pcb_gui->thindraw_pcb_polygon(Output.fgGC, &poly, i->drawn_area);
-	}
+	if (!polygon->Clipped)
+		return PCB_R_DIR_NOT_FOUND;
+
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(polygon->parent_type, &polygon->parent))
+		return PCB_R_DIR_NOT_FOUND;
+
+	pcb_poly_draw(i->layer, polygon, i->drawn_area, 0);
+
+	return PCB_R_DIR_FOUND_CONTINUE;
+}
+
+pcb_r_dir_t pcb_poly_draw_term_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_draw_info_t *i = cl;
+	pcb_polygon_t *polygon = (pcb_polygon_t *) b;
+
+	if (!polygon->Clipped)
+		return PCB_R_DIR_NOT_FOUND;
+
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(polygon->parent_type, &polygon->parent))
+		return PCB_R_DIR_NOT_FOUND;
+
+	pcb_poly_draw(i->layer, polygon, i->drawn_area, 1);
 
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
 /* erases a polygon on a layer */
-void ErasePolygon(pcb_polygon_t *Polygon)
+void pcb_poly_invalidate_erase(pcb_polygon_t *Polygon)
 {
 	pcb_draw_invalidate(Polygon);
 	pcb_flag_erase(&Polygon->Flags);
 }
 
-void DrawPolygon(pcb_layer_t *Layer, pcb_polygon_t *Polygon)
+void pcb_poly_invalidate_draw(pcb_layer_t *Layer, pcb_polygon_t *Polygon)
 {
 	pcb_draw_invalidate(Polygon);
 }
