@@ -4,9 +4,6 @@
  *  pcb-rnd, interactive printed circuit board design
  *  Copyright (C) 2017 Tibor 'Igor2' Palinkas
  *
- *  This module, rats.c, was written and is Copyright (C) 1997 by harry eaton
- *  this module is also subject to the GNU GPL as described below
- *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -25,6 +22,8 @@
 
 #include "config.h"
 
+#include <genvector/vtp0.h>
+
 #include "buffer.h"
 #include "board.h"
 #include "crosshair.h"
@@ -32,6 +31,9 @@
 #include "error.h"
 #include "obj_subc.h"
 #include "obj_subc_op.h"
+#include "obj_poly_op.h"
+#include "obj_line_op.h"
+#include "obj_term.h"
 #include "obj_text_draw.h"
 #include "rtree.h"
 #include "draw.h"
@@ -42,22 +44,38 @@
 #include "compat_misc.h"
 #include "math_helper.h"
 #include "pcb_minuid.h"
+#include "conf_core.h"
 
 #define SUBC_AUX_NAME "subc-aux"
 
+static void pcb_subc_attrib_post_change(pcb_attribute_list_t *list, const char *name, const char *value)
+{
+	pcb_subc_t *sc = (pcb_subc_t *)(((char *)list) - offsetof(pcb_subc_t, Attributes));
+	if (strcmp(name, "refdes") == 0) {
+		const char *inv;
+		sc->refdes = value;
+		inv = pcb_obj_id_invalid(sc->refdes);
+		if (inv != NULL)
+			pcb_message(PCB_MSG_ERROR, "Invalid character '%c' in subc refdes '%s'\n", *inv, sc->refdes);
+	}
+}
+
 pcb_subc_t *pcb_subc_alloc(void)
 {
 	pcb_subc_t *sc;
 	sc = calloc(sizeof(pcb_subc_t), 1);
+	sc->Attributes.post_change = pcb_subc_attrib_post_change;
 	sc->data = pcb_data_new(NULL);
 	sc->type = PCB_OBJ_SUBC;
 	PCB_SET_PARENT(sc->data, subc, sc);
 	minuid_gen(&pcb_minuid, sc->uid);
+	pcb_term_init(&sc->terminals);
 	return sc;
 }
 
 void pcb_subc_free(pcb_subc_t *sc)
 {
+	pcb_term_uninit(&sc->terminals);
 	pcb_subclist_remove(sc);
 	pcb_data_free(sc->data);
 	free(sc->data);
@@ -87,7 +105,7 @@ static pcb_layer_t *pcb_subc_layer_create_buff(pcb_subc_t *sc, pcb_layer_t *src)
 static pcb_line_t *add_aux_line(pcb_layer_t *aux, const char *key, const char *val, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
 {
 	pcb_line_t *l = pcb_line_new(aux, x1, y1, x2, y2, PCB_MM_TO_COORD(0.1), 0, pcb_no_flags());
-	pcb_attribute_put(&l->Attributes, key, val, 1);
+	pcb_attribute_put(&l->Attributes, key, val);
 	return l;
 }
 
@@ -155,11 +173,68 @@ static void pcb_subc_cache_invalidate(pcb_subc_t *sc)
 	sc->aux_cache[0] = NULL;
 }
 
+static pcb_polygon_t *sqline2term(pcb_layer_t *dst, pcb_line_t *line)
+{
+	pcb_polygon_t *poly;
+	pcb_coord_t x[4], y[4];
+	int n;
+
+	pcb_sqline_to_rect(line, x, y);
+
+	poly = pcb_poly_new(dst, line->Clearance, pcb_no_flags());
+	for(n = 0; n < 4; n++)
+		pcb_poly_point_new(poly, x[n], y[n]);
+	PCB_FLAG_SET(PCB_FLAG_CLEARPOLYPOLY, poly);
+	pcb_attribute_copy_all(&poly->Attributes, &line->Attributes);
+
+	pcb_poly_init_clip(dst->parent, dst, poly);
+	pcb_add_polygon_on_layer(dst, poly);
+
+	return poly;
+}
+
+extern unsigned long pcb_obj_type2oldtype(pcb_objtype_t type);
+
+/* Move the pad-side-effect objects to the appropriate layer */
+static void move_pad_side_effect(pcb_any_obj_t *o, pcb_layer_t *top, pcb_layer_t *bottom)
+{
+	pcb_layer_t *source = o->parent.layer;
+	pcb_layer_t *target = (source->meta.bound.type & PCB_LYT_TOP) ? top : bottom;
+	switch(o->type) {
+		case PCB_OBJ_POLYGON:
+			pcb_polyop_move_to_layer_low(NULL, source, (pcb_polygon_t *)o, target);
+			break;
+		case PCB_OBJ_LINE:
+			pcb_lineop_move_to_layer_low(NULL, source, (pcb_line_t *)o, target);
+			break;
+		default:
+			assert(!"internal error: invalid mask/paste side effect");
+	}
+}
+
+static pcb_coord_t read_mask(pcb_any_obj_t *obj)
+{
+	const char *smask = pcb_attribute_get(&obj->Attributes, "elem_smash_pad_mask");
+	pcb_coord_t mask = 0;
+
+	if (smask != NULL) {
+		pcb_bool success;
+		mask = pcb_get_value_ex(smask, NULL, NULL, NULL, "mm", &success);
+		if (!success)
+			mask = 0;
+	}
+	return mask;
+}
 
 int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 {
 	pcb_subc_t *sc;
-	int n;
+	int n, top_pads = 0, bottom_pads = 0, has_refdes_text = 0;
+	pcb_layer_t *dst_top_mask = NULL, *dst_bottom_mask = NULL, *dst_top_paste = NULL, *dst_bottom_paste = NULL, *dst_top_silk = NULL;
+	vtp0_t mask_pads, paste_pads;
+
+	vtp0_init(&mask_pads);
+	vtp0_init(&paste_pads);
 
 	sc = pcb_subc_alloc();
 	sc->ID = pcb_create_ID_get();
@@ -172,18 +247,85 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 		pcb_text_t *text;
 		pcb_polygon_t *poly;
 		pcb_arc_t *arc;
+		pcb_layer_type_t ltype;
 
 		if (pcb_layer_is_pure_empty(&buffer->Data->Layer[n]))
 			continue;
 
 		src = &buffer->Data->Layer[n];
 		dst = pcb_subc_layer_create_buff(sc, src);
+		ltype = dst->meta.bound.type;
+
+		if ((dst->comb & PCB_LYC_SUB) == 0) {
+			if ((ltype & PCB_LYT_PASTE) && (ltype & PCB_LYT_TOP))
+				dst_top_paste = dst;
+			else if ((ltype & PCB_LYT_PASTE) && (ltype & PCB_LYT_BOTTOM))
+				dst_bottom_paste = dst;
+			else if ((ltype & PCB_LYT_SILK) && (ltype & PCB_LYT_TOP))
+				dst_top_silk = dst;
+		}
+
+		if (dst->comb & PCB_LYC_SUB) {
+			if ((ltype & PCB_LYT_MASK) && (ltype & PCB_LYT_TOP))
+				dst_top_mask = dst;
+			else if ((ltype & PCB_LYT_MASK) && (ltype & PCB_LYT_BOTTOM))
+				dst_bottom_mask = dst;
+		}
 
 		while((line = linelist_first(&src->Line)) != NULL) {
-			linelist_remove(line);
-			linelist_append(&dst->Line, line);
-			PCB_SET_PARENT(line, layer, dst);
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, line);
+			pcb_coord_t mask = 0;
+			char *np, *sq, *termpad;
+			const char *term;
+
+			termpad = pcb_attribute_get(&line->Attributes, "elem_smash_pad");
+			if (termpad != NULL) {
+				if (ltype & PCB_LYT_TOP)
+					top_pads++;
+				else if (ltype & PCB_LYT_BOTTOM)
+					bottom_pads++;
+				mask = read_mask((pcb_any_obj_t *)line);
+			}
+			term = pcb_attribute_get(&line->Attributes, "term");
+			np = pcb_attribute_get(&line->Attributes, "elem_smash_nopaste");
+			sq = pcb_attribute_get(&line->Attributes, "elem_smash_shape_square");
+			if ((sq != NULL) && (*sq == '1')) { /* convert to polygon */
+				poly = sqline2term(dst, line);
+				if (termpad != NULL) {
+					if ((np == NULL) || (*np != '1')) {
+						poly = sqline2term(dst, line);
+						vtp0_append(&paste_pads, poly);
+					}
+					if (mask > 0) {
+						line->Thickness = mask;
+						poly = sqline2term(dst, line);
+						if (term != NULL)
+							pcb_attribute_put(&poly->Attributes, "term", term);
+						vtp0_append(&mask_pads, poly);
+					}
+				}
+
+				pcb_line_free(line);
+			}
+			else {
+				/* copy the line */
+				linelist_remove(line);
+				linelist_append(&dst->Line, line);
+				PCB_SET_PARENT(line, layer, dst);
+				PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, line);
+
+				if (termpad != NULL) {
+					pcb_line_t *nl = pcb_line_dup(dst, line);
+					if ((np == NULL) || (*np != '1'))
+						vtp0_append(&paste_pads, nl);
+					if (mask > 0) {
+						nl = pcb_line_dup(dst, line);
+						nl->Thickness = mask;
+						if (term != NULL)
+							pcb_attribute_put(&nl->Attributes, "term", term);
+						vtp0_append(&mask_pads, nl);
+					}
+				}
+			}
 		}
 
 		while((arc = arclist_first(&src->Arc)) != NULL) {
@@ -198,6 +340,8 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 			textlist_append(&dst->Text, text);
 			PCB_SET_PARENT(text, layer, dst);
 			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, text);
+			if (!has_refdes_text && (text->TextString != NULL) && (strstr(text->TextString, "%a.parent.refdes%") != NULL))
+				has_refdes_text = 1;
 		}
 
 		while((poly = polylist_first(&src->Polygon)) != NULL) {
@@ -208,34 +352,113 @@ int pcb_subc_convert_from_buffer(pcb_buffer_t *buffer)
 		}
 	}
 
+	{ /* check if we have pins - they need mask */
+		pcb_pin_t *via;
+
+		for(via = pinlist_first(&buffer->Data->Via); via != NULL; via = pinlist_next(via)) {
+			if (pcb_attribute_get(&via->Attributes, "elem_smash_pad") != NULL) {
+				top_pads++;
+				bottom_pads++;
+				break;
+			}
+		}
+	}
+
+	/* create paste and mask side effects - needed when importing from footprint */
+	{
+		if (top_pads > 0) {
+			if (dst_top_paste == NULL)
+				dst_top_paste = pcb_layer_new_bound(sc->data, PCB_LYT_TOP | PCB_LYT_PASTE, "top paste");
+			if (dst_top_mask == NULL) {
+				dst_top_mask = pcb_layer_new_bound(sc->data, PCB_LYT_TOP | PCB_LYT_MASK, "top mask");
+				dst_top_mask->comb = PCB_LYC_SUB;
+			}
+		}
+		if (bottom_pads > 0) {
+			if (dst_bottom_paste == NULL)
+				dst_bottom_paste = pcb_layer_new_bound(sc->data, PCB_LYT_BOTTOM | PCB_LYT_PASTE, "bottom paste");
+			if (dst_bottom_mask == NULL) {
+				dst_bottom_mask = pcb_layer_new_bound(sc->data, PCB_LYT_BOTTOM | PCB_LYT_MASK, "bottom mask");
+				dst_bottom_mask->comb = PCB_LYC_SUB;
+			}
+		}
+	}
+
+	{ /* convert globals */
+		pcb_pin_t *via;
+
+		while((via = pinlist_first(&buffer->Data->Via)) != NULL) {
+			const char *term;
+			pinlist_remove(via);
+			pinlist_append(&sc->data->Via, via);
+			PCB_SET_PARENT(via, data, sc->data);
+			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, via);
+			term = pcb_attribute_get(&via->Attributes, "term");
+			if (pcb_attribute_get(&via->Attributes, "elem_smash_pad") != NULL) {
+				pcb_coord_t mask = read_mask((pcb_any_obj_t *)via);
+				
+				if (mask == 0)
+					mask = via->Mask;
+
+				if (mask > 0) {
+					if (PCB_FLAG_TEST(PCB_FLAG_SQUARE, via)) {
+						pcb_line_t line;
+						pcb_polygon_t *poly;
+						memset(&line, 0, sizeof(line));
+						line.Point1.X = line.Point2.X = via->X;
+						line.Point1.Y = line.Point2.Y = via->Y;
+						line.Thickness = mask;
+						poly = sqline2term(dst_top_mask, &line);
+						if (term != NULL)
+							pcb_attribute_put(&poly->Attributes, "term", term);
+						poly = sqline2term(dst_bottom_mask, &line);
+						if (term != NULL)
+							pcb_attribute_put(&poly->Attributes, "term", term);
+					}
+					else {
+						pcb_line_t *line;
+						line = pcb_line_new(dst_top_mask, via->X, via->Y, via->X, via->Y, mask, 0, pcb_no_flags());
+						if (term != NULL)
+							pcb_attribute_put(&line->Attributes, "term", term);
+						line = pcb_line_new(dst_bottom_mask, via->X, via->Y, via->X, via->Y, mask, 0, pcb_no_flags());
+						if (term != NULL)
+							pcb_attribute_put(&line->Attributes, "term", term);
+					}
+				}
+			}
+		}
+	}
+
+	for(n = 0; n < vtp0_len(&paste_pads); n++)
+		move_pad_side_effect(paste_pads.array[n], dst_top_paste, dst_bottom_paste);
+	for(n = 0; n < vtp0_len(&mask_pads); n++)
+		move_pad_side_effect(mask_pads.array[n], dst_top_mask, dst_bottom_mask);
+
+	vtp0_uninit(&mask_pads);
+	vtp0_uninit(&paste_pads);
+
 	/* create aux layer */
 	{
-		pcb_layer_t *aux = &sc->data->Layer[sc->data->LayerN++];
 		pcb_coord_t unit = PCB_MM_TO_COORD(1);
-
-		memset(aux, 0, sizeof(pcb_layer_t));
-		aux->meta.bound.name = pcb_strdup(SUBC_AUX_NAME);
-		aux->meta.bound.type = PCB_LYT_VIRTUAL | PCB_LYT_NOEXPORT | PCB_LYT_MISC;
-		aux->grp = -1;
-		aux->parent = sc->data;
+		pcb_layer_t *aux = pcb_layer_new_bound(sc->data, PCB_LYT_VIRTUAL | PCB_LYT_NOEXPORT | PCB_LYT_MISC | PCB_LYT_TOP, SUBC_AUX_NAME);
 
 		add_aux_line(aux, "subc-role", "origin", buffer->X, buffer->Y, buffer->X, buffer->Y);
 		add_aux_line(aux, "subc-role", "x", buffer->X, buffer->Y, buffer->X+unit, buffer->Y);
 		add_aux_line(aux, "subc-role", "y", buffer->X, buffer->Y, buffer->X, buffer->Y+unit);
 	}
 
-
-	{ /* convert globals */
-		pcb_pin_t *via;
-
-		while((via = pinlist_first(&buffer->Data->Via)) != NULL) {
-			pinlist_remove(via);
-			pinlist_append(&sc->data->Via, via);
-			PCB_SET_PARENT(via, data, sc->data);
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, via);
-		}
+	/* Add refdes */
+	pcb_attribute_put(&sc->Attributes, "refdes", "U0");
+	if (!has_refdes_text) {
+		if (dst_top_silk == NULL)
+			dst_top_silk = pcb_layer_new_bound(sc->data, PCB_LYT_TOP | PCB_LYT_SILK, "top-silk");
+		if (dst_top_silk != NULL)
+			pcb_text_new(dst_top_silk, pcb_font(PCB, 0, 0), buffer->X, buffer->Y, 0, 100, "%a.parent.refdes%", pcb_flag_make(PCB_FLAG_DYNTEXT));
+		else
+			pcb_message(PCB_MSG_ERROR, "Error: can't create top silk layer in subc for placing the refdes\n");
 	}
 
+
 	return 0;
 }
 
@@ -281,7 +504,7 @@ void XORDrawSubc(pcb_subc_t *sc, pcb_coord_t DX, pcb_coord_t DY)
 			XORPolygon(poly, DX, DY, 0);
 
 		textlist_foreach(&ly->Text, &it, text)
-			XORDrawText(text, DX, DY);
+			pcb_text_draw_xor(text, DX, DY);
 	}
 
 	/* draw global objects */
@@ -314,7 +537,7 @@ static pcb_subc_t *pcb_subc_copy_meta(pcb_subc_t *dst, pcb_subc_t *src)
 {
 	if (dst == NULL)
 		return NULL;
-	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes, 0);
+	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes);
 	return dst;
 }
 
@@ -354,6 +577,7 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 			/* copy within the same board */
 			memcpy(&dl->meta.bound, &sl->meta.bound, sizeof(sl->meta.bound));
 			dl->meta.bound.name = pcb_strdup(sl->meta.bound.name);
+			dl->comb = sl->comb;
 			if (dl->meta.bound.real != NULL)
 				pcb_layer_link_trees(dl, dl->meta.bound.real);
 		}
@@ -362,9 +586,15 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 			memcpy(&dl->meta.bound, &sl->meta.bound, sizeof(sl->meta.bound));
 			dl->meta.bound.name = pcb_strdup(sl->meta.bound.name);
 			dl->meta.bound.real = pcb_layer_resolve_binding(pcb, sl);
+			dl->comb = sl->comb;
 
-			if (dl->meta.bound.real == NULL)
-				pcb_message(PCB_MSG_WARNING, "Couldn't bind a layer of subcricuit TODO1 while placing it\n");
+			if (dl->meta.bound.real == NULL) {
+				if (!(dl->meta.bound.type & PCB_LYT_VIRTUAL)) {
+					const char *name = dl->meta.bound.name;
+					if (name == NULL) name = "<anonymous>";
+					pcb_message(PCB_MSG_WARNING, "Couldn't bind a layer %s of subcricuit while placing it\n", name);
+				}
+			}
 			else
 				pcb_layer_link_trees(dl, dl->meta.bound.real);
 		}
@@ -374,6 +604,7 @@ pcb_subc_t *pcb_subc_dup_at(pcb_board_t *pcb, pcb_data_t *dst, pcb_subc_t *src,
 
 			dl->meta.bound.real = NULL;
 			dl->meta.bound.name = pcb_strdup(sl->meta.bound.name);
+			dl->comb = sl->comb;
 		}
 
 
@@ -559,6 +790,9 @@ void *pcb_subcop_copy(pcb_opctx_t *ctx, pcb_subc_t *src)
 
 	pcb_undo_add_obj_to_create(PCB_TYPE_SUBC, sc, sc, sc);
 
+	if (conf_core.editor.show_solder_side)
+		pcb_subc_change_side(sc, 2 * pcb_crosshair.Y - PCB->MaxHeight);
+
 	return (sc);
 }
 
@@ -578,6 +812,97 @@ void *pcb_subcop_rotate90(pcb_opctx_t *ctx, pcb_subc_t *sc)
 	return pcb_subc_op((pcb != NULL ? pcb->Data : NULL), sc, &Rotate90Functions, ctx);
 }
 
+static int subc_relocate_layer_objs(pcb_layer_t *dl, pcb_data_t *src_data, pcb_layer_t *sl, int src_has_real_layer, int dst_is_pcb)
+{
+	pcb_line_t *line;
+	pcb_text_t *text;
+	pcb_polygon_t *poly;
+	pcb_arc_t *arc;
+	gdl_iterator_t it;
+	int chg = 0;
+
+	linelist_foreach(&sl->Line, &it, line) {
+		if (src_has_real_layer) {
+			pcb_poly_restore_to_poly(src_data, PCB_TYPE_LINE, sl, line);
+			pcb_r_delete_entry(sl->line_tree, (pcb_box_t *)line);
+			chg++;
+		}
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, line);
+		if ((dl != NULL) && (dl->line_tree != NULL)) {
+			pcb_r_insert_entry(dl->line_tree, (pcb_box_t *)line, 0);
+			chg++;
+		}
+	}
+
+	arclist_foreach(&sl->Arc, &it, arc) {
+		if (src_has_real_layer) {
+			pcb_poly_restore_to_poly(src_data, PCB_TYPE_ARC, sl, arc);
+			pcb_r_delete_entry(sl->arc_tree, (pcb_box_t *)arc);
+			chg++;
+		}
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, arc);
+		if ((dl != NULL) && (dl->arc_tree != NULL)) {
+			pcb_r_insert_entry(dl->arc_tree, (pcb_box_t *)arc, 0);
+			chg++;
+		}
+	}
+
+	textlist_foreach(&sl->Text, &it, text) {
+		if (src_has_real_layer) {
+			pcb_poly_restore_to_poly(src_data, PCB_TYPE_LINE, sl, text);
+			pcb_r_delete_entry(sl->text_tree, (pcb_box_t *)text);
+			chg++;
+		}
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, text);
+		if ((dl != NULL) && (dl->text_tree != NULL)) {
+			pcb_r_insert_entry(dl->text_tree, (pcb_box_t *)text, 0);
+			chg++;
+		}
+	}
+
+	polylist_foreach(&sl->Polygon, &it, poly) {
+		if (src_has_real_layer) {
+			pcb_r_delete_entry(sl->polygon_tree, (pcb_box_t *)poly);
+			chg++;
+		}
+		PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, poly);
+		if ((dl != NULL) && (dl->polygon_tree != NULL)) {
+			pcb_r_insert_entry(dl->polygon_tree, (pcb_box_t *)poly, 0);
+			chg++;
+		}
+	}
+
+	if (!dst_is_pcb) {
+		/* keep only the layer binding match, unbound other aspects */
+		sl->meta.bound.real = NULL;
+		sl->arc_tree = sl->line_tree = sl->text_tree = sl->polygon_tree = NULL;
+		chg++;
+	}
+	else {
+		sl->meta.bound.real = dl;
+		chg++;
+	}
+	return chg;
+}
+
+static void subc_relocate_poly_clips(pcb_data_t *dst, pcb_subc_t *sc)
+{
+	int n;
+
+	for(n = 0; n < sc->data->LayerN; n++) {
+		pcb_polygon_t *poly;
+		gdl_iterator_t it;
+		pcb_layer_t *sl = sc->data->Layer + n;
+
+		if (sl->meta.bound.real == NULL)
+			continue;
+
+		polylist_foreach(&sl->Polygon, &it, poly) {
+			pcb_poly_init_clip(dst, sl->meta.bound.real, poly);
+		}
+	}
+}
+
 void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
 	int n;
@@ -603,11 +928,6 @@ void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 	for(n = 0; n < sc->data->LayerN; n++) {
 		pcb_layer_t *sl = sc->data->Layer + n;
 		pcb_layer_t *dl;
-		pcb_line_t *line;
-		pcb_text_t *text;
-		pcb_polygon_t *poly;
-		pcb_arc_t *arc;
-		gdl_iterator_t it;
 		int src_has_real_layer = (sl->meta.bound.real != NULL);
 
 		if (dst_is_pcb) {
@@ -627,54 +947,9 @@ void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 		else
 			dl = ctx->buffer.dst->Layer + n;
 
-		linelist_foreach(&sl->Line, &it, line) {
-			if (src_has_real_layer) {
-				pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_LINE, sl, line);
-				pcb_r_delete_entry(sl->line_tree, (pcb_box_t *)line);
-			}
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, line);
-			if ((dl != NULL) && (dl->line_tree != NULL))
-				pcb_r_insert_entry(dl->line_tree, (pcb_box_t *)line, 0);
-		}
-
-		arclist_foreach(&sl->Arc, &it, arc) {
-			if (src_has_real_layer) {
-				pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_ARC, sl, arc);
-				pcb_r_delete_entry(sl->arc_tree, (pcb_box_t *)arc);
-			}
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, arc);
-			if ((dl != NULL) && (dl->arc_tree != NULL))
-				pcb_r_insert_entry(dl->arc_tree, (pcb_box_t *)arc, 0);
-		}
-
-		textlist_foreach(&sl->Text, &it, text) {
-			if (src_has_real_layer) {
-				pcb_poly_restore_to_poly(ctx->buffer.src, PCB_TYPE_LINE, sl, text);
-				pcb_r_delete_entry(sl->text_tree, (pcb_box_t *)text);
-			}
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, text);
-			if ((dl != NULL) && (dl->text_tree != NULL))
-				pcb_r_insert_entry(dl->text_tree, (pcb_box_t *)text, 0);
-		}
-
-		polylist_foreach(&sl->Polygon, &it, poly) {
-			if (src_has_real_layer)
-				pcb_r_delete_entry(sl->polygon_tree, (pcb_box_t *)poly);
-			PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, poly);
-			if ((dl != NULL) && (dl->polygon_tree != NULL))
-				pcb_r_insert_entry(dl->polygon_tree, (pcb_box_t *)poly, 0);
-		}
-
-		if (!dst_is_pcb) {
-			/* keep only the layer binding match, unbound other aspects */
-			sl->meta.bound.real = NULL;
-			sl->arc_tree = sl->line_tree = sl->text_tree = sl->polygon_tree = NULL;
-		}
-		else
-			sl->meta.bound.real = dl;
+		subc_relocate_layer_objs(dl, ctx->buffer.src, sl, src_has_real_layer, dst_is_pcb);
 	}
 
-
 	/* move globals */
 	{
 		pcb_pin_t *via;
@@ -699,17 +974,8 @@ void *pcb_subcop_move_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 		sc->data->via_tree = NULL;
 
 	/* init poly clips */
-	if (dst_is_pcb) {
-		for(n = 0; n < sc->data->LayerN; n++) {
-			pcb_polygon_t *poly;
-			gdl_iterator_t it;
-			pcb_layer_t *sl = sc->data->Layer + n;
-
-			polylist_foreach(&sl->Polygon, &it, poly) {
-				pcb_poly_init_clip(ctx->buffer.dst, sl->meta.bound.real, poly);
-			}
-		}
-	}
+	if (dst_is_pcb)
+		subc_relocate_poly_clips(ctx->buffer.dst, sc);
 
 	PCB_FLAG_CLEAR(PCB_FLAG_WARN | PCB_FLAG_FOUND | PCB_FLAG_SELECTED, sc);
 	PCB_SET_PARENT(sc, data, ctx->buffer.dst);
@@ -729,7 +995,6 @@ void *pcb_subcop_add_to_buffer(pcb_opctx_t *ctx, pcb_subc_t *sc)
 pcb_bool pcb_subc_smash_buffer(pcb_buffer_t *buff)
 {
 	pcb_subc_t *subc;
-	int n;
 
 	if (pcb_subclist_length(&buff->Data->subc) != 1)
 		return (pcb_false);
@@ -745,6 +1010,46 @@ pcb_bool pcb_subc_smash_buffer(pcb_buffer_t *buff)
 	return (pcb_true);
 }
 
+int pcb_subc_rebind(pcb_board_t *pcb, pcb_subc_t *sc)
+{
+	int n, chgly = 0;
+	pcb_board_t *dst_top = pcb_data_get_top(sc->data);
+	int dst_is_pcb = ((dst_top != NULL) && (dst_top->Data == pcb->Data));
+
+	if (!dst_is_pcb)
+		return -1;
+
+	EraseSubc(sc);
+
+	/* relocate bindings */
+	for(n = 0; n < sc->data->LayerN; n++) {
+		pcb_layer_t *sl = sc->data->Layer + n;
+		pcb_layer_t *dl = pcb_layer_resolve_binding(dst_top, sl);
+		int src_has_real_layer = (sl->meta.bound.real != NULL);
+
+		if (dl != NULL) {
+			if (sl->meta.bound.real == dl)
+				continue;
+
+			/* make sure all trees exist on the dest layer - if these are the first objects there we may need to create them */
+			if (dl->line_tree == NULL) dl->line_tree = pcb_r_create_tree(NULL, 0, 0);
+			if (dl->arc_tree == NULL) dl->arc_tree = pcb_r_create_tree(NULL, 0, 0);
+			if (dl->text_tree == NULL) dl->text_tree = pcb_r_create_tree(NULL, 0, 0);
+			if (dl->polygon_tree == NULL) dl->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
+		}
+
+		if (subc_relocate_layer_objs(dl, pcb->Data, sl, src_has_real_layer, 1) > 0)
+			chgly++;
+
+		if (dl != NULL)
+			pcb_layer_link_trees(sl, dl);
+	}
+
+	if (chgly > 0)
+		subc_relocate_poly_clips(pcb->Data, sc);
+	return chgly;
+}
+
 
 void *pcb_subcop_change_size(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
@@ -778,8 +1083,8 @@ void *pcb_subcop_change_nonetlist(pcb_opctx_t *ctx, pcb_subc_t *sc)
 
 void *pcb_subcop_change_name(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
-#warning subc TODO
-	abort();
+	pcb_attribute_put(&sc->Attributes, "refdes", ctx->chgname.new_name);
+	return sc;
 }
 
 void *pcb_subcop_destroy(pcb_opctx_t *ctx, pcb_subc_t *sc)
@@ -840,11 +1145,15 @@ void *pcb_subcop_change_square(pcb_opctx_t *ctx, pcb_subc_t *sc)
 }
 
 
-#define PCB_SUBC_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_LOCK | PCB_FLAG_VISIT | PCB_FLAG_NONETLIST)
+
 void *pcb_subcop_change_flag(pcb_opctx_t *ctx, pcb_subc_t *sc)
 {
+	static pcb_flag_values_t pcb_subc_flags = 0;
+	if (pcb_subc_flags == 0)
+		pcb_subc_flags = pcb_obj_valid_flags(PCB_TYPE_SUBC);
+
 	pcb_subc_op(ctx->chgflag.pcb->Data, sc, &ChgFlagFunctions, ctx);
-	if ((ctx->chgflag.flag & PCB_SUBC_FLAGS) == ctx->chgflag.flag)
+	if ((ctx->chgflag.flag & pcb_subc_flags) == ctx->chgflag.flag)
 		PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, sc);
 	return sc;
 }
@@ -854,7 +1163,7 @@ void pcb_subc_select(pcb_board_t *pcb, pcb_subc_t *sc, pcb_change_flag_t how, in
 {
 	pcb_opctx_t ctx;
 
-	pcb_undo_add_obj_to_flag(PCB_TYPE_SUBC, sc, sc, sc);
+	pcb_undo_add_obj_to_flag(sc);
 
 	ctx.chgflag.pcb = pcb;
 	ctx.chgflag.how = how;
@@ -928,7 +1237,9 @@ pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl)
 	pcb_gui->set_color(Output.fgGC, conf_core.appearance.color.element);
 	pcb_gui->set_line_cap(Output.fgGC, Trace_Cap);
 	pcb_gui->set_line_width(Output.fgGC, 0);
+	pcb_gui->set_draw_xor(Output.fgGC, 1);
 	pcb_subc_draw_origin(subc, 0, 0);
+	pcb_gui->set_draw_xor(Output.fgGC, 0);
 
 	pcb_gui->set_color(Output.fgGC, selected ? conf_core.appearance.color.subc_selected : conf_core.appearance.color.subc);
 	pcb_gui->set_line_width(Output.fgGC, 0);
@@ -939,5 +1250,20 @@ pcb_r_dir_t draw_subc_mark_callback(const pcb_box_t *b, void *cl)
 	pcb_draw_dashed_line(Output.fgGC, bb->X2, bb->Y2, bb->X1, bb->Y2);
 	pcb_gui->set_draw_xor(Output.fgGC, 0);
 
+	if (subc->refdes != NULL)
+		pcb_term_label_draw(bb->X1, bb->Y1, 50.0, 0, 0, subc->refdes, subc->intconn);
+
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
+
+pcb_subc_t *pcb_subc_by_name(pcb_data_t *base, const char *name)
+{
+#warning subc TODO: hierarchy
+	PCB_SUBC_LOOP(base);
+	{
+		if ((subc->refdes != NULL) && (PCB_NSTRCMP(subc->refdes, name) == 0))
+			return subc;
+	}
+	PCB_END_LOOP;
+	return NULL;
+}
