@@ -60,6 +60,7 @@ pcb_text_t *pcb_text_alloc(pcb_layer_t * layer)
 
 	new_obj = calloc(sizeof(pcb_text_t), 1);
 	new_obj->type = PCB_OBJ_TEXT;
+	new_obj->Attributes.post_change = pcb_obj_attrib_post_change;
 	PCB_SET_PARENT(new_obj, layer, layer);
 
 	textlist_append(&layer->Text, new_obj);
@@ -97,6 +98,7 @@ pcb_text_t *pcb_text_new(pcb_layer_t *Layer, pcb_font_t *PCBFont, pcb_coord_t X,
 	text->Scale = Scale;
 	text->TextString = pcb_strdup(TextString);
 	text->fid = PCBFont->id;
+	text->ID = pcb_create_ID_get();
 
 	pcb_add_text_on_layer(Layer, text, PCBFont);
 
@@ -107,7 +109,7 @@ static pcb_text_t *pcb_text_copy_meta(pcb_text_t *dst, pcb_text_t *src)
 {
 	if (dst == NULL)
 		return NULL;
-	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes, 0);
+	pcb_attribute_copy_all(&dst->Attributes, &src->Attributes);
 	return dst;
 }
 
@@ -129,17 +131,69 @@ void pcb_add_text_on_layer(pcb_layer_t *Layer, pcb_text_t *text, pcb_font_t *PCB
 {
 	/* calculate size of the bounding box */
 	pcb_text_bbox(PCBFont, text);
-	text->ID = pcb_create_ID_get();
 	if (!Layer->text_tree)
 		Layer->text_tree = pcb_r_create_tree(NULL, 0, 0);
 	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) text, 0);
 }
 
+static int pcb_text_render_str_cb(void *ctx, gds_t *s, const char **input)
+{
+	pcb_text_t *text = ctx;
+	char *end, key[128], *path, *attrs;
+	size_t len;
+
+	end = strchr(*input, '%');
+	len = end - *input;
+	if (len > sizeof(key)-1)
+		return -1;
+
+	strncpy(key, *input, len);
+	*input += len+1;
+
+	if ((key[0] == 'a') && (key[1] == '.')) {
+		pcb_attribute_list_t *attr = &text->Attributes;
+		path = key+2;
+		if ((path[0] == 'p') && (memcmp(path, "parent.", 7) == 0)) {
+			pcb_data_t *par = text->parent.layer->parent;
+			if (par->parent_type == PCB_PARENT_SUBC)
+				attr = &par->parent.subc->Attributes;
+			else if (par->parent_type == PCB_PARENT_BOARD)
+				attr = &par->parent.board->Attributes;
+			else
+				attr = NULL;
+			path+=7;
+		}
+		if (attr != NULL) {
+			attrs = pcb_attribute_get(attr, path);
+			if (attrs != NULL)
+				gds_append_str(s, attrs);
+		}
+	}
+	return 0;
+}
+
+/* Render the string of a text, doing substitution if needed - don't allocate if there's no subst */
+static unsigned char *pcb_text_render_str(pcb_text_t *text)
+{
+	if (!PCB_FLAG_TEST(PCB_FLAG_DYNTEXT, text))
+		return (unsigned char *)text->TextString;
+
+	return (unsigned char *)pcb_strdup_subst(text->TextString, pcb_text_render_str_cb, text, PCB_SUBST_PERCENT | PCB_SUBST_CONF);
+}
+
+/* Free rendered if it was allocated */
+static void pcb_text_free_str(pcb_text_t *text, unsigned char *rendered)
+{
+	if ((unsigned char *)text->TextString != rendered)
+		free(rendered);
+}
+
+
 /* creates the bounding box of a text object */
 void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text)
 {
 	pcb_symbol_t *symbol;
-	unsigned char *s = (unsigned char *) Text->TextString;
+	unsigned char *s, *rendered = pcb_text_render_str(Text);
 	int i;
 	int space;
 	pcb_coord_t minx, miny, maxx, maxy, tx;
@@ -148,6 +202,8 @@ void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text)
 	pcb_bool first_time = pcb_true;
 	pcb_polygon_t *poly;
 
+	s = rendered;
+
 	if (FontPtr == NULL)
 		FontPtr = pcb_font(PCB, Text->fid, 1);
 
@@ -265,6 +321,7 @@ void pcb_text_bbox(pcb_font_t *FontPtr, pcb_text_t *Text)
 	Text->BoundingBox.X2 += PCB->Bloat;
 	Text->BoundingBox.Y2 += PCB->Bloat;
 	pcb_close_box(&Text->BoundingBox);
+	pcb_text_free_str(Text, rendered);
 }
 
 
@@ -304,21 +361,21 @@ void *pcb_textop_move_to_buffer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_text_
 /* changes the scaling factor of a text object */
 void *pcb_textop_change_size(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
-	int value = ctx->chgsize.absolute ? PCB_COORD_TO_MIL(ctx->chgsize.absolute)
-		: Text->Scale + PCB_COORD_TO_MIL(ctx->chgsize.delta);
+	int value = ctx->chgsize.is_absolute ? PCB_COORD_TO_MIL(ctx->chgsize.value)
+		: Text->Scale + PCB_COORD_TO_MIL(ctx->chgsize.value);
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
 		return (NULL);
 	if (value <= PCB_MAX_TEXTSCALE && value >= PCB_MIN_TEXTSCALE && value != Text->Scale) {
 		pcb_undo_add_obj_to_size(PCB_TYPE_TEXT, Layer, Text, Text);
-		EraseText(Layer, Text);
+		pcb_text_invalidate_erase(Layer, Text);
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 		Text->Scale = value;
 		pcb_text_bbox(pcb_font(PCB, Text->fid, 1), Text);
 		pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
-		DrawText(Layer, Text);
+		pcb_text_invalidate_draw(Layer, Text);
 		return (Text);
 	}
 	return (NULL);
@@ -332,7 +389,7 @@ void *pcb_textop_change_name(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
 		return (NULL);
-	EraseText(Layer, Text);
+	pcb_text_invalidate_erase(Layer, Text);
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *)Text);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	Text->TextString = ctx->chgname.new_name;
@@ -341,7 +398,7 @@ void *pcb_textop_change_name(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 	pcb_text_bbox(pcb_font(PCB, Text->fid, 1), Text);
 	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
-	DrawText(Layer, Text);
+	pcb_text_invalidate_draw(Layer, Text);
 	return (old);
 }
 
@@ -350,18 +407,18 @@ void *pcb_textop_change_join(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *T
 {
 	if (PCB_FLAG_TEST(PCB_FLAG_LOCK, Text))
 		return (NULL);
-	EraseText(Layer, Text);
+	pcb_text_invalidate_erase(Layer, Text);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_TEXT, Layer, Text, Text, pcb_false);
 		pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	}
-	pcb_undo_add_obj_to_flag(PCB_TYPE_TEXT, Layer, Text, Text);
+	pcb_undo_add_obj_to_flag(Text);
 	PCB_FLAG_TOGGLE(PCB_FLAG_CLEARLINE, Text);
 	if (PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, Text)) {
 		pcb_undo_add_obj_to_clear_poly(PCB_TYPE_TEXT, Layer, Text, Text, pcb_true);
 		pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	}
-	DrawText(Layer, Text);
+	pcb_text_invalidate_draw(Layer, Text);
 	return (Text);
 }
 
@@ -389,7 +446,7 @@ void *pcb_textop_copy(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	text = pcb_text_new(Layer, pcb_font(PCB, Text->fid, 1), Text->X + ctx->copy.DeltaX,
 											 Text->Y + ctx->copy.DeltaY, Text->Direction, Text->Scale, Text->TextString, pcb_flag_mask(Text->Flags, PCB_FLAG_FOUND));
 	pcb_text_copy_meta(text, Text);
-	DrawText(Layer, text);
+	pcb_text_invalidate_draw(Layer, text);
 	pcb_undo_add_obj_to_create(PCB_TYPE_TEXT, Layer, text, text);
 	return (text);
 }
@@ -400,9 +457,9 @@ void *pcb_textop_move(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 	if (Layer->meta.real.vis) {
-		EraseText(Layer, Text);
+		pcb_text_invalidate_erase(Layer, Text);
 		pcb_text_move(Text, ctx->move.dx, ctx->move.dy);
-		DrawText(Layer, Text);
+		pcb_text_invalidate_draw(Layer, Text);
 		pcb_draw();
 	}
 	else
@@ -448,10 +505,10 @@ void *pcb_textop_move_to_layer(pcb_opctx_t *ctx, pcb_layer_t * layer, pcb_text_t
 	if (ctx->move.dst_layer != layer) {
 		pcb_undo_add_obj_to_move_to_layer(PCB_TYPE_TEXT, layer, text, text);
 		if (layer->meta.real.vis)
-			EraseText(layer, text);
+			pcb_text_invalidate_erase(layer, text);
 		text = pcb_textop_move_to_layer_low(ctx, layer, text, ctx->move.dst_layer);
 		if (ctx->move.dst_layer->meta.real.vis)
-			DrawText(ctx->move.dst_layer, text);
+			pcb_text_invalidate_draw(ctx->move.dst_layer, text);
 		if (layer->meta.real.vis || ctx->move.dst_layer->meta.real.vis)
 			pcb_draw();
 	}
@@ -474,7 +531,7 @@ void *pcb_textop_remove(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	/* erase from screen */
 	if (Layer->meta.real.vis) {
-		EraseText(Layer, Text);
+		pcb_text_invalidate_erase(Layer, Text);
 		pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *)Text);
 		if (!ctx->remove.bulk)
 			pcb_draw();
@@ -515,13 +572,13 @@ void pcb_text_rotate90(pcb_text_t *Text, pcb_coord_t X, pcb_coord_t Y, unsigned
 /* rotates a text object and redraws it */
 void *pcb_textop_rotate90(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
-	EraseText(Layer, Text);
+	pcb_text_invalidate_erase(Layer, Text);
 	pcb_poly_restore_to_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
 	pcb_r_delete_entry(Layer->text_tree, (pcb_box_t *) Text);
 	pcb_text_rotate90(Text, ctx->rotate.center_x, ctx->rotate.center_y, ctx->rotate.number);
 	pcb_r_insert_entry(Layer->text_tree, (pcb_box_t *) Text, 0);
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, Layer, Text);
-	DrawText(Layer, Text);
+	pcb_text_invalidate_draw(Layer, Text);
 	pcb_draw();
 	return (Text);
 }
@@ -555,16 +612,34 @@ void pcb_text_update(pcb_layer_t *layer, pcb_text_t *text)
 	pcb_poly_clear_from_poly(PCB->Data, PCB_TYPE_TEXT, layer, text);
 }
 
-#define PCB_TEXT_FLAGS (PCB_FLAG_FOUND | PCB_FLAG_CLEARLINE | PCB_FLAG_SELECTED | PCB_FLAG_AUTO | PCB_FLAG_LOCK | PCB_FLAG_VISIT)
 void *pcb_textop_change_flag(pcb_opctx_t *ctx, pcb_layer_t *Layer, pcb_text_t *Text)
 {
-	if ((ctx->chgflag.flag & PCB_TEXT_FLAGS) != ctx->chgflag.flag)
+	static pcb_flag_values_t pcb_text_flags = 0;
+	if (pcb_text_flags == 0)
+		pcb_text_flags = pcb_obj_valid_flags(PCB_TYPE_TEXT);
+
+	if ((ctx->chgflag.flag & pcb_text_flags) != ctx->chgflag.flag)
 		return NULL;
-	pcb_undo_add_obj_to_flag(PCB_TYPE_TEXT, Text, Text, Text);
+	if ((ctx->chgflag.flag & PCB_FLAG_TERMNAME) && (Text->term == NULL))
+		return NULL;
+	pcb_undo_add_obj_to_flag(Text);
+
+	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
+		pcb_poly_restore_to_poly(ctx->chgflag.pcb->Data, PCB_TYPE_TEXT, Text->parent.layer, Text);
+
 	PCB_FLAG_CHANGE(ctx->chgflag.how, ctx->chgflag.flag, Text);
+
+	if (ctx->chgflag.flag & PCB_FLAG_CLEARLINE)
+		pcb_poly_clear_from_poly(ctx->chgflag.pcb->Data, PCB_TYPE_TEXT, Text->parent.layer, Text);
+
 	return Text;
 }
 
+void *pcb_textop_invalidate_label(pcb_opctx_t *ctx, pcb_layer_t *layer, pcb_text_t *text)
+{
+	pcb_text_name_invalidate_draw(text);
+	return text;
+}
 
 /*** draw ***/
 
@@ -611,10 +686,12 @@ static void draw_text_poly(pcb_text_t *Text, pcb_polygon_t *poly, pcb_coord_t x0
 static void DrawTextLowLevel_(pcb_text_t *Text, pcb_coord_t min_line_width, int xordraw, pcb_coord_t xordx, pcb_coord_t xordy)
 {
 	pcb_coord_t x = 0;
-	unsigned char *string = (unsigned char *) Text->TextString;
+	unsigned char *string, *rendered = pcb_text_render_str(Text);
 	pcb_cardinal_t n;
 	pcb_font_t *font = pcb_font(PCB, Text->fid, 1);
 
+	string = rendered;
+
 	while (string && *string) {
 		/* draw lines if symbol is valid and data is present */
 		if (*string <= PCB_MAX_FONTPOSITION && font->Symbol[*string].Valid) {
@@ -653,7 +730,7 @@ static void DrawTextLowLevel_(pcb_text_t *Text, pcb_coord_t min_line_width, int
 				if (xordraw)
 					pcb_gui->draw_line(pcb_crosshair.GC, xordx + newline.Point1.X, xordy + newline.Point1.Y, xordx + newline.Point2.X, xordy + newline.Point2.Y);
 				else
-					_draw_line(&newline);
+					pcb_line_draw_(&newline, 0);
 			}
 
 			/* draw the arcs */
@@ -679,7 +756,7 @@ static void DrawTextLowLevel_(pcb_text_t *Text, pcb_coord_t min_line_width, int
 				if (xordraw)
 					pcb_gui->draw_arc(pcb_crosshair.GC, xordx + newarc.X, xordy + newarc.Y, newarc.Width, newarc.Height, newarc.StartAngle, newarc.Delta);
 				else
-					_draw_arc(&newarc);
+					pcb_arc_draw_(&newarc, 0);
 			}
 
 			/* draw the polygons */
@@ -714,24 +791,58 @@ static void DrawTextLowLevel_(pcb_text_t *Text, pcb_coord_t min_line_width, int
 		}
 		string++;
 	}
+	pcb_text_free_str(Text, rendered);
 }
 
-void DrawTextLowLevel(pcb_text_t *Text, pcb_coord_t min_line_width)
+static pcb_bool is_text_term_vert(const pcb_text_t *text)
 {
-	DrawTextLowLevel_(Text, min_line_width, 0, 0, 0);
+	pcb_coord_t dx, dy;
+
+	dx = text->BoundingBox.X2 - text->BoundingBox.X1;
+	if (dx < 0)
+		dx = -dx;
+
+	dy = text->BoundingBox.Y2 - text->BoundingBox.Y1;
+	if (dy < 0)
+		dy = -dy;
+
+	return dx < dy;
 }
 
 
-pcb_r_dir_t draw_text_callback(const pcb_box_t * b, void *cl)
+void pcb_text_name_invalidate_draw(pcb_text_t *txt)
+{
+	if (txt->term != NULL) {
+		pcb_text_t text;
+		pcb_term_label_setup(&text, (txt->BoundingBox.X1 + txt->BoundingBox.X2)/2, (txt->BoundingBox.Y1 + txt->BoundingBox.Y2)/2,
+			100.0, is_text_term_vert(txt), pcb_true, txt->term, txt->intconn);
+		pcb_draw_invalidate(&text);
+	}
+}
+
+void pcb_text_draw_label(pcb_text_t *text)
+{
+	if (text->term != NULL)
+		pcb_term_label_draw((text->BoundingBox.X1 + text->BoundingBox.X2)/2, (text->BoundingBox.Y1 + text->BoundingBox.Y2)/2,
+			100.0, is_text_term_vert(text), pcb_true, text->term, text->intconn);
+}
+
+
+void pcb_text_draw_(pcb_text_t *text, pcb_coord_t min_line_width, int allow_term_gfx)
+{
+	DrawTextLowLevel_(text, min_line_width, 0, 0, 0);
+
+	if (text->term != NULL) {
+		if ((allow_term_gfx) && ((pcb_draw_doing_pinout) || PCB_FLAG_TEST(PCB_FLAG_TERMNAME, text)))
+			pcb_draw_delay_label_add((pcb_any_obj_t *)text);
+	}
+}
+
+static void pcb_text_draw(pcb_layer_t *layer, pcb_text_t *text, int allow_term_gfx)
 {
-	pcb_layer_t *layer = cl;
-	pcb_text_t *text = (pcb_text_t *) b;
 	int min_silk_line;
 	unsigned int flg = 0;
 
-	if (!PCB->SubcPartsOn && pcb_is_lobj_in_subc(text->parent_type, &text->parent))
-		return PCB_R_DIR_FOUND_CONTINUE;
-
 	if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, text))
 		pcb_gui->set_color(Output.fgGC, layer->meta.real.selected_color);
 	else
@@ -744,22 +855,46 @@ pcb_r_dir_t draw_text_callback(const pcb_box_t * b, void *cl)
 		min_silk_line = PCB->minSlk;
 	else
 		min_silk_line = PCB->minWid;
-	DrawTextLowLevel(text, min_silk_line);
+
+	pcb_text_draw_(text, min_silk_line, allow_term_gfx);
+}
+
+pcb_r_dir_t pcb_text_draw_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_layer_t *layer = cl;
+	pcb_text_t *text = (pcb_text_t *) b;
+
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(text->parent_type, &text->parent))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
+	pcb_text_draw(layer, text, 0);
+	return PCB_R_DIR_FOUND_CONTINUE;
+}
+
+pcb_r_dir_t pcb_text_draw_term_callback(const pcb_box_t * b, void *cl)
+{
+	pcb_layer_t *layer = cl;
+	pcb_text_t *text = (pcb_text_t *) b;
+
+	if (!PCB->SubcPartsOn && pcb_lobj_parent_subc(text->parent_type, &text->parent))
+		return PCB_R_DIR_FOUND_CONTINUE;
+
+	pcb_text_draw(layer, text, 1);
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
 /* erases a text on a layer */
-void EraseText(pcb_layer_t *Layer, pcb_text_t *Text)
+void pcb_text_invalidate_erase(pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	pcb_draw_invalidate(Text);
 }
 
-void DrawText(pcb_layer_t *Layer, pcb_text_t *Text)
+void pcb_text_invalidate_draw(pcb_layer_t *Layer, pcb_text_t *Text)
 {
 	pcb_draw_invalidate(Text);
 }
 
-void XORDrawText(pcb_text_t *text, pcb_coord_t x, pcb_coord_t y)
+void pcb_text_draw_xor(pcb_text_t *text, pcb_coord_t x, pcb_coord_t y)
 {
 	DrawTextLowLevel_(text, 0, 1, x, y);
 }
