@@ -66,6 +66,7 @@ static pcb_element_t *find_element_by_refdes(const char *refdes)
 	return NULL;
 }
 
+#warning subc TODO: remove this when removing element support
 static pcb_attribute_t *lookup_attr(pcb_attribute_list_t *list, const char *name)
 {
 	int i;
@@ -75,6 +76,7 @@ static pcb_attribute_t *lookup_attr(pcb_attribute_list_t *list, const char *name
 	return NULL;
 }
 
+#warning subc TODO: remove this when removing element support
 static void delete_attr(pcb_attribute_list_t *list, pcb_attribute_t * attr)
 {
 	int idx = attr - list->List;
@@ -87,10 +89,10 @@ static void delete_attr(pcb_attribute_list_t *list, pcb_attribute_t * attr)
 
 /* ------------------------------------------------------------ */
 
-static const char pcb_acts_Attributes[] = "Attributes(Layout|Layer|Element)\n" "Attributes(Layer,layername)";
+static const char pcb_acts_Attributes[] = "Attributes(Layout|Layer|Element|Subc)\n" "Attributes(Layer,layername)";
 
 static const char pcb_acth_Attributes[] =
-	"Let the user edit the attributes of the layout, current or given\n" "layer, or selected element.";
+	"Let the user edit the attributes of the layout, current or given\n" "layer, or selected element or subcircuit.";
 
 /* %start-doc actions Attributes
 
@@ -137,14 +139,10 @@ static int pcb_act_Attributes(int argc, const char **argv, pcb_coord_t x, pcb_co
 					return 1;
 				}
 			}
-			buf = (char *) malloc(strlen(layer->meta.real.name) + strlen("Layer X Attributes"));
-			sprintf(buf, "Layer %s Attributes", layer->meta.real.name);
-			pcb_gui->edit_attributes(buf, &(layer->meta.real.Attributes));
-			free(buf);
+			pcb_layer_edit_attrib(layer);
 			return 0;
 		}
 
-#warning subc TODO
 	case F_Element:
 		{
 			int n_found = 0;
@@ -184,6 +182,43 @@ static int pcb_act_Attributes(int argc, const char **argv, pcb_coord_t x, pcb_co
 			break;
 		}
 
+	case F_Subc:
+		{
+			int n_found = 0;
+			pcb_subc_t *s = NULL;
+			PCB_SUBC_LOOP(PCB->Data);
+			{
+				if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc)) {
+					s = subc;
+					n_found++;
+				}
+			}
+			PCB_END_LOOP;
+			if (n_found > 1) {
+				pcb_message(PCB_MSG_ERROR, _("Too many subcircuits selected\n"));
+				return 1;
+			}
+			if (n_found == 0) {
+				void *ptrtmp;
+				pcb_gui->get_coords(_("Click on a subcircuit"), &x, &y);
+				if ((pcb_search_screen(x, y, PCB_TYPE_SUBC, &ptrtmp, &ptrtmp, &ptrtmp)) != PCB_TYPE_NONE)
+					s = (pcb_subc_t *)ptrtmp;
+				else {
+					pcb_message(PCB_MSG_ERROR, _("No subcricuit found there\n"));
+					return 1;
+				}
+			}
+
+			if (s->refdes != NULL)
+				buf = pcb_strdup_printf("Subcircuit %s Attributes", s->refdes);
+			else
+				buf = pcb_strdup("Unnamed Subcircuit's Attributes");
+
+			pcb_gui->edit_attributes(buf, &(s->Attributes));
+			free(buf);
+			break;
+		}
+
 	default:
 		PCB_ACT_FAIL(Attributes);
 	}
@@ -656,6 +691,7 @@ static int pcb_act_ElementList(int argc, const char **argv, pcb_coord_t x, pcb_c
 }
 
 /* ---------------------------------------------------------------- */
+#warning subc TODO: remove this when removing element support
 static const char pcb_acts_ElementSetAttr[] = "ElementSetAttr(refdes,name[,value])";
 
 static const char pcb_acth_ElementSetAttr[] = "Sets or clears an element-specific attribute.";
@@ -706,7 +742,7 @@ static int pcb_act_ElementSetAttr(int argc, const char **argv, pcb_coord_t x, pc
 		delete_attr(&e->Attributes, attr);
 	}
 	if (!attr && value) {
-		pcb_attribute_put(&e->Attributes, name, value, 0);
+		pcb_attribute_put(&e->Attributes, name, value);
 	}
 
 	return 0;
@@ -977,7 +1013,7 @@ static int pcb_act_MinClearGap(int argc, const char **argv, pcb_coord_t x, pcb_c
 	{
 		if (!PCB_FLAGS_TEST(flags, line))
 			continue;
-		if (line->Clearance < value) {
+		if ((line->Clearance != 0) && (line->Clearance < value)) {
 			pcb_chg_obj_clear_size(PCB_TYPE_LINE, layer, line, 0, value, 1);
 			pcb_undo_restore_serial();
 		}
@@ -987,7 +1023,7 @@ static int pcb_act_MinClearGap(int argc, const char **argv, pcb_coord_t x, pcb_c
 	{
 		if (!PCB_FLAGS_TEST(flags, arc))
 			continue;
-		if (arc->Clearance < value) {
+		if ((arc->Clearance != 0) && (arc->Clearance < value)) {
 			pcb_chg_obj_clear_size(PCB_TYPE_ARC, layer, arc, 0, value, 1);
 			pcb_undo_restore_serial();
 		}
@@ -1007,6 +1043,7 @@ int pcb_act_ListRotations(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	}
 	PCB_END_LOOP;
 #warning subc TODO
+#warning cleanup TODO: move this to oldactions maybe?
 	return 0;
 }
 
