@@ -34,10 +34,6 @@
 #warning TODO: do not hardwire this, make a function to decide
 #define DEFAULT_FMT "pcb"
 
-/* for popen() */
-#define _DEFAULT_SOURCE
-#define _BSD_SOURCE
-
 #include "config.h"
 #include "conf_core.h"
 
@@ -48,6 +44,7 @@
 #include <string.h>
 
 #include "change.h"
+#include "conf.h"
 #include "data.h"
 #include "error.h"
 #include "plug_io.h"
@@ -63,7 +60,9 @@
 #include "route_style.h"
 #include "compat_fs.h"
 #include "compat_nls.h"
+#include "compat_lrealpath.h"
 #include "layer_vis.h"
+#include "safe_fs.h"
 
 /* for opendir */
 #include "compat_inc.h"
@@ -85,7 +84,7 @@ static void plug_io_err(int res, const char *what, const char *filename)
 			else {
 				FILE *f;
 				reason = "none of io plugins could successfully read file";
-				f = fopen(filename, "r");
+				f = pcb_fopen(filename, "r");
 				if (f != NULL) {
 					fclose(f);
 					comment = "(unknown/invalid file format?)";
@@ -103,6 +102,30 @@ static void plug_io_err(int res, const char *what, const char *filename)
 	}
 }
 
+void pcb_set_design_dir(const char *fn)
+{
+	char *end, *rlp = NULL;
+
+	if (fn != NULL)
+		rlp = pcb_lrealpath(fn);
+
+	if (conf_core.rc.path.design != NULL)
+		free((char *)(*((CFT_STRING *)(&conf_core.rc.path.design))));
+
+	if (rlp == NULL) {
+		conf_force_set_str(conf_core.rc.path.design, pcb_strdup("<invalid>"));
+		pcb_conf_ro("rc/path/design");
+		return;
+	}
+
+	end = strrchr(rlp, PCB_DIR_SEPARATOR_C);
+	if (end != NULL)
+		*end = '\0';
+
+	conf_force_set_str(conf_core.rc.path.design, rlp);
+	pcb_conf_ro("rc/path/design");
+}
+
 int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int load_settings, int ignore_missing)
 {
 	int res = -1, len, n;
@@ -111,7 +134,7 @@ int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int l
 	FILE *ft;
 	int accept_total = 0;
 
-	ft = fopen(Filename, "r");
+	ft = pcb_fopen(Filename, "r");
 	if (ft == NULL) {
 		if (!ignore_missing)
 			pcb_message(PCB_MSG_ERROR, "Error: can't open %s for reading (format is %s)\n", Filename, fmt);
@@ -179,6 +202,9 @@ int pcb_parse_pcb(pcb_board_t *Ptr, const char *Filename, const char *fmt, int l
 	if ((res == 0) && (load_settings))
 		conf_load_project(NULL, Filename);
 
+	if (res == 0)
+		pcb_set_design_dir(Filename);
+
 	if (load_settings)
 		pcb_event(PCB_EVENT_LOAD_POST, "si", Filename, res);
 
@@ -282,12 +308,12 @@ static pcb_plug_io_t *find_writer(pcb_plug_iot_t typ, const char *fmt)
 
 int pcb_write_buffer(FILE *f, pcb_buffer_t *buff, const char *fmt, pcb_bool elem_only)
 {
-	int res, newfmt = 0;
+	int res/*, newfmt = 0*/;
 	pcb_plug_io_t *p = find_writer(PCB_IOT_BUFFER, fmt);
 
 	if (p != NULL) {
 		res = p->write_buffer(p, f, buff, elem_only);
-		newfmt = 1;
+		/*newfmt = 1;*/
 	}
 
 /*	if ((res == 0) && (newfmt))
@@ -307,7 +333,7 @@ int pcb_write_element_data(FILE *f, pcb_data_t *e, const char *fmt)
 		newfmt = 1;
 	}
 
-	if (p != NULL)
+	if ((p != NULL) && (p->write_element != NULL))
 		res = p->write_element(p, f, e);
 
 	if ((res == 0) && (newfmt))
@@ -319,12 +345,12 @@ int pcb_write_element_data(FILE *f, pcb_data_t *e, const char *fmt)
 
 int pcb_write_font(pcb_font_t *Ptr, const char *Filename, const char *fmt)
 {
-	int res, newfmt = 0;
+	int res/*, newfmt = 0*/;
 	pcb_plug_io_t *p = find_writer(PCB_IOT_FONT, fmt);
 
 	if (p != NULL) {
 		res = p->write_font(p, Ptr, Filename);
-		newfmt = 1;
+		/*newfmt = 1;*/
 	}
 	else
 		res = -1;
@@ -362,6 +388,9 @@ static int pcb_write_pcb(FILE *f, const char *old_filename, const char *new_file
 	if ((res == 0) && (newfmt))
 		PCB->Data->loader = p;
 
+	if (res == 0)
+		pcb_set_design_dir(new_filename);
+
 	plug_io_err(res, "write pcb", NULL);
 	return res;
 }
@@ -591,7 +620,7 @@ FILE *pcb_check_and_open_file(const char *Filename, pcb_bool Confirm, pcb_bool A
 					*WasCancelButton = pcb_true;
 			}
 		}
-		if ((fp = fopen(Filename, "w")) == NULL)
+		if ((fp = pcb_fopen(Filename, "w")) == NULL)
 			pcb_open_error_message(Filename);
 	}
 	return (fp);
@@ -779,7 +808,8 @@ void pcb_enable_autosave(void)
  */
 void pcb_backup(void)
 {
-	char *filename = NULL, *fmt = NULL;
+	char *filename = NULL;
+	const char *fmt = NULL;
 
 	if (PCB && PCB->Filename) {
 		filename = (char *) malloc(sizeof(char) * (strlen(PCB->Filename) + 2));
@@ -856,7 +886,7 @@ int pcb_write_pcb_file(const char *Filename, pcb_bool thePcb, const char *fmt, p
 		fn_tmp = malloc(len+8);
 		memcpy(fn_tmp, Filename, len);
 		strcpy(fn_tmp+len, ".old");
-		if (rename(Filename, fn_tmp) != 0) {
+		if (pcb_rename(Filename, fn_tmp) != 0) {
 			if (emergency) {
 				/* Try an alternative emergency file */
 				strcpy(fn_tmp+len, ".emr");
@@ -869,7 +899,7 @@ int pcb_write_pcb_file(const char *Filename, pcb_bool thePcb, const char *fmt, p
 		}
 	}
 
-	if ((fp = fopen(Filename, "w")) == NULL) {
+	if ((fp = pcb_fopen(Filename, "w")) == NULL) {
 		pcb_open_error_message(Filename);
 		return (-1);
 	}
@@ -896,13 +926,17 @@ int pcb_write_pipe(const char *Filename, pcb_bool thePcb, const char *fmt, pcb_b
 	int result;
 	const char *p;
 	static gds_t command;
+	const char *save_cmd;
 
 	if (PCB_EMPTY_STRING_P(conf_core.rc.save_command))
 		return pcb_write_pcb_file(Filename, thePcb, fmt, pcb_false, elem_only);
 
+	if (!pcb_conf_cmd_is_safe(rc.save_command, &save_cmd, 1))
+		return -1;
+
 	/* setup commandline */
 	gds_truncate(&command,0);
-	for (p = conf_core.rc.save_command; *p; p++) {
+	for (p = save_cmd; *p; p++) {
 		/* copy character if not special or add string to command */
 		if (!(p[0] == '%' && p[1] == 'f'))
 			gds_append(&command, *p);
@@ -914,14 +948,14 @@ int pcb_write_pipe(const char *Filename, pcb_bool thePcb, const char *fmt, pcb_b
 		}
 	}
 	printf("write to pipe \"%s\"\n", command.array);
-	if ((fp = popen(command.array, "w")) == NULL) {
+	if ((fp = pcb_popen(command.array, "w")) == NULL) {
 		pcb_popen_error_message(command.array);
 		return (-1);
 	}
 
 	result = pcb_write_file(fp, thePcb, NULL, NULL, fmt, pcb_false, elem_only);
 
-	return (pclose(fp) ? (-1) : result);
+	return (pcb_pclose(fp) ? (-1) : result);
 }
 
 
