@@ -56,8 +56,12 @@
 #define SUBTRACT_PIN_VIA_BATCH_SIZE 100
 #define SUBTRACT_LINE_BATCH_SIZE 20
 
+#define sqr(x) ((x)*(x))
+
 static double rotate_circle_seg[4];
 
+static int Unsubtract(pcb_polyarea_t * np1, pcb_polygon_t * p);
+
 void pcb_polygon_init(void)
 {
 	double cos_ang = cos(2.0 * M_PI / PCB_POLY_CIRC_SEGS_F);
@@ -829,7 +833,7 @@ static int SubtractLine(pcb_line_t * line, pcb_polygon_t * p)
 {
 	pcb_polyarea_t *np;
 
-	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, line))
+	if (!PCB_NONPOLY_HAS_CLEARANCE(line))
 		return 0;
 	if (!(np = pcb_poly_from_line(line, line->Thickness + line->Clearance)))
 		return -1;
@@ -840,7 +844,7 @@ static int SubtractArc(pcb_arc_t * arc, pcb_polygon_t * p)
 {
 	pcb_polyarea_t *np;
 
-	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, arc))
+	if (!PCB_NONPOLY_HAS_CLEARANCE(arc))
 		return 0;
 	if (!(np = pcb_poly_from_arc(arc, arc->Thickness + arc->Clearance)))
 		return -1;
@@ -944,7 +948,7 @@ static pcb_r_dir_t arc_sub_callback(const pcb_box_t * b, void *cl)
 	/* don't subtract the object that was put back! */
 	if (b == info->other)
 		return PCB_R_DIR_NOT_FOUND;
-	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, arc))
+	if (!PCB_NONPOLY_HAS_CLEARANCE(arc))
 		return PCB_R_DIR_NOT_FOUND;
 	polygon = info->polygon;
 	if (SubtractArc(arc, polygon) < 0)
@@ -952,6 +956,141 @@ static pcb_r_dir_t arc_sub_callback(const pcb_box_t * b, void *cl)
 	return PCB_R_DIR_FOUND_CONTINUE;
 }
 
+/* quick create a polygon area from a line, knowing the coords and width */
+static pcb_polyarea_t *poly_sub_callback_line(pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2, pcb_coord_t width)
+{
+	static pcb_line_t lin;
+	static int inited = 0;
+
+	if (!inited) {
+		lin.type = PCB_TYPE_LINE;
+		lin.Flags = pcb_no_flags();
+		PCB_FLAG_SET(PCB_FLAG_CLEARLINE, &lin);
+		lin.Thickness = 0;
+		inited = 1;
+	}
+
+	lin.Point1.X = x1;
+	lin.Point1.Y = y1;
+	lin.Point2.X = x2;
+	lin.Point2.Y = y2;
+	lin.Clearance = width;
+
+	return pcb_poly_from_line(&lin, width);
+}
+
+#define pa_append(src) \
+	do { \
+		pcb_polyarea_boolean(ret, src, &tmp, PCB_PBO_UNITE); \
+		pcb_polyarea_free(&ret); \
+		ret = tmp; \
+	} while(0)
+
+
+/* Construct a poly area that represents the enlarged subpoly - so it can be
+   subtracted from the parent poly to form the clearance for subpoly */
+static pcb_polyarea_t *SubtractPolyPoly_construct(pcb_polygon_t *subpoly)
+{
+	pcb_poly_it_t it;
+	pcb_polyarea_t *ret = NULL, *pa, *lin, *tmp;
+	
+	/* iterate over all islands of a polygon */
+	for(pa = pcb_poly_island_first(subpoly, &it); pa != NULL; pa = pcb_poly_island_next(&it)) {
+		pcb_coord_t x, y, px, py, x0, y0;
+		pcb_pline_t *pl;
+		int go;
+		pcb_cardinal_t cnt;
+
+		if (ret != NULL)
+			pa_append(tmp);
+		else
+			pcb_polyarea_copy0(&ret, pa);
+
+		/* care about the outer contours only */
+		pl = pcb_poly_contour(&it);
+		if (pl != NULL) {
+			/* iterate over the vectors of the contour */
+			for(go = pcb_poly_vect_first(&it, &x, &y), cnt = 0; go; go = pcb_poly_vect_next(&it, &x, &y), cnt++) {
+				if (cnt > 0) {
+					lin = poly_sub_callback_line(px, py, x, y, subpoly->Clearance);
+					pa_append(lin);
+					pcb_polyarea_free(&lin);
+				}
+				else {
+					x0 = x;
+					y0 = y;
+				}
+				px = x;
+				py = y;
+			}
+			if (cnt > 0) {
+				lin = poly_sub_callback_line(px, py, x0, y0, subpoly->Clearance);
+				pa_append(lin);
+				pcb_polyarea_free(&lin);
+			}
+		}
+	}
+	return ret;
+}
+
+static int SubtractPolyPoly(pcb_polygon_t *subpoly, pcb_polygon_t *frompoly)
+{
+	pcb_polyarea_t *pa;
+
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, frompoly)) /* two clearing polys won't interact */
+		return -1;
+
+	pa = SubtractPolyPoly_construct(subpoly);
+	if (pa == NULL)
+		return -1;
+
+	Subtract(pa, frompoly, pcb_true);
+	frompoly->NoHolesValid = 0;
+	return 0;
+}
+
+
+static int UnsubtractPolyPoly(pcb_polygon_t *subpoly, pcb_polygon_t *frompoly)
+{
+	pcb_polyarea_t *pa;
+
+	if (PCB_FLAG_TEST(PCB_FLAG_CLEARPOLYPOLY, frompoly)) /* two clearing polys won't interact */
+		return -1;
+
+	pa = SubtractPolyPoly_construct(subpoly);
+	if (pa == NULL)
+		return -1;
+
+	if (!Unsubtract(pa, frompoly))
+		return -1;
+	frompoly->NoHolesValid = 0;
+	return 0;
+}
+
+static pcb_r_dir_t poly_sub_callback(const pcb_box_t *b, void *cl)
+{
+	pcb_polygon_t *subpoly = (pcb_polygon_t *)b;
+	struct cpInfo *info = (struct cpInfo *) cl;
+	pcb_polygon_t *polygon;
+
+	polygon = info->polygon;
+
+	/* don't do clearance in itself */
+	if (subpoly == polygon)
+		return PCB_R_DIR_NOT_FOUND;
+
+	/* don't subtract the object that was put back! */
+	if (b == info->other)
+		return PCB_R_DIR_NOT_FOUND;
+	if (!PCB_POLY_HAS_CLEARANCE(subpoly))
+		return PCB_R_DIR_NOT_FOUND;
+
+	if (SubtractPolyPoly(subpoly, polygon) < 0)
+		longjmp(info->env, 1);
+
+	return PCB_R_DIR_FOUND_CONTINUE;
+}
+
 static pcb_r_dir_t pad_sub_callback(const pcb_box_t * b, void *cl)
 {
 	pcb_pad_t *pad = (pcb_pad_t *) b;
@@ -983,7 +1122,7 @@ static pcb_r_dir_t line_sub_callback(const pcb_box_t * b, void *cl)
 	/* don't subtract the object that was put back! */
 	if (b == info->other)
 		return PCB_R_DIR_NOT_FOUND;
-	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, line))
+	if (!PCB_NONPOLY_HAS_CLEARANCE(line))
 		return PCB_R_DIR_NOT_FOUND;
 	polygon = info->polygon;
 
@@ -1024,8 +1163,15 @@ static int clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_polygon_t * polyg
 	struct cpInfo info;
 	pcb_layergrp_id_t group;
 	unsigned int gflg;
+	pcb_layer_type_t lf;
 
-	if (!(pcb_layer_flags_(PCB, Layer) & PCB_LYT_COPPER)) {
+#warning layer TODO#5: until we have an explicit flag for bound layers, pcb_layer_flags_() makes a mistake
+	if (Data == PCB->Data)
+		lf = pcb_layer_flags_(PCB, Layer);
+	else
+		lf = Layer->meta.bound.type;
+
+	if (!(lf & PCB_LYT_COPPER)) {
 		polygon->NoHolesValid = 0;
 		return 0;
 	}
@@ -1064,6 +1210,8 @@ static int clearPoly(pcb_data_t *Data, pcb_layer_t *Layer, pcb_polygon_t * polyg
 			r += seen;
 			pcb_r_search(layer->text_tree, &region, NULL, text_sub_callback, &info, &seen);
 			r += seen;
+			pcb_r_search(layer->polygon_tree, &region, NULL, poly_sub_callback, &info, &seen);
+			r += seen;
 		}
 		PCB_END_LOOP;
 		pcb_r_search(Data->via_tree, &region, NULL, pin_sub_callback, &info, &seen);
@@ -1131,7 +1279,7 @@ static int UnsubtractArc(pcb_arc_t * arc, pcb_layer_t * l, pcb_polygon_t * p)
 {
 	pcb_polyarea_t *np;
 
-	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, arc))
+	if (!PCB_NONPOLY_HAS_CLEARANCE(arc))
 		return 0;
 
 	/* overlap a bit to prevent gaps from rounding errors */
@@ -1149,7 +1297,7 @@ static int UnsubtractLine(pcb_line_t * line, pcb_layer_t * l, pcb_polygon_t * p)
 {
 	pcb_polyarea_t *np;
 
-	if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, line))
+	if (!PCB_NONPOLY_HAS_CLEARANCE(line))
 		return 0;
 
 	/* overlap a bit to prevent notches from rounding errors */
@@ -1382,19 +1530,23 @@ void pcb_polygon_copy_attached_to_layer(void)
 	int saveID;
 
 	/* move data to layer and clear attached struct */
-	polygon = pcb_poly_new(CURRENT, pcb_no_flags());
+	polygon = pcb_poly_new(CURRENT, 0, pcb_no_flags());
 	saveID = polygon->ID;
 	poly_copy_data(polygon, &pcb_crosshair.AttachedPolygon);
+	polygon->Clearance = 2 * conf_core.design.clearance;
 	polygon->ID = saveID;
 	PCB_FLAG_SET(PCB_FLAG_CLEARPOLY, polygon);
 	if (conf_core.editor.full_poly)
 		PCB_FLAG_SET(PCB_FLAG_FULLPOLY, polygon);
+	if (conf_core.editor.clear_polypoly)
+		PCB_FLAG_SET(PCB_FLAG_CLEARPOLYPOLY, polygon);
+
 	memset(&pcb_crosshair.AttachedPolygon, 0, sizeof(pcb_polygon_t));
 
 	pcb_add_polygon_on_layer(CURRENT, polygon);
 
 	pcb_poly_init_clip(PCB->Data, CURRENT, polygon);
-	DrawPolygon(CURRENT, polygon);
+	pcb_poly_invalidate_draw(CURRENT, polygon);
 	pcb_board_set_changed_flag(pcb_true);
 
 	/* reset state of attached line */
@@ -1519,6 +1671,13 @@ static pcb_r_dir_t subtract_plow(pcb_data_t *Data, pcb_layer_t *Layer, pcb_polyg
 		SubtractArc((pcb_arc_t *) ptr2, Polygon);
 		Polygon->NoHolesValid = 0;
 		return PCB_R_DIR_FOUND_CONTINUE;
+	case PCB_TYPE_POLYGON:
+		if (ptr2 != Polygon) {
+			SubtractPolyPoly((pcb_polygon_t *) ptr2, Polygon);
+			Polygon->NoHolesValid = 0;
+			return PCB_R_DIR_FOUND_CONTINUE;
+		}
+		break;
 	case PCB_TYPE_PAD:
 		SubtractPad((pcb_pad_t *) ptr2, Polygon);
 		Polygon->NoHolesValid = 0;
@@ -1544,6 +1703,12 @@ static pcb_r_dir_t add_plow(pcb_data_t *Data, pcb_layer_t *Layer, pcb_polygon_t
 	case PCB_TYPE_ARC:
 		UnsubtractArc((pcb_arc_t *) ptr2, Layer, Polygon);
 		return PCB_R_DIR_FOUND_CONTINUE;
+	case PCB_TYPE_POLYGON:
+		if (ptr2 != Polygon) {
+			UnsubtractPolyPoly((pcb_polygon_t *) ptr2, Polygon);
+			return PCB_R_DIR_FOUND_CONTINUE;
+		}
+		break;
 	case PCB_TYPE_PAD:
 		UnsubtractPad((pcb_pad_t *) ptr2, Layer, Polygon);
 		return PCB_R_DIR_FOUND_CONTINUE;
@@ -1618,16 +1783,27 @@ pcb_poly_plows(pcb_data_t * Data, int type, void *ptr1, void *ptr2,
 			PCB_END_LOOP;
 		}
 		break;
+	case PCB_TYPE_POLYGON:
+		if (!PCB_POLY_HAS_CLEARANCE((pcb_polygon_t *) ptr2))
+			return 0;
+		goto doit;
+
 	case PCB_TYPE_LINE:
 	case PCB_TYPE_ARC:
-	case PCB_TYPE_TEXT:
 		/* the cast works equally well for lines and arcs */
+		if (!PCB_NONPOLY_HAS_CLEARANCE((pcb_line_t *) ptr2))
+			return 0;
+		goto doit;
+
+	case PCB_TYPE_TEXT:
+		/* text has no clearance property */
 		if (!PCB_FLAG_TEST(PCB_FLAG_CLEARLINE, (pcb_line_t *) ptr2))
 			return 0;
 		/* non-copper (e.g. silk, outline) doesn't plow */
 #warning TODO: use pcb_layer_flags_ here - but what PCB?
 		if (!(pcb_layer_flags(PCB, pcb_layer_id(Data, (pcb_layer_t *) ptr1) & PCB_LYT_COPPER)))
 			return 0;
+		doit:;
 		PCB_COPPER_GROUP_LOOP(Data, pcb_layer_get_group(PCB, pcb_layer_id(Data, ((pcb_layer_t *) ptr1))));
 		{
 			info.layer = layer;
@@ -1676,16 +1852,14 @@ void pcb_poly_restore_to_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr
 {
 	if (type == PCB_TYPE_POLYGON)
 		pcb_poly_init_clip(PCB->Data, (pcb_layer_t *) ptr1, (pcb_polygon_t *) ptr2);
-	else
-		pcb_poly_plows(Data, type, ptr1, ptr2, add_plow);
+	pcb_poly_plows(Data, type, ptr1, ptr2, add_plow);
 }
 
 void pcb_poly_clear_from_poly(pcb_data_t * Data, int type, void *ptr1, void *ptr2)
 {
 	if (type == PCB_TYPE_POLYGON)
-		pcb_poly_init_clip(PCB->Data, (pcb_layer_t *) ptr1, (pcb_polygon_t *) ptr2);
-	else
-		pcb_poly_plows(Data, type, ptr1, ptr2, subtract_plow);
+		pcb_poly_init_clip(Data, (pcb_layer_t *) ptr1, (pcb_polygon_t *) ptr2);
+	pcb_poly_plows(Data, type, ptr1, ptr2, subtract_plow);
 }
 
 pcb_bool pcb_poly_isects_poly(pcb_polyarea_t * a, pcb_polygon_t *p, pcb_bool fr)
@@ -1827,7 +2001,7 @@ pcb_bool pcb_poly_morph(pcb_layer_t *layer, pcb_polygon_t *poly)
 		return pcb_false;
 	if (poly->Clipped->f == poly->Clipped)
 		return pcb_false;
-	ErasePolygon(poly);
+	pcb_poly_invalidate_erase(poly);
 	start = p = poly->Clipped;
 	/* This is ugly. The creation of the new polygons can cause
 	 * all of the polygon pointers (including the one we're called
@@ -1849,7 +2023,7 @@ pcb_bool pcb_poly_morph(pcb_layer_t *layer, pcb_polygon_t *poly)
 		pcb_polygon_t *newone;
 
 		if (p->contours->area > PCB->IsleArea) {
-			newone = pcb_poly_new(layer, flags);
+			newone = pcb_poly_new(layer, poly->Clearance, flags);
 			if (!newone)
 				return pcb_false;
 			many = pcb_true;
@@ -1866,7 +2040,7 @@ pcb_bool pcb_poly_morph(pcb_layer_t *layer, pcb_polygon_t *poly)
 			p = p->f;									/* go to next pline */
 			newone->Clipped->b = newone->Clipped->f = newone->Clipped;	/* unlink from others */
 			pcb_r_insert_entry(layer->polygon_tree, (pcb_box_t *) newone, 0);
-			DrawPolygon(layer, newone);
+			pcb_poly_invalidate_draw(layer, newone);
 		}
 		else {
 			pcb_polyarea_t *t = p;
@@ -1943,7 +2117,7 @@ void pcb_poly_to_polygons_on_layer(pcb_data_t * Destination, pcb_layer_t * Layer
 
 	pa = Input;
 	do {
-		Polygon = pcb_poly_new(Layer, Flags);
+		Polygon = pcb_poly_new(Layer, 2 * conf_core.design.clearance, Flags);
 
 		pline = pa->contours;
 		outer = pcb_true;
@@ -1968,7 +2142,7 @@ void pcb_poly_to_polygons_on_layer(pcb_data_t * Destination, pcb_layer_t * Layer
 			Layer->polygon_tree = pcb_r_create_tree(NULL, 0, 0);
 		pcb_r_insert_entry(Layer->polygon_tree, (pcb_box_t *) Polygon, 0);
 
-		DrawPolygon(Layer, Polygon);
+		pcb_poly_invalidate_draw(Layer, Polygon);
 		/* add to undo list */
 		pcb_undo_add_obj_to_create(PCB_TYPE_POLYGON, Layer, Polygon, Polygon);
 	}
@@ -2000,3 +2174,5 @@ pcb_bool pcb_pline_is_rectangle(pcb_pline_t *pl)
 
 	return pcb_false;
 }
+
+#include "polygon_offs.c"
