@@ -47,6 +47,8 @@
 #include "compat_nls.h"
 #include "vtptr.h"
 #include "obj_rat_draw.h"
+#include "obj_term.h"
+#include "obj_subc_parent.h"
 
 #define STEP_POINT 100
 
@@ -56,7 +58,7 @@
 /* ---------------------------------------------------------------------------
  * some forward declarations
  */
-static pcb_bool FindPad(const char *, const char *, pcb_connection_t *, pcb_bool);
+static pcb_bool pcb_term_find_name_ppt(const char *, const char *, pcb_connection_t *, pcb_bool);
 static pcb_bool ParseConnection(const char *, char *, char *);
 static pcb_bool DrawShortestRats(pcb_netlist_t *,
 														 void (*)(register pcb_connection_t *, register pcb_connection_t *, register pcb_route_style_t *));
@@ -100,23 +102,33 @@ static pcb_bool ParseConnection(const char *InString, char *ElementName, char *P
 }
 
 /* ---------------------------------------------------------------------------
- * Find a particular pad from an element name and pin number
+ * Find a particular terminal from an element/subc name and pin/pad/terminal number
  */
-static pcb_bool FindPad(const char *ElementName, const char *PinNum, pcb_connection_t * conn, pcb_bool Same)
+#warning term TODO: once pins and pads are gone, move this to obj_term.c
+static pcb_bool pcb_term_find_name_ppt(const char *ElementName, const char *PinNum, pcb_connection_t * conn, pcb_bool Same)
 {
 	pcb_element_t *element;
 	gdl_iterator_t it;
 	pcb_pad_t *pad;
 	pcb_pin_t *pin;
+	pcb_any_obj_t *obj;
+
+	/* first check for subcircuits; this is the only one thing we'll need to do
+	   once elements are removed */
+	obj = pcb_term_find_name(PCB, PCB->Data, PCB_LYT_COPPER, ElementName, PinNum, Same, (pcb_subc_t **)&conn->ptr1, &conn->group);
+	if (obj != NULL) {
+		conn->obj = obj;
+		pcb_obj_center(obj, &conn->X, &conn->Y);
+		return pcb_true;
+	}
 
 	if ((element = pcb_search_elem_by_name(PCB->Data, ElementName)) == NULL)
 		return pcb_false;
 
 	padlist_foreach(&element->Pad, &it, pad) {
 		if (PCB_NSTRCMP(PinNum, pad->Number) == 0 && (!Same || !PCB_FLAG_TEST(PCB_FLAG_DRC, pad))) {
-			conn->type = PCB_TYPE_PAD;
 			conn->ptr1 = element;
-			conn->ptr2 = pad;
+			conn->obj = (pcb_any_obj_t *)pad;
 			conn->group = PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) ? Sgrp : Cgrp;
 
 			if (PCB_FLAG_TEST(PCB_FLAG_EDGE2, pad)) {
@@ -133,9 +145,8 @@ static pcb_bool FindPad(const char *ElementName, const char *PinNum, pcb_connect
 
 	padlist_foreach(&element->Pin, &it, pin) {
 		if (!PCB_FLAG_TEST(PCB_FLAG_HOLE, pin) && pin->Number && PCB_NSTRCMP(PinNum, pin->Number) == 0 && (!Same || !PCB_FLAG_TEST(PCB_FLAG_DRC, pin))) {
-			conn->type = PCB_TYPE_PIN;
 			conn->ptr1 = element;
-			conn->ptr2 = pin;
+			conn->obj = (pcb_any_obj_t *)pin;
 			conn->group = Sgrp;			/* any layer will do */
 			conn->X = pin->X;
 			conn->Y = pin->Y;
@@ -164,7 +175,7 @@ pcb_bool pcb_rat_seek_pad(pcb_lib_entry_t * entry, pcb_connection_t * conn, pcb_
 		badnet = pcb_true;
 	}
 	else {
-		if (FindPad(ElementName, PinNum, conn, Same))
+		if (pcb_term_find_name_ppt(ElementName, PinNum, conn, Same))
 			return (pcb_true);
 		if (Same)
 			return (pcb_false);
@@ -177,10 +188,77 @@ pcb_bool pcb_rat_seek_pad(pcb_lib_entry_t * entry, pcb_connection_t * conn, pcb_
 	return (pcb_false);
 }
 
+static const char *get_refdes(void *ptr1)
+{
+	pcb_any_obj_t *obj = ptr1;
+	switch(obj->type) {
+		case PCB_OBJ_ELEMENT: return PCB_ELEM_NAME_REFDES((pcb_element_t *)ptr1);
+		case PCB_OBJ_SUBC:
+			if (((pcb_subc_t *)ptr1)->refdes != NULL)
+				return ((pcb_subc_t *)ptr1)->refdes;
+			return "<anon>";
+		default:
+			break;
+	}
+	return "<invalid>";
+}
+
+static const char *get_termid(pcb_any_obj_t *obj)
+{
+	switch(obj->type) {
+		case PCB_OBJ_PIN: return ((pcb_pin_t *)obj)->Number;
+		case PCB_OBJ_PAD: return ((pcb_pad_t *)obj)->Number;
+		default: return obj->term;
+	}
+}
+
+static void clear_drc_flag(int clear_ratconn)
+{
+	pcb_rtree_it_t it;
+	pcb_box_t *n;
+	int li;
+	pcb_layer_t *l;
+
+	for(n = pcb_r_first(PCB->Data->pin_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+		if (clear_ratconn)
+			((pcb_pin_t *)n)->ratconn = NULL;
+		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pin_t *)n);
+	}
+	pcb_r_end(&it);
+
+	for(n = pcb_r_first(PCB->Data->via_tree, &it); n != NULL; n = pcb_r_next(&it))
+		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pin_t *)n);
+	pcb_r_end(&it);
+
+	for(n = pcb_r_first(PCB->Data->pad_tree, &it); n != NULL; n = pcb_r_next(&it)) {
+		if (clear_ratconn)
+			((pcb_pad_t *)n)->ratconn = NULL;
+		PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_pad_t *)n);
+	}
+	pcb_r_end(&it);
+
+	for(li = 0, l = PCB->Data->Layer; li < PCB->Data->LayerN; li++,l++) {
+		for(n = pcb_r_first(l->line_tree, &it); n != NULL; n = pcb_r_next(&it))
+			PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_line_t *)n);
+		pcb_r_end(&it);
+
+		for(n = pcb_r_first(l->arc_tree, &it); n != NULL; n = pcb_r_next(&it))
+			PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_arc_t *)n);
+		pcb_r_end(&it);
+
+		for(n = pcb_r_first(l->polygon_tree, &it); n != NULL; n = pcb_r_next(&it))
+			PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_polygon_t *)n);
+		pcb_r_end(&it);
+
+		for(n = pcb_r_first(l->text_tree, &it); n != NULL; n = pcb_r_next(&it))
+			PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, (pcb_text_t *)n);
+		pcb_r_end(&it);
+	}
+}
+
 /* ---------------------------------------------------------------------------
  * Read the library-netlist build a pcb_true Netlist structure
  */
-
 pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 {
 	pcb_connection_t *connection;
@@ -201,18 +279,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 
 	Wantlist = (pcb_netlist_t *) calloc(1, sizeof(pcb_netlist_t));
 	if (Wantlist) {
-		PCB_PIN_ALL_LOOP(PCB->Data);
-		{
-			pin->Spare = NULL;
-			PCB_FLAG_CLEAR(PCB_FLAG_DRC, pin);
-		}
-		PCB_ENDALL_LOOP;
-		PCB_PAD_ALL_LOOP(PCB->Data);
-		{
-			pad->Spare = NULL;
-			PCB_FLAG_CLEAR(PCB_FLAG_DRC, pad);
-		}
-		PCB_ENDALL_LOOP;
+		clear_drc_flag(1);
 		PCB_MENU_LOOP(net_menu);
 		{
 			if (menu->Name[0] == '*' || menu->flag == 0) {
@@ -221,7 +288,7 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 			}
 			net = pcb_net_new(Wantlist);
 			if (menu->Style) {
-				int idx = pcb_route_style_lookup(&PCB->RouteStyle, 0, 0, 0, 0, menu->Style);
+				int idx = pcb_route_style_lookup(&PCB->RouteStyle, -1, -1, -1, -1, menu->Style);
 				if (idx >= 0)
 					net->Style = PCB->RouteStyle.array+idx;
 			}
@@ -230,23 +297,18 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 			PCB_ENTRY_LOOP(menu);
 			{
 				if (pcb_rat_seek_pad(entry, &LastPoint, pcb_false)) {
-					if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.ptr2))
+					if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.obj))
 						pcb_message(PCB_MSG_ERROR, _
 										("Error! Element %s pin %s appears multiple times in the netlist file.\n"),
-										PCB_ELEM_NAME_REFDES((pcb_element_t *) LastPoint.ptr1),
-										(LastPoint.type ==
-										 PCB_TYPE_PIN) ? ((pcb_pin_t *) LastPoint.ptr2)->Number : ((pcb_pad_t *) LastPoint.ptr2)->Number);
+										get_refdes(LastPoint.ptr1), get_termid(LastPoint.obj));
 					else {
 						connection = pcb_rat_connection_alloc(net);
 						*connection = LastPoint;
 						/* indicate expect net */
 						connection->menu = menu;
 						/* mark as visited */
-						PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.ptr2);
-						if (LastPoint.type == PCB_TYPE_PIN)
-							((pcb_pin_t *) LastPoint.ptr2)->Spare = (void *) menu;
-						else
-							((pcb_pad_t *) LastPoint.ptr2)->Spare = (void *) menu;
+						PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.obj);
+						LastPoint.obj->ratconn = (void *)menu;
 					}
 				}
 				else
@@ -258,28 +320,18 @@ pcb_netlist_t *pcb_rat_proc_netlist(pcb_lib_t *net_menu)
 					/* indicate expect net */
 					connection->menu = menu;
 					/* mark as visited */
-					PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.ptr2);
-					if (LastPoint.type == PCB_TYPE_PIN)
-						((pcb_pin_t *) LastPoint.ptr2)->Spare = (void *) menu;
-					else
-						((pcb_pad_t *) LastPoint.ptr2)->Spare = (void *) menu;
+					PCB_FLAG_SET(PCB_FLAG_DRC, (pcb_pin_t *) LastPoint.obj);
+					LastPoint.obj->ratconn = (void *)menu;
 				}
 			}
 			PCB_END_LOOP;
 		}
 		PCB_END_LOOP;
 	}
+
 	/* clear all visit marks */
-	PCB_PIN_ALL_LOOP(PCB->Data);
-	{
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC, pin);
-	}
-	PCB_ENDALL_LOOP;
-	PCB_PAD_ALL_LOOP(PCB->Data);
-	{
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC, pad);
-	}
-	PCB_ENDALL_LOOP;
+	clear_drc_flag(0);
+
 	return (Wantlist);
 }
 
@@ -307,10 +359,108 @@ static void TransferNet(pcb_netlist_t *Netl, pcb_net_t *SourceNet, pcb_net_t *De
 	memset(&Netl->Net[Netl->NetN], 0, sizeof(pcb_net_t));
 }
 
+static void **found_short(pcb_any_obj_t *parent, pcb_any_obj_t *term, vtptr_t *generic, pcb_lib_menu_t *theNet, void **menu)
+{
+	pcb_bool newone;
+	int i;
+
+	if (!term->ratconn) {
+		pcb_message(PCB_MSG_WARNING, _("Warning! Net \"%s\" is shorted to %s terminal %s\n"),
+						&theNet->Name[2], PCB_UNKNOWN(get_refdes(parent)), PCB_UNKNOWN(get_termid(term)));
+		pcb_stub_rat_found_short(term, &theNet->Name[2]);
+		return menu;
+	}
+
+	newone = pcb_true;
+	for(i = 0; i < vtptr_len(generic); i++) {
+		if (generic->array[i] == term->ratconn) {
+			newone = pcb_false;
+			break;
+		}
+	}
+
+	if (newone) {
+		menu = vtptr_alloc_append(generic, 1);
+		*menu = term->ratconn;
+		pcb_message(PCB_MSG_WARNING, _("Warning! Net \"%s\" is shorted to net \"%s\"\n"),
+						&theNet->Name[2], &((pcb_lib_menu_t *) (term->ratconn))->Name[2]);
+		pcb_stub_rat_found_short((pcb_any_obj_t *)term, &theNet->Name[2]);
+	}
+	return menu;
+}
+
+static void **find_shorts_in_subc(pcb_subc_t *subc_in, vtptr_t *generic, pcb_lib_menu_t *theNet, void **menu, pcb_bool *warn)
+{
+	if (PCB_FLAG_TEST(PCB_FLAG_NONETLIST, subc_in))
+		return menu;
+
+	PCB_VIA_LOOP(subc_in->data);
+	{
+		if (via->term == NULL)
+			continue;
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, via)) {
+			*warn = pcb_true;
+			menu = found_short((pcb_any_obj_t *)subc_in, (pcb_any_obj_t *)via, generic, theNet, menu);
+		}
+	}
+	PCB_END_LOOP;
+
+	PCB_LINE_ALL_LOOP(subc_in->data);
+	{
+		if (line->term == NULL)
+			continue;
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, line)) {
+			*warn = pcb_true;
+			menu = found_short((pcb_any_obj_t *)subc_in, (pcb_any_obj_t *)line, generic, theNet, menu);
+		}
+	}
+	PCB_ENDALL_LOOP;
+
+	PCB_ARC_ALL_LOOP(subc_in->data);
+	{
+		if (arc->term == NULL)
+			continue;
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, arc)) {
+			*warn = pcb_true;
+			menu = found_short((pcb_any_obj_t *)subc_in, (pcb_any_obj_t *)arc, generic, theNet, menu);
+		}
+	}
+	PCB_ENDALL_LOOP;
+
+	PCB_POLY_ALL_LOOP(subc_in->data);
+	{
+		if (polygon->term == NULL)
+			continue;
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, polygon)) {
+			*warn = pcb_true;
+			menu = found_short((pcb_any_obj_t *)subc_in, (pcb_any_obj_t *)polygon, generic, theNet, menu);
+		}
+	}
+	PCB_ENDALL_LOOP;
+
+	PCB_TEXT_ALL_LOOP(subc_in->data);
+	{
+		if (text->term == NULL)
+			continue;
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, text)) {
+			*warn = pcb_true;
+			menu = found_short((pcb_any_obj_t *)subc_in, (pcb_any_obj_t *)text, generic, theNet, menu);
+		}
+	}
+	PCB_ENDALL_LOOP;
+
+	PCB_SUBC_LOOP(subc_in->data);
+	{
+		menu = find_shorts_in_subc(subc, generic, theNet, menu, warn);
+	}
+	PCB_END_LOOP;
+	return menu;
+}
+
+
 static pcb_bool CheckShorts(pcb_lib_menu_t *theNet)
 {
-	pcb_bool newone, warn = pcb_false;
-	int i;
+	pcb_bool warn = pcb_false;
 	vtptr_t generic;
 	/* the first connection was starting point so
 	 * the menu is always non-null
@@ -324,29 +474,9 @@ static pcb_bool CheckShorts(pcb_lib_menu_t *theNet)
 	{
 		pcb_element_t *e = pin->Element;
 /* TODO: should be: !PCB_FLAG_TEST(PCB_FLAG_NONETLIST, (pcb_element_t *)pin->Element)*/
-		if ((PCB_FLAG_TEST(PCB_FLAG_DRC, pin)) && (!(e->Flags.f & PCB_FLAG_NONETLIST))) {
+		if ((PCB_FLAG_TEST(PCB_FLAG_DRC, pin)) && (!(e->Flags.f & PCB_FLAG_NONETLIST)) && (!(e->Name->Flags.f & PCB_FLAG_NONETLIST))) {
 			warn = pcb_true;
-			if (!pin->Spare) {
-				pcb_message(PCB_MSG_WARNING, _("Warning! Net \"%s\" is shorted to %s pin %s\n"),
-								&theNet->Name[2], PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)), PCB_UNKNOWN(pin->Number));
-				pcb_stub_rat_found_short(pin, NULL, &theNet->Name[2]);
-				continue;
-			}
-			newone = pcb_true;
-			for(i = 0; i < vtptr_len(&generic); i++) {
-				if (generic.array[i] == pin->Spare) {
-					newone = pcb_false;
-					break;
-				}
-			}
-
-			if (newone) {
-				menu = vtptr_alloc_append(&generic, 1);
-				*menu = pin->Spare;
-				pcb_message(PCB_MSG_WARNING, _("Warning! Net \"%s\" is shorted to net \"%s\"\n"),
-								&theNet->Name[2], &((pcb_lib_menu_t *) (pin->Spare))->Name[2]);
-				pcb_stub_rat_found_short(pin, NULL, &theNet->Name[2]);
-			}
+			menu = found_short((pcb_any_obj_t *)element, (pcb_any_obj_t *)pin, &generic, theNet, menu);
 		}
 	}
 	PCB_ENDALL_LOOP;
@@ -356,34 +486,89 @@ static pcb_bool CheckShorts(pcb_lib_menu_t *theNet)
 /* TODO: should be: !PCB_FLAG_TEST(PCB_FLAG_NONETLIST, (pcb_element_t *)pad->Element)*/
 		if ((PCB_FLAG_TEST(PCB_FLAG_DRC, pad)) && (!(e->Flags.f & PCB_FLAG_NONETLIST)) && (!(e->Name->Flags.f & PCB_FLAG_NONETLIST))) {
 			warn = pcb_true;
-			if (!pad->Spare) {
-				pcb_message(PCB_MSG_WARNING, _("Warning! Net \"%s\" is shorted  to %s pad %s\n"),
-								&theNet->Name[2], PCB_UNKNOWN(PCB_ELEM_NAME_REFDES(element)), PCB_UNKNOWN(pad->Number));
-				pcb_stub_rat_found_short(NULL, pad, &theNet->Name[2]);
-				continue;
-			}
-			newone = pcb_true;
-			for(i = 0; i < vtptr_len(&generic); i++) {
-				if (generic.array[i] == pad->Spare) {
-					newone = pcb_false;
-					break;
-				}
-			}
-
-			if (newone) {
-				menu = vtptr_alloc_append(&generic, 1);
-				*menu = pad->Spare;
-				pcb_message(PCB_MSG_WARNING, _("Warning! Net \"%s\" is shorted to net \"%s\"\n"),
-								&theNet->Name[2], &((pcb_lib_menu_t *) (pad->Spare))->Name[2]);
-				pcb_stub_rat_found_short(NULL, pad, &theNet->Name[2]);
-			}
+			menu = found_short((pcb_any_obj_t *)element, (pcb_any_obj_t *)pad, &generic, theNet, menu);
 		}
 	}
 	PCB_ENDALL_LOOP;
+	PCB_SUBC_LOOP(PCB->Data);
+	{
+		menu = find_shorts_in_subc(subc, &generic, theNet, menu, &warn);
+	}
+	PCB_END_LOOP;
 	vtptr_uninit(&generic);
 	return (warn);
 }
 
+static void gather_subnet_objs(pcb_data_t *data, pcb_netlist_t *Netl, pcb_net_t *a)
+{
+	pcb_connection_t *conn;
+
+	/* now add other possible attachment points to the subnet */
+	/* e.g. line end-points and vias */
+	/* don't add non-manhattan lines, the auto-router can't route to them */
+	PCB_LINE_COPPER_LOOP(data);
+	{
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, line)
+				&& ((line->Point1.X == line->Point2.X)
+						|| (line->Point1.Y == line->Point2.Y))) {
+			conn = pcb_rat_connection_alloc(a);
+			conn->X = line->Point1.X;
+			conn->Y = line->Point1.Y;
+			conn->ptr1 = layer;
+			conn->obj = (pcb_any_obj_t *)line;
+			conn->group = pcb_layer_get_group_(layer);
+			conn->menu = NULL;			/* agnostic view of where it belongs */
+			conn = pcb_rat_connection_alloc(a);
+			conn->X = line->Point2.X;
+			conn->Y = line->Point2.Y;
+			conn->ptr1 = layer;
+			conn->obj = (pcb_any_obj_t *)line;
+			conn->group = pcb_layer_get_group_(layer);
+			conn->menu = NULL;
+			if PCB_FLAG_TEST(PCB_FLAG_DRC_INTCONN, line)
+				PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, line);
+		}
+	}
+	PCB_ENDALL_LOOP;
+#warning term TODO: and what about arcs and text?
+	/* add polygons so the auto-router can see them as targets */
+	PCB_POLY_COPPER_LOOP(data);
+	{
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, polygon)) {
+			conn = pcb_rat_connection_alloc(a);
+			/* make point on a vertex */
+			conn->X = polygon->Clipped->contours->head.point[0];
+			conn->Y = polygon->Clipped->contours->head.point[1];
+			conn->ptr1 = layer;
+			conn->obj = (pcb_any_obj_t *)polygon;
+			conn->group = pcb_layer_get_group_(layer);
+			conn->menu = NULL;			/* agnostic view of where it belongs */
+			if PCB_FLAG_TEST(PCB_FLAG_DRC_INTCONN, polygon)
+				PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, polygon);
+		}
+	}
+	PCB_ENDALL_LOOP;
+	PCB_VIA_LOOP(data);
+	{
+		if (PCB_FLAG_TEST(PCB_FLAG_DRC, via)) {
+			conn = pcb_rat_connection_alloc(a);
+			conn->X = via->X;
+			conn->Y = via->Y;
+			conn->ptr1 = via;
+			conn->obj = (pcb_any_obj_t *)via;
+			conn->group = Sgrp;
+			if PCB_FLAG_TEST(PCB_FLAG_DRC_INTCONN, via)
+				PCB_FLAG_CLEAR(PCB_FLAG_DRC | PCB_FLAG_DRC_INTCONN, via);
+		}
+	}
+	PCB_END_LOOP;
+
+	PCB_SUBC_LOOP(data);
+	{
+		gather_subnet_objs(subc->data, Netl, a);
+	}
+	PCB_END_LOOP;
+}
 
 /* ---------------------------------------------------------------------------
  * Determine existing interconnections of the net and gather into sub-nets
@@ -394,83 +579,29 @@ static pcb_bool CheckShorts(pcb_lib_menu_t *theNet)
 static pcb_bool GatherSubnets(pcb_netlist_t *Netl, pcb_bool NoWarn, pcb_bool AndRats)
 {
 	pcb_net_t *a, *b;
-	pcb_connection_t *conn;
-	pcb_cardinal_t m, n;
 	pcb_bool Warned = pcb_false;
+	pcb_cardinal_t m, n;
 
 	for (m = 0; Netl->NetN > 0 && m < Netl->NetN; m++) {
 		a = &Netl->Net[m];
 		pcb_reset_conns(pcb_false);
-		pcb_rat_find_hook(a->Connection[0].type, a->Connection[0].ptr1, a->Connection[0].ptr2, a->Connection[0].ptr2, pcb_false, AndRats);
+		pcb_rat_find_hook(a->Connection[0].obj, pcb_false, AndRats);
+
 		/* now anybody connected to the first point has PCB_FLAG_DRC set */
 		/* so move those to this subnet */
-		PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_pin_t *) a->Connection[0].ptr2);
+		PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_pin_t *) a->Connection[0].obj);
 		for (n = m + 1; n < Netl->NetN; n++) {
 			b = &Netl->Net[n];
 			/* There can be only one connection in net b */
-			if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_pin_t *) b->Connection[0].ptr2)) {
-				PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_pin_t *) b->Connection[0].ptr2);
+			if (PCB_FLAG_TEST(PCB_FLAG_DRC, (pcb_pin_t *) b->Connection[0].obj)) {
+				PCB_FLAG_CLEAR(PCB_FLAG_DRC, (pcb_pin_t *) b->Connection[0].obj);
 				TransferNet(Netl, b, a);
 				/* back up since new subnet is now at old index */
 				n--;
 			}
 		}
-		/* now add other possible attachment points to the subnet */
-		/* e.g. line end-points and vias */
-		/* don't add non-manhattan lines, the auto-router can't route to them */
-		PCB_LINE_ALL_LOOP(PCB->Data);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_DRC, line)
-					&& ((line->Point1.X == line->Point2.X)
-							|| (line->Point1.Y == line->Point2.Y))) {
-				conn = pcb_rat_connection_alloc(a);
-				conn->X = line->Point1.X;
-				conn->Y = line->Point1.Y;
-				conn->type = PCB_TYPE_LINE;
-				conn->ptr1 = layer;
-				conn->ptr2 = line;
-				conn->group = pcb_layer_get_group_(layer);
-				conn->menu = NULL;			/* agnostic view of where it belongs */
-				conn = pcb_rat_connection_alloc(a);
-				conn->X = line->Point2.X;
-				conn->Y = line->Point2.Y;
-				conn->type = PCB_TYPE_LINE;
-				conn->ptr1 = layer;
-				conn->ptr2 = line;
-				conn->group = pcb_layer_get_group_(layer);
-				conn->menu = NULL;
-			}
-		}
-		PCB_ENDALL_LOOP;
-		/* add polygons so the auto-router can see them as targets */
-		PCB_POLY_ALL_LOOP(PCB->Data);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_DRC, polygon)) {
-				conn = pcb_rat_connection_alloc(a);
-				/* make point on a vertex */
-				conn->X = polygon->Clipped->contours->head.point[0];
-				conn->Y = polygon->Clipped->contours->head.point[1];
-				conn->type = PCB_TYPE_POLYGON;
-				conn->ptr1 = layer;
-				conn->ptr2 = polygon;
-				conn->group = pcb_layer_get_group_(layer);
-				conn->menu = NULL;			/* agnostic view of where it belongs */
-			}
-		}
-		PCB_ENDALL_LOOP;
-		PCB_VIA_LOOP(PCB->Data);
-		{
-			if (PCB_FLAG_TEST(PCB_FLAG_DRC, via)) {
-				conn = pcb_rat_connection_alloc(a);
-				conn->X = via->X;
-				conn->Y = via->Y;
-				conn->type = PCB_TYPE_VIA;
-				conn->ptr1 = via;
-				conn->ptr2 = via;
-				conn->group = Sgrp;
-			}
-		}
-		PCB_END_LOOP;
+
+		gather_subnet_objs(PCB->Data, Netl, a);
 		if (!NoWarn)
 			Warned |= CheckShorts(a->Connection[0].menu);
 	}
@@ -566,20 +697,20 @@ DrawShortestRats(pcb_netlist_t *Netl,
 					 * not a daisy chain).  Further prefer to pick an existing
 					 * via in the Net to make that connection.
 					 */
-					if (conn1->type == PCB_TYPE_POLYGON &&
-							(polygon = (pcb_polygon_t *) conn1->ptr2) &&
+					if (conn1->obj->type == PCB_OBJ_POLYGON &&
+							(polygon = (pcb_polygon_t *) conn1->obj) &&
 							!(distance == 0 &&
-								firstpoint && firstpoint->type == PCB_TYPE_VIA) && pcb_poly_is_point_in_p_ignore_holes(conn2->X, conn2->Y, polygon)) {
+								firstpoint && firstpoint->obj->type == PCB_OBJ_VIA) && pcb_poly_is_point_in_p_ignore_holes(conn2->X, conn2->Y, polygon)) {
 						distance = 0;
 						firstpoint = conn2;
 						secondpoint = conn1;
 						theSubnet = next;
 						havepoints = pcb_true;
 					}
-					else if (conn2->type == PCB_TYPE_POLYGON &&
-									 (polygon = (pcb_polygon_t *) conn2->ptr2) &&
+					else if (conn2->obj->type == PCB_OBJ_POLYGON &&
+									 (polygon = (pcb_polygon_t *) conn2->obj) &&
 									 !(distance == 0 &&
-										 firstpoint && firstpoint->type == PCB_TYPE_VIA) && pcb_poly_is_point_in_p_ignore_holes(conn1->X, conn1->Y, polygon)) {
+										 firstpoint && firstpoint->obj->type == PCB_OBJ_VIA) && pcb_poly_is_point_in_p_ignore_holes(conn1->X, conn1->Y, polygon)) {
 						distance = 0;
 						firstpoint = conn1;
 						secondpoint = conn2;
@@ -614,7 +745,7 @@ DrawShortestRats(pcb_netlist_t *Netl,
 					if (distance == 0)
 						PCB_FLAG_SET(PCB_FLAG_VIA, line);
 					pcb_undo_add_obj_to_create(PCB_TYPE_RATLINE, line, line, line);
-					DrawRat(line);
+					pcb_rat_invalidate_draw(line);
 					changed = pcb_true;
 				}
 			}
@@ -681,7 +812,7 @@ pcb_rat_add_all(pcb_bool SelectedOnly,
 	{
 		PCB_CONNECTION_LOOP(net);
 		{
-			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) connection->ptr2)) {
+			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) connection->obj)) {
 				lonesome = pcb_net_new(Nets);
 				onepin = pcb_rat_connection_alloc(lonesome);
 				*onepin = *connection;
@@ -766,7 +897,7 @@ pcb_netlist_list_t pcb_rat_collect_subnets(pcb_bool SelectedOnly)
 		Nets = pcb_netlist_new(&result);
 		PCB_CONNECTION_LOOP(net);
 		{
-			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) connection->ptr2)) {
+			if (!SelectedOnly || PCB_FLAG_TEST(PCB_FLAG_SELECTED, (pcb_pin_t *) connection->obj)) {
 				lonesome = pcb_net_new(Nets);
 				onepin = pcb_rat_connection_alloc(lonesome);
 				*onepin = *connection;
@@ -802,15 +933,13 @@ static int rat_used(char *name)
 	return 0;
 }
 
-	/* These next two functions moved from the original netlist.c as part of the
-	   |  gui code separation for the Gtk port.
-	 */
 pcb_rat_t *pcb_rat_add_net(void)
 {
 	static int ratDrawn = 0;
 	char name1[256], *name2;
 	pcb_cardinal_t group1, group2;
 	char ratname[20];
+	const char *rd;
 	int found;
 	void *ptr1, *ptr2, *ptr3;
 	pcb_lib_menu_t *menu;
@@ -826,7 +955,8 @@ pcb_rat_t *pcb_rat_add_net(void)
 		pcb_message(PCB_MSG_ERROR, _("No pad/pin under rat line\n"));
 		return (NULL);
 	}
-	if (PCB_ELEM_NAME_REFDES((pcb_element_t *) ptr1) == NULL || *PCB_ELEM_NAME_REFDES((pcb_element_t *) ptr1) == 0) {
+	rd = get_refdes(ptr1);
+	if ((rd == NULL) || (*rd == 0) || (*rd == '<')) {
 		pcb_message(PCB_MSG_ERROR, _("You must name the starting element first\n"));
 		return (NULL);
 	}
@@ -837,19 +967,20 @@ pcb_rat_t *pcb_rat_add_net(void)
 
 	/* will work for pins to since the FLAG is common */
 	group1 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? Sgrp : Cgrp);
-	strcpy(name1, pcb_connection_name(found, ptr1, ptr2));
+	strcpy(name1, pcb_connection_name(ptr2));
 	found = pcb_search_obj_by_location(PCB_TYPE_PAD | PCB_TYPE_PIN, &ptr1, &ptr2, &ptr3,
 																 pcb_crosshair.AttachedLine.Point2.X, pcb_crosshair.AttachedLine.Point2.Y, 5);
 	if (found == PCB_TYPE_NONE) {
 		pcb_message(PCB_MSG_ERROR, _("No pad/pin under rat line\n"));
 		return (NULL);
 	}
-	if (PCB_ELEM_NAME_REFDES((pcb_element_t *) ptr1) == NULL || *PCB_ELEM_NAME_REFDES((pcb_element_t *) ptr1) == 0) {
+	rd = get_refdes(ptr1);
+	if ((rd == NULL) || (*rd == 0) || (*rd == '<')) {
 		pcb_message(PCB_MSG_ERROR, _("You must name the ending element first\n"));
 		return (NULL);
 	}
 	group2 = (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, (pcb_pad_t *) ptr2) ? Sgrp : Cgrp);
-	name2 = pcb_connection_name(found, ptr1, ptr2);
+	name2 = pcb_connection_name(ptr2);
 
 	menu = pcb_netnode_to_netname(name1);
 	if (menu) {
@@ -904,22 +1035,30 @@ ratIt:
 }
 
 
-char *pcb_connection_name(int type, void *ptr1, void *ptr2)
+char *pcb_connection_name(pcb_any_obj_t *obj)
 {
 	static char name[256];
-	char *num;
+	const char *num, *parent = NULL;
+	pcb_subc_t *subc;
 
-	switch (type) {
-	case PCB_TYPE_PIN:
-		num = ((pcb_pin_t *) ptr2)->Number;
+	switch (obj->type) {
+	case PCB_OBJ_PIN:
+		num = ((pcb_pin_t *)obj)->Number;
+		parent = PCB_ELEM_NAME_REFDES((pcb_element_t*)((pcb_pin_t *)obj)->Element);
 		break;
-	case PCB_TYPE_PAD:
-		num = ((pcb_pad_t *) ptr2)->Number;
+	case PCB_OBJ_PAD:
+		num = ((pcb_pad_t *)obj)->Number;
+		parent = PCB_ELEM_NAME_REFDES((pcb_element_t*)((pcb_pad_t *)obj)->Element);
 		break;
 	default:
-		return (NULL);
+		if (obj->term == NULL)
+			return NULL;
+		num = obj->term;
+		subc = pcb_obj_parent_subc(obj);
+		if (subc != NULL)
+			parent = subc->refdes;
 	}
-	strcpy(name, PCB_UNKNOWN(PCB_ELEM_NAME_REFDES((pcb_element_t *) ptr1)));
+	strcpy(name, PCB_UNKNOWN(parent));
 	strcat(name, "-");
 	strcat(name, PCB_UNKNOWN(num));
 	return (name);
