@@ -35,6 +35,7 @@
 #include "data.h"
 #include "draw.h"
 #include "error.h"
+#include "polygon.h"
 #include "search.h"
 #include "select.h"
 #include "undo.h"
@@ -58,33 +59,33 @@ void pcb_select_element(pcb_board_t *pcb, pcb_element_t *element, pcb_change_fla
 	/* select all pins and names of the element */
 	PCB_PIN_LOOP(element);
 	{
-		pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, pin, pin);
+		pcb_undo_add_obj_to_flag(pin);
 		PCB_FLAG_CHANGE(how, PCB_FLAG_SELECTED, pin);
 	}
 	PCB_END_LOOP;
 	PCB_PAD_LOOP(element);
 	{
-		pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, element, pad, pad);
+		pcb_undo_add_obj_to_flag(pad);
 		PCB_FLAG_CHANGE(how, PCB_FLAG_SELECTED, pad);
 	}
 	PCB_END_LOOP;
 	PCB_ELEMENT_PCB_TEXT_LOOP(element);
 	{
-		pcb_undo_add_obj_to_flag(PCB_TYPE_ELEMENT_NAME, element, text, text);
+		pcb_undo_add_obj_to_flag(text);
 		PCB_FLAG_CHANGE(how, PCB_FLAG_SELECTED, text);
 	}
 	PCB_END_LOOP;
-	pcb_undo_add_obj_to_flag(PCB_TYPE_ELEMENT, element, element, element);
+	pcb_undo_add_obj_to_flag(element);
 	PCB_FLAG_CHANGE(how, PCB_FLAG_SELECTED, element);
 
 	if (redraw) {
 		if (pcb_silk_on(pcb) && ((PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) != 0) == PCB_SWAP_IDENT || pcb->InvisibleObjectsOn))
 			if (pcb_silk_on(pcb)) {
-				DrawElementName(element);
-				DrawElementPackage(element);
+				pcb_elem_name_invalidate_draw(element);
+				pcb_elem_package_invalidate_draw(element);
 			}
 		if (pcb->PinOn)
-			DrawElementPinsAndPads(element);
+			pcb_elem_pp_invalidate_draw(element);
 	}
 }
 
@@ -93,13 +94,13 @@ void pcb_select_element_name(pcb_element_t *element, pcb_change_flag_t how, int
 	/* select all names of the element */
 	PCB_ELEMENT_PCB_TEXT_LOOP(element);
 	{
-		pcb_undo_add_obj_to_flag(PCB_TYPE_ELEMENT_NAME, element, text, text);
+		pcb_undo_add_obj_to_flag(text);
 		PCB_FLAG_CHANGE(how, PCB_FLAG_SELECTED, text);
 	}
 	PCB_END_LOOP;
 
 	if (redraw)
-		DrawElementName(element);
+		pcb_elem_name_invalidate_draw(element);
 }
 
 
@@ -120,9 +121,9 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 		return (pcb_false);
 	switch (type) {
 	case PCB_TYPE_VIA:
-		pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, ptr1, ptr1, ptr1);
+		pcb_undo_add_obj_to_flag(ptr1);
 		PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, (pcb_pin_t *) ptr1);
-		DrawVia((pcb_pin_t *) ptr1);
+		pcb_via_invalidate_draw((pcb_pin_t *) ptr1);
 		break;
 
 	case PCB_TYPE_LINE:
@@ -130,9 +131,9 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 			pcb_line_t *line = (pcb_line_t *) ptr2;
 
 			layer = (pcb_layer_t *) ptr1;
-			pcb_undo_add_obj_to_flag(PCB_TYPE_LINE, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, line);
-			DrawLine(layer, line);
+			pcb_line_invalidate_draw(layer, line);
 			break;
 		}
 
@@ -140,9 +141,9 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 		{
 			pcb_rat_t *rat = (pcb_rat_t *) ptr2;
 
-			pcb_undo_add_obj_to_flag(PCB_TYPE_RATLINE, ptr1, ptr1, ptr1);
+			pcb_undo_add_obj_to_flag(ptr1);
 			PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, rat);
-			DrawRat(rat);
+			pcb_rat_invalidate_draw(rat);
 			break;
 		}
 
@@ -151,9 +152,9 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 			pcb_arc_t *arc = (pcb_arc_t *) ptr2;
 
 			layer = (pcb_layer_t *) ptr1;
-			pcb_undo_add_obj_to_flag(PCB_TYPE_ARC, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, arc);
-			DrawArc(layer, arc);
+			pcb_arc_invalidate_draw(layer, arc);
 			break;
 		}
 
@@ -162,9 +163,9 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 			pcb_text_t *text = (pcb_text_t *) ptr2;
 
 			layer = (pcb_layer_t *) ptr1;
-			pcb_undo_add_obj_to_flag(PCB_TYPE_TEXT, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, text);
-			DrawText(layer, text);
+			pcb_text_invalidate_draw(layer, text);
 			break;
 		}
 
@@ -173,23 +174,23 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
 			pcb_polygon_t *poly = (pcb_polygon_t *) ptr2;
 
 			layer = (pcb_layer_t *) ptr1;
-			pcb_undo_add_obj_to_flag(PCB_TYPE_POLYGON, ptr1, ptr2, ptr2);
+			pcb_undo_add_obj_to_flag(ptr2);
 			PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, poly);
-			DrawPolygon(layer, poly);
+			pcb_poly_invalidate_draw(layer, poly);
 			/* changing memory order no longer effects draw order */
 			break;
 		}
 
 	case PCB_TYPE_PIN:
-		pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, ptr1, ptr2, ptr2);
+		pcb_undo_add_obj_to_flag(ptr2);
 		PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, (pcb_pin_t *) ptr2);
-		DrawPin((pcb_pin_t *) ptr2);
+		pcb_pin_invalidate_draw((pcb_pin_t *) ptr2);
 		break;
 
 	case PCB_TYPE_PAD:
-		pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, ptr1, ptr2, ptr2);
+		pcb_undo_add_obj_to_flag(ptr2);
 		PCB_FLAG_TOGGLE(PCB_FLAG_SELECTED, (pcb_pad_t *) ptr2);
-		DrawPad((pcb_pad_t *) ptr2);
+		pcb_pad_invalidate_draw((pcb_pad_t *) ptr2);
 		break;
 
 	case PCB_TYPE_ELEMENT_NAME:
@@ -218,7 +219,7 @@ pcb_bool pcb_select_object(pcb_board_t *pcb)
  *  returns a list of object IDs matched the search and loads len with the
  *  length of the list. Returns NULL on no match.
  */
-#warning TODO: should be rewritten with generic ops and rtree
+#warning cleanup TODO: should be rewritten with generic ops and rtree
 static long int *ListBlock_(pcb_board_t *pcb, pcb_box_t *Box, pcb_bool Flag, int *len)
 {
 	int changed = 0;
@@ -269,7 +270,7 @@ do { \
 #define append(undo_type, p1, obj) \
 do { \
 	if (len == NULL) { \
-		pcb_undo_add_obj_to_flag(undo_type, p1, obj, obj); \
+		pcb_undo_add_obj_to_flag(obj); \
 		PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, obj); \
 	} \
 	else { \
@@ -295,7 +296,7 @@ do { \
 		if (PCB_LINE_NEAR_BOX((pcb_line_t *) line, Box) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, line) && PCB_FLAG_TEST(PCB_FLAG_SELECTED, line) != Flag) {
 			append(PCB_TYPE_RATLINE, line, line);
 			if (pcb->RatOn)
-				DrawRat(line);
+				pcb_rat_invalidate_draw(line);
 		}
 	}
 	PCB_END_LOOP;
@@ -323,7 +324,7 @@ do { \
 					&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, line) != Flag) {
 				append(PCB_TYPE_LINE, layer, line);
 				if (layer->meta.real.vis)
-					DrawLine(layer, line);
+					pcb_line_invalidate_draw(layer, line);
 			}
 		}
 		PCB_END_LOOP;
@@ -334,7 +335,7 @@ do { \
 					&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, arc) != Flag) {
 				append(PCB_TYPE_ARC, layer, arc);
 				if (layer->meta.real.vis)
-					DrawArc(layer, arc);
+					pcb_arc_invalidate_draw(layer, arc);
 			}
 		}
 		PCB_END_LOOP;
@@ -346,7 +347,7 @@ do { \
 						&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, text) != Flag) {
 					append(PCB_TYPE_TEXT, layer, text);
 					if (pcb_text_is_visible(PCB, layer, text))
-						DrawText(layer, text);
+						pcb_text_invalidate_draw(layer, text);
 				}
 			}
 		}
@@ -358,7 +359,7 @@ do { \
 					&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, polygon) != Flag) {
 				append(PCB_TYPE_POLYGON, layer, polygon);
 				if (layer->meta.real.vis)
-					DrawPolygon(layer, polygon);
+					pcb_poly_invalidate_draw(layer, polygon);
 			}
 		}
 		PCB_END_LOOP;
@@ -368,7 +369,7 @@ do { \
 	if (PCB->SubcOn) {
 		PCB_SUBC_LOOP(pcb->Data);
 		{
-			if (PCB_POLYGON_NEAR_BOX(subc, Box)
+			if (PCB_SUBC_NEAR_BOX(subc, Box)
 					&& !PCB_FLAG_TEST(PCB_FLAG_LOCK, subc)
 					&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, subc) != Flag) {
 
@@ -408,7 +409,7 @@ do { \
 					}
 					PCB_END_LOOP;
 					if (pcb_silk_on(pcb))
-						DrawElementName(element);
+						pcb_elem_name_invalidate_draw(element);
 				}
 				if ((pcb->PinOn || !Flag) && PCB_ELEMENT_NEAR_BOX(element, Box))
 					if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, element) != Flag) {
@@ -418,7 +419,7 @@ do { \
 							if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, pin) != Flag) {
 								append(PCB_TYPE_PIN, element, pin);
 								if (pcb->PinOn)
-									DrawPin(pin);
+									pcb_pin_invalidate_draw(pin);
 							}
 						}
 						PCB_END_LOOP;
@@ -427,12 +428,12 @@ do { \
 							if (PCB_FLAG_TEST(PCB_FLAG_SELECTED, pad) != Flag) {
 								append(PCB_TYPE_PAD, element, pad);
 								if (pcb->PinOn)
-									DrawPad(pad);
+									pcb_pad_invalidate_draw(pad);
 							}
 						}
 						PCB_END_LOOP;
 						if (pcb->PinOn)
-							DrawElement(element);
+							pcb_elem_invalidate_draw(element);
 						gotElement = pcb_true;
 					}
 			}
@@ -443,7 +444,7 @@ do { \
 							 && PCB_FLAG_TEST(PCB_FLAG_SELECTED, pin) != Flag)) {
 						append(PCB_TYPE_PIN, element, pin);
 						if (pcb->PinOn)
-							DrawPin(pin);
+							pcb_pin_invalidate_draw(pin);
 					}
 				}
 				PCB_END_LOOP;
@@ -454,7 +455,7 @@ do { \
 							&& (PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) == PCB_SWAP_IDENT || pcb->InvisibleObjectsOn || !Flag)) {
 						append(PCB_TYPE_PAD, element, pad);
 						if (pcb->PinOn)
-							DrawPad(pad);
+							pcb_pad_invalidate_draw(pad);
 					}
 				}
 				PCB_END_LOOP;
@@ -471,7 +472,7 @@ do { \
 				&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, via) != Flag) {
 			append(PCB_TYPE_VIA, via, via);
 			if (pcb->ViaOn)
-				DrawVia(via);
+				pcb_via_invalidate_draw(via);
 		}
 	}
 	PCB_END_LOOP;
@@ -527,9 +528,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 		PCB_RAT_LOOP(pcb->Data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_FOUND, line)) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_RATLINE, line, line, line);
+			pcb_undo_add_obj_to_flag(line);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, line);
-			DrawRat(line);
+			pcb_rat_invalidate_draw(line);
 			changed = pcb_true;
 		}
 	}
@@ -538,9 +539,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 	PCB_LINE_VISIBLE_LOOP(pcb->Data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_FOUND, line) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, line)) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_LINE, layer, line, line);
+			pcb_undo_add_obj_to_flag(line);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, line);
-			DrawLine(layer, line);
+			pcb_line_invalidate_draw(layer, line);
 			changed = pcb_true;
 		}
 	}
@@ -548,9 +549,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 	PCB_ARC_VISIBLE_LOOP(pcb->Data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_FOUND, arc) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, arc)) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_ARC, layer, arc, arc);
+			pcb_undo_add_obj_to_flag(arc);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, arc);
-			DrawArc(layer, arc);
+			pcb_arc_invalidate_draw(layer, arc);
 			changed = pcb_true;
 		}
 	}
@@ -558,9 +559,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 	PCB_POLY_VISIBLE_LOOP(pcb->Data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_FOUND, polygon) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, polygon)) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_POLYGON, layer, polygon, polygon);
+			pcb_undo_add_obj_to_flag(polygon);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, polygon);
-			DrawPolygon(layer, polygon);
+			pcb_poly_invalidate_draw(layer, polygon);
 			changed = pcb_true;
 		}
 	}
@@ -570,9 +571,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 		PCB_PIN_ALL_LOOP(pcb->Data);
 		{
 			if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, element) && PCB_FLAG_TEST(PCB_FLAG_FOUND, pin)) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, pin, pin);
+				pcb_undo_add_obj_to_flag(pin);
 				PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, pin);
-				DrawPin(pin);
+				pcb_pin_invalidate_draw(pin);
 				changed = pcb_true;
 			}
 		}
@@ -580,9 +581,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 		PCB_PAD_ALL_LOOP(pcb->Data);
 		{
 			if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, element) && PCB_FLAG_TEST(PCB_FLAG_FOUND, pad)) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, element, pad, pad);
+				pcb_undo_add_obj_to_flag(pad);
 				PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, pad);
-				DrawPad(pad);
+				pcb_pad_invalidate_draw(pad);
 				changed = pcb_true;
 			}
 		}
@@ -593,9 +594,9 @@ pcb_bool pcb_select_connection(pcb_board_t *pcb, pcb_bool Flag)
 		PCB_VIA_LOOP(pcb->Data);
 	{
 		if (PCB_FLAG_TEST(PCB_FLAG_FOUND, via) && !PCB_FLAG_TEST(PCB_FLAG_LOCK, via)) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, via, via, via);
+			pcb_undo_add_obj_to_flag(via);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, via);
-			DrawVia(via);
+			pcb_via_invalidate_draw(via);
 			changed = pcb_true;
 		}
 	}
@@ -687,9 +688,9 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 				&& pcb_text_is_visible(PCB, layer, text)
 				&& text->TextString && REGEXEC(text->TextString)
 				&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, text) != Flag) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_TEXT, layer, text, text);
+			pcb_undo_add_obj_to_flag(text);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, text);
-			DrawText(layer, text);
+			pcb_text_invalidate_draw(layer, text);
 			changed = pcb_true;
 		}
 	}
@@ -703,28 +704,28 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 				&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, element) != Flag) {
 			const char* name = PCB_ELEM_NAME_VISIBLE(PCB, element);
 			if (name && REGEXEC(name)) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_ELEMENT, element, element, element);
+				pcb_undo_add_obj_to_flag(element);
 				PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, element);
 				PCB_PIN_LOOP(element);
 				{
-					pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, pin, pin);
+					pcb_undo_add_obj_to_flag(pin);
 					PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, pin);
 				}
 				PCB_END_LOOP;
 				PCB_PAD_LOOP(element);
 				{
-					pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, element, pad, pad);
+					pcb_undo_add_obj_to_flag(pad);
 					PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, pad);
 				}
 				PCB_END_LOOP;
 				PCB_ELEMENT_PCB_TEXT_LOOP(element);
 				{
-					pcb_undo_add_obj_to_flag(PCB_TYPE_ELEMENT_NAME, element, text, text);
+					pcb_undo_add_obj_to_flag(text);
 					PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, text);
 				}
 				PCB_END_LOOP;
-				DrawElementName(element);
-				DrawElement(element);
+				pcb_elem_name_invalidate_draw(element);
+				pcb_elem_invalidate_draw(element);
 				changed = pcb_true;
 			}
 		}
@@ -736,9 +737,9 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 		if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, element)
 				&& pin->Name && REGEXEC(pin->Name)
 				&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, pin) != Flag) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_PIN, element, pin, pin);
+			pcb_undo_add_obj_to_flag(pin);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, pin);
-			DrawPin(pin);
+			pcb_pin_invalidate_draw(pin);
 			changed = pcb_true;
 		}
 	}
@@ -750,9 +751,9 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 				&& ((PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, pad) != 0) == PCB_SWAP_IDENT || pcb->InvisibleObjectsOn)
 				&& PCB_FLAG_TEST(PCB_FLAG_SELECTED, pad) != Flag)
 			if (pad->Name && REGEXEC(pad->Name)) {
-				pcb_undo_add_obj_to_flag(PCB_TYPE_PAD, element, pad, pad);
+				pcb_undo_add_obj_to_flag(pad);
 				PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, pad);
-				DrawPad(pad);
+				pcb_pad_invalidate_draw(pad);
 				changed = pcb_true;
 			}
 	}
@@ -762,9 +763,9 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 	{
 		if (!PCB_FLAG_TEST(PCB_FLAG_LOCK, via)
 				&& via->Name && REGEXEC(via->Name) && PCB_FLAG_TEST(PCB_FLAG_SELECTED, via) != Flag) {
-			pcb_undo_add_obj_to_flag(PCB_TYPE_VIA, via, via, via);
+			pcb_undo_add_obj_to_flag(via);
 			PCB_FLAG_ASSIGN(PCB_FLAG_SELECTED, Flag, via);
-			DrawVia(via);
+			pcb_via_invalidate_draw(via);
 			changed = pcb_true;
 		}
 	}
@@ -783,7 +784,7 @@ pcb_bool pcb_select_object_by_name(pcb_board_t *pcb, int Type, const char *name_
 			if (menu->Name && menu->Name[0] != '\0' && menu->Name[1] != '\0' && REGEXEC(menu->Name + 2)) {
 				for (i = menu->EntryN, entry = menu->Entry; i; i--, entry++)
 					if (pcb_rat_seek_pad(entry, &conn, pcb_false))
-						pcb_rat_find_hook(conn.type, conn.ptr1, conn.ptr2, conn.ptr2, pcb_true, pcb_true);
+						pcb_rat_find_hook(conn.obj, pcb_true, pcb_true);
 			}
 		}
 		PCB_END_LOOP;
