@@ -773,6 +773,46 @@ lht_doc_t *lht_dom_load(const char *fn, char **errmsg)
 	return doc;
 }
 
+lht_doc_t *lht_dom_load_string(const char *str, const char *fn, char **errmsg)
+{
+	lht_doc_t *doc;
+	int error = 0;
+
+	doc = lht_dom_init();
+	lht_dom_loc_newfile(doc, fn);
+
+	while(*str != '\0') {
+		lht_err_t err;
+		int c = *str++;
+		err = lht_dom_parser_char(doc, c);
+		if (err != LHTE_SUCCESS) {
+			if (err != LHTE_STOP) {
+				if (errmsg != NULL) {
+					const char *fn;
+					int line, col;
+					const char *msg;
+
+					lht_dom_loc_active(doc, &fn, &line, &col);
+					msg = lht_err_str(err);
+					*errmsg = malloc(strlen(msg) + strlen(fn) + 128);
+					sprintf(*errmsg, "%s (%s:%d.%d)\n", msg, fn, line+1, col+1);
+				}
+				error = 1;
+				break;
+			}
+			break; /* error or stop, do not read anymore (would get LHTE_STOP without any processing all the time) */
+		}
+	}
+
+	if (error) {
+		lht_dom_uninit(doc);
+		doc = NULL;
+	}
+
+	return doc;
+}
+
+
 lht_node_t *lht_dom_duptree(lht_node_t *src)
 {
 	lht_dom_iterator_t it;
