@@ -67,7 +67,7 @@ static pcb_text_t *dtext(int x, int y, int scale, int dir, const char *txt)
 	t.Scale = scale;
 	t.fid = 0; /* use the default font */
 	t.Flags = pcb_no_flags();
-	DrawTextLowLevel(&t, 0);
+	pcb_text_draw_(&t, 0, 0);
 	return &t;
 }
 
@@ -83,7 +83,7 @@ static pcb_text_t *dtext_(pcb_coord_t x, pcb_coord_t y, int scale, int dir, cons
 	t.Scale = scale;
 	t.fid = 0; /* use the default font */
 	t.Flags = pcb_no_flags();
-	DrawTextLowLevel(&t, th);
+	pcb_text_draw_(&t, th, 0);
 	return &t;
 }
 
@@ -102,11 +102,11 @@ static pcb_text_t *dtext_bg(pcb_hid_gc_t gc, int x, int y, int scale, int dir, c
 
 	if (pcb_gui->gui) { /* it is unreadable on black&white and on most exporters */
 		pcb_gui->set_color(gc, bgcolor);
-		DrawTextLowLevel(&t, 1000000);
+		pcb_text_draw_(&t, 1000000, 0);
 	}
 
 	pcb_gui->set_color(gc, fgcolor);
-	DrawTextLowLevel(&t, 0);
+	pcb_text_draw_(&t, 0, 0);
 
 	return &t;
 }
@@ -122,7 +122,7 @@ static void dline(int x1, int y1, int x2, int y2, float thick)
 	l.Thickness = PCB_MM_TO_COORD(thick);
 	if (l.Thickness == 0)
 		l.Thickness = 1;
-	_draw_line(&l);
+	pcb_line_draw_(&l, 0);
 }
 
 /* draw a line of a specific thickness */
@@ -136,7 +136,7 @@ static void dline_(pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y
 	l.Thickness = PCB_MM_TO_COORD(thick);
 	if (l.Thickness == 0)
 		l.Thickness = 1;
-	_draw_line(&l);
+	pcb_line_draw_(&l, 0);
 }
 
 
