@@ -51,6 +51,7 @@
 #include "hid_helper.h"
 #include "layer.h"
 #include "compat_misc.h"
+#include "safe_fs.h"
 
 #include "hid.h"
 #include <gd.h>
@@ -309,7 +310,7 @@ static void gcode_start_png(const char *basename, const char *suffix)
 
 	/* Nelma only works with true color images */
 	gcode_im = gdImageCreate(w, h);
-	gcode_f = fopen(buf, "wb");
+	gcode_f = pcb_fopen(buf, "wb");
 
 	gcode_alloc_colors();
 
@@ -442,7 +443,7 @@ static void gcode_do_export(pcb_hid_attr_val_t * options)
 					BM_PUT(bm, r, c, p);
 				}
 			}
-			gcode_f2 = fopen(filename, "wb");
+			gcode_f2 = pcb_fopen(filename, "wb");
 			if (!gcode_f2) {
 				perror(filename);
 				return;
@@ -487,7 +488,7 @@ static void gcode_do_export(pcb_hid_attr_val_t * options)
 				d = 0;
 				drill = sort_drill(drill, n_drill);
 				sprintf(filename, "%s.drill.cnc", gcode_basename);
-				gcode_f2 = fopen(filename, "wb");
+				gcode_f2 = pcb_fopen(filename, "wb");
 				if (!gcode_f2) {
 					perror(filename);
 					return;
@@ -587,11 +588,6 @@ static void gcode_destroy_gc(pcb_hid_gc_t gc)
 	free(gc);
 }
 
-static void gcode_use_mask(pcb_mask_op_t use_it)
-{
-	/* does nothing */
-}
-
 static void gcode_set_color(pcb_hid_gc_t gc, const char *name)
 {
 	if (gcode_im == NULL) {
@@ -896,7 +892,6 @@ int pplg_init_export_gcode(void)
 	gcode_hid.name = "gcode";
 	gcode_hid.description = "G-CODE export";
 	gcode_hid.exporter = 1;
-	gcode_hid.poly_before = 1;
 
 	gcode_hid.get_export_options = gcode_get_export_options;
 	gcode_hid.do_export = gcode_do_export;
@@ -904,7 +899,6 @@ int pplg_init_export_gcode(void)
 	gcode_hid.set_layer_group = gcode_set_layer_group;
 	gcode_hid.make_gc = gcode_make_gc;
 	gcode_hid.destroy_gc = gcode_destroy_gc;
-	gcode_hid.use_mask = gcode_use_mask;
 	gcode_hid.set_color = gcode_set_color;
 	gcode_hid.set_line_cap = gcode_set_line_cap;
 	gcode_hid.set_line_width = gcode_set_line_width;
