@@ -22,6 +22,7 @@
 #include "plugins.h"
 #include "hid_helper.h"
 #include "compat_misc.h"
+#include "safe_fs.h"
 
 #include "hid.h"
 #include "hid_nogui.h"
@@ -35,16 +36,17 @@ const char *gerber_cookie = "gerber HID";
 
 #define CRASH(func) fprintf(stderr, "HID error: pcb called unimplemented Gerber function %s.\n", func); abort()
 
-/*----------------------------------------------------------------------------*/
-/* Function prototypes                                                        */
-/*----------------------------------------------------------------------------*/
+#define SUFF_LEN 256
+
+#if SUFF_LEN < PCB_DERIVE_FN_SUFF_LEN
+#	error SUFF_LEN needs at least PCB_DERIVE_FN_SUFF_LEN
+#endif
 
 static pcb_hid_attribute_t *gerber_get_export_options(int *n);
 static void gerber_do_export(pcb_hid_attr_val_t * options);
 static void gerber_parse_arguments(int *argc, char ***argv);
 static pcb_hid_gc_t gerber_make_gc(void);
 static void gerber_destroy_gc(pcb_hid_gc_t gc);
-static void gerber_use_mask(pcb_mask_op_t use_it);
 static void gerber_set_color(pcb_hid_gc_t gc, const char *name);
 static void gerber_set_line_cap(pcb_hid_gc_t gc, pcb_cap_style_t style);
 static void gerber_set_line_width(pcb_hid_gc_t gc, pcb_coord_t width);
@@ -80,13 +82,12 @@ static int verbose;
 static int all_layers;
 static int is_mask, was_drill;
 static int is_drill;
-static int current_mask;
+static pcb_composite_op_t gerber_drawing_mode, drawing_mode_issued;
 static int flash_drills;
 static int copy_outline_mode;
 static int name_style;
 static int want_cross_sect;
 static pcb_layer_t *outline_layer;
-static int mask_issued;
 static int gerber_debug;
 
 enum ApertureShape {
@@ -305,6 +306,8 @@ static const char *name_style_names[] = {
 	"eagle",
 #define NAME_STYLE_HACKVANA 4
 	"hackvana",
+#define NAME_STYLE_UNIVERSAL 5
+	"universal",
 	NULL
 };
 
@@ -491,11 +494,68 @@ static void assign_hackvana_file_suffix(char *dest, pcb_layer_id_t lid, unsigned
 	strcpy(dest, suff);
 }
 
+/* Very similar to layer_type_to_file_name() but appends the group name _and_ the magic suffix */
+static void assign_universal_file_suffix(char *dest, pcb_layergrp_id_t gid, unsigned int flags)
+{
+	char *suff;
+	int name_len;
+	pcb_layergrp_t *g;
+
+	if (fmatch(flags, PCB_LYT_TOP | PCB_LYT_COPPER))
+		suff = "gtl";
+	else if (fmatch(flags, PCB_LYT_BOTTOM | PCB_LYT_COPPER))
+		suff = "gbl";
+	else if (fmatch(flags, PCB_LYT_TOP | PCB_LYT_SILK))
+		suff = "gto";
+	else if (fmatch(flags, PCB_LYT_BOTTOM | PCB_LYT_SILK))
+		suff = "gbo";
+	else if (fmatch(flags, PCB_LYT_TOP | PCB_LYT_MASK))
+		suff = "gts";
+	else if (fmatch(flags, PCB_LYT_BOTTOM | PCB_LYT_MASK))
+		suff = "gbs";
+	else if (fmatch(flags, PCB_LYT_PDRILL))
+		suff = "drl";
+	else if (fmatch(flags, PCB_LYT_UDRILL))
+		suff = "_NPTH.drl";
+	else if (fmatch(flags, PCB_LYT_TOP | PCB_LYT_PASTE))
+		suff = "gtp";
+	else if (fmatch(flags, PCB_LYT_BOTTOM | PCB_LYT_PASTE))
+		suff = "gbp";
+	else if (fmatch(flags, PCB_LYT_INVIS))
+		suff = "inv";
+	else if (fmatch(flags, PCB_LYT_FAB))
+		suff = "fab";
+	else if (fmatch(flags, PCB_LYT_TOP | PCB_LYT_ASSY))
+		suff = "ast";
+	else if (fmatch(flags, PCB_LYT_BOTTOM | PCB_LYT_ASSY))
+		suff = "asb";
+	else if (fmatch(flags, PCB_LYT_OUTLINE))
+		suff = "gko";
+	else {
+		static char buf[20];
+		sprintf(buf, "g%ld", gid);
+		suff = buf;
+	}
+
+	/* insert group name if available */
+	g = pcb_get_layergrp(PCB, gid);
+	if (g != NULL) {
+		name_len = strlen(g->name);
+		if (name_len >= SUFF_LEN-5)
+			name_len = SUFF_LEN-5; /* truncate group name */
+		memcpy(dest, g->name, name_len);
+		dest += name_len;
+		*dest = '.';
+		dest++;
+	}
+	strcpy(dest, suff);
+}
+
 
 #undef fmatch
 
 
-static void assign_file_suffix(char *dest, pcb_layer_id_t lid, unsigned int flags)
+static void assign_file_suffix(char *dest, pcb_layergrp_id_t gid, pcb_layer_id_t lid, unsigned int flags)
 {
 	int fns_style;
 	const char *sext = ".gbr";
@@ -517,6 +577,9 @@ static void assign_file_suffix(char *dest, pcb_layer_id_t lid, unsigned int flag
 	case NAME_STYLE_HACKVANA:
 		assign_hackvana_file_suffix(dest, lid, flags);
 		return;
+	case NAME_STYLE_UNIVERSAL:
+		assign_universal_file_suffix(dest, gid, flags);
+		return;
 	}
 
 	if ((flags & PCB_LYT_PDRILL) || (flags & PCB_LYT_UDRILL))
@@ -537,6 +600,8 @@ static void gerber_do_export(pcb_hid_attr_val_t * options)
 	conf_force_set_bool(conf_core.editor.thin_draw_poly, 0);
 	conf_force_set_bool(conf_core.editor.check_planes, 0);
 
+	drawing_mode_issued = PCB_HID_COMP_POSITIVE;
+
 	if (!options) {
 		gerber_get_export_options(NULL);
 		for (i = 0; i < NUM_OPTIONS; i++)
@@ -564,7 +629,7 @@ static void gerber_do_export(pcb_hid_attr_val_t * options)
 			outline_layer = PCB->Data->Layer + i;
 
 	i = strlen(fnbase);
-	filename = (char *) realloc(filename, i + 40);
+	filename = (char *) realloc(filename, i + SUFF_LEN);
 	strcpy(filename, fnbase);
 	strcat(filename, ".");
 	filesuff = filename + strlen(filename);
@@ -600,7 +665,7 @@ static void gerber_do_export(pcb_hid_attr_val_t * options)
 	maybe_close_f(f);
 	f = NULL;
 	pcb_hid_restore_layer_ons(save_ons);
-	conf_update(NULL); /* resotre forced sets */
+	conf_update(NULL, -1); /* resotre forced sets */
 }
 
 static void gerber_parse_arguments(int *argc, char ***argv)
@@ -676,7 +741,6 @@ static int gerber_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer,
 
 	is_drill = ((flags & PCB_LYT_PDRILL) || (flags & PCB_LYT_UDRILL));
 	is_mask = !!(flags & PCB_LYT_MASK);
-	current_mask = 0;
 	if (group < 0 || group != lastgroup) {
 		time_t currenttime;
 		char utcTime[64];
@@ -702,8 +766,8 @@ static int gerber_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer,
 		f = NULL;
 
 		pagecount++;
-		assign_file_suffix(filesuff, layer, flags);
-		f = fopen(filename, "wb");	/* Binary needed to force CR-LF */
+		assign_file_suffix(filesuff, group, layer, flags);
+		f = pcb_fopen(filename, "wb"); /* Binary needed to force CR-LF */
 		if (f == NULL) {
 			pcb_message(PCB_MSG_ERROR, "Error:  Could not open %s for writing.\n", filename);
 			return 1;
@@ -821,14 +885,11 @@ static void gerber_destroy_gc(pcb_hid_gc_t gc)
 	free(gc);
 }
 
-
-static void gerber_use_mask(pcb_mask_op_t use_it)
+static void gerber_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct, const pcb_box_t *drw_screen)
 {
-	current_mask = use_it;
+	gerber_drawing_mode = op;
 	if ((f != NULL) && (gerber_debug))
-		fprintf(f, "G04 hid debug mask: %d*\r\n", use_it);
-	if ((use_it == HID_MASK_OFF) || (use_it == HID_MASK_INIT))
-		mask_issued = -1;
+		fprintf(f, "G04 hid debug composite: %d*\r\n", op);
 }
 
 static void gerber_set_color(pcb_hid_gc_t gc, const char *name)
@@ -867,14 +928,14 @@ static void gerber_set_draw_xor(pcb_hid_gc_t gc, int xor_)
 
 static void use_gc(pcb_hid_gc_t gc, int radius)
 {
-	if ((f != NULL) && (current_mask != mask_issued)) {
-		if (current_mask == HID_MASK_SET) {
+	if ((f != NULL) && (gerber_drawing_mode != drawing_mode_issued)) {
+		if (gerber_drawing_mode == PCB_HID_COMP_POSITIVE) {
 			fprintf(f, "%%LPD*%%\r\n");
-			mask_issued = current_mask;
+			drawing_mode_issued = gerber_drawing_mode;
 		}
-		else if (current_mask == HID_MASK_CLEAR) {
+		else if (gerber_drawing_mode == PCB_HID_COMP_NEGATIVE) {
 			fprintf(f, "%%LPC*%%\r\n");
-			mask_issued = current_mask;
+			drawing_mode_issued = gerber_drawing_mode;
 		}
 	}
 
@@ -1148,7 +1209,7 @@ static void gerber_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x,
 	int firstTime = 1;
 	pcb_coord_t startX = 0, startY = 0;
 
-	if (is_mask && current_mask == HID_MASK_BEFORE)
+	if (is_mask && (gerber_drawing_mode != PCB_HID_COMP_POSITIVE) && (gerber_drawing_mode != PCB_HID_COMP_NEGATIVE))
 		return;
 
 	use_gc(gc, 10 * 100);
@@ -1251,7 +1312,7 @@ int pplg_init_export_gerber(void)
 	gerber_hid.set_layer_group = gerber_set_layer_group;
 	gerber_hid.make_gc = gerber_make_gc;
 	gerber_hid.destroy_gc = gerber_destroy_gc;
-	gerber_hid.use_mask = gerber_use_mask;
+	gerber_hid.set_drawing_mode = gerber_set_drawing_mode;
 	gerber_hid.set_color = gerber_set_color;
 	gerber_hid.set_line_cap = gerber_set_line_cap;
 	gerber_hid.set_line_width = gerber_set_line_width;
