@@ -1,12 +1,8 @@
-/*!
+/*
  *
- * \brief IPC-D-356 Netlist export.
+ * IPC-D-356 Netlist export.
  *
- * \author Copyright (C) 2012 Jerome Marchand (Jerome.Marchand@gmail.com)
- *
- * <hr>
- *
- * <h1><b>Copyright.</b></h1>\n
+ * Copyright (C) 2012 Jerome Marchand (Jerome.Marchand@gmail.com)
  *
  * PCB, interactive printed circuit board design
  *
@@ -48,6 +44,8 @@
 #include "netlist.h"
 #include "conf_core.h"
 #include "obj_pinvia.h"
+#include "compat_misc.h"
+#include "safe_fs.h"
 
 #include "hid.h"
 #include "hid_nogui.h"
@@ -125,7 +123,6 @@ static pcb_hid_attribute_t *IPCD356_get_export_options(int *n)
  */
 void IPCD356_WriteHeader(FILE * fd)
 {
-	time_t currenttime;
 	char utcTime[64];
 
 	pcb_print_utc(utcTime, sizeof utcTime, 0);
@@ -401,7 +398,7 @@ int IPCD356_Netlist(void)
 	if (IPCD356_filename == NULL)
 		return 1;
 
-	fp = fopen(IPCD356_filename, "w+");
+	fp = pcb_fopen(IPCD356_filename, "w+");
 	if (fp == NULL) {
 		pcb_message(PCB_MSG_ERROR, "error opening %s\n", IPCD356_filename);
 		return 1;
