@@ -68,6 +68,7 @@
 #include "rats.h"
 #include "plugins.h"
 #include "hid_helper.h"
+#include "safe_fs.h"
 
 #include "hid.h"
 #include "hid_nogui.h"
@@ -538,7 +539,7 @@ static void nelma_start_png(const char *basename, const char *suffix)
 
 	/* Nelma only works with true color images */
 	nelma_im = gdImageCreate(w, h);
-	nelma_f = fopen(buf, "wb");
+	nelma_f = pcb_fopen(buf, "wb");
 
 	nelma_alloc_colors();
 
@@ -629,7 +630,7 @@ static void nelma_do_export(pcb_hid_attr_val_t * options)
 	buf = (char *) malloc(sizeof(*buf) * len);
 
 	sprintf(buf, "%s.em", nelma_basename);
-	nelma_config = fopen(buf, "w");
+	nelma_config = pcb_fopen(buf, "w");
 
 	free(buf);
 
@@ -693,11 +694,6 @@ static void nelma_destroy_gc(pcb_hid_gc_t gc)
 	free(gc);
 }
 
-static void nelma_use_mask(pcb_mask_op_t use_it)
-{
-	/* does nothing */
-}
-
 static void nelma_set_color(pcb_hid_gc_t gc, const char *name)
 {
 	if (nelma_im == NULL) {
@@ -968,6 +964,7 @@ int pplg_check_ver_export_nelma(int ver_needed) { return 0; }
 int pplg_uninit_export_nelma(void)
 {
 	pcb_hid_remove_attributes_by_cookie(nelma_cookie);
+	return 0;
 }
 
 int pplg_init_export_nelma(void)
@@ -981,7 +978,6 @@ int pplg_init_export_nelma(void)
 	nelma_hid.name = "nelma";
 	nelma_hid.description = "Numerical analysis package export";
 	nelma_hid.exporter = 1;
-	nelma_hid.poly_before = 1;
 
 	nelma_hid.get_export_options = nelma_get_export_options;
 	nelma_hid.do_export = nelma_do_export;
@@ -989,7 +985,6 @@ int pplg_init_export_nelma(void)
 	nelma_hid.set_layer_group = nelma_set_layer_group;
 	nelma_hid.make_gc = nelma_make_gc;
 	nelma_hid.destroy_gc = nelma_destroy_gc;
-	nelma_hid.use_mask = nelma_use_mask;
 	nelma_hid.set_color = nelma_set_color;
 	nelma_hid.set_line_cap = nelma_set_line_cap;
 	nelma_hid.set_line_width = nelma_set_line_width;
