@@ -1,7 +1,3 @@
-/* for popen() */
-#define _DEFAULT_SOURCE
-#define _BSD_SOURCE
-
 #include "config.h"
 
 #include <stdio.h>
@@ -20,6 +16,7 @@
 #include "pcb-printf.h"
 #include "plugins.h"
 #include "hid_helper.h"
+#include "safe_fs.h"
 
 #include "hid.h"
 #include "hid_nogui.h"
@@ -318,7 +315,7 @@ Paper width. Used for x-Axis calibration.
 %end-doc
 */
 	{"xcalib", "Paper width. Used for x-Axis calibration",
-	 PCB_HATT_REAL, 0, 0, {0, 0, 1.0}, 0, 0},
+	 PCB_HATT_REAL, 0, 2, {0, 0, 1.0}, 0, 0},
 #define HA_xcalib 14
 
 /* %start-doc options "91 Postscript Export"
@@ -329,7 +326,7 @@ Paper height. Used for y-Axis calibration.
 %end-doc
 */
 	{"ycalib", "Paper height. Used for y-Axis calibration",
-	 PCB_HATT_REAL, 0, 0, {0, 0, 1.0}, 0, 0},
+	 PCB_HATT_REAL, 0, 2, {0, 0, 1.0}, 0, 0},
 #define HA_ycalib 15
 
 /* %start-doc options "91 Postscript Export"
@@ -414,7 +411,7 @@ static struct {
 
 	double polygrid;
 
-	pcb_mask_op_t mask_mode;
+	pcb_composite_op_t drawing_mode;
 } global;
 
 static pcb_hid_attribute_t *ps_get_export_options(int *n)
@@ -575,7 +572,7 @@ static FILE *psopen(const char *base, const char *which)
 	char *buf, *suff, *buf2;
 
 	if (!global.multi_file)
-		return fopen(base, "w");
+		return pcb_fopen(base, "w");
 
 	buf = (char *) malloc(strlen(base) + strlen(which) + 5);
 
@@ -589,7 +586,7 @@ static FILE *psopen(const char *base, const char *which)
 		sprintf(buf, "%s.%s.ps", base, which);
 	}
 	printf("PS: open %s\n", buf);
-	ps_open_file = fopen(buf, "w");
+	ps_open_file = pcb_fopen(buf, "w");
 	free(buf);
 	return ps_open_file;
 }
@@ -686,7 +683,7 @@ void ps_hid_export_to_file(FILE * the_file, pcb_hid_attr_val_t * options)
 		fprintf(the_file, "showpage\n");
 
 	memcpy(pcb_layer_stack, saved_layer_stack, sizeof(pcb_layer_stack));
-	conf_update(NULL); /* restore forced sets */
+	conf_update(NULL, -1); /* restore forced sets */
 }
 
 static void ps_do_export(pcb_hid_attr_val_t * options)
@@ -962,7 +959,10 @@ static int ps_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, uns
 			global.outline_layer != NULL &&
 			global.outline_layer != pcb_get_layer(layer) &&
 			!(flags & PCB_LYT_OUTLINE)) {
+		int save_drill = global.is_drill;
+		global.is_drill = 0;
 		pcb_draw_layer(global.outline_layer, &global.exps.view);
+		global.is_drill = save_drill;
 	}
 
 	return 1;
@@ -981,20 +981,24 @@ static void ps_destroy_gc(pcb_hid_gc_t gc)
 	free(gc);
 }
 
-static void ps_use_mask(pcb_mask_op_t use_it)
+static void ps_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct, const pcb_box_t *screen)
 {
-	global.mask_mode = use_it;
+	global.drawing_mode = op;
 }
 
+
 static void ps_set_color(pcb_hid_gc_t gc, const char *name)
 {
-	if (global.mask_mode == HID_MASK_CLEAR) {
+	if (global.drawing_mode == PCB_HID_COMP_NEGATIVE) {
 		gc->r = gc->g = gc->b = 255;
 		gc->erase = 0;
 	}
-	else if (global.mask_mode == HID_MASK_SET) {
-		gc->r = gc->g = gc->b = 255;
-		gc->erase = 1;
+	else if (global.drawing_mode == PCB_HID_COMP_POSITIVE) {
+		gc->r = gc->g = gc->b = 0;
+		if (strcmp(name, "erase") == 0)
+			gc->erase = 1;
+		else
+			gc->erase = 0;
 	}
 	else if (strcmp(name, "erase") == 0 || strcmp(name, "drill") == 0) {
 		gc->r = gc->g = gc->b = 255;
@@ -1519,20 +1523,18 @@ void ps_calibrate_1(double xval, double yval, int use_command)
 		ps_calib_attribute_list[0].default_val.str_value = pcb_strdup("lpr");
 	}
 
-	if (pcb_gui->
-			attribute_dialog(ps_calib_attribute_list, 1, vals, _("Print Calibration Page"),
-											 _("Generates a printer calibration page")))
+	if (pcb_gui->attribute_dialog(ps_calib_attribute_list, 1, vals, _("Print Calibration Page"), _("Generates a printer calibration page"), NULL))
 		return;
 
 	if (use_command || strchr(vals[0].str_value, '|')) {
 		const char *cmd = vals[0].str_value;
 		while (*cmd == ' ' || *cmd == '|')
 			cmd++;
-		ps_cal_file = popen(cmd, "w");
+		ps_cal_file = pcb_popen(cmd, "w");
 		used_popen = 1;
 	}
 	else
-		ps_cal_file = fopen(vals[0].str_value, "w");
+		ps_cal_file = pcb_fopen(vals[0].str_value, "w");
 
 	for (c = 0; calib_lines[c]; c++)
 		fputs(calib_lines[c], ps_cal_file);
@@ -1550,7 +1552,7 @@ void ps_calibrate_1(double xval, double yval, int use_command)
 	fprintf(ps_cal_file, "%%%%EOF\n");
 
 	if (used_popen)
-		pclose(ps_cal_file);
+		pcb_pclose(ps_cal_file);
 	else
 		fclose(ps_cal_file);
 }
@@ -1592,7 +1594,7 @@ void ps_ps_init(pcb_hid_t * hid)
 	hid->set_layer_group = ps_set_layer_group;
 	hid->make_gc = ps_make_gc;
 	hid->destroy_gc = ps_destroy_gc;
-	hid->use_mask = ps_use_mask;
+	hid->set_drawing_mode = ps_set_drawing_mode;
 	hid->set_color = ps_set_color;
 	hid->set_line_cap = ps_set_line_cap;
 	hid->set_line_width = ps_set_line_width;
