@@ -25,41 +25,46 @@ static void ghid_gdk_screen_update(void);
 */
 #define USE_GC(gc) if (!use_gc(gc)) return
 
-static int cur_mask = -1;
-static int mask_seq = 0;
-
 typedef struct render_priv_s {
 	GdkGC *bg_gc;
 	GdkColor bg_color;
 	GdkGC *offlimits_gc;
 	GdkColor offlimits_color;
-	GdkGC *mask_gc;
-	GdkGC *u_gc;
 	GdkGC *grid_gc;
+	GdkGC *clear_gc, *copy_gc;
 	GdkColor grid_color;
-	GdkPixmap *pixmap, *mask;
-	GdkDrawable *drawable;				/* Current drawable for drawing routines */
-	pcb_bool clip;
 	GdkRectangle clip_rect;
+	pcb_bool clip_rect_valid;
+	pcb_bool direct;
 	int attached_invalidate_depth;
 	int mark_invalidate_depth;
+
+	/* available canvases */
+	GdkPixmap *base_pixel;                  /* base canvas, pixel colors only */
+	GdkPixmap *sketch_pixel, *sketch_clip;  /* sketch canvas for compositing (non-direct) */
+
+	/* currently active: drawing routines draw to these*/
+	GdkDrawable *out_pixel, *out_clip;
+	GdkGC *pixel_gc, *clip_gc;
+	GdkColor clip_color;
 } render_priv_t;
 
 
 typedef struct hid_gc_s {
 	pcb_hid_t *me_pointer;
-	GdkGC *gc;
+	GdkGC *pixel_gc;
+	GdkGC *clip_gc;
 
 	gchar *colorname;
 	pcb_coord_t width;
 	gint cap, join;
 	gchar xor_mask;
-	gint mask_seq;
+	gint sketch_seq;
 } hid_gc_s;
 
 static void draw_lead_user(render_priv_t *priv_);
 
-static const gchar *get_color_name(GdkColor * color)
+static const gchar *get_color_name(pcb_gtk_color_t * color)
 {
 	static char tmp[16];
 
@@ -71,7 +76,7 @@ static const gchar *get_color_name(GdkColor * color)
 }
 
 /** Returns TRUE only if \p color_string has been allocated to \p color. */
-static pcb_bool map_color_string(const char *color_string, GdkColor * color)
+static pcb_bool map_color_string(const char *color_string, pcb_gtk_color_t * color)
 {
 	static GdkColormap *colormap = NULL;
 	pcb_bool parsed;
@@ -138,8 +143,10 @@ static int ghid_gdk_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t laye
 
 static void ghid_gdk_destroy_gc(pcb_hid_gc_t gc)
 {
-	if (gc->gc)
-		g_object_unref(gc->gc);
+	if (gc->pixel_gc)
+		g_object_unref(gc->pixel_gc);
+	if (gc->clip_gc)
+		g_object_unref(gc->clip_gc);
 	if (gc->colorname != NULL)
 		g_free(gc->colorname);
 	g_free(gc);
@@ -155,12 +162,11 @@ static pcb_hid_gc_t ghid_gdk_make_gc(void)
 	return rv;
 }
 
-static void set_clip(render_priv_t * priv, GdkGC * gc)
+static void set_clip(render_priv_t *priv, GdkGC *gc)
 {
 	if (gc == NULL)
 		return;
-
-	if (priv->clip)
+	if (priv->clip_rect_valid)
 		gdk_gc_set_clip_rectangle(gc, &priv->clip_rect);
 	else
 		gdk_gc_set_clip_mask(gc, NULL);
@@ -222,7 +228,7 @@ static inline void ghid_gdk_draw_grid_global(void)
 	for (y = y1; y <= y2; y += grd) {
 		for (i = 0; i < n; i++)
 			points[i].y = Vy(y);
-		gdk_draw_points(priv->drawable, priv->grid_gc, points, n);
+		gdk_draw_points(priv->out_pixel, priv->grid_gc, points, n);
 	}
 }
 
@@ -277,7 +283,7 @@ static void ghid_gdk_draw_grid_local_(pcb_coord_t cx, pcb_coord_t cy, int radius
 		points_abs[n].y = Vy(points_base[n].y + cy);
 	}
 
-	gdk_draw_points(priv->drawable, priv->grid_gc, points_abs, npoints);
+	gdk_draw_points(priv->out_pixel, priv->grid_gc, points_abs, npoints);
 }
 
 
@@ -327,7 +333,7 @@ static void ghid_gdk_draw_grid(void)
 			priv->grid_color.blue ^= priv->bg_color.blue;
 			gdk_color_alloc(colormap, &priv->grid_color);
 		}
-		priv->grid_gc = gdk_gc_new(priv->drawable);
+		priv->grid_gc = gdk_gc_new(priv->out_pixel);
 		gdk_gc_set_function(priv->grid_gc, GDK_XOR);
 		gdk_gc_set_foreground(priv->grid_gc, &priv->grid_color);
 		gdk_gc_set_clip_origin(priv->grid_gc, 0, 0);
@@ -392,76 +398,74 @@ static void ghid_gdk_draw_bg_image(void)
 	}
 
 	if (pixbuf)
-		gdk_pixbuf_render_to_drawable(pixbuf, priv->drawable, priv->bg_gc, src_x, src_y, dst_x, dst_y, w - src_x, h - src_y, GDK_RGB_DITHER_NORMAL, 0, 0);
+		gdk_pixbuf_render_to_drawable(pixbuf, priv->out_pixel, priv->bg_gc, src_x, src_y, dst_x, dst_y, w - src_x, h - src_y, GDK_RGB_DITHER_NORMAL, 0, 0);
 }
 
-#define WHICH_GC(gc) (((cur_mask == HID_MASK_CLEAR) || (cur_mask == HID_MASK_SET)) ? priv->mask_gc : (gc)->gc)
+static pcb_composite_op_t curr_drawing_mode;
 
-static void ghid_mask_setup(render_priv_t *priv)
+static void ghid_sketch_setup(render_priv_t *priv)
 {
-	if (!priv->mask)
-		priv->mask = gdk_pixmap_new(0, gport->view.canvas_width, gport->view.canvas_height, 1);
-	priv->drawable = priv->mask;
-	if (!priv->mask_gc) {
-		priv->mask_gc = gdk_gc_new(priv->drawable);
-		gdk_gc_set_clip_origin(priv->mask_gc, 0, 0);
-		set_clip(priv, priv->mask_gc);
-	}
+	if (priv->sketch_pixel == NULL)
+		priv->sketch_pixel = gdk_pixmap_new(gtk_widget_get_window(gport->drawing_area), gport->view.canvas_width, gport->view.canvas_height, -1);
+	if (priv->sketch_clip == NULL)
+		priv->sketch_clip = gdk_pixmap_new(0, gport->view.canvas_width, gport->view.canvas_height, 1);
+
+	priv->out_pixel = priv->sketch_pixel;
+	priv->out_clip = priv->sketch_clip;
 }
 
-static void ghid_gdk_use_mask(pcb_mask_op_t use_it)
+static void ghid_gdk_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct, const pcb_box_t *screen)
 {
-	static int mask_seq_id = 0;
-	GdkColor color;
 	render_priv_t *priv = gport->render_priv;
 
-	if (!priv->pixmap)
+	if (!priv->base_pixel) {
+		abort();
 		return;
-	if (use_it == cur_mask)
-		return;
-	switch (use_it) {
-	case HID_MASK_OFF:
-		priv->drawable = priv->pixmap;
-		mask_seq = 0;
-		break;
-
-	case HID_MASK_BEFORE:
-		/* The HID asks not to receive this mask type, so warn if we get it */
-		g_return_if_reached();
-
-	case HID_MASK_INIT:
-		ghid_mask_setup(priv);
-		mask_seq = 0;
-
-		/* clear the mask */
-		color.pixel = 0;
-		gdk_gc_set_foreground(priv->mask_gc, &color);
-		gdk_draw_rectangle(priv->drawable, priv->mask_gc, TRUE, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
-		break;
-
-	case HID_MASK_CLEAR:
-		color.pixel = 0;
-		gdk_gc_set_foreground(priv->mask_gc, &color);
-		break;
-
-	case HID_MASK_SET:
-		color.pixel = 1;
-		gdk_gc_set_foreground(priv->mask_gc, &color);
-		break;
-
-	case HID_MASK_AFTER:
-		mask_seq_id++;
-		if (!mask_seq_id)
-			mask_seq_id = 1;
-		mask_seq = mask_seq_id;
-
-		priv->drawable = priv->pixmap;
-		break;
-
 	}
-	cur_mask = use_it;
-}
 
+	priv->direct = direct;
+
+	if (direct) {
+		priv->out_pixel = priv->base_pixel;
+		priv->out_clip = NULL;
+		curr_drawing_mode = PCB_HID_COMP_POSITIVE;
+		return;
+	}
+
+	switch(op) {
+		case PCB_HID_COMP_RESET:
+			ghid_sketch_setup(priv);
+
+		/* clear the canvas */
+			priv->clip_color.pixel = 0;
+			if (priv->clear_gc == NULL)
+				priv->clear_gc = gdk_gc_new(priv->out_clip);
+			gdk_gc_set_foreground(priv->clear_gc, &priv->clip_color);
+			set_clip(priv, priv->clear_gc);
+			gdk_draw_rectangle(priv->out_clip, priv->clear_gc, TRUE, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
+			break;
+
+		case PCB_HID_COMP_POSITIVE:
+			priv->clip_color.pixel = 1;
+			break;
+
+		case PCB_HID_COMP_NEGATIVE:
+			priv->clip_color.pixel = 0;
+			break;
+
+		case PCB_HID_COMP_FLUSH:
+			if (priv->copy_gc == NULL)
+				priv->copy_gc = gdk_gc_new(priv->out_pixel);
+			gdk_gc_set_clip_mask(priv->copy_gc, priv->sketch_clip);
+			gdk_gc_set_clip_origin(priv->copy_gc, 0, 0);
+			gdk_draw_drawable(priv->base_pixel, priv->copy_gc, priv->sketch_pixel, 0, 0, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
+
+			priv->out_pixel = priv->base_pixel;
+			priv->out_clip = NULL;
+			break;
+	}
+	curr_drawing_mode = op;
+}
 
 typedef struct {
 	int color_set;
@@ -490,7 +494,7 @@ static void set_special_grid_color(void)
 		gdk_gc_set_foreground(priv->grid_gc, &priv->grid_color);
 }
 
-static void ghid_gdk_set_special_colors(conf_native_t *cfg)
+static void ghid_gdk_set_special_colors(conf_native_t *cfg, int arr_idx)
 {
 	render_priv_t *priv = gport->render_priv;
 
@@ -533,16 +537,15 @@ static void ghid_gdk_set_color(pcb_hid_gc_t gc, const char *name)
 		gc->colorname = g_strdup(name);
 	}
 
-	if (!gc->gc)
+	if (!gc->pixel_gc)
 		return;
 	if (colormap == NULL)
 		colormap = gtk_widget_get_colormap(gport->top_window);
 
 	if (strcmp(name, "erase") == 0) {
-		gdk_gc_set_foreground(gc->gc, &priv->bg_color);
 	}
 	else if (strcmp(name, "drill") == 0) {
-		gdk_gc_set_foreground(gc->gc, &priv->offlimits_color);
+		gdk_gc_set_foreground(gc->pixel_gc, &priv->offlimits_color);
 	}
 	else {
 		ColorCache *cc;
@@ -570,18 +573,16 @@ static void ghid_gdk_set_color(pcb_hid_gc_t gc, const char *name)
 				gdk_color_alloc(colormap, &cc->xor_color);
 				cc->xor_set = 1;
 			}
-			gdk_gc_set_foreground(gc->gc, &cc->xor_color);
+			gdk_gc_set_foreground(gc->pixel_gc, &cc->xor_color);
 		}
 		else {
-			gdk_gc_set_foreground(gc->gc, &cc->color);
+			gdk_gc_set_foreground(gc->pixel_gc, &cc->color);
 		}
 	}
 }
 
 static void ghid_gdk_set_line_cap(pcb_hid_gc_t gc, pcb_cap_style_t style)
 {
-	render_priv_t *priv = gport->render_priv;
-
 	switch (style) {
 	case Trace_Cap:
 	case Round_Cap:
@@ -594,56 +595,84 @@ static void ghid_gdk_set_line_cap(pcb_hid_gc_t gc, pcb_cap_style_t style)
 		gc->join = GDK_JOIN_MITER;
 		break;
 	}
-	if (gc->gc)
-		gdk_gc_set_line_attributes(WHICH_GC(gc), Vz(gc->width), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
+	if (gc->pixel_gc)
+		gdk_gc_set_line_attributes(gc->pixel_gc, Vz(gc->width), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
 }
 
 static void ghid_gdk_set_line_width(pcb_hid_gc_t gc, pcb_coord_t width)
 {
-	render_priv_t *priv = gport->render_priv;
-
 	gc->width = width;
-	if (gc->gc)
-		gdk_gc_set_line_attributes(WHICH_GC(gc), Vz(gc->width), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
+	if (gc->pixel_gc)
+		gdk_gc_set_line_attributes(gc->pixel_gc, Vz(gc->width), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
+	if (gc->clip_gc)
+		gdk_gc_set_line_attributes(gc->clip_gc, Vz(gc->width), GDK_LINE_SOLID, (GdkCapStyle) gc->cap, (GdkJoinStyle) gc->join);
 }
 
 static void ghid_gdk_set_draw_xor(pcb_hid_gc_t gc, int xor_mask)
 {
+	render_priv_t *priv = gport->render_priv;
+
 	gc->xor_mask = xor_mask;
-	if (!gc->gc)
-		return;
-	gdk_gc_set_function(gc->gc, xor_mask ? GDK_XOR : GDK_COPY);
+	if (gc->pixel_gc != NULL)
+		gdk_gc_set_function(gc->pixel_gc, xor_mask ? GDK_XOR : GDK_COPY);
+	if (gc->clip_gc != NULL)
+		gdk_gc_set_function(gc->clip_gc, xor_mask ? GDK_XOR : GDK_COPY);
 	ghid_gdk_set_color(gc, gc->colorname);
+
+	/* If not in direct mode then select the correct drawables so that the sketch and clip
+	 * drawables are not drawn to in XOR mode
+	 */
+	if(!priv->direct)	{
+		/* If xor mode then draw directly to the base drawable */
+		if(xor_mask) {
+			priv->out_pixel = priv->base_pixel;
+			priv->out_clip = NULL;
+		}
+		/* If not in direct mode then draw to the sketch and clip drawables */
+		else
+			ghid_sketch_setup(priv);
+	}
 }
 
 static int use_gc(pcb_hid_gc_t gc)
 {
 	render_priv_t *priv = gport->render_priv;
 	GdkWindow *window = gtk_widget_get_window(gport->top_window);
+	int need_setup = 0;
+
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
 
 	if (gc->me_pointer != &gtk2_gdk_hid) {
 		fprintf(stderr, "Fatal: GC from another HID passed to GTK HID\n");
 		abort();
 	}
 
-	if (!priv->pixmap)
+	if (!priv->base_pixel)
 		return 0;
-	if (!gc->gc) {
-		gc->gc = gdk_gc_new(window);
+
+	if ((!gc->clip_gc) && (priv->out_clip != NULL)) {
+		gc->clip_gc = gdk_gc_new(priv->out_clip);
+		need_setup = 1;
+	}
+
+	if (!gc->pixel_gc) {
+		gc->pixel_gc = gdk_gc_new(window);
+		need_setup = 1;
+	}
+
+	if (need_setup) {
 		ghid_gdk_set_color(gc, gc->colorname);
 		ghid_gdk_set_line_width(gc, gc->width);
 		ghid_gdk_set_line_cap(gc, (pcb_cap_style_t) gc->cap);
 		ghid_gdk_set_draw_xor(gc, gc->xor_mask);
-		gdk_gc_set_clip_origin(gc->gc, 0, 0);
+		gdk_gc_set_clip_origin(gc->pixel_gc, 0, 0);
 	}
-	if (gc->mask_seq != mask_seq) {
-		if (mask_seq)
-			gdk_gc_set_clip_mask(gc->gc, priv->mask);
-		else
-			set_clip(priv, gc->gc);
-		gc->mask_seq = mask_seq;
-	}
-	priv->u_gc = WHICH_GC(gc);
+
+	if (priv->out_clip != NULL)
+		gdk_gc_set_foreground(gc->clip_gc, &priv->clip_color);
+
+	priv->pixel_gc = gc->pixel_gc;
+	priv->clip_gc = gc->clip_gc;
 	return 1;
 }
 
@@ -652,6 +681,8 @@ static void ghid_gdk_draw_line(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1,
 	double dx1, dy1, dx2, dy2;
 	render_priv_t *priv = gport->render_priv;
 
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
+
 	dx1 = Vx((double) x1);
 	dy1 = Vy((double) y1);
 	dx2 = Vx((double) x2);
@@ -661,7 +692,10 @@ static void ghid_gdk_draw_line(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1,
 		return;
 
 	USE_GC(gc);
-	gdk_draw_line(priv->drawable, priv->u_gc, dx1, dy1, dx2, dy2);
+	gdk_draw_line(priv->out_pixel, priv->pixel_gc, dx1, dy1, dx2, dy2);
+
+	if (priv->out_clip != NULL)
+		gdk_draw_line(priv->out_clip, priv->clip_gc, dx1, dy1, dx2, dy2);
 }
 
 static void ghid_gdk_draw_arc(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy, pcb_coord_t xradius, pcb_coord_t yradius, pcb_angle_t start_angle, pcb_angle_t delta_angle)
@@ -670,6 +704,8 @@ static void ghid_gdk_draw_arc(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy, p
 	double w, h, radius;
 	render_priv_t *priv = gport->render_priv;
 
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
+
 	w = gport->view.canvas_width * gport->view.coord_per_px;
 	h = gport->view.canvas_height * gport->view.coord_per_px;
 	radius = (xradius > yradius) ? xradius : yradius;
@@ -700,7 +736,13 @@ static void ghid_gdk_draw_arc(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy, p
 	if (start_angle >= 180)
 		start_angle -= 360;
 
-	gdk_draw_arc(priv->drawable, priv->u_gc, 0,
+	gdk_draw_arc(priv->out_pixel, priv->pixel_gc, 0,
+							 pcb_round(Vxd(cx) - Vzd(xradius) + 0.5), pcb_round(Vyd(cy) - Vzd(yradius) + 0.5),
+							 pcb_round(vrx2), pcb_round(vry2),
+							 (start_angle + 180) * 64, delta_angle * 64);
+
+	if (priv->out_clip != NULL)
+		gdk_draw_arc(priv->out_clip, priv->clip_gc, 0,
 							 pcb_round(Vxd(cx) - Vzd(xradius) + 0.5), pcb_round(Vyd(cy) - Vzd(yradius) + 0.5),
 							 pcb_round(vrx2), pcb_round(vry2),
 							 (start_angle + 180) * 64, delta_angle * 64);
@@ -711,6 +753,8 @@ static void ghid_gdk_draw_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1,
 	gint w, h, lw;
 	render_priv_t *priv = gport->render_priv;
 
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
+
 	lw = gc->width;
 	w = gport->view.canvas_width * gport->view.coord_per_px;
 	h = gport->view.canvas_height * gport->view.coord_per_px;
@@ -738,7 +782,10 @@ static void ghid_gdk_draw_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1,
 	}
 
 	USE_GC(gc);
-	gdk_draw_rectangle(priv->drawable, priv->u_gc, FALSE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
+	gdk_draw_rectangle(priv->out_pixel, priv->pixel_gc, FALSE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
+
+	if (priv->out_clip != NULL)
+		gdk_draw_rectangle(priv->out_clip, priv->clip_gc, FALSE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
 }
 
 
@@ -747,6 +794,8 @@ static void ghid_gdk_fill_circle(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy
 	gint w, h, vr;
 	render_priv_t *priv = gport->render_priv;
 
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
+
 	w = gport->view.canvas_width * gport->view.coord_per_px;
 	h = gport->view.canvas_height * gport->view.coord_per_px;
 	if (SIDE_X(cx) < gport->view.x0 - radius
@@ -756,7 +805,10 @@ static void ghid_gdk_fill_circle(pcb_hid_gc_t gc, pcb_coord_t cx, pcb_coord_t cy
 
 	USE_GC(gc);
 	vr = Vz(radius);
-	gdk_draw_arc(priv->drawable, priv->u_gc, TRUE, Vx(cx) - vr, Vy(cy) - vr, vr * 2, vr * 2, 0, 360 * 64);
+	gdk_draw_arc(priv->out_pixel, priv->pixel_gc, TRUE, Vx(cx) - vr, Vy(cy) - vr, vr * 2, vr * 2, 0, 360 * 64);
+
+	if (priv->out_clip != NULL)
+		gdk_draw_arc(priv->out_clip, priv->clip_gc, TRUE, Vx(cx) - vr, Vy(cy) - vr, vr * 2, vr * 2, 0, 360 * 64);
 }
 
 static void ghid_gdk_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x, pcb_coord_t * y)
@@ -767,6 +819,8 @@ static void ghid_gdk_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x
 	render_priv_t *priv = gport->render_priv;
 	USE_GC(gc);
 
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
+
 	if (npoints < n_coords) {
 		npoints = n_coords + 1;
 		points = (GdkPoint *) realloc(points, npoints * sizeof(GdkPoint));
@@ -775,7 +829,9 @@ static void ghid_gdk_fill_polygon(pcb_hid_gc_t gc, int n_coords, pcb_coord_t * x
 		points[i].x = Vx(x[i]);
 		points[i].y = Vy(y[i]);
 	}
-	gdk_draw_polygon(priv->drawable, priv->u_gc, 1, points, n_coords);
+	gdk_draw_polygon(priv->out_pixel, priv->pixel_gc, 1, points, n_coords);
+	if (priv->out_clip != NULL)
+		gdk_draw_polygon(priv->out_clip, priv->clip_gc, 1, points, n_coords);
 }
 
 static void ghid_gdk_fill_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
@@ -783,6 +839,8 @@ static void ghid_gdk_fill_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1,
 	gint w, h, lw, xx, yy;
 	render_priv_t *priv = gport->render_priv;
 
+	assert((curr_drawing_mode == PCB_HID_COMP_POSITIVE) || (curr_drawing_mode == PCB_HID_COMP_NEGATIVE));
+
 	lw = gc->width;
 	w = gport->view.canvas_width * gport->view.coord_per_px;
 	h = gport->view.canvas_height * gport->view.coord_per_px;
@@ -808,7 +866,10 @@ static void ghid_gdk_fill_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1,
 		y2 = yy;
 	}
 	USE_GC(gc);
-	gdk_draw_rectangle(priv->drawable, priv->u_gc, TRUE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
+	gdk_draw_rectangle(priv->out_pixel, priv->pixel_gc, TRUE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
+
+	if (priv->out_clip != NULL)
+		gdk_draw_rectangle(priv->out_clip, priv->clip_gc, TRUE, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
 }
 
 static void redraw_region(GdkRectangle * rect)
@@ -817,24 +878,23 @@ static void redraw_region(GdkRectangle * rect)
 	pcb_hid_expose_ctx_t ctx;
 	render_priv_t *priv = gport->render_priv;
 
-	if (!priv->pixmap)
+	if (!priv->base_pixel)
 		return;
 
 	if (rect != NULL) {
 		priv->clip_rect = *rect;
-		priv->clip = pcb_true;
+		priv->clip_rect_valid = pcb_true;
 	}
 	else {
 		priv->clip_rect.x = 0;
 		priv->clip_rect.y = 0;
 		priv->clip_rect.width = gport->view.canvas_width;
 		priv->clip_rect.height = gport->view.canvas_height;
-		priv->clip = pcb_false;
+		priv->clip_rect_valid = pcb_false;
 	}
 
 	set_clip(priv, priv->bg_gc);
 	set_clip(priv, priv->offlimits_gc);
-	set_clip(priv, priv->mask_gc);
 	set_clip(priv, priv->grid_gc);
 
 	ctx.view.X1 = MIN(Px(priv->clip_rect.x), Px(priv->clip_rect.x + priv->clip_rect.width + 1));
@@ -866,23 +926,23 @@ static void redraw_region(GdkRectangle * rect)
 	}
 
 	if (eleft > 0)
-		gdk_draw_rectangle(priv->drawable, priv->offlimits_gc, 1, 0, 0, eleft, gport->view.canvas_height);
+		gdk_draw_rectangle(priv->out_pixel, priv->offlimits_gc, 1, 0, 0, eleft, gport->view.canvas_height);
 	else
 		eleft = 0;
 	if (eright < gport->view.canvas_width)
-		gdk_draw_rectangle(priv->drawable, priv->offlimits_gc, 1, eright, 0, gport->view.canvas_width - eright, gport->view.canvas_height);
+		gdk_draw_rectangle(priv->out_pixel, priv->offlimits_gc, 1, eright, 0, gport->view.canvas_width - eright, gport->view.canvas_height);
 	else
 		eright = gport->view.canvas_width;
 	if (etop > 0)
-		gdk_draw_rectangle(priv->drawable, priv->offlimits_gc, 1, eleft, 0, eright - eleft + 1, etop);
+		gdk_draw_rectangle(priv->out_pixel, priv->offlimits_gc, 1, eleft, 0, eright - eleft + 1, etop);
 	else
 		etop = 0;
 	if (ebottom < gport->view.canvas_height)
-		gdk_draw_rectangle(priv->drawable, priv->offlimits_gc, 1, eleft, ebottom, eright - eleft + 1, gport->view.canvas_height - ebottom);
+		gdk_draw_rectangle(priv->out_pixel, priv->offlimits_gc, 1, eleft, ebottom, eright - eleft + 1, gport->view.canvas_height - ebottom);
 	else
 		ebottom = gport->view.canvas_height;
 
-	gdk_draw_rectangle(priv->drawable, priv->bg_gc, 1, eleft, etop, eright - eleft + 1, ebottom - etop + 1);
+	gdk_draw_rectangle(priv->out_pixel, priv->bg_gc, 1, eleft, etop, eright - eleft + 1, ebottom - etop + 1);
 
 	ghid_gdk_draw_bg_image();
 
@@ -899,7 +959,7 @@ static void redraw_region(GdkRectangle * rect)
 
 	draw_lead_user(priv);
 
-	priv->clip = pcb_false;
+	priv->clip_rect_valid = pcb_false;
 
 	/* Rest the clip for bg_gc, as it is used outside this function */
 	gdk_gc_set_clip_mask(priv->bg_gc, NULL);
@@ -1163,22 +1223,22 @@ static void ghid_gdk_drawing_area_configure_hook(void *vport)
 	static int done_once = 0;
 	render_priv_t *priv = port->render_priv;
 
-	if (priv->pixmap)
-		gdk_pixmap_unref(priv->pixmap);
+	if (priv->base_pixel)
+		gdk_pixmap_unref(priv->base_pixel);
 
-	priv->pixmap = gdk_pixmap_new(gtk_widget_get_window(gport->drawing_area),
+	priv->base_pixel = gdk_pixmap_new(gtk_widget_get_window(gport->drawing_area),
                                 gport->view.canvas_width, gport->view.canvas_height, -1);
-	priv->drawable = priv->pixmap;
+	priv->out_pixel = priv->base_pixel;
 	gport->drawing_allowed = pcb_true;
 
 	if (!done_once) {
-		priv->bg_gc = gdk_gc_new(priv->drawable);
+		priv->bg_gc = gdk_gc_new(priv->out_pixel);
 		if (!map_color_string(conf_core.appearance.color.background, &priv->bg_color))
 			map_color_string("white", &priv->bg_color);
 		gdk_gc_set_foreground(priv->bg_gc, &priv->bg_color);
 		gdk_gc_set_clip_origin(priv->bg_gc, 0, 0);
 
-		priv->offlimits_gc = gdk_gc_new(priv->drawable);
+		priv->offlimits_gc = gdk_gc_new(priv->out_pixel);
 		if (!map_color_string(conf_core.appearance.color.off_limit, &priv->offlimits_color))
 			map_color_string("white", &priv->offlimits_color);
 		gdk_gc_set_foreground(priv->offlimits_gc, &priv->offlimits_color);
@@ -1186,9 +1246,13 @@ static void ghid_gdk_drawing_area_configure_hook(void *vport)
 		done_once = 1;
 	}
 
-	if (priv->mask) {
-		gdk_pixmap_unref(priv->mask);
-		priv->mask = gdk_pixmap_new(0, port->view.canvas_width, port->view.canvas_height, 1);
+	if (priv->sketch_pixel) {
+		gdk_pixmap_unref(priv->sketch_pixel);
+		priv->sketch_pixel = gdk_pixmap_new(gtk_widget_get_window(gport->drawing_area), port->view.canvas_width, port->view.canvas_height, -1);
+	}
+	if (priv->sketch_clip) {
+		gdk_pixmap_unref(priv->sketch_clip);
+		priv->sketch_clip = gdk_pixmap_new(0, port->view.canvas_width, port->view.canvas_height, 1);
 	}
 }
 
@@ -1197,10 +1261,10 @@ static void ghid_gdk_screen_update(void)
 	render_priv_t *priv = gport->render_priv;
 	GdkWindow *window = gtk_widget_get_window(gport->drawing_area);
 
-	if (priv->pixmap == NULL)
+	if (priv->base_pixel == NULL)
 		return;
 
-	gdk_draw_drawable(window, priv->bg_gc, priv->pixmap, 0, 0, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
+	gdk_draw_drawable(window, priv->bg_gc, priv->base_pixel, 0, 0, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
 	show_crosshair(TRUE);
 }
 
@@ -1210,7 +1274,7 @@ static gboolean ghid_gdk_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_expo
 	render_priv_t *priv = port->render_priv;
 	GdkWindow *window = gtk_widget_get_window(gport->drawing_area);
 
-	gdk_draw_drawable(window, priv->bg_gc, priv->pixmap,
+	gdk_draw_drawable(window, priv->bg_gc, priv->base_pixel,
 										ev->area.x, ev->area.y, ev->area.x, ev->area.y, ev->area.width, ev->area.height);
 	show_crosshair(TRUE);
 	return FALSE;
@@ -1235,7 +1299,7 @@ static gboolean ghid_gdk_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev
 
 	/* Setup drawable and zoom factor for drawing routines
 	 */
-	save_drawable = priv->drawable;
+	save_drawable = priv->base_pixel;
 	save_view = gport->view;
 	save_width = gport->view.canvas_width;
 	save_height = gport->view.canvas_height;
@@ -1248,7 +1312,7 @@ static gboolean ghid_gdk_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev
 	else
 		gport->view.coord_per_px = yz;
 
-	priv->drawable = window;
+	priv->base_pixel = priv->out_pixel = window;
 	gport->view.canvas_width = allocation.width;
 	gport->view.canvas_height = allocation.height;
 	gport->view.width = allocation.width * gport->view.coord_per_px;
@@ -1262,7 +1326,7 @@ static gboolean ghid_gdk_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev
 	/* call the drawing routine */
 	expcall(&gtk2_gdk_hid, ctx);
 
-	priv->drawable = save_drawable;
+	priv->out_pixel = priv->base_pixel = save_drawable;
 	gport->view = save_view;
 	gport->view.canvas_width = save_width;
 	gport->view.canvas_height = save_height;
@@ -1270,63 +1334,6 @@ static gboolean ghid_gdk_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev
 	return FALSE;
 }
 
-static void *ghid_gdk_render_pixmap(int cx, int cy, double zoom, int width, int height, int depth)
-{
-	GdkPixmap *pixmap;
-	GdkDrawable *save_drawable;
-	pcb_gtk_view_t save_view;
-	int save_width, save_height;
-	pcb_hid_expose_ctx_t ectx;
-	render_priv_t *priv = gport->render_priv;
-
-	save_drawable = priv->drawable;
-	save_view = gport->view;
-	save_width = gport->view.canvas_width;
-	save_height = gport->view.canvas_height;
-
-	pixmap = gdk_pixmap_new(NULL, width, height, depth);
-
-	/* Setup drawable and zoom factor for drawing routines
-	 */
-
-	priv->drawable = pixmap;
-	gport->view.coord_per_px = zoom;
-	gport->view.canvas_width = width;
-	gport->view.canvas_height = height;
-	gport->view.width = width * gport->view.coord_per_px;
-	gport->view.height = height * gport->view.coord_per_px;
-	gport->view.x0 = conf_core.editor.view.flip_x ? PCB->MaxWidth - cx : cx;
-	gport->view.x0 -= gport->view.height / 2;
-	gport->view.y0 = conf_core.editor.view.flip_y ? PCB->MaxHeight - cy : cy;
-	gport->view.y0 -= gport->view.width / 2;
-
-	/* clear background */
-	gdk_draw_rectangle(pixmap, priv->bg_gc, TRUE, 0, 0, width, height);
-
-	/* call the drawing routine */
-	ectx.view.X1 = MIN(Px(0), Px(gport->view.canvas_width + 1));
-	ectx.view.Y1 = MIN(Py(0), Py(gport->view.canvas_height + 1));
-	ectx.view.X2 = MAX(Px(0), Px(gport->view.canvas_width + 1));
-	ectx.view.Y2 = MAX(Py(0), Py(gport->view.canvas_height + 1));
-
-	ectx.view.X1 = MAX(0, MIN(PCB->MaxWidth, ectx.view.X1));
-	ectx.view.X2 = MAX(0, MIN(PCB->MaxWidth, ectx.view.X2));
-	ectx.view.Y1 = MAX(0, MIN(PCB->MaxHeight, ectx.view.Y1));
-	ectx.view.Y2 = MAX(0, MIN(PCB->MaxHeight, ectx.view.Y2));
-
-	ectx.force = 0;
-	ectx.content.elem = NULL;
-
-	pcb_hid_expose_all(&gtk2_gdk_hid, &ectx);
-
-	priv->drawable = save_drawable;
-	gport->view = save_view;
-	gport->view.canvas_width = save_width;
-	gport->view.canvas_height = save_height;
-
-	return pixmap;
-}
-
 static pcb_hid_t *ghid_gdk_request_debug_draw(void)
 {
 	/* No special setup requirements, drawing goes into
@@ -1389,7 +1396,7 @@ static void draw_lead_user(render_priv_t *priv)
 			radius += PCB_MM_TO_COORD(LEAD_USER_INITIAL_RADIUS);
 
 		/* Draw an arc at radius */
-		gdk_draw_arc(gport->render_priv->drawable, lead_gc, FALSE,
+		gdk_draw_arc(gport->render_priv->out_pixel, lead_gc, FALSE,
 								 Vx(lead_user->x - radius), Vy(lead_user->y - radius), Vz(2. * radius), Vz(2. * radius), 0, 360 * 64);
 	}
 }
@@ -1397,44 +1404,43 @@ static void draw_lead_user(render_priv_t *priv)
 void ghid_gdk_install(pcb_gtk_common_t *common, pcb_hid_t *hid)
 {
 	if (common != NULL) {
-	common->render_pixmap = ghid_gdk_render_pixmap;
-	common->init_drawing_widget = ghid_gdk_init_drawing_widget;
-	common->drawing_realize = ghid_gdk_port_drawing_realize_cb;
-	common->drawing_area_expose = ghid_gdk_drawing_area_expose_cb;
-	common->preview_expose = ghid_gdk_preview_expose;
-	common->invalidate_all = ghid_gdk_invalidate_all;
-	common->set_special_colors = ghid_gdk_set_special_colors;
-	common->init_renderer = ghid_gdk_init_renderer;
-	common->screen_update = ghid_gdk_screen_update;
-	common->draw_grid_local = ghid_gdk_draw_grid_local;
-	common->drawing_area_configure_hook = ghid_gdk_drawing_area_configure_hook;
-	common->shutdown_renderer = ghid_gdk_shutdown_renderer;
-	common->get_color_name = get_color_name;
-	common->map_color_string = map_color_string;
+		common->init_drawing_widget = ghid_gdk_init_drawing_widget;
+		common->drawing_realize = ghid_gdk_port_drawing_realize_cb;
+		common->drawing_area_expose = ghid_gdk_drawing_area_expose_cb;
+		common->preview_expose = ghid_gdk_preview_expose;
+		common->invalidate_all = ghid_gdk_invalidate_all;
+		common->set_special_colors = ghid_gdk_set_special_colors;
+		common->init_renderer = ghid_gdk_init_renderer;
+		common->screen_update = ghid_gdk_screen_update;
+		common->draw_grid_local = ghid_gdk_draw_grid_local;
+		common->drawing_area_configure_hook = ghid_gdk_drawing_area_configure_hook;
+		common->shutdown_renderer = ghid_gdk_shutdown_renderer;
+		common->get_color_name = get_color_name;
+		common->map_color_string = map_color_string;
 	}
 
 	if (hid != NULL) {
-	hid->invalidate_lr = ghid_gdk_invalidate_lr;
-	hid->invalidate_all = ghid_gdk_invalidate_all;
-	hid->notify_crosshair_change = ghid_gdk_notify_crosshair_change;
-	hid->notify_mark_change = ghid_gdk_notify_mark_change;
-	hid->set_layer_group = ghid_gdk_set_layer_group;
-	hid->make_gc = ghid_gdk_make_gc;
-	hid->destroy_gc = ghid_gdk_destroy_gc;
-	hid->use_mask = ghid_gdk_use_mask;
-	hid->set_color = ghid_gdk_set_color;
-	hid->set_line_cap = ghid_gdk_set_line_cap;
-	hid->set_line_width = ghid_gdk_set_line_width;
-	hid->set_draw_xor = ghid_gdk_set_draw_xor;
-	hid->draw_line = ghid_gdk_draw_line;
-	hid->draw_arc = ghid_gdk_draw_arc;
-	hid->draw_rect = ghid_gdk_draw_rect;
-	hid->fill_circle = ghid_gdk_fill_circle;
-	hid->fill_polygon = ghid_gdk_fill_polygon;
-	hid->fill_rect = ghid_gdk_fill_rect;
+		hid->invalidate_lr = ghid_gdk_invalidate_lr;
+		hid->invalidate_all = ghid_gdk_invalidate_all;
+		hid->notify_crosshair_change = ghid_gdk_notify_crosshair_change;
+		hid->notify_mark_change = ghid_gdk_notify_mark_change;
+		hid->set_layer_group = ghid_gdk_set_layer_group;
+		hid->make_gc = ghid_gdk_make_gc;
+		hid->destroy_gc = ghid_gdk_destroy_gc;
+		hid->set_drawing_mode = ghid_gdk_set_drawing_mode;
+		hid->set_color = ghid_gdk_set_color;
+		hid->set_line_cap = ghid_gdk_set_line_cap;
+		hid->set_line_width = ghid_gdk_set_line_width;
+		hid->set_draw_xor = ghid_gdk_set_draw_xor;
+		hid->draw_line = ghid_gdk_draw_line;
+		hid->draw_arc = ghid_gdk_draw_arc;
+		hid->draw_rect = ghid_gdk_draw_rect;
+		hid->fill_circle = ghid_gdk_fill_circle;
+		hid->fill_polygon = ghid_gdk_fill_polygon;
+		hid->fill_rect = ghid_gdk_fill_rect;
 
-	hid->request_debug_draw = ghid_gdk_request_debug_draw;
-	hid->flush_debug_draw = ghid_gdk_flush_debug_draw;
-	hid->finish_debug_draw = ghid_gdk_finish_debug_draw;
+		hid->request_debug_draw = ghid_gdk_request_debug_draw;
+		hid->flush_debug_draw = ghid_gdk_flush_debug_draw;
+		hid->finish_debug_draw = ghid_gdk_finish_debug_draw;
 	}
 }
