@@ -18,44 +18,35 @@
 #include "../src_plugins/lib_gtk_hid/coord_conv.h"
 #include "../src_plugins/lib_gtk_hid/render.h"
 
-
-/* The Linux OpenGL ABI 1.0 spec requires that we define
- * GL_GLEXT_PROTOTYPES before including gl.h or glx.h for extensions
- * in order to get prototypes:
- *   http://www.opengl.org/registry/ABI/
- */
-
-#define GL_GLEXT_PROTOTYPES 1
-#ifdef HAVE_OPENGL_GL_H
-#include <OpenGL/gl.h>
-#else
-#include <GL/gl.h>
-#endif
-
+#include "opengl.h"
 #include <gtk/gtkgl.h>
+#include "draw_gl.h"
 #include "hidgl.h"
 #include "hid_draw_helpers.h"
+#include "stencil_gl.h"
 
 #include "../src_plugins/lib_gtk_config/hid_gtk_conf.h"
 
+#define Z_NEAR 3.0
 extern pcb_hid_t gtk2_gl_hid;
 
 static pcb_hid_gc_t current_gc = NULL;
-
 /* Sets gport->u_gc to the "right" GC to use (wrt mask or window)
 */
 #define USE_GC(gc) if (!use_gc(gc)) return
 
-static int cur_mask = -1;
+/*static int cur_mask = -1;*/
 
 typedef struct render_priv_s {
-	GdkGLConfig *glconfig;
-	GdkColor bg_color, offlimits_color, grid_color;
-	pcb_bool trans_lines;
-	pcb_bool in_context;
-	int subcomposite_stencil_bit;
-	char *current_colorname;
-	double current_alpha_mult;
+	GdkGLConfig *				glconfig;
+	GdkColor 						bg_color;
+	GdkColor 						offlimits_color;
+	GdkColor 						grid_color;
+	pcb_bool 						trans_lines;
+	pcb_bool 						in_context;
+	int 								subcomposite_stencil_bit;
+	char *							current_colorname;
+	double 							current_alpha_mult;
 } render_priv_t;
 
 
@@ -71,7 +62,12 @@ typedef struct hid_gc_s {
 
 static void draw_lead_user(render_priv_t * priv);
 
-static const gchar *get_color_name(GdkColor * color)
+void ghid_gl_render_burst(pcb_burst_op_t op, const pcb_box_t *screen)
+{
+
+}
+
+static const gchar *get_color_name(pcb_gtk_color_t * color)
 {
 	static char tmp[16];
 
@@ -83,7 +79,7 @@ static const gchar *get_color_name(GdkColor * color)
 }
 
 /** Returns TRUE only if \p color_string has been allocated to \p color. */
-static pcb_bool map_color_string(const char *color_string, GdkColor * color)
+static pcb_bool map_color_string(const char *color_string, pcb_gtk_color_t * color)
 {
 	static GdkColormap *colormap = NULL;
 	GHidPort *out = &ghid_port;
@@ -103,13 +99,14 @@ static pcb_bool map_color_string(const char *color_string, GdkColor * color)
 }
 
 
+#if 0
 static void start_subcomposite(void)
 {
 	render_priv_t *priv = gport->render_priv;
 	int stencil_bit;
 
 	/* Flush out any existing geoemtry to be rendered */
-	hidgl_flush_triangles(&buffer);
+	hidgl_flush_triangles();
 
 	glEnable(GL_STENCIL_TEST);		/* Enable Stencil test */
 	glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);	/* Stencil pass => replace stencil value (with 1) */
@@ -126,7 +123,7 @@ static void end_subcomposite(void)
 	render_priv_t *priv = gport->render_priv;
 
 	/* Flush out any existing geoemtry to be rendered */
-	hidgl_flush_triangles(&buffer);
+	hidgl_flush_triangles();
 
 	hidgl_return_stencil_bit(priv->subcomposite_stencil_bit);	/* Relinquish any bitplane we previously used */
 
@@ -136,12 +133,13 @@ static void end_subcomposite(void)
 
 	priv->subcomposite_stencil_bit = 0;
 }
-
+#endif
 
 int ghid_gl_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsigned int flags, int is_empty)
 {
 	render_priv_t *priv = gport->render_priv;
 	int idx = group;
+
 	if (idx >= 0 && idx < pcb_max_group(PCB)) {
 		int n = PCB->LayerGroups.grp[group].len;
 		for (idx = 0; idx < n - 1; idx++) {
@@ -152,9 +150,27 @@ int ghid_gl_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsig
 		idx = PCB->LayerGroups.grp[group].lid[idx];
 	}
 
+	glMatrixMode(GL_MODELVIEW);
+	glLoadIdentity();
+	glTranslatef(0.0f, 0.0f, -Z_NEAR);
+		
+	glScalef((conf_core.editor.view.flip_x ? -1. : 1.) / gport->view.coord_per_px,
+					 (conf_core.editor.view.flip_y ? -1. : 1.) / gport->view.coord_per_px,
+					 ((conf_core.editor.view.flip_x == conf_core.editor.view.flip_y) ? 1. : -1.) / gport->view.coord_per_px);
+	glTranslatef(conf_core.editor.view.flip_x ? gport->view.x0 - PCB->MaxWidth :
+							 -gport->view.x0, conf_core.editor.view.flip_y ? gport->view.y0 - PCB->MaxHeight : -gport->view.y0, 0);
+
+	/* Put the renderer into a good state so that any drawing is done in standard mode */
+	
+	drawgl_flush();
+	drawgl_reset();
+	glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
+	glDisable(GL_STENCIL_TEST);
+	
+/*
 	end_subcomposite();
 	start_subcomposite();
-
+*/
 	priv->trans_lines = pcb_true;
 
 	/* non-virtual layers with group visibility */
@@ -188,7 +204,7 @@ int ghid_gl_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsig
 			/* Disable stencil for cross-section drawing; that code
 			 * relies on overdraw doing the right thing and doesn't
 			 * use layers */
-			glDisable(GL_STENCIL_TEST);
+			/*glDisable(GL_STENCIL_TEST);*/
 			return 0;
 		}
 	}
@@ -197,7 +213,8 @@ int ghid_gl_set_layer_group(pcb_layergrp_id_t group, pcb_layer_id_t layer, unsig
 
 static void ghid_gl_end_layer(void)
 {
-	end_subcomposite();
+	drawgl_flush();
+	drawgl_reset();
 }
 
 void ghid_gl_destroy_gc(pcb_hid_gc_t gc)
@@ -216,6 +233,7 @@ pcb_hid_gc_t ghid_gl_make_gc(void)
 	return rv;
 }
 
+
 void ghid_gl_draw_grid_local(pcb_coord_t cx, pcb_coord_t cy)
 {
 #warning draw_grid_local stubbed out for now
@@ -293,58 +311,6 @@ static void ghid_gl_draw_bg_image(void)
 	glDisable(GL_TEXTURE_2D);
 }
 
-void ghid_gl_use_mask(pcb_mask_op_t use_it)
-{
-	static int stencil_bit = 0;
-
-	if (use_it == cur_mask)
-		return;
-
-	/* Flush out any existing geoemtry to be rendered */
-	hidgl_flush_triangles(&buffer);
-
-	switch (use_it) {
-	case HID_MASK_BEFORE:
-		/* The HID asks not to receive this mask type, so warn if we get it */
-		g_return_if_reached();
-
-	case HID_MASK_INIT:
-		glColorMask(0, 0, 0, 0);		/* Disable writting in color buffer */
-		glEnable(GL_STENCIL_TEST);	/* Enable Stencil test */
-		stencil_bit = hidgl_assign_clear_stencil_bit();	/* Get a new (clean) bitplane to stencil with */
-		break;
-
-	case HID_MASK_SET:
-		/* Write '1' to the stencil buffer where we should draw (on mask: "red", "solid")*/
-		glStencilFunc(GL_ALWAYS, stencil_bit, stencil_bit);	/* Always pass stencil test, write stencil_bit */
-		glStencilMask(stencil_bit);	/* Only write to our subcompositing stencil bitplane */
-		glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);	/* Stencil pass => replace stencil value (with 1) */
-		break;
-
-	case HID_MASK_CLEAR:
-		/* Write '0' to the stencil buffer where we should clear (on mask: "cutout", "transparent"). */
-		glStencilFunc(GL_ALWAYS, stencil_bit, stencil_bit);	/* Always pass stencil test, write stencil_bit */
-		glStencilMask(stencil_bit);	/* Only write to our subcompositing stencil bitplane */
-		glStencilOp(GL_KEEP, GL_KEEP, GL_ZERO);	/* Stencil pass => replace stencil value (with 1) */
-		break;
-
-	case HID_MASK_AFTER:
-		/* Drawing operations as masked to areas where the stencil buffer is '0' */
-		glColorMask(1, 1, 1, 1);		/* Enable drawing of r, g, b & a */
-		glStencilFunc(GL_NOTEQUAL, 0, stencil_bit);	/* Draw only where our bit of the stencil buffer is clear */
-		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);	/* Stencil buffer read only */
-		break;
-
-	case HID_MASK_OFF:
-		/* Disable stenciling */
-		hidgl_return_stencil_bit(stencil_bit);	/* Relinquish any bitplane we previously used */
-		glDisable(GL_STENCIL_TEST);	/* Disable Stencil test */
-		break;
-	}
-	cur_mask = use_it;
-}
-
-
 	/* Config helper functions for when the user changes color preferences.
 	   |  set_special colors used in the gtkhid.
 	 */
@@ -357,7 +323,7 @@ static void set_special_grid_color(void)
 	priv->grid_color.blue ^= priv->bg_color.blue;
 }
 
-void ghid_gl_set_special_colors(conf_native_t *cfg)
+void ghid_gl_set_special_colors(conf_native_t *cfg, int arr_idx)
 {
 	render_priv_t *priv = gport->render_priv;
 
@@ -478,8 +444,16 @@ static void set_gl_color_for_gc(pcb_hid_gc_t gc)
 	if (!priv->in_context)
 		return;
 
-	hidgl_flush_triangles(&buffer);
-	glColor4d(r, g, b, a);
+
+	/* We need to flush the draw buffer when changing colour so that new primtives
+	 * don't get merged. This actually isn't a problem with the colour but due to 
+	 * way that the final render pass iterates through the primtive buffer in 
+	 * reverse order. If the new primitives are merged with previous ones then they
+	 * will be drawn in the wrong order.
+	 */
+	drawgl_flush();	
+
+	drawgl_set_colour(r,g,b,a);
 }
 
 void ghid_gl_set_color(pcb_hid_gc_t gc, const char *name)
@@ -544,6 +518,7 @@ static int use_gc(pcb_hid_gc_t gc)
 	return 1;
 }
 
+
 void ghid_gl_draw_line(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_coord_t x2, pcb_coord_t y2)
 {
 	USE_GC(gc);
@@ -562,7 +537,7 @@ void ghid_gl_draw_rect(pcb_hid_gc_t gc, pcb_coord_t x1, pcb_coord_t y1, pcb_coor
 {
 	USE_GC(gc);
 
-	hidgl_draw_rect(x1, y1, x2, y2);
+	drawgl_add_rectangle(x1, y1, x2, y2);
 }
 
 
@@ -771,7 +746,7 @@ void ghid_gl_init_renderer(int *argc, char ***argv, void *vport)
 {
 	GHidPort * port = vport;
 	render_priv_t *priv;
-
+	
 	port->render_priv = priv = g_new0(render_priv_t, 1);
 
 	gtk_gl_init(argc, argv);
@@ -833,7 +808,7 @@ gboolean ghid_gl_start_drawing(GHidPort * port)
 	GtkWidget *widget = port->drawing_area;
 	GdkGLContext *pGlContext = gtk_widget_get_gl_context(widget);
 	GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable(widget);
-
+	
 	/* make GL-context "current" */
 	if (!gdk_gl_drawable_gl_begin(pGlDrawable, pGlContext))
 		return FALSE;
@@ -863,7 +838,6 @@ void ghid_gl_screen_update(void)
 {
 }
 
-#define Z_NEAR 3.0
 gboolean ghid_gl_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_expose_t *ev, void *vport)
 {
 	GHidPort * port = vport;
@@ -880,7 +854,7 @@ gboolean ghid_gl_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_expose_t *ev
 	/* If we don't have any stencil bits available,
 	   we can't use the hidgl polygon drawing routine */
 	/* TODO: We could use the GLU tessellator though */
-	if (hidgl_stencil_bits() == 0)
+	if (stencilgl_bit_count() == 0)
 		gtk2_gl_hid.fill_pcb_polygon = pcb_dhlp_fill_pcb_polygon;
 
 	glEnable(GL_BLEND);
@@ -910,7 +884,7 @@ gboolean ghid_gl_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_expose_t *ev
 	glStencilMask(~0);
 	glClearStencil(0);
 	glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
-	hidgl_reset_stencil_usage();
+	stencilgl_reset_stencil_usage();
 
 	/* Disable the stencil test until we need it - otherwise it gets dirty */
 	glDisable(GL_STENCIL_TEST);
@@ -938,10 +912,13 @@ gboolean ghid_gl_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_expose_t *ev
 
 	ghid_gl_draw_bg_image();
 
-	hidgl_init_triangle_array(&buffer);
 	ghid_gl_invalidate_current_gc();
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
 	pcb_hid_expose_all(&gtk2_gl_hid, &ctx);
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
 
 	ghid_gl_draw_grid(&ctx.view);
 
@@ -949,11 +926,11 @@ gboolean ghid_gl_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_expose_t *ev
 
 	pcb_draw_attached();
 	pcb_draw_mark();
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
 
 	ghid_gl_show_crosshair(TRUE);
 
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
 
 	draw_lead_user(priv);
 
@@ -971,7 +948,7 @@ void ghid_gl_port_drawing_realize_cb(GtkWidget * widget, gpointer data)
 {
 	GdkGLContext *glcontext = gtk_widget_get_gl_context(widget);
 	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(widget);
-
+	
 	if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext))
 		return;
 
@@ -981,7 +958,7 @@ void ghid_gl_port_drawing_realize_cb(GtkWidget * widget, gpointer data)
 
 gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hid_expose_t expcall, const pcb_hid_expose_ctx_t *ctx)
 {
-	GdkWindow *window = gtk_widget_get_window(widget);
+/*	GdkWindow *window = gtk_widget_get_window(widget);*/
 	GdkGLContext *pGlContext = gtk_widget_get_gl_context(widget);
 	GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable(widget);
 	GtkAllocation allocation;
@@ -1045,13 +1022,12 @@ gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hi
 	glClearStencil(0);
 	glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
 
-	hidgl_reset_stencil_usage();
+	stencilgl_reset_stencil_usage();
 	glDisable(GL_STENCIL_TEST);
 	glStencilMask(0);
 	glStencilFunc(GL_ALWAYS, 0, 0);
 
 	/* call the drawing routine */
-	hidgl_init_triangle_array(&buffer);
 	ghid_gl_invalidate_current_gc();
 	glPushMatrix();
 	glScalef((conf_core.editor.view.flip_x ? -1. : 1.) / gport->view.coord_per_px,
@@ -1061,7 +1037,7 @@ gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hi
 
 	expcall(&gtk2_gl_hid, ctx);
 
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
 	glPopMatrix();
 
 	if (gdk_gl_drawable_is_double_buffered(pGlDrawable))
@@ -1080,113 +1056,6 @@ gboolean ghid_gl_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *ev, pcb_hi
 	return FALSE;
 }
 
-/*GdkPixmap*/void *ghid_gl_render_pixmap(int cx, int cy, double zoom, int width, int height, int depth)
-{
-	GdkGLConfig *glconfig;
-	GdkPixmap *pixmap;
-	GdkGLPixmap *glpixmap;
-	GdkGLContext *glcontext;
-	GdkGLDrawable *gldrawable;
-	render_priv_t *priv = gport->render_priv;
-	pcb_gtk_view_t save_view;
-	int save_width, save_height;
-	pcb_hid_expose_ctx_t ctx;
-
-	save_view = gport->view;
-	save_width = gport->view.canvas_width;
-	save_height = gport->view.canvas_height;
-
-	/* Setup rendering context for drawing routines
-	 */
-
-	glconfig = gdk_gl_config_new_by_mode(GDK_GL_MODE_RGB | GDK_GL_MODE_STENCIL | GDK_GL_MODE_SINGLE);
-
-	pixmap = gdk_pixmap_new(NULL, width, height, depth);
-	glpixmap = gdk_pixmap_set_gl_capability(pixmap, glconfig, NULL);
-	gldrawable = GDK_GL_DRAWABLE(glpixmap);
-	glcontext = gdk_gl_context_new(gldrawable, NULL, TRUE, GDK_GL_RGBA_TYPE);
-
-	/* Setup zoom factor for drawing routines */
-
-	gport->view.coord_per_px = zoom;
-	gport->view.canvas_width = width;
-	gport->view.canvas_height = height;
-	gport->view.width = width * gport->view.coord_per_px;
-	gport->view.height = height * gport->view.coord_per_px;
-	gport->view.x0 = conf_core.editor.view.flip_x ? PCB->MaxWidth - cx : cx;
-	gport->view.x0 -= gport->view.height / 2;
-	gport->view.y0 = conf_core.editor.view.flip_y ? PCB->MaxHeight - cy : cy;
-	gport->view.y0 -= gport->view.width / 2;
-
-	/* make GL-context "current" */
-	if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext)) {
-		return NULL;
-	}
-	gport->render_priv->in_context = pcb_true;
-
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-	glViewport(0, 0, width, height);
-
-	glEnable(GL_SCISSOR_TEST);
-	glScissor(0, 0, width, height);
-
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	glOrtho(0, width, height, 0, 0, 100);
-	glMatrixMode(GL_MODELVIEW);
-	glLoadIdentity();
-	glTranslatef(0.0f, 0.0f, -Z_NEAR);
-
-	glClearColor(priv->bg_color.red / 65535., priv->bg_color.green / 65535., priv->bg_color.blue / 65535., 1.);
-	glStencilMask(~0);
-	glClearStencil(0);
-	glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
-	hidgl_reset_stencil_usage();
-
-	/* call the drawing routine */
-	hidgl_init_triangle_array(&buffer);
-	ghid_gl_invalidate_current_gc();
-	glPushMatrix();
-	glScalef((conf_core.editor.view.flip_x ? -1. : 1.) / gport->view.coord_per_px,
-					 (conf_core.editor.view.flip_y ? -1. : 1.) / gport->view.coord_per_px, 1);
-	glTranslatef(conf_core.editor.view.flip_x ? gport->view.x0 - PCB->MaxWidth :
-		     -gport->view.x0,
-		     conf_core.editor.view.flip_y ? gport->view.y0 - PCB->MaxHeight : -gport->view.y0, 0);
-
-	ctx.view.X1 = MIN(Px(0), Px(gport->view.canvas_width + 1));
-	ctx.view.Y1 = MIN(Py(0), Py(gport->view.canvas_height + 1));
-	ctx.view.X2 = MAX(Px(0), Px(gport->view.canvas_width + 1));
-	ctx.view.Y2 = MAX(Py(0), Py(gport->view.canvas_height + 1));
-
-	ctx.view.X1 = MAX(0, MIN(PCB->MaxWidth, ctx.view.X1));
-	ctx.view.X2 = MAX(0, MIN(PCB->MaxWidth, ctx.view.X2));
-	ctx.view.Y1 = MAX(0, MIN(PCB->MaxHeight, ctx.view.Y1));
-	ctx.view.Y2 = MAX(0, MIN(PCB->MaxHeight, ctx.view.Y2));
-
-	pcb_hid_expose_all(&gtk2_gl_hid, &ctx);
-	hidgl_flush_triangles(&buffer);
-	glPopMatrix();
-
-	glFlush();
-
-	/* end drawing to current GL-context */
-	gport->render_priv->in_context = pcb_false;
-	gdk_gl_drawable_gl_end(gldrawable);
-
-	gdk_pixmap_unset_gl_capability(pixmap);
-
-	g_object_unref(glconfig);
-	g_object_unref(glcontext);
-
-	gport->view = save_view;
-	gport->view.canvas_width = save_width;
-	gport->view.canvas_height = save_height;
-
-	return pixmap;
-}
-
 pcb_hid_t *ghid_gl_request_debug_draw(void)
 {
 	GHidPort *port = gport;
@@ -1206,7 +1075,7 @@ pcb_hid_t *ghid_gl_request_debug_draw(void)
 	glLoadIdentity();
 	glTranslatef(0.0f, 0.0f, -Z_NEAR);
 
-	hidgl_init_triangle_array(&buffer);
+/*	hidgl_init_triangle_array(&buffer);*/
 	ghid_gl_invalidate_current_gc();
 
 	/* Setup stenciling */
@@ -1226,7 +1095,7 @@ void ghid_gl_flush_debug_draw(void)
 	GtkWidget *widget = gport->drawing_area;
 	GdkGLDrawable *pGlDrawable = gtk_widget_get_gl_drawable(widget);
 
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
 
 	if (gdk_gl_drawable_is_double_buffered(pGlDrawable))
 		gdk_gl_drawable_swap_buffers(pGlDrawable);
@@ -1236,7 +1105,7 @@ void ghid_gl_flush_debug_draw(void)
 
 void ghid_gl_finish_debug_draw(void)
 {
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
 	glPopMatrix();
 
 	ghid_gl_end_drawing(gport);
@@ -1269,14 +1138,17 @@ static void draw_lead_user(render_priv_t * priv)
 		hidgl_draw_arc(width, lead_user->x, lead_user->y, radius, radius, 0, 360, gport->view.coord_per_px);
 	}
 
-	hidgl_flush_triangles(&buffer);
+	drawgl_flush();
 	glPopAttrib();
 }
 
+
+
+
 void ghid_gl_install(pcb_gtk_common_t *common, pcb_hid_t *hid)
 {
+
 	if (common != NULL) {
-	common->render_pixmap = ghid_gl_render_pixmap;
 	common->init_drawing_widget = ghid_gl_init_drawing_widget;
 	common->drawing_realize = ghid_gl_port_drawing_realize_cb;
 	common->drawing_area_expose = ghid_gl_drawing_area_expose_cb;
@@ -1300,7 +1172,6 @@ void ghid_gl_install(pcb_gtk_common_t *common, pcb_hid_t *hid)
 	hid->set_layer_group = ghid_gl_set_layer_group;
 	hid->make_gc = ghid_gl_make_gc;
 	hid->destroy_gc = ghid_gl_destroy_gc;
-	hid->use_mask = ghid_gl_use_mask;
 	hid->set_color = ghid_gl_set_color;
 	hid->set_line_cap = ghid_gl_set_line_cap;
 	hid->set_line_width = ghid_gl_set_line_width;
@@ -1312,9 +1183,14 @@ void ghid_gl_install(pcb_gtk_common_t *common, pcb_hid_t *hid)
 	hid->fill_polygon = ghid_gl_fill_polygon;
 	hid->fill_rect = ghid_gl_fill_rect;
 
+	hid->set_drawing_mode = hidgl_set_drawing_mode;
+	hid->render_burst = ghid_gl_render_burst;
+
 	hid->request_debug_draw = ghid_gl_request_debug_draw;
 	hid->flush_debug_draw = ghid_gl_flush_debug_draw;
 	hid->finish_debug_draw = ghid_gl_finish_debug_draw;
+
+	hid->holes_after = 1;
 	}
 }
 
