@@ -51,8 +51,9 @@ static void ghid_cairo_screen_update(void);
 */
 #define USE_GC(gc) if (!use_gc(gc)) return
 
-static int cur_mask = -1;
-static int mask_seq = 0;
+//static int cur_mask = -1;
+//static int mask_seq = 0;
+static pcb_composite_op_t curr_drawing_mode;
 
 typedef struct render_priv_s {
 	GdkRGBA bg_color;											/**< cached back-ground color           */
@@ -66,9 +67,6 @@ typedef struct render_priv_s {
 	cairo_surface_t *surf_da;							/**< cairo surface connected to gport->drawing_area */
 	cairo_t *cr_drawing_area;							/**< cairo context created from surf_da             */
 
-	cairo_surface_t *surf_ui;							/**< cairo surface gathering visual feedback to user */
-	cairo_t *cr_ui;												/**< cairo context created from surf_ui             */
-
 	cairo_surface_t *surf_layer;					/**< cairo surface for temporary layer composition  */
 	cairo_t *cr_layer;										/**< cairo context created from surf_layer          */
 
@@ -81,6 +79,7 @@ typedef struct render_priv_s {
 	//GdkGC *grid_gc;
 	pcb_bool clip;
 	GdkRectangle clip_rect;
+	int xor_mode;													/**< 1 if drawn in XOR mode, 0 otherwise*/
 	int attached_invalidate_depth;
 	int mark_invalidate_depth;
 } render_priv_t;
@@ -95,16 +94,11 @@ typedef struct hid_gc_s {
 	cairo_line_cap_t cap;
 	cairo_line_join_t join;
 	gchar xor_mask;
-	gint mask_seq;
+	//gint mask_seq;
 } hid_gc_s;
 
 static void draw_lead_user(render_priv_t * priv_);
 
-#warning TODO: replace GdkRGBA with pcb_gtk_color_t to avoid incompatible pointer type
-/*../src_plugins/hid_gtk3_cairo/gtkhid-cairo.c:1541:26: warning: assignment from incompatible pointer type [-Wincompatible-pointer-types]
-   common->get_color_name = get_color_name;
-*/
-
 /** Duplicates the source fields to destination. */
 static void copy_color(GdkRGBA * dest, GdkRGBA * source)
 {
@@ -114,7 +108,7 @@ static void copy_color(GdkRGBA * dest, GdkRGBA * source)
 	dest->alpha = source->alpha;
 }
 
-static const gchar *get_color_name(GdkRGBA * color)
+static const gchar *get_color_name(pcb_gtk_color_t * color)
 {
 	static char tmp[16];
 
@@ -127,7 +121,7 @@ static const gchar *get_color_name(GdkRGBA * color)
 }
 
 /** Returns TRUE if \p color_string has been successfully parsed to \p color. */
-static pcb_bool map_color_string(const char *color_string, GdkRGBA * color)
+static pcb_bool map_color_string(const char *color_string, pcb_gtk_color_t * color)
 {
 	pcb_bool parsed;
 
@@ -379,8 +373,8 @@ static inline void ghid_cairo_draw_grid_global(cairo_t *cr)
 			cairo_move_to(cr, points[i].x, points[i].y);
 			cairo_line_to(cr, points[i].x, points[i].y);
 		}
+		cairo_stroke(cr);
 	}
-	cairo_stroke(cr);
 }
 
 static void ghid_cairo_draw_grid_local_(pcb_coord_t cx, pcb_coord_t cy, int radius)
@@ -567,58 +561,71 @@ static void ghid_cairo_draw_bg_image(void)
 	}
 }
 
-#define WHICH_GC(gc) (cur_mask == HID_MASK_CLEAR ? priv->mask_gc : (gc)->gc)
-
-static void ghid_cairo_use_mask(pcb_mask_op_t use_it)
+void ghid_cairo_render_burst(pcb_burst_op_t op, const pcb_box_t *screen)
 {
-	static int mask_seq_id = 0;
-	GdkColor color;
-	render_priv_t *priv = gport->render_priv;
-
-	if (use_it == cur_mask)
-		return;
-	switch (use_it) {
-	case HID_MASK_OFF:
-		//priv->drawable = priv->pixmap;
-		mask_seq = 0;
-		break;
-
-	case HID_MASK_BEFORE:
-		/* The HID asks not to receive this mask type, so warn if we get it */
-		g_return_if_reached();
-
-	case HID_MASK_INIT:
-		//ghid_mask_setup(priv);
-		mask_seq = 0;
-
-		/* clear the mask */
-		color.pixel = 0;
-		//gdk_gc_set_foreground(priv->mask_gc, &color);
-		//gdk_draw_rectangle(priv->drawable, priv->mask_gc, TRUE, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
-		break;
-
-	case HID_MASK_CLEAR:
-		//color.pixel = 0;
-		//gdk_gc_set_foreground(priv->mask_gc, &color);
-		break;
-
-	case HID_MASK_SET:
-		color.pixel = 1;
-		//gdk_gc_set_foreground(priv->mask_gc, &color);
-		break;
-
-	case HID_MASK_AFTER:
-		mask_seq_id++;
-		if (!mask_seq_id)
-			mask_seq_id = 1;
-		mask_seq = mask_seq_id;
-
-		//priv->drawable = priv->pixmap;
-		break;
-	}
-	cur_mask = use_it;
 }
 
+static void ghid_cairo_set_drawing_mode(pcb_composite_op_t op, pcb_bool direct, const pcb_box_t *screen)
+{
+	render_priv_t *priv = gport->render_priv;
+
+	if (!priv->cr) {
+		//abort();
+		return;
+	}
+
+	if (direct) {
+		//priv->out_pixel = priv->base_pixel;
+		//priv->out_clip = NULL;
+		curr_drawing_mode = PCB_HID_COMP_POSITIVE;
+		return;
+	}
+
+	switch(op) {
+		case PCB_HID_COMP_RESET:
+			//ghid_sketch_setup(priv);
+			start_subcomposite();
+
+		/* clear the canvas */
+			//priv->clip_color.pixel = 0;
+			//if (priv->clear_gc == NULL)
+			//	priv->clear_gc = gdk_gc_new(priv->out_clip);
+			//gdk_gc_set_foreground(priv->clear_gc, &priv->clip_color);
+			//set_clip(priv, priv->clear_gc);
+			//gdk_draw_rectangle(priv->out_clip, priv->clear_gc, TRUE, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
+			break;
+
+		case PCB_HID_COMP_POSITIVE:
+			//priv->clip_color.pixel = 1;
+			cairo_set_operator(priv->cr, CAIRO_OPERATOR_SOURCE);
+			break;
+
+		case PCB_HID_COMP_NEGATIVE:
+			//priv->clip_color.pixel = 0;
+			cairo_set_operator(priv->cr, CAIRO_OPERATOR_CLEAR);
+			break;
+
+		case PCB_HID_COMP_FLUSH:
+			priv->cr = priv->cr_target;
+			//cairo_set_operator(priv->cr, CAIRO_OPERATOR_OVER);
+			//cairo_set_source_surface(priv->cr, priv->surf_layer, 0, 0);
+			cairo_mask_surface(priv->cr, priv->surf_layer, 0, 0);
+			cairo_fill(priv->cr);
+			//cairo_paint_with_alpha(priv->cr, conf_core.appearance.layer_alpha);
+			//cairo_paint_with_alpha(priv->cr, 1.0);
+
+			//if (priv->copy_gc == NULL)
+			//	priv->copy_gc = gdk_gc_new(priv->out_pixel);
+			//gdk_gc_set_clip_mask(priv->copy_gc, priv->sketch_clip);
+			//gdk_gc_set_clip_origin(priv->copy_gc, 0, 0);
+			//gdk_draw_drawable(priv->base_pixel, priv->copy_gc, priv->sketch_pixel, 0, 0, 0, 0, gport->view.canvas_width, gport->view.canvas_height);
+      //
+			//priv->out_pixel = priv->base_pixel;
+			//priv->out_clip = NULL;
+			break;
+	}
+	curr_drawing_mode = op;
+}
 
 typedef struct {
 	int color_set;
@@ -785,6 +792,9 @@ static void ghid_cairo_set_line_width(pcb_hid_gc_t gc, pcb_coord_t width)
 
 static void ghid_cairo_set_draw_xor(pcb_hid_gc_t gc, int xor_mask)
 {
+	render_priv_t *priv = gport->render_priv;
+
+	priv->xor_mode = (xor_mask) ? 1 : 0;
 	//gc->xor_mask = xor_mask;
 	//if (!gc->gc)
 	//  return;
@@ -1104,16 +1114,13 @@ static void redraw_region(GdkRectangle * rect)
 
 	ghid_cairo_draw_bg_image();
 
-	pcb_hid_expose_all(&gtk3_cairo_hid, &ctx);
+	pcb_hid_expose_all(pcb_gui, &ctx);
 	ghid_cairo_draw_grid();
 
-	/* In some cases we are called with the crosshair still off */
-	if (priv->attached_invalidate_depth == 0)
-		pcb_draw_attached();
-
-	/* In some cases we are called with the mark still off */
-	if (priv->mark_invalidate_depth == 0)
-		pcb_draw_mark();
+	/* Draws "GUI" information on top of design */
+	priv->cr = priv->cr_drawing_area;
+	pcb_draw_attached();
+	pcb_draw_mark();
 
 	draw_lead_user(priv);
 
@@ -1163,29 +1170,16 @@ static void ghid_cairo_notify_crosshair_change(pcb_bool changes_complete)
 	if (gport->drawing_area == NULL)
 		return;
 
+	/* Keeps track of calls (including XOR draw mode), and draws only when complete and final. */
 	if (changes_complete)
 		priv->attached_invalidate_depth--;
+	else
+		priv->attached_invalidate_depth = 1 + priv->xor_mode;
 
-	if (priv->attached_invalidate_depth < 0) {
-		priv->attached_invalidate_depth = 0;
-		/* A mismatch of changes_complete == pcb_false and == pcb_true notifications
-		   is not expected to occur, but we will try to handle it gracefully.
-		   As we know the crosshair will have been shown already, we must
-		   repaint the entire view to be sure not to leave an artefact.
-		 */
+	if (changes_complete && gport->drawing_area != NULL && priv->attached_invalidate_depth <= 0) {
+		/* Queues a GTK redraw when changes are complete */
 		ghid_cairo_invalidate_all();
-		return;
-	}
-
-	if (priv->attached_invalidate_depth == 0)
-		pcb_draw_attached();
-
-	if (!changes_complete) {
-		priv->attached_invalidate_depth++;
-	}
-	else if (gport->drawing_area != NULL) {
-		/* Queue a GTK expose when changes are complete */
-		ghid_draw_area_update(gport, NULL);
+		priv->attached_invalidate_depth = 0;
 	}
 }
 
@@ -1380,8 +1374,6 @@ static void ghid_cairo_init_renderer(int *argc, char ***argv, void *vport)
 	port->render_priv->cr = NULL;
 	port->render_priv->surf_da = NULL;
 	port->render_priv->cr_drawing_area = NULL;
-	port->render_priv->surf_ui = NULL;
-	port->render_priv->cr_ui = NULL;
 }
 
 static void ghid_cairo_shutdown_renderer(void *vport)
@@ -1441,15 +1433,8 @@ static void ghid_cairo_drawing_area_configure_hook(void *vport)
 
 	/* Creates a single cairo surface/context for off-line painting */
 	cr_create_similar_surface_and_context(&priv->surf_da, &priv->cr_drawing_area, port);
-
-	/* Creates a surface/context for painting user inter-action feedback (ghosts) */
-	cr_create_similar_surface_and_context(&priv->surf_ui, &priv->cr_ui, port);
-	/* Erases the surface, and points to it */
-	cr = priv->cr_ui;
-	cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
-	cairo_paint_with_alpha(cr, 1.0);
-	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-	priv->cr = priv->cr_ui;
+  cr_create_similar_surface_and_context(&priv->surf_layer, &priv->cr_layer, port);
+	priv->cr = priv->cr_drawing_area;
 }
 
 /* GtkDrawingArea -> GtkWidget "draw" signal Call-Back function */
@@ -1463,14 +1448,10 @@ static gboolean ghid_cairo_drawing_area_expose_cb(GtkWidget * widget, pcb_gtk_ex
 	//                  ev->area.x, ev->area.y, ev->area.x, ev->area.y, ev->area.width, ev->area.height);
 
 	cr_paint_from_surface(p, priv->surf_da);
-	cr_paint_from_surface(p, priv->surf_ui);
-	cairo_set_operator(priv->cr_ui, CAIRO_OPERATOR_CLEAR);
-	cairo_paint_with_alpha(priv->cr_ui, 1.0);
-	cairo_set_operator(priv->cr_ui, CAIRO_OPERATOR_SOURCE);
 
 	priv->cr = p;
 	show_crosshair(TRUE);
-	priv->cr = priv->cr_ui;
+	priv->cr = priv->cr_drawing_area;
 
 	return FALSE;
 }
@@ -1490,17 +1471,6 @@ static void ghid_cairo_screen_update(void)
 
 static void ghid_cairo_port_drawing_realize_cb(GtkWidget * widget, gpointer data)
 {
-	//GHidPort *port = data;
-	//render_priv_t *priv = port->render_priv;
-  //cairo_surface_t *surf;
-  //
-	//surf = gdk_window_create_similar_surface(gtk_widget_get_window(widget),
-	//																				 CAIRO_CONTENT_COLOR_ALPHA,
-	//																				 gport->view.canvas_width, gport->view.canvas_height);
-	////surf = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,
-	////																	gport->view.canvas_width, gport->view.canvas_height);
-	//priv->cr = cairo_create(surf);
-	//cairo_surface_destroy(surf);
 }
 
 static gboolean ghid_cairo_preview_expose(GtkWidget * widget, pcb_gtk_expose_t * p,
@@ -1557,12 +1527,6 @@ static gboolean ghid_cairo_preview_expose(GtkWidget * widget, pcb_gtk_expose_t *
 	return FALSE;
 }
 
-/* TODO: obsolete this function ; retire 'render_pixmap()' from 'struct pcb_gtk_common_t'. */
-static void *ghid_cairo_render_pixmap(int cx, int cy, double zoom, int width, int height, int depth)
-{
-	return NULL;
-}
-
 static pcb_hid_t *ghid_cairo_request_debug_draw(void)
 {
 	/* No special setup requirements, drawing goes into
@@ -1629,7 +1593,6 @@ static void draw_lead_user(render_priv_t * priv)
 void ghid_cairo_install(pcb_gtk_common_t * common, pcb_hid_t * hid)
 {
 	if (common != NULL) {
-		common->render_pixmap = ghid_cairo_render_pixmap;
 		common->init_drawing_widget = ghid_cairo_init_drawing_widget;
 		common->drawing_realize = ghid_cairo_port_drawing_realize_cb;
 		common->drawing_area_expose = ghid_cairo_drawing_area_expose_cb;
@@ -1641,7 +1604,6 @@ void ghid_cairo_install(pcb_gtk_common_t * common, pcb_hid_t * hid)
 		common->draw_grid_local = ghid_cairo_draw_grid_local;
 		common->drawing_area_configure_hook = ghid_cairo_drawing_area_configure_hook;
 		common->shutdown_renderer = ghid_cairo_shutdown_renderer;
-#warning TODO: change common API to use compatibility layer :  pcb_gtk_color_t
 		common->get_color_name = get_color_name;
 		common->map_color_string = map_color_string;
 	}
@@ -1655,7 +1617,8 @@ void ghid_cairo_install(pcb_gtk_common_t * common, pcb_hid_t * hid)
 		hid->end_layer = ghid_cairo_end_layer_group;
 		hid->make_gc = ghid_cairo_make_gc;
 		hid->destroy_gc = ghid_cairo_destroy_gc;
-		hid->use_mask = ghid_cairo_use_mask;
+		hid->render_burst = ghid_cairo_render_burst;
+		hid->set_drawing_mode = ghid_cairo_set_drawing_mode;
 		hid->set_color = ghid_cairo_set_color;
 		hid->set_line_cap = ghid_cairo_set_line_cap;
 		hid->set_line_width = ghid_cairo_set_line_width;
@@ -1670,5 +1633,6 @@ void ghid_cairo_install(pcb_gtk_common_t * common, pcb_hid_t * hid)
 		hid->request_debug_draw = ghid_cairo_request_debug_draw;
 		hid->flush_debug_draw = ghid_cairo_flush_debug_draw;
 		hid->finish_debug_draw = ghid_cairo_finish_debug_draw;
+
 	}
 }
