@@ -583,126 +583,240 @@ static Widget create_form_ok_dialog(const char *name, int ok)
 	return topform;
 }
 
-int lesstif_attribute_dialog(pcb_hid_attribute_t * attrs, int n_attrs, pcb_hid_attr_val_t * results, const char *title, const char *descr)
+static Widget pcb_motif_box(Widget parent, char *name, char type, int num_table_rows, int want_frame)
 {
-	Widget dialog, topform, lform, form;
+	Widget cnt;
+
+	if (want_frame) {
+		/* create and insert frame around the content table */
+		stdarg(XmNalignment, XmALIGNMENT_END);
+		parent = XmCreateFrame(parent, XmStrCast("box-frame"), stdarg_args, stdarg_n);
+		XtManageChild(parent);
+		stdarg_n = 0;
+	}
+
+	switch(type) {
+		case 'h': /* "hbox" */
+			stdarg(XmNorientation, XmHORIZONTAL);
+			stdarg(XmNpacking, XmPACK_TIGHT);
+			break;
+		case 'v': /* "vbox" */
+			stdarg(XmNorientation, XmVERTICAL);
+			stdarg(XmNpacking, XmPACK_COLUMN);
+			break;
+		case 't': /* "table" */
+			stdarg(XmNorientation, XmHORIZONTAL);
+			stdarg(XmNpacking, XmPACK_COLUMN);
+			stdarg(XmNnumColumns, num_table_rows);
+			stdarg(XmNisAligned, True);
+			stdarg(XmNentryAlignment, XmALIGNMENT_END);
+			break;
+		default:
+			abort();
+	}
+	cnt = XmCreateRowColumn(parent, name, stdarg_args, stdarg_n);
+	return cnt;
+}
+
+typedef struct {
+	pcb_hid_attribute_t *attrs;
+	int n_attrs, actual_nattrs;
 	Widget *wl;
-	int i, rv;
+	Widget **btn; /* enum value buttons */
+	pcb_hid_attr_val_t *results;
+	void *caller_data;
+	Widget dialog;
+	pcb_hid_attr_val_t property[PCB_HATP_max];
+	unsigned inhibit_valchg:1;
+} lesstif_attr_dlg_t;
+
+static void attribute_dialog_readres(lesstif_attr_dlg_t *ctx, int widx)
+{
+	char *cp;
+
+	if (ctx->attrs[widx].help_text == ATTR_UNDOCUMENTED)
+		return;
+
+	switch(ctx->attrs[widx].type) {
+		case PCB_HATT_BOOL:
+			ctx->attrs[widx].default_val.int_value = ctx->results[widx].int_value = XmToggleButtonGetState(ctx->wl[widx]);
+			break;
+		case PCB_HATT_STRING:
+			ctx->results[widx].str_value = pcb_strdup(XmTextGetString(ctx->wl[widx]));
+			break;
+		case PCB_HATT_INTEGER:
+			cp = XmTextGetString(ctx->wl[widx]);
+			sscanf(cp, "%d", &ctx->results[widx].int_value);
+			break;
+		case PCB_HATT_COORD:
+			cp = XmTextGetString(ctx->wl[widx]);
+			ctx->results[widx].coord_value = pcb_get_value(cp, NULL, NULL, NULL);
+			break;
+		case PCB_HATT_REAL:
+			cp = XmTextGetString(ctx->wl[widx]);
+			sscanf(cp, "%lg", &ctx->results[widx].real_value);
+			break;
+		case PCB_HATT_ENUM:
+			{
+				const char **uptr;
+				Widget btn;
+
+				stdarg_n = 0;
+				stdarg(XmNmenuHistory, &btn);
+				XtGetValues(ctx->wl[widx], stdarg_args, stdarg_n);
+				stdarg_n = 0;
+				stdarg(XmNuserData, &uptr);
+				XtGetValues(btn, stdarg_args, stdarg_n);
+				ctx->results[widx].int_value = uptr - ctx->attrs[widx].enumerations;
+			}
+			break;
+		default:
+			break;
+	}
+	ctx->attrs[widx].default_val = ctx->results[widx];
+}
+
+static void valchg(Widget w, XtPointer dlg_widget_, XtPointer call_data)
+{
+	lesstif_attr_dlg_t *ctx;
+	Widget dlg_widget = (Widget)dlg_widget_; /* ctx->wl[i] */
+	int widx;
+
+	if (dlg_widget == NULL)
+		return;
+
+	XtVaGetValues(dlg_widget, XmNuserData, &ctx, NULL);
+
+	if (ctx == NULL)
+		return;
+
+	if (ctx->inhibit_valchg)
+		return;
+
+	for(widx = 0; widx < ctx->n_attrs; widx++)
+		if (ctx->wl[widx] == dlg_widget)
+			break;
+
+	if (widx >= ctx->n_attrs)
+		return;
+
+	ctx->attrs[widx].changed = 1;
+
+	if ((ctx->attrs[widx].change_cb == NULL) && (ctx->property[PCB_HATP_GLOBAL_CALLBACK].func == NULL))
+		return;
+
+	attribute_dialog_readres(ctx, widx);
+	if (ctx->property[PCB_HATP_GLOBAL_CALLBACK].func != NULL)
+		ctx->property[PCB_HATP_GLOBAL_CALLBACK].func(ctx, ctx->caller_data, &ctx->attrs[widx]);
+	if (ctx->attrs[widx].change_cb != NULL)
+		ctx->attrs[widx].change_cb(ctx, ctx->caller_data, &ctx->attrs[widx]);
+}
+
+/* returns the index of HATT_END where the loop had to stop */
+static int attribute_dialog_add(lesstif_attr_dlg_t *ctx, Widget parent, int start_from, int add_labels)
+{
+	int len, i, numch, numcol;
 	static XmString empty = 0;
-	int actual_nattrs = 0;
-	int attrcount = 0;
 
 	if (!empty)
 		empty = XmStringCreatePCB(" ");
 
-	for (i = 0; i < n_attrs; i++) {
-		if (attrs[i].help_text != ATTR_UNDOCUMENTED)
-			actual_nattrs++;
-		results[i] = attrs[i].default_val;
-		if (results[i].str_value)
-			results[i].str_value = pcb_strdup(results[i].str_value);
-	}
-
-	wl = (Widget *) malloc(n_attrs * sizeof(Widget));
-
-	topform = create_form_ok_dialog(title, 1);
-	dialog = XtParent(topform);
-
-	stdarg_n = 0;
-	stdarg(XmNfractionBase, n_attrs);
-	XtSetValues(topform, stdarg_args, stdarg_n);
-
-	stdarg_n = 0;
-	stdarg(XmNtopAttachment, XmATTACH_FORM);
-	stdarg(XmNbottomAttachment, XmATTACH_FORM);
-	stdarg(XmNleftAttachment, XmATTACH_FORM);
-	stdarg(XmNfractionBase, actual_nattrs);
-	lform = XmCreateForm(topform, XmStrCast("attributes"), stdarg_args, stdarg_n);
-	XtManageChild(lform);
-
-	stdarg_n = 0;
-	stdarg(XmNtopAttachment, XmATTACH_FORM);
-	stdarg(XmNbottomAttachment, XmATTACH_FORM);
-	stdarg(XmNleftAttachment, XmATTACH_WIDGET);
-	stdarg(XmNleftWidget, lform);
-	stdarg(XmNrightAttachment, XmATTACH_FORM);
-	stdarg(XmNfractionBase, actual_nattrs);
-	form = XmCreateForm(topform, XmStrCast("attributes"), stdarg_args, stdarg_n);
-	XtManageChild(form);
-
-	attrcount = -1;
-	for (i = 0; i < n_attrs; i++) {
+	for (i = start_from; i < ctx->n_attrs; i++) {
+		char buf[30];
 		Widget w;
 
-		if (attrs[i].help_text == ATTR_UNDOCUMENTED)
+		if (ctx->attrs[i].type == PCB_HATT_END)
+			break;
+
+		if (ctx->attrs[i].help_text == ATTR_UNDOCUMENTED)
 			continue;
-		attrcount++;
 
+		/* Add label */
+		if ((add_labels) && (ctx->attrs[i].type != PCB_HATT_LABEL)) {
+			stdarg_n = 0;
+			stdarg(XmNalignment, XmALIGNMENT_END);
+			w = XmCreateLabel(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtManageChild(w);
+		}
+
+		/* Add content */
 		stdarg_n = 0;
-		stdarg(XmNleftAttachment, XmATTACH_FORM);
-		stdarg(XmNrightAttachment, XmATTACH_FORM);
-		stdarg(XmNtopAttachment, XmATTACH_POSITION);
-		stdarg(XmNtopPosition, attrcount);
-		stdarg(XmNbottomAttachment, XmATTACH_POSITION);
-		stdarg(XmNbottomPosition, attrcount + 1);
 		stdarg(XmNalignment, XmALIGNMENT_END);
-		w = XmCreateLabel(lform, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
-		XtManageChild(w);
-	}
+		stdarg(XmNuserData, ctx);
 
-	attrcount = -1;
-	for (i = 0; i < n_attrs; i++) {
-		static char buf[30];
-		stdarg_n = 0;
+		switch(ctx->attrs[i].type) {
+		case PCB_HATT_BEGIN_HBOX:
+			w = pcb_motif_box(parent, XmStrCast(ctx->attrs[i].name), 'h', 0, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_FRAME));
+			XtManageChild(w);
+			i = attribute_dialog_add(ctx, w, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+			break;
 
-		if (attrs[i].help_text == ATTR_UNDOCUMENTED)
-			continue;
-		attrcount++;
+		case PCB_HATT_BEGIN_VBOX:
+			w = pcb_motif_box(parent, XmStrCast(ctx->attrs[i].name), 'v', 0, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_FRAME));
+			XtManageChild(w);
+			i = attribute_dialog_add(ctx, w, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+			break;
 
-		stdarg(XmNleftAttachment, XmATTACH_FORM);
-		stdarg(XmNrightAttachment, XmATTACH_FORM);
-		stdarg(XmNtopAttachment, XmATTACH_POSITION);
-		stdarg(XmNtopPosition, attrcount);
-		stdarg(XmNbottomAttachment, XmATTACH_POSITION);
-		stdarg(XmNbottomPosition, attrcount + 1);
-		stdarg(XmNalignment, XmALIGNMENT_END);
+		case PCB_HATT_BEGIN_TABLE:
+			/* create content table */
+			numcol = ctx->attrs[i].pcb_hatt_table_cols;
+			len = pcb_hid_atrdlg_num_children(ctx->attrs, i+1, ctx->n_attrs);
+			numch = len  / numcol + !!(len % numcol);
+			w = pcb_motif_box(parent, XmStrCast(ctx->attrs[i].name), 't', numch, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_FRAME));
 
-		switch (attrs[i].type) {
+			i = attribute_dialog_add(ctx, w, i+1, (ctx->attrs[i].pcb_hatt_flags & PCB_HATF_LABEL));
+			while((len % numcol) != 0) {
+				Widget pad;
+
+				stdarg_n = 0;
+				stdarg(XmNalignment, XmALIGNMENT_END);
+				pad = XmCreateLabel(w, XmStrCast("."), stdarg_args, stdarg_n);
+				XtManageChild(pad);
+				len++;
+			}
+			XtManageChild(w);
+			break;
 		case PCB_HATT_LABEL:
-			stdarg(XmNlabelString, empty);
-			wl[i] = XmCreateLabel(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
+			stdarg_n = 0;
+			stdarg(XmNalignment, XmALIGNMENT_BEGINNING);
+			ctx->wl[i] = XmCreateLabel(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
 			break;
 		case PCB_HATT_BOOL:
 			stdarg(XmNlabelString, empty);
-			stdarg(XmNset, results[i].int_value);
-			wl[i] = XmCreateToggleButton(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
+			stdarg(XmNset, ctx->results[i].int_value);
+			ctx->wl[i] = XmCreateToggleButton(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtAddCallback(ctx->wl[i], XmNvalueChangedCallback, valchg, ctx->wl[i]);
 			break;
 		case PCB_HATT_STRING:
 			stdarg(XmNcolumns, 40);
 			stdarg(XmNresizeWidth, True);
-			stdarg(XmNvalue, results[i].str_value);
-			wl[i] = XmCreateTextField(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
+			stdarg(XmNvalue, ctx->results[i].str_value);
+			ctx->wl[i] = XmCreateTextField(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtAddCallback(ctx->wl[i], XmNvalueChangedCallback, valchg, ctx->wl[i]);
 			break;
 		case PCB_HATT_INTEGER:
 			stdarg(XmNcolumns, 13);
 			stdarg(XmNresizeWidth, True);
-			sprintf(buf, "%d", results[i].int_value);
+			sprintf(buf, "%d", ctx->results[i].int_value);
 			stdarg(XmNvalue, buf);
-			wl[i] = XmCreateTextField(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
+			ctx->wl[i] = XmCreateTextField(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtAddCallback(ctx->wl[i], XmNvalueChangedCallback, valchg, ctx->wl[i]);
 			break;
 		case PCB_HATT_COORD:
 			stdarg(XmNcolumns, 13);
 			stdarg(XmNresizeWidth, True);
-			pcb_snprintf(buf, sizeof(buf), "%$mS", results[i].coord_value);
+			pcb_snprintf(buf, sizeof(buf), "%$mS", ctx->results[i].coord_value);
 			stdarg(XmNvalue, buf);
-			wl[i] = XmCreateTextField(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
+			ctx->wl[i] = XmCreateTextField(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtAddCallback(ctx->wl[i], XmNvalueChangedCallback, valchg, ctx->wl[i]);
 			break;
 		case PCB_HATT_REAL:
 			stdarg(XmNcolumns, 16);
 			stdarg(XmNresizeWidth, True);
-			pcb_snprintf(buf, sizeof(buf), "%g", results[i].real_value);
+			pcb_snprintf(buf, sizeof(buf), "%g", ctx->results[i].real_value);
 			stdarg(XmNvalue, buf);
-			wl[i] = XmCreateTextField(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
+			ctx->wl[i] = XmCreateTextField(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtAddCallback(ctx->wl[i], XmNvalueChangedCallback, valchg, ctx->wl[i]);
 			break;
 		case PCB_HATT_ENUM:
 			{
@@ -713,93 +827,221 @@ int lesstif_attribute_dialog(pcb_hid_attribute_t * attrs, int n_attrs, pcb_hid_a
 				if (empty == 0)
 					empty = XmStringCreatePCB("");
 
-				submenu = XmCreatePulldownMenu(form, XmStrCast(attrs[i].name), stdarg_args + sn, stdarg_n - sn);
+				submenu = XmCreatePulldownMenu(parent, XmStrCast(ctx->attrs[i].name == NULL ? "anon" : ctx->attrs[i].name), stdarg_args + sn, stdarg_n - sn);
 
 				stdarg_n = sn;
 				stdarg(XmNlabelString, empty);
 				stdarg(XmNsubMenuId, submenu);
-				wl[i] = XmCreateOptionMenu(form, XmStrCast(attrs[i].name), stdarg_args, stdarg_n);
-
-				for (sn = 0; attrs[i].enumerations[sn]; sn++) {
+				ctx->wl[i] = XmCreateOptionMenu(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+				for (sn = 0; ctx->attrs[i].enumerations[sn]; sn++);
+				ctx->btn[i] = calloc(sizeof(Widget), sn);
+				for (sn = 0; ctx->attrs[i].enumerations[sn]; sn++) {
 					Widget btn;
 					XmString label;
 					stdarg_n = 0;
-					label = XmStringCreatePCB(attrs[i].enumerations[sn]);
-					stdarg(XmNuserData, &attrs[i].enumerations[sn]);
+					label = XmStringCreatePCB(ctx->attrs[i].enumerations[sn]);
+					stdarg(XmNuserData, &ctx->attrs[i].enumerations[sn]);
 					stdarg(XmNlabelString, label);
 					btn = XmCreatePushButton(submenu, XmStrCast("menubutton"), stdarg_args, stdarg_n);
 					XtManageChild(btn);
 					XmStringFree(label);
-					if (sn == attrs[i].default_val.int_value)
+					if (sn == ctx->attrs[i].default_val.int_value)
 						default_button = btn;
+					XtAddCallback(btn, XmNactivateCallback, valchg, ctx->wl[i]);
+					(ctx->btn[i])[sn] = btn;
 				}
 				if (default_button) {
 					stdarg_n = 0;
 					stdarg(XmNmenuHistory, default_button);
-					XtSetValues(wl[i], stdarg_args, stdarg_n);
+					XtSetValues(ctx->wl[i], stdarg_args, stdarg_n);
 				}
 			}
 			break;
+		case PCB_HATT_BUTTON:
+			stdarg(XmNlabelString, XmStringCreatePCB(ctx->attrs[i].default_val.str_value));
+			ctx->wl[i] = XmCreatePushButton(parent, XmStrCast(ctx->attrs[i].name), stdarg_args, stdarg_n);
+			XtAddCallback(ctx->wl[i], XmNactivateCallback, valchg, ctx->wl[i]);
+			break;
 		default:
-			wl[i] = XmCreateLabel(form, XmStrCast("UNIMPLEMENTED"), stdarg_args, stdarg_n);
+			ctx->wl[i] = XmCreateLabel(parent, XmStrCast("UNIMPLEMENTED"), stdarg_args, stdarg_n);
 			break;
 		}
-
-		XtManageChild(wl[i]);
+		if (ctx->wl[i] != NULL)
+			XtManageChild(ctx->wl[i]);
 	}
+	return i;
+}
 
-	rv = wait_for_dialog(dialog);
 
-	for (i = 0; i < n_attrs; i++) {
-		char *cp;
+static int attribute_dialog_set(lesstif_attr_dlg_t *ctx, int idx, const pcb_hid_attr_val_t *val)
+{
+	char buf[30];
+	int save, n;
 
-		if (attrs[i].help_text == ATTR_UNDOCUMENTED)
-			continue;
-
-		switch (attrs[i].type) {
+	save = ctx->inhibit_valchg;
+	ctx->inhibit_valchg = 1;
+	switch(ctx->attrs[idx].type) {
+		case PCB_HATT_BEGIN_HBOX:
+		case PCB_HATT_BEGIN_VBOX:
+		case PCB_HATT_BEGIN_TABLE:
+		case PCB_HATT_END:
+		case PCB_HATT_BUTTON:
+			goto err;
+		case PCB_HATT_LABEL:
+			XtVaSetValues(ctx->wl[idx], XmNlabelString, XmStrCast(val->str_value), NULL);
+			break;
 		case PCB_HATT_BOOL:
-			results[i].int_value = XmToggleButtonGetState(wl[i]);
+			XtVaSetValues(ctx->wl[idx], XmNset, val->int_value, NULL);
 			break;
 		case PCB_HATT_STRING:
-			results[i].str_value = XmTextGetString(wl[i]);
+			XtVaSetValues(ctx->wl[idx], XmNvalue, XmStrCast(val->str_value), NULL);
 			break;
 		case PCB_HATT_INTEGER:
-			cp = XmTextGetString(wl[i]);
-			sscanf(cp, "%d", &results[i].int_value);
+			sprintf(buf, "%d", val->int_value);
+			XtVaSetValues(ctx->wl[idx], XmNvalue, XmStrCast(buf), NULL);
 			break;
 		case PCB_HATT_COORD:
-			cp = XmTextGetString(wl[i]);
-			results[i].coord_value = pcb_get_value(cp, NULL, NULL, NULL);
+			pcb_snprintf(buf, sizeof(buf), "%$mS", val->coord_value);
+			XtVaSetValues(ctx->wl[idx], XmNvalue, XmStrCast(buf), NULL);
 			break;
 		case PCB_HATT_REAL:
-			cp = XmTextGetString(wl[i]);
-			sscanf(cp, "%lg", &results[i].real_value);
+			pcb_snprintf(buf, sizeof(buf), "%g", val->real_value);
+			XtVaSetValues(ctx->wl[idx], XmNvalue, XmStrCast(buf), NULL);
 			break;
 		case PCB_HATT_ENUM:
-			{
-				const char **uptr;
-				Widget btn;
-
-				stdarg_n = 0;
-				stdarg(XmNmenuHistory, &btn);
-				XtGetValues(wl[i], stdarg_args, stdarg_n);
-				stdarg_n = 0;
-				stdarg(XmNuserData, &uptr);
-				XtGetValues(btn, stdarg_args, stdarg_n);
-				results[i].int_value = uptr - attrs[i].enumerations;
+			for (n = 0; ctx->attrs[idx].enumerations[n]; n++) {
+				if (n == val->int_value) {
+					stdarg_n = 0;
+					stdarg(XmNmenuHistory, (ctx->btn[idx])[n]);
+					XtSetValues(ctx->wl[idx], stdarg_args, stdarg_n);
+					goto ok;
+				}
 			}
-			break;
+			goto err;
 		default:
-			break;
-		}
+			goto err;
 	}
 
-	free(wl);
-	XtDestroyWidget(dialog);
+	ok:;
+	ctx->attrs[idx].default_val = *val;
+	ctx->inhibit_valchg = save;
+	return 0;
+
+	err:;
+	ctx->inhibit_valchg = save;
+	return -1;
+}
+
+void *lesstif_attr_dlg_new(pcb_hid_attribute_t *attrs, int n_attrs, pcb_hid_attr_val_t *results, const char *title, const char *descr, void *caller_data)
+{
+	Widget topform, main_tbl;
+	int i;
+	lesstif_attr_dlg_t *ctx;
+
+	ctx = calloc(sizeof(lesstif_attr_dlg_t), 1);
+	ctx->attrs = attrs;
+	ctx->results = results;
+	ctx->n_attrs = n_attrs;
+	ctx->caller_data = caller_data;
+
+	for (i = 0; i < n_attrs; i++) {
+		if (attrs[i].help_text != ATTR_UNDOCUMENTED)
+			ctx->actual_nattrs++;
+		results[i] = attrs[i].default_val;
+		if (PCB_HAT_IS_STR(attrs[i].type) && (results[i].str_value))
+			results[i].str_value = pcb_strdup(results[i].str_value);
+		else
+			results[i].str_value = NULL;
+	}
+
+	ctx->wl = (Widget *) calloc(n_attrs, sizeof(Widget));
+	ctx->btn = (Widget **) calloc(n_attrs, sizeof(Widget *));
+
+	topform = create_form_ok_dialog(title, 1);
+	ctx->dialog = XtParent(topform);
+
+	stdarg_n = 0;
+	stdarg(XmNfractionBase, ctx->n_attrs);
+	XtSetValues(topform, stdarg_args, stdarg_n);
+
+	if (!PCB_HATT_IS_COMPOSITE(attrs[0].type)) {
+		stdarg_n = 0;
+		main_tbl = pcb_motif_box(topform, XmStrCast("layout"), 't', pcb_hid_atrdlg_num_children(ctx->attrs, 0, ctx->n_attrs), 0);
+		XtManageChild(main_tbl);
+		attribute_dialog_add(ctx, main_tbl, 0, 1);
+	}
+	else
+		attribute_dialog_add(ctx, topform, 0, (ctx->attrs[0].pcb_hatt_flags & PCB_HATF_LABEL));
+	return ctx;
+}
+
+int lesstif_attr_dlg_run(void *hid_ctx)
+{
+	lesstif_attr_dlg_t *ctx = hid_ctx;
+	return wait_for_dialog(ctx->dialog);
+}
+
+void lesstif_attr_dlg_free(void *hid_ctx)
+{
+	lesstif_attr_dlg_t *ctx = hid_ctx;
+	int i;
+
+	for (i = 0; i < ctx->n_attrs; i++) {
+		attribute_dialog_readres(ctx, i);
+		free(ctx->btn[i]);
+	}
+
+	free(ctx->wl);
+	XtDestroyWidget(ctx->dialog);
+	free(ctx);
+}
+
+void lesstif_attr_dlg_property(void *hid_ctx, pcb_hat_property_t prop, const pcb_hid_attr_val_t *val)
+{
+	lesstif_attr_dlg_t *ctx = hid_ctx;
+	if ((prop >= 0) && (prop < PCB_HATP_max))
+		ctx->property[prop] = *val;
+}
+
+int lesstif_attribute_dialog(pcb_hid_attribute_t * attrs, int n_attrs, pcb_hid_attr_val_t * results, const char *title, const char *descr, void *caller_data)
+{
+	int rv;
+	void *hid_ctx;
+	
+	hid_ctx = lesstif_attr_dlg_new(attrs, n_attrs, results, title, descr, caller_data);
+	rv = lesstif_attr_dlg_run(hid_ctx);
+	lesstif_attr_dlg_free(hid_ctx);
 
 	return rv ? 0 : 1;
 }
 
+int lesstif_attr_dlg_widget_state(void *hid_ctx, int idx, pcb_bool enabled)
+{
+	lesstif_attr_dlg_t *ctx = hid_ctx;
+
+	if ((idx < 0) || (idx >= ctx->n_attrs) || (ctx->wl[idx] == NULL))
+		return -1;
+
+	XtSetSensitive(ctx->wl[idx], enabled);
+	return 0;
+}
+
+int lesstif_attr_dlg_set_value(void *hid_ctx, int idx, const pcb_hid_attr_val_t *val)
+{
+	lesstif_attr_dlg_t *ctx = hid_ctx;
+
+	if ((idx < 0) || (idx >= ctx->n_attrs))
+		return -1;
+
+	if (attribute_dialog_set(ctx, idx, val) == 0) {
+		ctx->results[idx] = *val;
+		return 0;
+	}
+
+	return -1;
+}
+
+
 /* ------------------------------------------------------------ */
 
 static const char dowindows_syntax[] = "DoWindows(1|2|3|4)\n" "DoWindows(Layout|Library|Log|Netlist)";
@@ -911,7 +1153,7 @@ static int Print(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 	}
 	opts = printer->get_export_options(&n);
 	vals = (pcb_hid_attr_val_t *) calloc(n, sizeof(pcb_hid_attr_val_t));
-	if (lesstif_attribute_dialog(opts, n, vals, "Print", "")) {
+	if (lesstif_attribute_dialog(opts, n, vals, "Print", "", NULL)) {
 		free(vals);
 		return 1;
 	}
@@ -948,7 +1190,7 @@ static int PrintCalibrate(int argc, const char **argv, pcb_coord_t x, pcb_coord_
 	printer->calibrate(0.0, 0.0);
 	if (pcb_gui->attribute_dialog(printer_calibrate_attrs, 3,
 														printer_calibrate_values,
-														"Printer Calibration Values", "Enter calibration values for your printer"))
+														"Printer Calibration Values", "Enter calibration values for your printer", NULL))
 		return 1;
 	printer->calibrate(printer_calibrate_values[1].real_value, printer_calibrate_values[2].real_value);
 	return 0;
@@ -1025,7 +1267,7 @@ static int ExportGUI(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 
 	opts = printer->get_export_options(&n);
 	vals = (pcb_hid_attr_val_t *) calloc(n, sizeof(pcb_hid_attr_val_t));
-	if (lesstif_attribute_dialog(opts, n, vals, "Export", NULL)) {
+	if (lesstif_attribute_dialog(opts, n, vals, "Export", NULL, NULL)) {
 		free(vals);
 		return 1;
 	}
@@ -1561,25 +1803,10 @@ void lesstif_attributes_dialog(const char *owner, pcb_attribute_list_t * attrs_l
 	if (wait_for_dialog(attr_dialog) == 0) {
 		int i;
 		/* Copy the values back */
-		for (i = 0; i < attributes_list->Number; i++) {
-			if (attributes_list->List[i].name)
-				free(attributes_list->List[i].name);
-			if (attributes_list->List[i].value)
-				free(attributes_list->List[i].value);
-		}
-		if (attributes_list->Max < attr_num_rows) {
-			int sz = attr_num_rows * sizeof(pcb_attribute_t);
-			if (attributes_list->List == NULL)
-				attributes_list->List = (pcb_attribute_t *) malloc(sz);
-			else
-				attributes_list->List = (pcb_attribute_t *) realloc(attributes_list->List, sz);
-			attributes_list->Max = attr_num_rows;
-		}
-		for (i = 0; i < attr_num_rows; i++) {
-			attributes_list->List[i].name = pcb_strdup(XmTextFieldGetString(attr_row[i].w_name));
-			attributes_list->List[i].value = pcb_strdup(XmTextFieldGetString(attr_row[i].w_value));
-			attributes_list->Number = attr_num_rows;
-		}
+		pcb_attribute_copyback_begin(attributes_list);
+		for (i = 0; i < attr_num_rows; i++)
+			pcb_attribute_copyback(attributes_list, XmTextFieldGetString(attr_row[i].w_name), XmTextFieldGetString(attr_row[i].w_value));
+		pcb_attribute_copyback_end(attributes_list);
 	}
 
 	return;
