@@ -47,6 +47,7 @@
 #include "move.h"
 #include "macro.h"
 #include "obj_all.h"
+#include "safe_fs.h"
 #include "rotate.h"
 #include "../src_plugins/boardflip/boardflip.h"
 #include "hid_actions.h"
@@ -689,7 +690,7 @@ static int autotrax_parse_fill(read_state_t *st, FILE *FP, pcb_element_t *el)
 	}
 
 	if (PCB_layer >= 0 && el == NULL) {
-		polygon = pcb_poly_new(&st->pcb->Data->Layer[PCB_layer], flags);
+		polygon = pcb_poly_new(&st->pcb->Data->Layer[PCB_layer], 0, flags);
 	} else if (PCB_layer < 0 && el == NULL){
 		pcb_message(PCB_MSG_ERROR, "Invalid free fill layer found, %s:%d\n", st->Filename, st->lineno);
 	}
@@ -1011,7 +1012,7 @@ int io_autotrax_read_pcb(pcb_plug_io_t *ctx, pcb_board_t *Ptr, const char *Filen
 	char line[1024];
 	char *s;
 
-	FP = fopen(Filename, "r");
+	FP = pcb_fopen(Filename, "r");
 	if (FP == NULL)
 		return -1;
 
