@@ -35,10 +35,9 @@
 #include "layer.h"
 #include "const.h"
 #include "obj_all.h"
+#include "polygon_offs.h"
 #include "../lib_polyhelp/polyhelp.h"
 
-#define F2S(OBJ, TYPE) pcb_strflg_f2s((OBJ)->Flags, TYPE)
-
 /* ---------------------------------------------------------------------------
  * writes autotrax vias to file
  */
