@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
+#include <stdarg.h>
 
 #include "eagle_bin.h"
 #include "egb_tree.h"
@@ -553,7 +554,7 @@ static const pcb_eagle_script_t pcb_eagle_script[] = {
 			{"dx",  T_INT, 12, 2},
 			{"dx_bin_doubling",  T_INT, 12, 2},
 			{"dy",  T_INT, 14, 2},
-			{"dy_bin_doubling",  T_INT, 12, 2},
+			{"dy_bin_doubling",  T_INT, 14, 2},
 			{"rot",  T_UBF, 16, BITFIELD(2, 0, 11)},
 			{"stop",  T_BMB, 18, 0x01},
 			{"cream",  T_BMB, 18, 0x02},
@@ -765,7 +766,9 @@ static const pcb_eagle_script_t pcb_eagle_script[] = {
 		},
 		{ /* attributes */
 			{"partnumber",  T_INT, 4, 2},
+			{"element",  T_INT, 4, 2}, /*still need to translate element ID number i.e. "5647907" to an instance number, i.e. "Can't find 6 pin 2 called for in netlist."  */
 			{"pin",  T_INT, 6, 2},
+			{"pad",  T_INT, 6, 2}, /*read.c signal dispatch table needs this to behave */
 			TERM
 		},
 	},
@@ -1281,7 +1284,6 @@ static egb_node_t *find_node_name(egb_node_t *first, const char *id_name)
 static int arc_decode(void *ctx, egb_node_t *elem, int arctype, int linetype)
 {
 	htss_entry_t *e;
-	egb_node_t *n;
 
 	char itoa_buffer[64];
 	long c, cx, cy, x1, x2, y1, y2, x3, y3, radius;
@@ -1369,7 +1371,7 @@ static int arc_decode(void *ctx, egb_node_t *elem, int arctype, int linetype)
 
 			theta_1 = 180 - theta_1; /* eagle coord system */
 			theta_2 = 180 - theta_2; /* eagle coord system */
-			
+
 			delta_theta = (theta_2 - theta_1);
 
 			if (!clockwise) {
@@ -1459,7 +1461,7 @@ static int arc_decode(void *ctx, egb_node_t *elem, int arctype, int linetype)
 		}
 
 		radius = (long)(pcb_distance((double)cx, (double)cy, (double)x2, (double)y2));
-		pcb_trace("Using radius for post-processed arc: %ld\n", radius);	
+		pcb_trace("Using radius for post-processed arc: %ld\n", radius);
 		sprintf(itoa_buffer, "%ld", radius);
 		egb_node_prop_set(elem, "radius", itoa_buffer);
 
@@ -1467,15 +1469,63 @@ static int arc_decode(void *ctx, egb_node_t *elem, int arctype, int linetype)
 		egb_node_prop_set(elem, "x", itoa_buffer);
 		sprintf(itoa_buffer, "%ld", cy);
 		egb_node_prop_set(elem, "y", itoa_buffer);
-		
+
 	}
 	return 0;
 }
 
+/* Return the idxth element instance from the elements subtree, or NULL if not found */
+static egb_node_t *elem_ref_by_idx(egb_node_t *elements, long idx)
+{
+	egb_node_t *n;
+
+	/* count children of elelements */
+	for(n = elements->first_child; (n != NULL) && (idx > 1); n = n->next, idx--) ;
+	if (n == NULL)
+		pcb_message(PCB_MSG_ERROR, "io_eagle bin: eagle_elem_ref_by_idx() can't find element placement index %ld\n", idx);
+	return n;
+}
+
+/* Return the refdes of the idxth element instance from the elements subtree, or NULL if not found */
+static const char *elem_refdes_by_idx(egb_node_t *elements, long idx)
+{
+	egb_node_t *e = elem_ref_by_idx(elements, idx);
+	if (e == NULL)
+		return NULL;
+	return egb_node_prop_get(e, "name");
+}
+
+/* bin path walk; the ... is a 0 terminated list of pcb_eagle_binkw_t IDs */
+static egb_node_t *tree_id_path(egb_node_t *subtree, ...)
+{
+	egb_node_t *nd = subtree;
+	pcb_eagle_binkw_t target;
+	va_list ap;
+
+	va_start(ap, subtree);
+
+	/* get next path element */
+	while((target = va_arg(ap, pcb_eagle_binkw_t)) != 0) {
+		/* look for target on this level */
+		for(nd = nd->first_child;;nd = nd->next) {
+			if (nd == NULL) {/* target not found on this level */
+				va_end(ap);
+				return NULL;
+			}
+			if (nd->id == target) /* found, skip to next level */
+				break;
+		}
+	}
+
+	va_end(ap);
+	return nd;
+}
+
+
 static int postprocess_wires(void *ctx, egb_node_t *root)
 {
 	htss_entry_t *e;
-	egb_node_t *n, *first_c, *wire_t;
+	egb_node_t *n;
 	int line_type = -1;
 
 	if (root->id == PCB_EGKW_SECT_LINE) {
@@ -1525,7 +1575,7 @@ static int postprocess_wires(void *ctx, egb_node_t *root)
 static int postprocess_arcs(void *ctx, egb_node_t *root)
 {
 	htss_entry_t *e;
-	egb_node_t *n, *first_c, *arc_t;
+	egb_node_t *n;
 	int arc_type = -1;
 
 	if (root->id == PCB_EGKW_SECT_ARC) {
@@ -1607,7 +1657,7 @@ static int postproc_elements(void *ctx, egb_node_t *root)
 		if (n->first_child && n->first_child->id == PCB_EGKW_SECT_ELEMENT) {
 			pcb_trace("Found PCB_EKGW_SECT_ELEMENT\n");
 			el1 = n->first_child;
-		} 
+		}
 	}
 
 	for(n = el1; n != NULL; n = next) {
