@@ -203,7 +203,7 @@ int io_eagle_test_parse_pcb_bin(pcb_plug_io_t *ctx, pcb_board_t *Ptr, const char
 	unsigned char buff[2];
 	int read_length = fread(buff, 1, 2, f);
 	if ((read_length == 2) && (buff[0] == 0x10) && (buff[1] == 0x00)) {
-		return 1; /* Eagle v4, v5 */ 
+		return 1; /* Eagle v4, v5 */
 	} else if ((read_length == 2) && (buff[0] == 0x10) && (buff[1] == 0x80)) {
 		return 1; /* Eagle v3 */
 	}
@@ -327,7 +327,7 @@ static int eagle_read_layers(read_state_t *st, trnode_t *subtree, void *obj, int
 					reuse = 1;
 					typ = PCB_LYT_SILK | PCB_LYT_BOTTOM;
 					break;
-				case 199:
+				case 20: /*199:   20 is dimension, 199 is contour */
 					grp = pcb_get_grp_new_intern(st->pcb, -1);
 					ly->ly = pcb_layer_create(grp - st->pcb->LayerGroups.grp, ly->name);
 					pcb_layergrp_fix_turn_to_outline(grp);
@@ -554,7 +554,7 @@ static int eagle_read_circle(read_state_t *st, trnode_t *subtree, void *obj, int
 	circ->StartAngle = eagle_degrees_to_pcb_degrees(eagle_get_attrl(st, subtree, "StartAngle", 0));
 	circ->Delta = -eagle_get_attrl(st, subtree, "Delta", -360);
 	circ->Thickness = eagle_get_attrc(st, subtree, "width", -1);
-        circ->Thickness += eagle_get_attrc(st, subtree, "width_doubling_bin", 0);	
+        circ->Thickness += eagle_get_attrc(st, subtree, "width_doubling_bin", 0);
 	circ->Clearance = st->md_wire_wire*2;
 	circ->Flags = pcb_flag_make(PCB_FLAG_CLEARLINE);
 
@@ -680,7 +680,7 @@ static int eagle_read_wire(read_state_t * st, trnode_t * subtree, void *obj, int
 		pcb_trace("Found null wire layer number 'ln'");
 	}
 
-	ly = eagle_layer_get(st, ln); 
+	ly = eagle_layer_get(st, ln);
 	if (ly != NULL) {
 		pcb_trace("Allocated layer 'ly' via eagle_layer_get(st, ln)\n");
 	}
@@ -739,6 +739,7 @@ static int eagle_read_smd(read_state_t *st, trnode_t *subtree, void *obj, int ty
 	long ln = eagle_get_attrl(st, subtree, "layer", -1);
 	const char *name, *rot;
 	int deg = 0;
+	long roundness = 0;
 
 	assert(type == IN_ELEM);
 
@@ -753,7 +754,9 @@ static int eagle_read_smd(read_state_t *st, trnode_t *subtree, void *obj, int ty
 	rot = eagle_get_attrs(st, subtree, "rot", NULL);
 	deg = eagle_rot2degrees(rot);
 
-#warning TODO need to load thermals flags to set clearance.
+	roundness = eagle_get_attrl(st, subtree, "roundness", 0);
+
+#warning TODO need to load thermals flags to set clearance; may in fact be more contactref related.
 
 	if (dx < 0) {
 		x -= dx;
@@ -810,6 +813,10 @@ static int eagle_read_smd(read_state_t *st, trnode_t *subtree, void *obj, int ty
 
 	pcb_trace("%mm %mm -> %mm %mm\n", x, y, dx, dy);
 
+#warning TODO padstacks - consider roundrect, oval etc shapes when padstacks available
+	if (roundness >= 65) /* round smd pads found in fiducials, some discretes, it seems */
+		PCB_FLAG_CLEAR(PCB_FLAG_SQUARE, pad);
+
 	if (ln == 16)
 		PCB_FLAG_SET(PCB_FLAG_ONSOLDER, pad);
 
@@ -830,7 +837,6 @@ static int eagle_read_pad_or_hole(read_state_t *st, trnode_t *subtree, void *obj
 	dia = eagle_get_attrc(st, subtree, "diameter", drill * (1.0+st->rv_pad_top*2.0));
 	shape = eagle_get_attrs(st, subtree, "shape", 0);
 
-
 	if ((dia - drill) / 2.0 < st->ms_width)
 		dia = drill + 2*st->ms_width;
 
@@ -847,10 +853,11 @@ static int eagle_read_pad_or_hole(read_state_t *st, trnode_t *subtree, void *obj
 
 	if (hole)
 		PCB_FLAG_SET(PCB_FLAG_ONSOLDER, pin);
-
-	if ((shape != NULL) && (strcmp(shape, "octagon") == 0))
+#warning TODO padstacks will allow more pad shapes
+ 	/* shape = {square, round, octagon, long, offset} binary */
+	if ((shape != NULL) && ((strcmp(shape, "octagon") == 0) || (strcmp(shape, "2") == 0)))
 		PCB_FLAG_SET(PCB_FLAG_OCTAGON, pin);
-	else if ((shape != NULL) && (strcmp(shape, "square") == 0))
+	else if ((shape != NULL) && ((strcmp(shape, "square") == 0) || (strcmp(shape, "0") == 0)))
 		PCB_FLAG_SET(PCB_FLAG_SQUARE, pin);
 	else
 		PCB_FLAG_SET(PCB_FLAG_VIA, pin);
@@ -908,7 +915,7 @@ static int eagle_read_pkg_txt(read_state_t *st, trnode_t *subtree, void *obj, in
 		size = eagle_get_attrc(st, subtree, "size", EAGLE_TEXT_SIZE_100);
 		st->value_scale = (int)(((double)size/ (double)EAGLE_TEXT_SIZE_100) * 100.0);
 		st->value_x = eagle_get_attrc(st, subtree, "x", 0);
-		st->value_y = eagle_get_attrc(st, subtree, "y", 0); 
+		st->value_y = eagle_get_attrc(st, subtree, "y", 0);
 	} else {
 		return 0;
 	}
@@ -1028,8 +1035,8 @@ static int eagle_read_contactref(read_state_t *st, trnode_t *subtree, void *obj,
 
 
 	if ((elem == NULL) || (pad == NULL)) {
-		pcb_message(PCB_MSG_WARNING, "Ignoring contactref: missing element or pad\n");
-		return 0;
+		pcb_message(PCB_MSG_WARNING, "Failed to parse contactref node: missing \"element\" or \"pad\" netlist attributes\n");
+		return -1;
 	}
 
 	net = eagle_get_attrs(st, PARENT(subtree), "name", NULL);
@@ -1055,7 +1062,7 @@ static int eagle_read_poly(read_state_t *st, trnode_t *subtree, void *obj, int t
 		return 0;
 	}
 
-	poly = pcb_poly_new(&st->pcb->Data->Layer[ly->ly], pcb_flag_make(PCB_FLAG_CLEARPOLY));
+	poly = pcb_poly_new(&st->pcb->Data->Layer[ly->ly], 0, pcb_flag_make(PCB_FLAG_CLEARPOLY));
 
 	for(n = CHILDREN(subtree); n != NULL; n = NEXT(n)) {
 		if (STRCMP(NODENAME(n), "vertex") == 0) {
@@ -1100,7 +1107,7 @@ static int eagle_read_signals(read_state_t *st, trnode_t *subtree, void *obj, in
 {
 	trnode_t *n;
 	static const dispatch_t disp[] = { /* possible children of <library> */
-		{"contactref",  eagle_read_contactref},
+		{"contactref",  eagle_read_contactref}, /* if this fails, rest of disp acts up */
 		{"wire",        eagle_read_wire},
 		{"arc",         eagle_read_circle}, /*binary format */
 		{"polygon",     eagle_read_poly},
@@ -1118,7 +1125,7 @@ static int eagle_read_signals(read_state_t *st, trnode_t *subtree, void *obj, in
 			const char *name = eagle_get_attrs(st, n, "name", NULL);
 			if (name == NULL) {
 				pcb_message(PCB_MSG_WARNING, "Ignoring signal with no name\n");
-				continue; 
+				continue;
 			}
 			eagle_foreach_dispatch(st, CHILDREN(n), disp, (char *)name, ON_BOARD);
 		}
@@ -1134,7 +1141,8 @@ static void eagle_read_elem_text(read_state_t *st, trnode_t *nd, pcb_element_t *
 {
 	int direction = 0;
 	pcb_flag_t TextFlags = pcb_no_flags();
-        pcb_coord_t size;
+	pcb_coord_t size;
+	int TextScale;
 
 	x += def_text->X;
 	y += def_text->Y + EAGLE_TEXT_SIZE_100;
@@ -1153,13 +1161,13 @@ static void eagle_read_elem_text(read_state_t *st, trnode_t *nd, pcb_element_t *
 	}
 #warning subc TODO can have unique text scaling in subcircuits
 
-	int TextScale = (def_text->Scale == 0 ? 100 : def_text->Scale);
+	TextScale = (def_text->Scale == 0 ? 100 : def_text->Scale);
 #warning TODO scaling not behaving due to size read issue so hard wired for now
 /*	if (size >= 0)
 		TextScale = (int)(((double)size/ (double)EAGLE_TEXT_SIZE_100) * 100.0);
 */
 	TextScale = 100; /* hardwired for now */
-/*	pcb_trace("About to use text scale %d for element.\n", TextScale); 
+/*	pcb_trace("About to use text scale %d for element.\n", TextScale);
 
 	pcb_element_text_set(text, pcb_font(st->pcb, 0, 1), x, y, direction, str, TextScale, TextFlags);
 */
@@ -1327,7 +1335,6 @@ static int eagle_read_board(read_state_t *st, trnode_t *subtree, void *obj, int
 
 static int eagle_read_drawing(read_state_t *st, trnode_t *subtree, void *obj, int type)
 {
-	
 	static const dispatch_t disp[] = { /* possible children of <drawing> */
 		{"settings",  eagle_read_nop},
 		{"layers",    eagle_read_layers},
