@@ -45,6 +45,7 @@
 #include "macro.h"
 #include "obj_all.h"
 #include "rotate.h"
+#include "safe_fs.h"
 
 typedef struct {
 	pcb_board_t *PCB;
@@ -1015,7 +1016,7 @@ static int kicad_get_layeridx(read_state_t *st, const char *kicad_name)
 			/* Workaround: specal case InX.Cu, where X is an integer */
 			char *end;
 			long id = strtol(kicad_name+2, &end, 10);
-			if ((strcasecmp(end, ".Cu") == 0) && (id >= 1) && (id <=30)) {
+			if ((pcb_strcasecmp(end, ".Cu") == 0) && (id >= 1) && (id <=30)) {
 				if (kicad_reg_layer(st, kicad_name, PCB_LYT_COPPER | PCB_LYT_INTERN) == 0) {
 					pcb_trace("Created implicit copper layer %s as %d\n", kicad_name, id);
 					return kicad_get_layeridx(st, kicad_name);
@@ -2143,7 +2144,7 @@ static int kicad_parse_zone(read_state_t *st, gsxl_node_t *subtree)
 					if (PCBLayer < 0) {
 						return kicad_warning(subtree, "parse error: zone layer <0.");
 					}
-					polygon = pcb_poly_new(&st->PCB->Data->Layer[PCBLayer], flags);
+					polygon = pcb_poly_new(&st->PCB->Data->Layer[PCBLayer], 0, flags);
 				} else {
 					return kicad_error(subtree, "unexpected zone layer null node.");
 				}
@@ -2285,7 +2286,7 @@ int io_kicad_read_pcb(pcb_plug_io_t *ctx, pcb_board_t *Ptr, const char *Filename
 	gsx_parse_res_t res;
 	FILE *FP;
 
-		FP = fopen(Filename, "r");
+	FP = pcb_fopen(Filename, "r");
 	if (FP == NULL)
 		return -1;
 
