@@ -48,8 +48,9 @@
 #include "obj_subc.h"
 #include "pcb_minuid.h"
 #include "io_lihata.h"
+#include "safe_fs.h"
 
-#warning TODO: put these in a gloal load-context-struct
+#warning cleanup TODO: put these in a gloal load-context-struct
 vtptr_t post_ids, post_thermal;
 static int rdver;
 
@@ -75,7 +76,7 @@ static int parse_attributes(pcb_attribute_list_t *list, lht_node_t *nd)
 
 	for(n = lht_dom_first(&it, nd); n != NULL; n = lht_dom_next(&it)) {
 		if (n->type == LHT_TEXT)
-			pcb_attribute_put(list, n->name, n->data.text.value, 0);
+			pcb_attribute_put(list, n->name, n->data.text.value);
 	}
 
 	return 0;
@@ -329,7 +330,7 @@ static int post_thermal_assign(vtptr_t *pt)
 /* NOTE: in case of objects with thermal, f must point to the object's
    flags because termals will be filled in at the end, in a 2nd pass and
    we need to store the f pointer. */
-static int parse_flags(pcb_flag_t *f, lht_node_t *fn, int object_type)
+static int parse_flags(pcb_flag_t *f, lht_node_t *fn, int object_type, unsigned char *intconn)
 {
 	io_lihata_flag_holder fh;
 
@@ -355,8 +356,9 @@ static int parse_flags(pcb_flag_t *f, lht_node_t *fn, int object_type)
 		if (parse_int(&n, lht_dom_hash_get(fn, "shape")) == 0)
 			fh.Flags.q = n;
 
-		if (parse_int(&n, lht_dom_hash_get(fn, "intconn")) == 0)
-			fh.Flags.int_conn_grp = n;
+		if ((intconn != NULL) && (rdver < 3))
+			if (parse_int(&n, lht_dom_hash_get(fn, "intconn")) == 0)
+				*intconn = n;
 	}
 
 	*f = fh.Flags;
@@ -367,6 +369,7 @@ static int parse_flags(pcb_flag_t *f, lht_node_t *fn, int object_type)
 static int parse_line(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, int no_id, pcb_coord_t dx, pcb_coord_t dy)
 {
 	pcb_line_t *line;
+	unsigned char intconn = 0;
 
 	if (ly != NULL)
 		line = pcb_line_alloc(ly);
@@ -379,8 +382,9 @@ static int parse_line(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, int n
 		line->ID = 0;
 	else
 		parse_id(&line->ID, obj, 5);
+	parse_flags(&line->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE, &intconn);
+	pcb_attrib_compat_set_intconn(&line->Attributes, intconn);
 	parse_attributes(&line->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&line->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE);
 
 	parse_coord(&line->Thickness, lht_dom_hash_get(obj, "thickness"));
 	parse_coord(&line->Clearance, lht_dom_hash_get(obj, "clearance"));
@@ -408,18 +412,21 @@ static int parse_line(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, int n
 static int parse_rat(pcb_data_t *dt, lht_node_t *obj)
 {
 	pcb_rat_t rat, *new_rat;
+	int tmp;
 
 	parse_id(&rat.ID, obj, 4);
+	parse_flags(&rat.Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE, NULL);
 	parse_attributes(&rat.Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&rat.Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_LINE);
 
 	parse_coord(&rat.Point1.X, lht_dom_hash_get(obj, "x1"));
 	parse_coord(&rat.Point1.Y, lht_dom_hash_get(obj, "y1"));
 	parse_coord(&rat.Point2.X, lht_dom_hash_get(obj, "x2"));
 	parse_coord(&rat.Point2.Y, lht_dom_hash_get(obj, "y2"));
 
-	parse_int(&rat.group1, lht_dom_hash_get(obj, "lgrp1"));
-	parse_int(&rat.group2, lht_dom_hash_get(obj, "lgrp2"));
+	parse_int(&tmp, lht_dom_hash_get(obj, "lgrp1"));
+	rat.group1 = tmp;
+	parse_int(&tmp, lht_dom_hash_get(obj, "lgrp2"));
+	rat.group2 = tmp;
 
 	post_id_req(&rat.Point1);
 	post_id_req(&rat.Point2);
@@ -435,6 +442,7 @@ static int parse_rat(pcb_data_t *dt, lht_node_t *obj)
 static int parse_arc(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy)
 {
 	pcb_arc_t *arc;
+	unsigned char intconn = 0;
 
 	if (ly != NULL)
 		arc = pcb_arc_alloc(ly);
@@ -444,8 +452,9 @@ static int parse_arc(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj, pcb_co
 		return -1;
 
 	parse_id(&arc->ID, obj, 4);
+	parse_flags(&arc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ARC, &intconn);
+	pcb_attrib_compat_set_intconn(&arc->Attributes, intconn);
 	parse_attributes(&arc->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&arc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ARC);
 
 	parse_coord(&arc->Thickness, lht_dom_hash_get(obj, "thickness"));
 	parse_coord(&arc->Clearance, lht_dom_hash_get(obj, "clearance"));
@@ -471,10 +480,15 @@ static int parse_polygon(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj)
 	pcb_polygon_t *poly = pcb_poly_alloc(ly);
 	lht_node_t *geo;
 	pcb_cardinal_t n = 0, c;
+	unsigned char intconn = 0;
 
 	parse_id(&poly->ID, obj, 8);
+	parse_flags(&poly->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_POLYGON, &intconn);
+	pcb_attrib_compat_set_intconn(&poly->Attributes, intconn);
 	parse_attributes(&poly->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&poly->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_POLYGON);
+
+	if (rdver >= 3)
+		parse_coord(&poly->Clearance, lht_dom_hash_get(obj, "clearance"));
 
 	geo = lht_dom_hash_get(obj, "geometry");
 	if ((geo != NULL) && (geo->type == LHT_LIST)) {
@@ -523,6 +537,7 @@ static int parse_pcb_text(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj)
 	pcb_text_t *text;
 	lht_node_t *role;
 	int tmp;
+	unsigned char intconn = 0;
 
 	role = lht_dom_hash_get(obj, "role");
 
@@ -543,8 +558,9 @@ static int parse_pcb_text(pcb_layer_t *ly, pcb_element_t *el, lht_node_t *obj)
 
 	parse_id(&text->ID, obj, 5);
 
+	parse_flags(&text->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_TEXT, &intconn);
+	pcb_attrib_compat_set_intconn(&text->Attributes, intconn);
 	parse_attributes(&text->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&text->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_TEXT);
 	parse_int(&text->Scale, lht_dom_hash_get(obj, "scale"));
 	parse_int(&tmp, lht_dom_hash_get(obj, "fid"));
 	text->fid = tmp;
@@ -593,30 +609,7 @@ static int parse_data_layer(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *grp, i
 
 	ly->parent = dt;
 
-	if (bound) {
-		ly->meta.bound.name = pcb_strdup(grp->name);
-		parse_int(&dt->Layer[layer_id].meta.bound.stack_offs, lht_dom_hash_get(grp, "stack_offs"));
-		parse_layer_type(&dt->Layer[layer_id].meta.bound.type, lht_dom_hash_get(grp, "type"), "bound layer");
-		if (pcb != NULL) {
-			dt->Layer[layer_id].meta.bound.real = pcb_layer_resolve_binding(pcb, &dt->Layer[layer_id]);
-			if (dt->Layer[layer_id].meta.bound.real != NULL)
-				pcb_layer_link_trees(&dt->Layer[layer_id], dt->Layer[layer_id].meta.bound.real);
-			else
-				pcb_message(PCB_MSG_WARNING, "Can't bind subcircuit layer %s: can't find anything similar on the current board\n", dt->Layer[layer_id].meta.bound.name);
-			dt->via_tree = subc_parent->via_tree;
-		}
-	}
-	else {
-		/* real */
-	ly->meta.real.name = pcb_strdup(grp->name);
-		parse_bool(&ly->meta.real.vis, lht_dom_hash_get(grp, "visible"));
-		if (pcb != NULL) {
-			int grp_id;
-			parse_int(&grp_id, lht_dom_hash_get(grp, "group"));
-			dt->Layer[layer_id].grp = grp_id;
-	/*		pcb_trace("parse_data_layer name: %d,%d '%s' grp=%d\n", layer_id, dt->LayerN-1, ly->meta.real.name, grp_id);*/
-		}
-	}
+	parse_attributes(&ly->meta.real.Attributes, lht_dom_hash_get(grp, "attributes"));
 
 	ncmb = lht_dom_hash_get(grp, "combining");
 	if (ncmb != NULL) {
@@ -630,11 +623,36 @@ static int parse_data_layer(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *grp, i
 			}
 			cval = pcb_layer_comb_str2bit(n->name);
 			if (cval == 0)
-				pcb_message(PCB_MSG_WARNING, "Ignoring unknown cmbining flag: '%s'\n", n->name);
+				pcb_message(PCB_MSG_WARNING, "Ignoring unknown combining flag: '%s'\n", n->name);
 			ly->comb |= cval;
 		}
 	}
 
+	if (bound) {
+		ly->meta.bound.name = pcb_strdup(grp->name);
+		parse_int(&dt->Layer[layer_id].meta.bound.stack_offs, lht_dom_hash_get(grp, "stack_offs"));
+		parse_layer_type(&dt->Layer[layer_id].meta.bound.type, lht_dom_hash_get(grp, "type"), "bound layer");
+		if (pcb != NULL) {
+			dt->Layer[layer_id].meta.bound.real = pcb_layer_resolve_binding(pcb, &dt->Layer[layer_id]);
+			if (dt->Layer[layer_id].meta.bound.real != NULL)
+				pcb_layer_link_trees(&dt->Layer[layer_id], dt->Layer[layer_id].meta.bound.real);
+			else if (!(dt->Layer[layer_id].meta.bound.type & PCB_LYT_VIRTUAL))
+				pcb_message(PCB_MSG_WARNING, "Can't bind subcircuit layer %s: can't find anything similar on the current board\n", dt->Layer[layer_id].meta.bound.name);
+			dt->via_tree = subc_parent->via_tree;
+		}
+	}
+	else {
+		/* real */
+	ly->meta.real.name = pcb_strdup(grp->name);
+		parse_bool(&ly->meta.real.vis, lht_dom_hash_get(grp, "visible"));
+		if (pcb != NULL) {
+			int grp_id;
+			parse_int(&grp_id, lht_dom_hash_get(grp, "group"));
+			dt->Layer[layer_id].grp = grp_id;
+	/*		pcb_trace("parse_data_layer name: %d,%d '%s' grp=%d\n", layer_id, dt->LayerN-1, ly->meta.real.name, grp_id);*/
+		}
+	}
+
 	lst = lht_dom_hash_get(grp, "objects");
 	if (lst != NULL) {
 		if (lst->type != LHT_LIST)
@@ -672,6 +690,7 @@ static int parse_data_layers(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *grp,
 static int parse_pin(pcb_data_t *dt, pcb_element_t *el, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy)
 {
 	pcb_pin_t *via;
+	unsigned char intconn = 0;
 
 	if (dt != NULL)
 		via = pcb_via_alloc(dt);
@@ -681,8 +700,9 @@ static int parse_pin(pcb_data_t *dt, pcb_element_t *el, lht_node_t *obj, pcb_coo
 		return -1;
 
 	parse_id(&via->ID, obj, 4);
+	parse_flags(&via->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_VIA, &intconn);
+	pcb_attrib_compat_set_intconn(&via->Attributes, intconn);
 	parse_attributes(&via->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&via->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_VIA);
 
 	parse_coord(&via->Thickness, lht_dom_hash_get(obj, "thickness"));
 	parse_coord(&via->Clearance, lht_dom_hash_get(obj, "clearance"));
@@ -707,12 +727,14 @@ static int parse_pin(pcb_data_t *dt, pcb_element_t *el, lht_node_t *obj, pcb_coo
 static int parse_pad(pcb_element_t *el, lht_node_t *obj, pcb_coord_t dx, pcb_coord_t dy)
 {
 	pcb_pad_t *pad;
+	unsigned char intconn = 0;
 
 	pad = pcb_pad_alloc(el);
 
 	parse_id(&pad->ID, obj, 4);
+	parse_flags(&pad->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_PAD, &intconn);
+	pcb_attrib_compat_set_intconn(&pad->Attributes, intconn);
 	parse_attributes(&pad->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&pad->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_PAD);
 
 	parse_coord(&pad->Thickness, lht_dom_hash_get(obj, "thickness"));
 	parse_coord(&pad->Clearance, lht_dom_hash_get(obj, "clearance"));
@@ -745,8 +767,8 @@ static int parse_element(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj)
 	int onsld;
 
 	parse_id(&elem->ID, obj, 8);
+	parse_flags(&elem->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT, NULL);
 	parse_attributes(&elem->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&elem->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT);
 	parse_coord(&elem->MarkX, lht_dom_hash_get(obj, "x"));
 	parse_coord(&elem->MarkY, lht_dom_hash_get(obj, "y"));
 
@@ -783,17 +805,20 @@ static int parse_element(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj)
 	return 0;
 }
 
-static int parse_subc(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj)
+static int parse_subc(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj, pcb_subc_t **subc_out)
 {
-	pcb_subc_t *sc = pcb_subc_alloc(dt);
+	pcb_subc_t *sc = pcb_subc_alloc();
+	unsigned char intconn = 0;
 
 	parse_id(&sc->ID, obj, 5);
+	parse_flags(&sc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT, &intconn);
+	pcb_attrib_compat_set_intconn(&sc->Attributes, intconn);
 	parse_attributes(&sc->Attributes, lht_dom_hash_get(obj, "attributes"));
-	parse_flags(&sc->Flags, lht_dom_hash_get(obj, "flags"), PCB_TYPE_ELEMENT);
 	parse_minuid(sc->uid, lht_dom_hash_get(obj, "uid"));
 
 	if (!dt->via_tree)
 		dt->via_tree = pcb_r_create_tree(NULL, 0, 0);
+	sc->data->via_tree = dt->via_tree;
 
 	pcb_add_subc_to_data(dt, sc);
 
@@ -805,6 +830,9 @@ static int parse_subc(pcb_board_t *pcb, pcb_data_t *dt, lht_node_t *obj)
 		dt->subc_tree = pcb_r_create_tree(NULL, 0, 0);
 	pcb_r_insert_entry(dt->subc_tree, (pcb_box_t *)sc, 0);
 
+	if (subc_out != NULL)
+		*subc_out = sc;
+
 	return 0;
 }
 
@@ -825,7 +853,7 @@ static int parse_data_objects(pcb_board_t *pcb_for_font, pcb_data_t *dt, lht_nod
 		else if (strncmp(n->name, "element.", 8) == 0)
 			parse_element(pcb_for_font, dt, n);
 		else if (strncmp(n->name, "subc.", 5) == 0)
-			parse_subc(pcb_for_font, dt, n);
+			parse_subc(pcb_for_font, dt, n, NULL);
 	}
 
 	return 0;
@@ -1251,7 +1279,7 @@ static void parse_conf(pcb_board_t *pcb, lht_node_t *sub)
 	if (conf_insert_tree_as(CFR_DESIGN, sub) != 0)
 		pcb_message(PCB_MSG_ERROR, "Failed to insert the config subtree found in %s\n", pcb->Filename);
 	else
-		conf_update(NULL);
+		conf_update(NULL, -1);
 }
 
 
@@ -1260,10 +1288,6 @@ static int parse_board(pcb_board_t *pcb, lht_node_t *nd)
 	lht_node_t *sub;
 	pcb_plug_io_t *loader;
 
-	if ((nd->type != LHT_HASH) || (strncmp(nd->name, "pcb-rnd-board-v", 15) != 0)) {
-		pcb_message(PCB_MSG_ERROR, "Not a board lihata.\n");
-		return -1;
-	}
 	rdver = atoi(nd->name+15);
 	switch(rdver) {
 		case 1: loader = &plug_io_lihata_v1; break;
@@ -1343,15 +1367,46 @@ static int parse_board(pcb_board_t *pcb, lht_node_t *nd)
 int io_lihata_parse_pcb(pcb_plug_io_t *ctx, pcb_board_t *Ptr, const char *Filename, conf_role_t settings_dest)
 {
 	int res;
-	char *errmsg;
-	lht_doc_t *doc = lht_dom_load(Filename, &errmsg);
+	char *errmsg, *realfn;
+	lht_doc_t *doc = NULL;
+	pcb_board_t *old_pcb;
+
+	realfn = pcb_fopen_check(Filename, "r");
+	if (realfn != NULL)
+		doc = lht_dom_load(realfn, &errmsg);
+	free(realfn);
 
 	if (doc == NULL) {
 		pcb_message(PCB_MSG_ERROR, "Error loading '%s': %s\n", Filename, errmsg);
 		return -1;
 	}
 
-	res = parse_board(Ptr, doc->root);
+#warning TODO#5: not faking PCB would trigger layer binding bugs because the "is bound" flag is not explicit in pcb_layer_t
+	old_pcb = PCB;
+	PCB = Ptr;
+
+	if ((doc->root->type == LHT_HASH) && (strncmp(doc->root->name, "pcb-rnd-board-v", 15) == 0)) {
+		res = parse_board(Ptr, doc->root);
+	}
+	else if ((doc->root->type == LHT_LIST) && (strncmp(doc->root->name, "pcb-rnd-subcircuit-v", 20) == 0)) {
+		pcb_subc_t *sc;
+
+		rdver = atoi(doc->root->name+20);
+		Ptr->is_footprint = 1;
+		res = parse_subc(NULL, Ptr->Data, doc->root->data.list.first, &sc);
+
+		pcb_layer_group_setup_default(&Ptr->LayerGroups);
+		pcb_layer_group_setup_silks(&Ptr->LayerGroups);
+		pcb_layer_create_all_for_recipe(Ptr, sc->data->Layer, sc->data->LayerN);
+		pcb_subc_rebind(Ptr, sc);
+	}
+	else {
+		pcb_message(PCB_MSG_ERROR, "Error loading '%s': neither a board nor a subcircuit\n", Filename);
+		res = -1;
+	}
+
+	PCB = old_pcb;
+
 	lht_dom_uninit(doc);
 	return res;
 }
@@ -1371,6 +1426,8 @@ void test_parse_ev(lht_parse_t *ctx, lht_event_t ev, lht_node_type_t nt, const c
 	if (ev == LHT_OPEN) {
 		if ((nt == LHT_HASH) && (strncmp(name, "pcb-rnd-board-v", 15) == 0))
 			*state = TPS_GOOD;
+		else if ((nt == LHT_LIST) && (strncmp(name, "pcb-rnd-subcircuit-v", 20) == 0))
+			*state = TPS_GOOD;
 		else
 			*state = TPS_BAD;
 	}
@@ -1410,8 +1467,13 @@ int io_lihata_test_parse_pcb(pcb_plug_io_t *plug_ctx, pcb_board_t *Ptr, const ch
 int io_lihata_parse_font(pcb_plug_io_t *ctx, pcb_font_t *Ptr, const char *Filename)
 {
 	int res;
-	char *errmsg;
-	lht_doc_t *doc = lht_dom_load(Filename, &errmsg);
+	char *errmsg, *realfn;
+	lht_doc_t *doc = NULL;
+
+	realfn = pcb_fopen_check(Filename, "r");
+	if (realfn != NULL)
+		doc = lht_dom_load(realfn, &errmsg);
+	free(realfn);
 
 	if (doc == NULL) {
 		if (!pcb_io_err_inhibit)
@@ -1433,8 +1495,13 @@ int io_lihata_parse_font(pcb_plug_io_t *ctx, pcb_font_t *Ptr, const char *Filena
 int io_lihata_parse_element(pcb_plug_io_t *ctx, pcb_data_t *Ptr, const char *name)
 {
 	int res;
-	char *errmsg;
-	lht_doc_t *doc = lht_dom_load(name, &errmsg);
+	char *errmsg, *realfn;
+	lht_doc_t *doc = NULL;
+
+	realfn = pcb_fopen_check(name, "r");
+	if (realfn != NULL)
+		doc = lht_dom_load(name, &errmsg);
+	free(realfn);
 
 	if (doc == NULL) {
 		if (!pcb_io_err_inhibit)
@@ -1448,7 +1515,7 @@ int io_lihata_parse_element(pcb_plug_io_t *ctx, pcb_data_t *Ptr, const char *nam
 		return -1;
 	}
 
-	res = parse_subc(NULL, Ptr, doc->root->data.list.first);
+	res = parse_subc(NULL, Ptr, doc->root->data.list.first, NULL);
 
 	lht_dom_uninit(doc);
 	return res;
