@@ -45,6 +45,7 @@
 #include "paths.h"
 #include "obj_subc_list.h"
 #include "pcb_minuid.h"
+#include "safe_fs.h"
 
 /*#define CFMT "%[9]"*/
 #define CFMT "%.08$$mH"
@@ -65,7 +66,7 @@ do { \
 static lht_node_t *dummy_node(const char *name)
 {
 	lht_node_t *n;
-	n = lht_dom_node_alloc(LHT_TEXT, "attributes");
+	n = lht_dom_node_alloc(LHT_TEXT, name);
 	n->type = LHT_INVALID_TYPE;
 	return n;
 }
@@ -160,13 +161,16 @@ static lht_node_t *build_attributes(pcb_attribute_list_t *lst)
 		return dummy_node("attributes");
 	ln = lht_dom_node_alloc(LHT_HASH, "attributes");
 
-	for (n = 0; n < lst->Number; n++)
+	for (n = 0; n < lst->Number; n++) {
+		if ((wrver < 3) && (strcmp(lst->List[n].name, "intconn") == 0))
+			continue; /* do not write intconn as attribute for v1 and v2, we used a flag for those */
 		lht_dom_hash_put(ln, build_text(lst->List[n].name, lst->List[n].value));
+	}
 
 	return ln;
 }
 
-static lht_node_t *build_flags(pcb_flag_t *f, int object_type)
+static lht_node_t *build_flags(pcb_flag_t *f, int object_type, int intconn)
 {
 	int n, layer, added = 0, thrm = 0;
 	lht_node_t *hsh, *txt, *lst;
@@ -217,8 +221,8 @@ static lht_node_t *build_flags(pcb_flag_t *f, int object_type)
 		added++;
 	}
 
-	if (f->int_conn_grp > 0) {
-		lht_dom_hash_put(hsh, build_textf("intconn", "%d", f->int_conn_grp));
+	if ((intconn > 0) && (wrver < 3)) {
+		lht_dom_hash_put(hsh, build_textf("intconn", "%d", intconn));
 		added++;
 	}
 
@@ -240,7 +244,7 @@ static lht_node_t *build_line(pcb_line_t *line, int local_id, pcb_coord_t dx, pc
 
 	if (!simple) {
 		lht_dom_hash_put(obj, build_attributes(&line->Attributes));
-		lht_dom_hash_put(obj, build_flags(&line->Flags, PCB_TYPE_LINE));
+		lht_dom_hash_put(obj, build_flags(&line->Flags, PCB_TYPE_LINE, line->intconn));
 		lht_dom_hash_put(obj, build_textf("clearance", CFMT, line->Clearance));
 	}
 
@@ -297,7 +301,7 @@ static lht_node_t *build_rat(pcb_rat_t *rat)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&rat->Attributes));
-	lht_dom_hash_put(obj, build_flags(&rat->Flags, PCB_TYPE_LINE));
+	lht_dom_hash_put(obj, build_flags(&rat->Flags, PCB_TYPE_LINE, rat->intconn));
 	lht_dom_hash_put(obj, build_textf("x1", CFMT, rat->Point1.X));
 	lht_dom_hash_put(obj, build_textf("y1", CFMT, rat->Point1.Y));
 	lht_dom_hash_put(obj, build_textf("x2", CFMT, rat->Point2.X));
@@ -317,7 +321,7 @@ static lht_node_t *build_arc(pcb_arc_t *arc, pcb_coord_t dx, pcb_coord_t dy)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&arc->Attributes));
-	lht_dom_hash_put(obj, build_flags(&arc->Flags, PCB_TYPE_ARC));
+	lht_dom_hash_put(obj, build_flags(&arc->Flags, PCB_TYPE_ARC, arc->intconn));
 	lht_dom_hash_put(obj, build_textf("thickness", CFMT, arc->Thickness));
 	lht_dom_hash_put(obj, build_textf("clearance", CFMT, arc->Clearance));
 	lht_dom_hash_put(obj, build_textf("x", CFMT, arc->X+dx));
@@ -339,7 +343,7 @@ static lht_node_t *build_pin(pcb_pin_t *pin, int is_via, pcb_coord_t dx, pcb_coo
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&pin->Attributes));
-	lht_dom_hash_put(obj, build_flags(&pin->Flags, PCB_TYPE_VIA));
+	lht_dom_hash_put(obj, build_flags(&pin->Flags, PCB_TYPE_VIA, pin->intconn));
 	lht_dom_hash_put(obj, build_textf("thickness", CFMT, pin->Thickness));
 	lht_dom_hash_put(obj, build_textf("clearance", CFMT, pin->Clearance));
 	lht_dom_hash_put(obj, build_textf("mask", CFMT, pin->Mask));
@@ -360,7 +364,7 @@ static lht_node_t *build_pad(pcb_pad_t *pad, pcb_coord_t dx, pcb_coord_t dy)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&pad->Attributes));
-	lht_dom_hash_put(obj, build_flags(&pad->Flags, PCB_TYPE_PAD));
+	lht_dom_hash_put(obj, build_flags(&pad->Flags, PCB_TYPE_PAD, pad->intconn));
 	lht_dom_hash_put(obj, build_textf("thickness", CFMT, pad->Thickness));
 	lht_dom_hash_put(obj, build_textf("clearance", CFMT, pad->Clearance));
 	lht_dom_hash_put(obj, build_textf("mask", CFMT, pad->Mask));
@@ -383,7 +387,10 @@ static lht_node_t *build_polygon(pcb_polygon_t *poly)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&poly->Attributes));
-	lht_dom_hash_put(obj, build_flags(&poly->Flags, PCB_TYPE_POLYGON));
+	lht_dom_hash_put(obj, build_flags(&poly->Flags, PCB_TYPE_POLYGON, poly->intconn));
+
+	if ((wrver >= 3) && (poly->Clearance > 0))
+		lht_dom_hash_put(obj, build_textf("clearance", CFMT, poly->Clearance));
 
 	geo = lht_dom_node_alloc(LHT_LIST, "geometry");
 	lht_dom_hash_put(obj, geo);
@@ -419,7 +426,7 @@ static lht_node_t *build_pcb_text(const char *role, pcb_text_t *text)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&text->Attributes));
-	lht_dom_hash_put(obj, build_flags(&text->Flags, PCB_TYPE_TEXT));
+	lht_dom_hash_put(obj, build_flags(&text->Flags, PCB_TYPE_TEXT, text->intconn));
 	lht_dom_hash_put(obj, build_text("string", text->TextString));
 	lht_dom_hash_put(obj, build_textf("fid", "%ld", text->fid));
 	lht_dom_hash_put(obj, build_textf("scale", "%d", text->Scale));
@@ -446,7 +453,7 @@ static lht_node_t *build_element(pcb_element_t *elem)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&elem->Attributes));
-	lht_dom_hash_put(obj, build_flags(&elem->Flags, PCB_TYPE_ELEMENT));
+	lht_dom_hash_put(obj, build_flags(&elem->Flags, PCB_TYPE_ELEMENT, 0));
 
 
 	/* build drawing primitives */
@@ -494,7 +501,7 @@ static lht_node_t *build_subc(pcb_subc_t *sc)
 	obj = lht_dom_node_alloc(LHT_HASH, buff);
 
 	lht_dom_hash_put(obj, build_attributes(&sc->Attributes));
-	lht_dom_hash_put(obj, build_flags(&sc->Flags, PCB_TYPE_SUBC));
+	lht_dom_hash_put(obj, build_flags(&sc->Flags, PCB_TYPE_SUBC, 0));
 	lht_dom_hash_put(obj, build_data(sc->data));
 	lht_dom_hash_put(obj, build_minuid("uid", sc->uid));
 
@@ -568,7 +575,9 @@ static lht_node_t *build_data_layer(pcb_data_t *data, pcb_layer_t *layer, pcb_la
 		else
 			pcb_message(PCB_MSG_WARNING, "io_lihata: attempting to save bound layers in lihata version lower than 3; feature not supported by the format.\n");
 	}
-	lht_dom_hash_put(obj, build_attributes(&layer->meta.real.Attributes));
+
+	if (PCB_LAYER_IS_REAL(layer))
+		lht_dom_hash_put(obj, build_attributes(&layer->meta.real.Attributes));
 
 	if (wrver >= 2) {
 		lht_dom_hash_put(obj, build_textf("lid", "%ld", lid));
@@ -598,11 +607,6 @@ static lht_node_t *build_data_layer(pcb_data_t *data, pcb_layer_t *layer, pcb_la
 		added++;
 	}
 
-	if ((added == 0) && (!io_lihata_full_tree)) {
-		lht_dom_node_free(grp);
-		grp = dummy_node("objects");
-	}
-
 	lht_dom_hash_put(obj, grp);
 
 	return obj;
@@ -786,7 +790,7 @@ static lht_node_t *build_font(pcb_font_t *font)
 
 static lht_node_t *build_fontkit(pcb_fontkit_t *fk)
 {
-	lht_node_t *syms, *ndt, *frt;
+	lht_node_t *ndt, *frt;
 
 	frt = lht_dom_node_alloc(LHT_HASH, "font");
 
@@ -1016,6 +1020,9 @@ static lhtpers_ev_res_t check_text(void *ev_ctx, lht_perstyle_t *style, lht_node
 		/* else fall back to the string compare below */
 	}
 
+	if (inmem_node->data.text.value == NULL)
+		return LHTPERS_INHIBIT;
+
 	/* classic method: string mismatch => overwrite from mem */
 	if (strcmp(inmem_node->data.text.value, ondisk_value) != 0)
 		return LHTPERS_MEM;
@@ -1053,7 +1060,7 @@ static int io_lihata_write_pcb(pcb_plug_io_t *ctx, FILE * FP, const char *old_fi
 		   Thus this feature should be disabled */
 		if ((fnpat != NULL) && (*fnpat != '\0')) {
 			char *orig_fn, *end;
-			char *pcb_fn = pcb_strdup_subst(fnpat, pcb_build_fn_cb, NULL);
+			char *pcb_fn = pcb_strdup_subst(fnpat, pcb_build_fn_cb, NULL, PCB_SUBST_ALL);
 			
 			orig_fn = PCB->Filename;
 			PCB->Filename = NULL;
@@ -1085,7 +1092,7 @@ static int io_lihata_write_pcb(pcb_plug_io_t *ctx, FILE * FP, const char *old_fi
 		lhtpers_ev_t events;
 
 		if (old_filename != NULL)
-			inf = fopen(old_filename, "r");
+			inf = pcb_fopen(old_filename, "r");
 
 		memset(&events, 0, sizeof(events));
 		events.text = check_text;
@@ -1102,7 +1109,7 @@ static int io_lihata_write_pcb(pcb_plug_io_t *ctx, FILE * FP, const char *old_fi
 		if (res != 0) {
 			FILE *fe;
 			char *fe_name = pcb_concat(old_filename, ".mem.lht", NULL);
-			fe = fopen(fe_name, "w");
+			fe = pcb_fopen(fe_name, "w");
 			if (fe != NULL) {
 				res = lht_dom_export(brd->root, fe, "");
 				fclose(fe);
@@ -1141,7 +1148,7 @@ int io_lihata_write_font(pcb_plug_io_t *ctx, pcb_font_t *font, const char *Filen
 	lht_doc_t *doc;
 
 
-	f = fopen(Filename, "w");
+	f = pcb_fopen(Filename, "w");
 	if (f == NULL) {
 		pcb_message(PCB_MSG_ERROR, "Failed to open font file %s for write\n", Filename);
 		return -1;
@@ -1161,20 +1168,16 @@ int io_lihata_write_font(pcb_plug_io_t *ctx, pcb_font_t *font, const char *Filen
 	return res;
 }
 
-int io_lihata_write_buffer(pcb_plug_io_t *ctx, FILE *f, pcb_buffer_t *buff, pcb_bool elem_only)
+static int io_lihata_dump_1st_subc(pcb_plug_io_t *ctx, FILE *f, pcb_data_t *data, int enforce1)
 {
 	int res;
 	lht_doc_t *doc;
 
-	if (!elem_only) {
-		pcb_message(PCB_MSG_ERROR, "Can't save full buffer (yet), only a single subcircuits from a buffer\n");
-		return -1;
-	}
-	if (pcb_subclist_length(&buff->Data->subc) > 1) {
+	if ((enforce1) && (pcb_subclist_length(&data->subc)) > 1) {
 		pcb_message(PCB_MSG_ERROR, "Can't save more than one subcircuit from a buffer\n");
 		return -1;
 	}
-	if (pcb_subclist_length(&buff->Data->subc) < 1) {
+	if (pcb_subclist_length(&data->subc) < 1) {
 		pcb_message(PCB_MSG_ERROR, "there's no subcircuit in the buffer\n");
 		return -1;
 	}
@@ -1184,7 +1187,7 @@ int io_lihata_write_buffer(pcb_plug_io_t *ctx, FILE *f, pcb_buffer_t *buff, pcb_
 	doc = lht_dom_init();
 	wrver = 3;
 	doc->root = lht_dom_node_alloc(LHT_LIST, "pcb-rnd-subcircuit-v3");
-	lht_dom_list_append(doc->root, build_subc(pcb_subclist_first(&buff->Data->subc)));
+	lht_dom_list_append(doc->root, build_subc(pcb_subclist_first(&data->subc)));
 
 	res = lht_dom_export(doc->root, f, "");
 
@@ -1193,3 +1196,17 @@ int io_lihata_write_buffer(pcb_plug_io_t *ctx, FILE *f, pcb_buffer_t *buff, pcb_
 	return res;
 }
 
+int io_lihata_write_buffer(pcb_plug_io_t *ctx, FILE *f, pcb_buffer_t *buff, pcb_bool elem_only)
+{
+	if (!elem_only) {
+		pcb_message(PCB_MSG_ERROR, "Can't save full buffer (yet), only a single subcircuits from a buffer\n");
+		return -1;
+	}
+
+	return io_lihata_dump_1st_subc(ctx, f, buff->Data, 1);
+}
+
+int io_lihata_write_element(pcb_plug_io_t *ctx, FILE *f, pcb_data_t *dt)
+{
+	return io_lihata_dump_1st_subc(ctx, f, dt, 1);
+}
