@@ -64,6 +64,7 @@
 #include "const.h"
 #include "plug_footprint.h"
 #include "compat_nls.h"
+#include "compat_misc.h"
 
 #include <gdk/gdkkeysyms.h>
 
@@ -158,7 +159,7 @@ static GObjectClass *library_window_parent_class = NULL;
  */
 static gboolean lib_model_filter_visible_func(GtkTreeModel * model, GtkTreeIter * iter, gpointer data)
 {
-	pcb_gtk_library_t *library_window = (pcb_gtk_library_t *) data;
+	pcb_gtk_library_t *lib_window = (pcb_gtk_library_t *) data;
 	const gchar *compname;
 	gchar *compname_upper, *text_upper, *pattern;
 	const gchar *text_;
@@ -169,7 +170,7 @@ static gboolean lib_model_filter_visible_func(GtkTreeModel * model, GtkTreeIter
 
 	g_assert(GHID_IS_LIBRARY_WINDOW(data));
 
-	text_ = gtk_entry_get_text(library_window->entry_filter);
+	text_ = gtk_entry_get_text(lib_window->entry_filter);
 	if (strcmp(text_, "") == 0) {
 		return TRUE;
 	}
@@ -428,6 +429,7 @@ static void library_window_callback_tree_selection_changed(GtkTreeSelection * se
 	pcb_gtk_library_t *library_window = (pcb_gtk_library_t *) user_data;
 	char *name = NULL;
 	pcb_fplibrary_t *entry = NULL;
+	int norefresh = 0;
 
 	lib_param_del_timer(library_window);
 
@@ -441,15 +443,28 @@ static void library_window_callback_tree_selection_changed(GtkTreeSelection * se
 
 	if ((entry->type == LIB_FOOTPRINT) && (entry->data.fp.type == PCB_FP_PARAMETRIC)) {
 		const char *in_para = gtk_entry_get_text(library_window->entry_filter);
+		char *orig = pcb_strdup(in_para);
 		name = pcb_gtk_library_param_ui(library_window, entry, in_para, lib_param_chg);
 		lib_param_del_timer(library_window);
 		if (name == NULL) {
-#warning TODO: refresh the display with empty - also for the above returns!
-			return;
+			gtk_entry_set_text(library_window->entry_filter, orig);
+			g_source_remove(library_window->filter_timeout); /* block the above edit from collapsing the tree */
+			gtk_tree_model_filter_refilter((GtkTreeModelFilter *) model);
+
+			/* cancel means we should skip refreshing the preview with the invalid
+			   text we may have in the filter; also dismiss the preview and clear
+			   the buffer: we may have a valid parametric footprint output in there
+			   that happened before the cancel */
+			norefresh = 1;
+			g_object_set(library_window->preview, "element-data", NULL, NULL);
+			pcb_hid_actionl("PasteBuffer", "clear", NULL);
 		}
-		gtk_entry_set_text(library_window->entry_filter, name);
+		else
+			gtk_entry_set_text(library_window->entry_filter, name);
+		free(orig);
 	}
-	library_window_preview_refresh(library_window, name, entry);
+	if (!norefresh)
+		library_window_preview_refresh(library_window, name, entry);
 	free(name);
 }
 
@@ -775,7 +790,7 @@ static pcb_gtk_common_t *lwcom;
 static GObject *library_window_constructor(GType type, guint n_construct_properties, GObjectConstructParam * construct_params)
 {
 	GObject *object;
-	pcb_gtk_library_t *library_window;
+	pcb_gtk_library_t *lib_window;
 	GtkWidget *content_area;
 	GtkWidget *hpaned, *notebook;
 	GtkWidget *libview;
@@ -784,8 +799,8 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 
 	/* chain up to constructor of parent class */
 	object = G_OBJECT_CLASS(library_window_parent_class)->constructor(type, n_construct_properties, construct_params);
-	library_window = GHID_LIBRARY_WINDOW(object);
-	library_window->param_timer = 0;
+	lib_window = GHID_LIBRARY_WINDOW(object);
+	lib_window->param_timer = 0;
 
 	/* dialog initialization */
 	g_object_set(object,
@@ -795,19 +810,19 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 							 "default-height", 300, "default-width", 400, "modal", FALSE, "window-position", GTK_WIN_POS_NONE,
 							 /* GtkDialog */
 							 "has-separator", TRUE, NULL);
-	g_object_set(gtk_dialog_get_content_area(GTK_DIALOG(library_window)), "homogeneous", FALSE, NULL);
+	g_object_set(gtk_dialog_get_content_area(GTK_DIALOG(lib_window)), "homogeneous", FALSE, NULL);
 
 	/* horizontal pane containing selection and preview */
 	hpaned = GTK_WIDGET(g_object_new(GTK_TYPE_HPANED,
 																	 /* GtkContainer */
 																	 "border-width", 5, NULL));
-	library_window->hpaned = hpaned;
+	lib_window->hpaned = hpaned;
 
 	/* notebook for library views */
 	notebook = GTK_WIDGET(g_object_new(GTK_TYPE_NOTEBOOK, "show-tabs", FALSE, NULL));
-	library_window->viewtabs = GTK_NOTEBOOK(notebook);
+	lib_window->viewtabs = GTK_NOTEBOOK(notebook);
 
-	libview = create_lib_treeview(library_window);
+	libview = create_lib_treeview(lib_window);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), libview, gtk_label_new(_("Libraries")));
 
 	/* include the vertical box in horizontal box */
@@ -839,19 +854,19 @@ static GObject *library_window_constructor(GType type, guint n_construct_propert
 		gtk_container_add(GTK_CONTAINER(frame), alignment);
 	}
 
-	/* set preview of library_window */
-	library_window->preview = preview;
-	library_window->preview_text = preview_text;
+	/* set preview of library window */
+	lib_window->preview = preview;
+	lib_window->preview_text = preview_text;
 
 	gtk_paned_pack2(GTK_PANED(hpaned), frame, FALSE, FALSE);
 
 	/* add the hpaned to the dialog content area */
-	content_area = gtk_dialog_get_content_area(GTK_DIALOG(library_window));
+	content_area = gtk_dialog_get_content_area(GTK_DIALOG(lib_window));
 	gtk_box_pack_start(GTK_BOX(content_area), hpaned, TRUE, TRUE, 0);
 	gtk_widget_show_all(hpaned);
 
 	/* now add buttons in the action area */
-	gtk_dialog_add_buttons(GTK_DIALOG(library_window),
+	gtk_dialog_add_buttons(GTK_DIALOG(lib_window),
 												 /*  - close button */
 												 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 
