@@ -86,13 +86,13 @@ static void preview_set_data(pcb_gtk_preview_t * preview, pcb_element_t * elemen
 	pcb_element_copy(NULL, &preview->element, element, FALSE, 0, 0);
 	PCB_PIN_LOOP(&preview->element);
 	{
-		PCB_FLAG_SET(PCB_FLAG_DISPLAYNAME, pin);
+		PCB_FLAG_SET(PCB_FLAG_TERMNAME, pin);
 	}
 	PCB_END_LOOP;
 
 	PCB_PAD_LOOP(&preview->element);
 	{
-		PCB_FLAG_SET(PCB_FLAG_DISPLAYNAME, pad);
+		PCB_FLAG_SET(PCB_FLAG_TERMNAME, pad);
 	}
 	PCB_END_LOOP;
 
@@ -538,7 +538,7 @@ static GtkWidget *pcb_gtk_preview_generic_new(pcb_gtk_common_t * com, pcb_gtk_in
 	prv->expose_data.force = 1;
 	prv->init_drawing_widget(GTK_WIDGET(prv), prv->gport);
 
-	gtk_widget_add_events(GTK_WIDGET(prv), GDK_EXPOSURE_MASK
+	gtk_widget_add_events(GTK_WIDGET(prv), GDK_EXPOSURE_MASK | GDK_SCROLL_MASK
 												| GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK | GDK_BUTTON_RELEASE_MASK
 												| GDK_BUTTON_PRESS_MASK | GDK_KEY_RELEASE_MASK | GDK_KEY_PRESS_MASK
 												| GDK_FOCUS_CHANGE_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
