@@ -53,6 +53,7 @@
 #include "compat_misc.h"
 #include "compat_nls.h"
 #include "fptr_cast.h"
+#include "safe_fs.h"
 #include <liblihata/tree.h>
 
 #include "gtk_conf_list.h"
@@ -271,7 +272,7 @@ void config_any_replace(save_ctx_t * ctx, const char **paths)
 					if (conf_replace_subtree(ctx->dst_role, e->key, ctx->src_role, e->key) != 0)
 						pcb_message(PCB_MSG_ERROR, "Error: failed to save config item %s\n", *p);
 					if (ctx->dst_role < CFR_max_real) {
-						conf_update(e->key);
+						conf_update(e->key, -1);
 						need_update++;
 					}
 				}
@@ -282,7 +283,7 @@ void config_any_replace(save_ctx_t * ctx, const char **paths)
 			if (conf_replace_subtree(ctx->dst_role, *p, ctx->src_role, *p) != 0)
 				pcb_message(PCB_MSG_ERROR, "Error: failed to save config item %s\n", *p);
 			if (ctx->dst_role < CFR_max_real) {
-				conf_update(*p);
+				conf_update(*p, -1);
 				need_update++;
 			}
 		}
@@ -1136,7 +1137,7 @@ static void pre_rebuild(gtk_conf_list_t * cl)
 
 static void post_rebuild(gtk_conf_list_t * cl)
 {
-	conf_update("rc/library_search_paths");
+	conf_update("rc/library_search_paths", -1);
 }
 
 
@@ -1352,7 +1353,7 @@ void config_layers_save(GtkButton * widget, save_ctx_t * ctx)
 			conf_makedirty(ctx->dst_role);
 		}
 	}
-	conf_update("design/default_layer_name");
+	conf_update("design/default_layer_name", -1);
 	conf_set(CFR_DESIGN, "design/groups", -1, s, POL_OVERWRITE);
 	g_free(s);
 	config_any_replace(ctx, paths);
@@ -2280,7 +2281,7 @@ static void config_auto_save(conf_role_t role)
 		const char *fn = conf_get_project_conf_name(NULL, pcbfn, &try);
 		if (fn == NULL) {
 			FILE *f;
-			f = fopen(try, "w");
+			f = pcb_fopen(try, "w");
 			if (f == NULL) {
 				pcb_message(PCB_MSG_ERROR, "can not create config to project file: %s\n", try);
 				return;
@@ -2353,7 +2354,7 @@ static void config_auto_apply_cb(GtkButton * btn, void *data)
 				conf_set_dry(role, nat->hash_path, -1, pcb_strdup(s), (n == 0) ? POL_OVERWRITE : POL_APPEND);
 				g_free(s);
 			}
-			conf_update(nat->hash_path);
+			conf_update(nat->hash_path, -1);
 			config_auto_save(role);
 		}
 		new_val = NULL;							/* do not run conf_set, but run the rest of the updates */
