@@ -122,13 +122,23 @@ This is a shortcut for @code{Report(Object)}.
 
 %end-doc */
 
+#define gen_locked(obj) (PCB_FLAG_TEST(PCB_FLAG_LOCK, obj) ? "It is LOCKED.\n" : "")
+#define gen_term(obj) \
+	(((obj)->term != NULL) ? "It is terminal " : ""), \
+	(((obj)->term != NULL) ? (obj)->term : ""), \
+	(((obj)->term != NULL) ? "\n" : "")
+
 static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 {
 	void *ptr1, *ptr2, *ptr3;
-	int type;
+	int type = REPORT_TYPES;
 	char *report = NULL;
 
-	type = pcb_search_screen(x, y, REPORT_TYPES, &ptr1, &ptr2, &ptr3);
+	if ((argv != NULL) && (argv[0] != NULL)) {
+		if (pcb_strncasecmp(argv[0], "Subc", 4) == 0)
+			type = PCB_TYPE_SUBC;
+	}
+	type = pcb_search_screen(x, y, type, &ptr1, &ptr2, &ptr3);
 	if (type == PCB_TYPE_NONE)
 		type = pcb_search_screen(x, y, REPORT_TYPES | PCB_TYPE_LOCKED, &ptr1, &ptr2, &ptr3);
 
@@ -148,24 +158,26 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 										"(X,Y) = %$mD.\n"
 										"It is a pure hole of diameter %$mS.\n"
 										"Name = \"%s\"."
-										"%s", USER_UNITMASK, via->ID, pcb_strflg_f2s(via->Flags, PCB_TYPE_VIA),
-										via->X, via->Y, via->DrillingHole, PCB_EMPTY(via->Name), PCB_FLAG_TEST(PCB_FLAG_LOCK, via) ? "It is LOCKED.\n" : "");
+										"%s"
+										"%s%s%s", USER_UNITMASK, via->ID, pcb_strflg_f2s(via->Flags, PCB_TYPE_VIA, NULL),
+										via->X, via->Y, via->DrillingHole, PCB_EMPTY(via->Name), gen_locked(via), gen_term(via));
 			else
 				report = pcb_strdup_printf("%m+VIA ID# %ld;  Flags:%s\n"
 										"(X,Y) = %$mD.\n"
 										"Copper width = %$mS. Drill width = %$mS.\n"
-										"Clearance width in polygons = %$mS.\n"
+										"Clearance = %$mS.\n"
 										"Annulus = %$mS.\n"
 										"Solder mask hole = %$mS (gap = %$mS).\n"
 										"Name = \"%s\"."
-										"%s", USER_UNITMASK, via->ID, pcb_strflg_f2s(via->Flags, PCB_TYPE_VIA),
+										"%s"
+										"%s%s%s", USER_UNITMASK, via->ID, pcb_strflg_f2s(via->Flags, PCB_TYPE_VIA, NULL),
 										via->X, via->Y,
 										via->Thickness,
 										via->DrillingHole,
 										via->Clearance / 2,
 										(via->Thickness - via->DrillingHole) / 2,
 										via->Mask,
-										(via->Mask - via->Thickness) / 2, PCB_EMPTY(via->Name), PCB_FLAG_TEST(PCB_FLAG_LOCK, via) ? "It is LOCKED.\n" : "");
+										(via->Mask - via->Thickness) / 2, PCB_EMPTY(via->Name), gen_locked(via), gen_term(via));
 			break;
 		}
 	case PCB_TYPE_PIN:
@@ -192,20 +204,20 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 										"(X,Y) = %$mD.\n"
 										"It is a mounting hole. Drill width = %$mS.\n"
 										"It is owned by element %$mS.\n"
-										"%s", USER_UNITMASK, Pin->ID, pcb_strflg_f2s(Pin->Flags, PCB_TYPE_PIN),
+										"%s", USER_UNITMASK, Pin->ID, pcb_strflg_f2s(Pin->Flags, PCB_TYPE_PIN, NULL),
 										Pin->X, Pin->Y, Pin->DrillingHole,
-										PCB_EMPTY(element->Name[1].TextString), PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin) ? "It is LOCKED.\n" : "");
+										PCB_EMPTY(element->Name[1].TextString), gen_locked(Pin));
 			else
 				report = pcb_strdup_printf(
 										"%m+PIN ID# %ld;  Flags:%s\n" "(X,Y) = %$mD.\n"
 										"Copper width = %$mS. Drill width = %$mS.\n"
-										"Clearance width to Polygon = %$mS.\n"
+										"Clearance = %$mS.\n"
 										"Annulus = %$mS.\n"
 										"Solder mask hole = %$mS (gap = %$mS).\n"
 										"Name = \"%s\".\n"
 										"It is owned by element %s\n as pin number %s.\n"
 										"%s", USER_UNITMASK,
-										Pin->ID, pcb_strflg_f2s(Pin->Flags, PCB_TYPE_PIN),
+										Pin->ID, pcb_strflg_f2s(Pin->Flags, PCB_TYPE_PIN, NULL),
 										Pin->X, Pin->Y, Pin->Thickness,
 										Pin->DrillingHole,
 										Pin->Clearance / 2,
@@ -213,7 +225,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 										Pin->Mask,
 										(Pin->Mask - Pin->Thickness) / 2,
 										PCB_EMPTY(Pin->Name),
-										PCB_EMPTY(element->Name[1].TextString), PCB_EMPTY(Pin->Number), PCB_FLAG_TEST(PCB_FLAG_LOCK, Pin) ? "It is LOCKED.\n" : "");
+										PCB_EMPTY(element->Name[1].TextString), PCB_EMPTY(Pin->Number), gen_locked(Pin));
 			break;
 		}
 	case PCB_TYPE_LINE:
@@ -230,16 +242,17 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 			report = pcb_strdup_printf("%m+LINE ID# %ld;  Flags:%s\n"
 									"FirstPoint(X,Y)  = %$mD, ID = %ld.\n"
 									"SecondPoint(X,Y) = %$mD, ID = %ld.\n"
-									"Width = %$mS.\nClearance width in polygons = %$mS.\n"
+									"Width = %$mS.\nClearance = %$mS.\n"
 									"It is on layer %d\n"
 									"and has name \"%s\".\n"
-									"%s", USER_UNITMASK,
-									line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE),
+									"%s"
+									"%s%s%s", USER_UNITMASK,
+									line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE, NULL),
 									line->Point1.X, line->Point1.Y, line->Point1.ID,
 									line->Point2.X, line->Point2.Y, line->Point2.ID,
 									line->Thickness, line->Clearance / 2,
 									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1),
-									PCB_UNKNOWN(line->Number), PCB_FLAG_TEST(PCB_FLAG_LOCK, line) ? "It is LOCKED.\n" : "");
+									PCB_UNKNOWN(line->Number), gen_locked(line), gen_term(line));
 			break;
 		}
 	case PCB_TYPE_RATLINE:
@@ -257,7 +270,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"connects to layer group %d.\n"
 									"SecondPoint(X,Y) = %$mD; ID = %ld; "
 									"connects to layer group %d.\n",
-									USER_UNITMASK, line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE),
+									USER_UNITMASK, line->ID, pcb_strflg_f2s(line->Flags, PCB_TYPE_LINE, NULL),
 									line->Point1.X, line->Point1.Y,
 									line->Point1.ID, line->group1, line->Point2.X, line->Point2.Y, line->Point2.ID, line->group2);
 			break;
@@ -279,21 +292,21 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 
 			report = pcb_strdup_printf("%m+ARC ID# %ld;  Flags:%s\n"
 									"CenterPoint(X,Y) = %$mD.\n"
-									"Radius = %$mS, Thickness = %$mS.\n"
-									"Clearance width in polygons = %$mS.\n"
-									"StartAngle = %ma degrees, DeltaAngle = %ma degrees.\n"
+									"Width = %$mS.\nClearance = %$mS.\n"
+									"Radius = %$mS, StartAngle = %ma degrees, DeltaAngle = %ma degrees.\n"
 									"Bounding Box is %$mD, %$mD.\n"
 									"That makes the end points at %$mD and %$mD.\n"
 									"It is on layer %d.\n"
-									"%s", USER_UNITMASK, Arc->ID, pcb_strflg_f2s(Arc->Flags, PCB_TYPE_ARC),
+									"%s"
+									"%s%s%s", USER_UNITMASK, Arc->ID, pcb_strflg_f2s(Arc->Flags, PCB_TYPE_ARC, NULL),
 									Arc->X, Arc->Y,
-									Arc->Width, Arc->Thickness,
-									Arc->Clearance / 2, Arc->StartAngle, Arc->Delta,
+									Arc->Thickness, Arc->Clearance / 2,
+									Arc->Width, Arc->StartAngle, Arc->Delta,
 									Arc->BoundingBox.X1, Arc->BoundingBox.Y1,
 									Arc->BoundingBox.X2, Arc->BoundingBox.Y2,
 									box.X1, box.Y1,
 									box.X2, box.Y2,
-									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1), PCB_FLAG_TEST(PCB_FLAG_LOCK, Arc) ? "It is LOCKED.\n" : "");
+									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1), gen_locked(Arc), gen_term(Arc));
 			break;
 		}
 	case PCB_TYPE_POLYGON:
@@ -313,13 +326,14 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"It has %d points and could store %d more\n"
 									"  without using more memory.\n"
 									"It has %d holes and resides on layer %d.\n"
-									"%s", USER_UNITMASK, Polygon->ID,
-									pcb_strflg_f2s(Polygon->Flags, PCB_TYPE_POLYGON),
+									"%s"
+									"%s%s%s", USER_UNITMASK, Polygon->ID,
+									pcb_strflg_f2s(Polygon->Flags, PCB_TYPE_POLYGON, NULL),
 									Polygon->BoundingBox.X1, Polygon->BoundingBox.Y1,
 									Polygon->BoundingBox.X2, Polygon->BoundingBox.Y2,
 									Polygon->PointN, Polygon->PointMax - Polygon->PointN,
 									Polygon->HoleIndexN,
-									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1), PCB_FLAG_TEST(PCB_FLAG_LOCK, Polygon) ? "It is LOCKED.\n" : "");
+									pcb_layer_id(PCB->Data, (pcb_layer_t *) ptr1), gen_locked(Polygon), gen_term(Polygon));
 			break;
 		}
 	case PCB_TYPE_PAD:
@@ -349,14 +363,14 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"FirstPoint(X,Y)  = %$mD; ID = %ld.\n"
 									"SecondPoint(X,Y) = %$mD; ID = %ld.\n"
 									"Width = %$mS.  Length = %$mS.\n"
-									"Clearance width in polygons = %$mS.\n"
+									"Clearance = %$mS.\n"
 									"Solder mask = %$mS x %$mS (gap = %$mS).\n"
 									"Name = \"%s\".\n"
 									"It is owned by SMD element %s\n"
 									"  as pin number %s and is on the %s\n"
 									"side of the board.\n"
 									"%s", USER_UNITMASK, Pad->ID,
-									pcb_strflg_f2s(Pad->Flags, PCB_TYPE_PAD),
+									pcb_strflg_f2s(Pad->Flags, PCB_TYPE_PAD, NULL),
 									Pad->Point1.X, Pad->Point1.Y, Pad->Point1.ID,
 									Pad->Point2.X, Pad->Point2.Y, Pad->Point2.ID,
 									Pad->Thickness, len + Pad->Thickness,
@@ -367,7 +381,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									PCB_EMPTY(element->Name[1].TextString),
 									PCB_EMPTY(Pad->Number),
 									PCB_FLAG_TEST(PCB_FLAG_ONSOLDER,
-														Pad) ? "solder (bottom)" : "component", PCB_FLAG_TEST(PCB_FLAG_LOCK, Pad) ? "It is LOCKED.\n" : "");
+														Pad) ? "solder (bottom)" : "component", gen_locked(Pad));
 			break;
 		}
 	case PCB_TYPE_ELEMENT:
@@ -389,7 +403,7 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"Mark located at point (X,Y) = %$mD.\n"
 									"It is on the %s side of the board.\n"
 									"%s", USER_UNITMASK,
-									element->ID, pcb_strflg_f2s(element->Flags, PCB_TYPE_ELEMENT),
+									element->ID, pcb_strflg_f2s(element->Flags, PCB_TYPE_ELEMENT, NULL),
 									element->BoundingBox.X1, element->BoundingBox.Y1,
 									element->BoundingBox.X2, element->BoundingBox.Y2,
 									PCB_EMPTY(element->Name[0].TextString),
@@ -400,7 +414,28 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									PCB_FLAG_TEST(PCB_FLAG_HIDENAME, element) ? ",\n  but it's hidden" : "",
 									element->MarkX, element->MarkY,
 									PCB_FLAG_TEST(PCB_FLAG_ONSOLDER, element) ? "solder (bottom)" : "component",
-									PCB_FLAG_TEST(PCB_FLAG_LOCK, element) ? "It is LOCKED.\n" : "");
+									gen_locked(element));
+			break;
+		}
+	case PCB_TYPE_SUBC:
+		{
+			pcb_subc_t *subc;
+#ifndef NDEBUG
+			if (pcb_gui->shift_is_pressed()) {
+				pcb_r_dump_tree(PCB->Data->element_tree->root, 0);
+				return 0;
+			}
+#endif
+			subc = (pcb_subc_t *)ptr2;
+			report = pcb_strdup_printf("%m+SUBCIRCUIT ID# %ld;  Flags:%s\n"
+									"BoundingBox %$mD %$mD.\n"
+									"Refdes \"%s\".\n"
+									"%s", USER_UNITMASK,
+									subc->ID, pcb_strflg_f2s(subc->Flags, PCB_TYPE_ELEMENT, NULL),
+									subc->BoundingBox.X1, subc->BoundingBox.Y1,
+									subc->BoundingBox.X2, subc->BoundingBox.Y2,
+									PCB_EMPTY(subc->refdes),
+									gen_locked(subc));
 			break;
 		}
 	case PCB_TYPE_TEXT:
@@ -432,12 +467,13 @@ static int ReportDialog(int argc, const char **argv, pcb_coord_t x, pcb_coord_t
 									"Direction is %d.\n"
 									"The bounding box is %$mD %$mD.\n"
 									"%s\n"
-									"%s", USER_UNITMASK, text->ID, pcb_strflg_f2s(text->Flags, PCB_TYPE_TEXT),
+									"%s"
+									"%s%s%s", USER_UNITMASK, text->ID, pcb_strflg_f2s(text->Flags, PCB_TYPE_TEXT, NULL),
 									text->X, text->Y, PCB_SCALE_TEXT(PCB_FONT_CAPHEIGHT, text->Scale),
 									text->TextString, text->Direction,
 									text->BoundingBox.X1, text->BoundingBox.Y1,
 									text->BoundingBox.X2, text->BoundingBox.Y2,
-									(type == PCB_TYPE_TEXT) ? laynum : "It is an element name.", PCB_FLAG_TEST(PCB_FLAG_LOCK, text) ? "It is LOCKED.\n" : "");
+									(type == PCB_TYPE_TEXT) ? laynum : "It is an element name.", gen_locked(text), gen_term(text));
 			break;
 		}
 	case PCB_TYPE_LINE_POINT:
@@ -720,7 +756,7 @@ got_net_name:
 	return 0;
 }
 
-static int ReportNetLengthByName(const char *tofind, int x, int y)
+static int ReportNetLengthByName(const char *tofind, pcb_coord_t x, pcb_coord_t y)
 {
 	char *netname = 0;
 	pcb_coord_t length = 0;
@@ -766,20 +802,11 @@ static int ReportNetLengthByName(const char *tofind, int x, int y)
 		}
 
 		if (pcb_rat_seek_pad(net->Entry, &conn, pcb_false)) {
-			switch (conn.type) {
-			case PCB_TYPE_PIN:
-				x = ((pcb_pin_t *) (conn.ptr2))->X;
-				y = ((pcb_pin_t *) (conn.ptr2))->Y;
-				net_found = 1;
-				break;
-			case PCB_TYPE_PAD:
-				x = ((pcb_pad_t *) (conn.ptr2))->Point1.X;
-				y = ((pcb_pad_t *) (conn.ptr2))->Point1.Y;
+			pcb_obj_center(conn.obj, &x, &y);
+			if ((conn.obj->type == PCB_OBJ_PIN) || (conn.obj->type == PCB_OBJ_PAD) || (conn.obj->term != NULL)) {
 				net_found = 1;
 				break;
 			}
-			if (net_found)
-				break;
 		}
 	}
 
@@ -875,6 +902,10 @@ static int Report(int argc, const char **argv, pcb_coord_t x, pcb_coord_t y)
 		pcb_gui->get_coords("Click on an object", &x, &y);
 		return ReportDialog(argc - 1, argv + 1, x, y);
 	}
+	else if (pcb_strncasecmp(argv[0], "Subc", 4) == 0) {
+		pcb_gui->get_coords("Click on a subcircuit", &x, &y);
+		return ReportDialog(argc, argv, x, y);
+	}
 	else if (pcb_strcasecmp(argv[0], "DrillReport") == 0)
 		return ReportDrills(argc - 1, argv + 1, x, y);
 	else if (pcb_strcasecmp(argv[0], "FoundPins") == 0)
