@@ -32,6 +32,6 @@
 #define REPORT_TYPES \
 	(PCB_TYPE_VIA | PCB_TYPE_LINE | PCB_TYPE_TEXT | PCB_TYPE_POLYGON | PCB_TYPE_ELEMENT | \
 	 PCB_TYPE_RATLINE | PCB_TYPE_PIN | PCB_TYPE_PAD | PCB_TYPE_ELEMENT_NAME | PCB_TYPE_ARC \
-	 | PCB_TYPE_POLYGON_POINT | PCB_TYPE_LINE_POINT)
+	 | PCB_TYPE_POLYGON_POINT | PCB_TYPE_LINE_POINT | PCB_TYPE_SUBC_PART | PCB_TYPE_SUBC)
 
 #endif
